Package: switchSelection
Type: Package
Title: Endogenous Switching and Sample Selection Regression Models
Version: 2.1.0
Date: 2026-04-15
Authors@R: c(person("Bogdan", "Potanin", email = "bogdanpotanin@gmail.com", role = c("aut", "cre", "ctb")), person("Sofiia", "Dolgikh", email = "sidolgikh@gmail.com", role = c("ctb")))
Description: Estimate the parameters of multivariate endogenous switching and sample selection models using methods described in Newey (2009) <doi:10.1111/j.1368-423X.2008.00263.x>, E. Kossova, B. Potanin (2018) <https://ideas.repec.org/a/ris/apltrx/0346.html>, E. Kossova, L. Kupriianova, B. Potanin (2020) <https://ideas.repec.org/a/ris/apltrx/0391.html> and E. Kossova, B. Potanin (2022) <https://ideas.repec.org/a/ris/apltrx/0455.html>. 
License: GPL (>= 2)
Imports: Rcpp (>= 1.1.1), hpa (>= 1.3.4), mnorm (>= 1.2.3), gena (>=
        1.0.1), dplyr (>= 1.1.4), methods
LinkingTo: Rcpp, RcppArmadillo, hpa, mnorm
Depends: R (>= 3.5.0)
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2026-04-15 15:00:33 UTC; Bogdan
Author: Bogdan Potanin [aut, cre, ctb],
  Sofiia Dolgikh [ctb]
Maintainer: Bogdan Potanin <bogdanpotanin@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-15 15:30:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 08:38:18 UTC; unix
Archs: switchSelection.so.dSYM

Package: switchSelection
Type: Package
Title: Endogenous Switching and Sample Selection Regression Models
Version: 2.1.0
Date: 2026-04-15
Authors@R: c(person("Bogdan", "Potanin", email = "bogdanpotanin@gmail.com", role = c("aut", "cre", "ctb")), person("Sofiia", "Dolgikh", email = "sidolgikh@gmail.com", role = c("ctb")))
Description: Estimate the parameters of multivariate endogenous switching and sample selection models using methods described in Newey (2009) <doi:10.1111/j.1368-423X.2008.00263.x>, E. Kossova, B. Potanin (2018) <https://ideas.repec.org/a/ris/apltrx/0346.html>, E. Kossova, L. Kupriianova, B. Potanin (2020) <https://ideas.repec.org/a/ris/apltrx/0391.html> and E. Kossova, B. Potanin (2022) <https://ideas.repec.org/a/ris/apltrx/0455.html>. 
License: GPL (>= 2)
Imports: Rcpp (>= 1.1.1), hpa (>= 1.3.4), mnorm (>= 1.2.3), gena (>=
        1.0.1), dplyr (>= 1.1.4), methods
LinkingTo: Rcpp, RcppArmadillo, hpa, mnorm
Depends: R (>= 3.5.0)
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2026-04-15 15:00:33 UTC; Bogdan
Author: Bogdan Potanin [aut, cre, ctb],
  Sofiia Dolgikh [ctb]
Maintainer: Bogdan Potanin <bogdanpotanin@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-15 15:30:02 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-04-17 07:42:37 UTC; unix
Archs: switchSelection.so.dSYM

useDynLib(switchSelection, .registration=TRUE)
importFrom(Rcpp, evalCpp)
import(hpa, mnorm, gena, dplyr, methods)

importFrom("stats", "AIC", "as.formula", "binomial", "coef",
           "complete.cases", "cov", "glm", "lm", "lm.influence",
           "logLik", "model.frame", "na.exclude", "na.omit", "na.pass",
           "optim", "pchisq", "pnorm", "predict", "qnorm", "residuals",
           "terms", "update", "nobs", "sigma", "fitted", "vcov",
           "printCoefmat", "rnorm", "quantile", "sd")
importFrom("utils", "hasName", "methods")
importFrom("methods", "is")
importFrom("dplyr", "full_join")

export("msel")

export("loocv", "boot")

export("formula_merge", "formula_split")

export("starsVector")

export("test_msel", "lrtest_msel")

export("update_msel")

export("bootstrap_msel", "bootstrap_combine_msel")

export("struct_msel")

S3method("predict", "msel")
S3method("summary", "msel")
S3method("logLik", "msel")
S3method("print", "summary.msel")
S3method("print", "msel")
S3method("nobs", "msel")
S3method("coef", "msel")
S3method("vcov", "msel")
S3method("formula", "msel")
S3method("sigma", "msel")
S3method("fitted", "msel")

S3method("summary", "test_msel")
S3method("print", "summary.test_msel")

S3method("print", "lrtest_msel")
S3method("summary", "lrtest_msel")
S3method("print", "summary.lrtest_msel")

S3method("print", "struct_msel")

export("vcov.msel")

# Store parameters for msel
par_msel <- function(object, par, type = "par")
{
  # Get some variables
  is_het           <- object$other$is_het
  is_marginal      <- object$other$is_marginal
  n_coef2          <- object$control_lnL$n_coef2
  n_coef3          <- object$control_lnL$n_coef3
  n_sigma          <- object$other$n_sigma
  n_sigma2         <- object$other$n_sigma2
  coef_ind         <- object$ind$coef
  coef_var_ind     <- object$ind$coef_var
  cuts_ind         <- object$ind$cuts
  coef2_ind        <- object$ind$coef2
  sigma_ind        <- object$ind$sigma
  var2_ind         <- object$ind$var2
  cov2_ind         <- object$ind$cov2
  sigma2_ind       <- object$ind$sigma2
  sigma3_ind       <- object$ind$sigma3
  marginal_par_ind <- object$ind$marginal_par
  marginal_par_n   <- object$control_lnL$marginal_par_n
  sigma_omit       <- object$control_lnL$sigma_omit
  cov2_omit        <- object$other$cov2_omit
  coef3_ind        <- object$ind$coef3
  y_names          <- object$other$y_names
  z_names          <- object$other$z_names
  n_eq             <- object$control_lnL$n_eq
  n_eq2            <- object$control_lnL$n_eq2
  n_eq3            <- object$other$n_eq3
  n_regimes        <- object$control_lnL$n_regimes
  regimes_pair     <- object$other$regimes_pair
  colnames_X       <- object$other$colnames_X
  colnames_W_mean  <- object$other$colnames_W_mean
  colnames_W_var   <- object$other$colnames_W_var
  colnames_W_mn    <- object$other$colnames_W_mn
  estimator        <- object$estimator
  degrees          <- object$degrees
  is1              <- object$other$is1
  is2              <- object$other$is2
  is3              <- object$other$is3
  type3            <- object$type3
  
  # Coefficients of the ordered equations
  coef     <- vector(mode = "list", length = n_eq)
  coef_var <- vector(mode = "list", length = n_eq)
  cuts     <- vector(mode = "list", length = n_eq)
  if (is1)
  {
    for (i in 1:n_eq)
    {
      coef[[i]] <- par[coef_ind[[i]]]
      names(coef[[i]]) <- colnames_W_mean[[i]]
      if (is_het[i])
      {
        coef_var[[i]] <- par[coef_var_ind[[i]]]
        names(coef_var[[i]]) <- colnames_W_var[[i]]
      }
      cuts[[i]] <- par[cuts_ind[[i]]]
      names(cuts[[i]]) <- paste0("cut ", 1:length(cuts[[i]]))
    }
    names(coef)     <- z_names
    names(coef_var) <- z_names
    names(cuts)     <- z_names
  }
  
  # Coefficients of the continuous equations
  coef2 <- list()
  if (is2)
  {
    coef2 <- vector(mode = "list", length = n_eq2)
    for (i in 1:n_eq2)
    {
      coef2[[i]] <- matrix(nrow = n_regimes[i], ncol = n_coef2[i])
      for (j in 1:n_regimes[i])
      {
        coef2[[i]][j ,] <- par[coef2_ind[[i]][j ,]]
      }
      rownames(coef2[[i]]) <- paste0("regime ", 0:(n_regimes[i] - 1))
      colnames(coef2[[i]]) <- colnames_X[[i]]
    }
    names(coef2) <- y_names
  }
  
  # Variances of the continuous equations
  var2 <- list()
  if (is2 & (estimator == "ml"))
  {
    var2 <- vector(mode = "list", length = n_eq2)
    for (i in 1:n_eq2)
    {
      var2[[i]] <- par[var2_ind[[i]]]
      names(var2[[i]]) <- paste0("regime ", 0:(n_regimes[i] - 1))
    }
    names(var2)  <- y_names
  }
  
  # Covariances between the continuous and the ordered equations
  cov2 <- list()
  if (is1 & is2 & (estimator == "ml"))
  {
    cov2 <- vector(mode = "list", length = n_eq2)
    for (i in 1:n_eq2)
    {
      cov2[[i]] <- matrix(0, nrow = n_regimes[i], ncol = n_eq)
      for (j in 1:n_regimes[i])
      {
        cov2[[i]][j, cov2_omit[[i]][j, ] == 0] <- par[
          cov2_ind[[i]][j, cov2_omit[[i]][j, ] == 0]]
      }
      rownames(cov2[[i]]) <- paste0("regime ", 0:(n_regimes[i] - 1))
      colnames(cov2[[i]]) <- z_names
    }
    names(cov2)  <- y_names
  }
  
  # Covariances between the ordered equations
  sigma <- matrix(NA, nrow = n_eq, ncol = n_eq)
  sigma_vec_ind <- numeric()
  if (is1)
  {
    diag(sigma) <- rep(1, n_eq)
    if (n_eq >= 2)
    {
      sigma_vec <- par[sigma_ind]
      sigma_vec_ind <- matrix(NA, nrow = n_sigma, ncol = 2)
      counter <- 1
      for (i in 2:n_eq)
      {
        for (j in 1:(i - 1))
        {
          if (sigma_omit[i, j] == 0)
          {
            sigma_vec_ind[counter, ] <- c(i, j)
            sigma[i, j] <- sigma_vec[counter]
            sigma[j, i] <- sigma[i, j]
            counter <- counter + 1
          }
        }
      }
    }
    colnames(sigma) <- z_names
    rownames(sigma) <- z_names
  }
  
  # Covariances between the continuous equations
  sigma2  <- list()
  counter <- 1
  if ((n_eq2 >= 2) & (estimator == "ml"))
  {
    sigma2 <- vector(mode = "list", length = n_sigma2)
    for (i in 2:n_eq2)
    {
      for (j in 1:(i - 1))
      {
        sigma2[[counter]] <- par[sigma2_ind[[counter]]]
        names(sigma2)[counter] <- paste0(y_names[i], ", ", y_names[j])
        if (length(regimes_pair[[counter]]) > 0)
        {
          for (t in 1:nrow(regimes_pair[[counter]]))
          {
            names(sigma2[[counter]])[t] <- paste0("(",
                                                  regimes_pair[[counter]][t, 1], 
                                                  ",",
                                                  regimes_pair[[counter]][t, 2],
                                                  ")")
          }
        }
        counter <- counter + 1
      }
    }
  }
  
  # Marginal distribution parameters
  marginal_par <- vector(mode = "list", length = n_eq)
  if (is_marginal)
  {
    for (i in 1:n_eq)
    {
      if (marginal_par_n[i] > 0)
      {
        marginal_par[[i]]        <- par[marginal_par_ind[[i]]]
        names(marginal_par[[i]]) <- paste0("par", 1:marginal_par_n[i])
      }
    }
    names(marginal_par) <- z_names
  }
  
  
  # Coefficients of the multinomial equations
  coef3 <- numeric()
  if (is3)
  {
    coef3 <- matrix(NA, nrow = n_eq3 - 1, ncol = n_coef3)
    rownames(coef3) <- 1:(n_eq3 - 1)
    for (i in 1:(n_eq3 - 1))
    {
      coef3[i, ] <- par[coef3_ind[i, ]]
    }
    colnames(coef3) <- colnames_W_mn
    rownames(coef3) <- paste0("alt", 0:(n_eq3 - 2))
  }
  
  # Covariances of the multinomial equation
  sigma3 <- matrix(1)
  if(is3 & (type3 == "probit"))
  {
    sigma3       <- matrix(NA, nrow = n_eq3 - 1, ncol = n_eq3 - 1)
    sigma3[1, 1] <- 1
    sigma3_vec   <- NULL
    if (n_eq3 > 2)
    {
      counter <- 1;
      sigma3_vec <- par[sigma3_ind]
      for (i in 1:(n_eq3 - 1))
      {
        for (j in 1:i)
        {
          if (!((i == 1) & (j == 1)))
          {
            sigma3[i, j] <- sigma3_vec[counter]
            sigma3[j, i] <- sigma3[i, j];
            counter <- counter + 1
          }
        }
      }
    }
    colnames(sigma3) <- paste0("alt", 0:(n_eq3 - 2))
    rownames(sigma3) <- colnames(sigma3)
  }
  
  # If the model should be updated
  if (type == "object")
  {
    object$par                 <- par
    object$coef                <- coef
    object$coef_var            <- coef_var
    object$cuts                <- cuts
    object$coef2               <- coef2
    object$var2                <- var2
    object$cov2                <- cov2
    object$sigma               <- sigma
    object$sigma2              <- sigma2
    object$marginal_par        <- marginal_par
    object$other$sigma_vec_ind <- sigma_vec_ind
    object$coef3               <- coef3
    object$sigma3              <- sigma3
    
    return(object)
  }
  
  # Store the output
  out <- list(par           = par,           coef         = coef, 
              coef_var      = coef_var,      cuts         = cuts,                 
              coef2         = coef2,         var2         = var2,                 
              cov2          = cov2,          sigma        = sigma,               
              sigma2        = sigma2,        marginal_par = marginal_par, 
              sigma_vec_ind = sigma_vec_ind, coef3        = coef3,
              sigma3        = sigma3)
  
  # Return the result
  return(out)
}

#' Stars for p-values
#' @description This function assigns stars 
#' (associated with different significance levels) to p-values.
#' @param p_value vector of values between 0 and 1 representing p-values.
#' @details Three stars are assigned to p-values not greater than 0.01.
#' Two stars are assigned to p-values greater than 0.01 and not greater 
#' than 0.05.
#' One star is assigned to p-values greater than 0.05 and not greater than 0.1.
#' @return The function returns a string vector of stars assigned according
#' to the rules described in 'Details' section.
#' @examples 
#' p_value <- c(0.002, 0.2, 0.03, 0.08)
#' starsVector(p_value)
starsVector <- function(p_value)
{
  n <- length(p_value)
  stars <- rep(NA, n)
  
  for (i in 1:n)
  {
    if (p_value[i] > 0.1)
    {
      stars[i] <- ""
      next
    }
    
    if ((p_value[i] <= 0.1) & (p_value[i] > 0.05))
    {
      stars[i] <- "*"
      next
    }
    
    if ((p_value[i] <= 0.05) & (p_value[i] > 0.01))
    {
      stars[i] <- "**"
      next
    }
    if (p_value[i] <= 0.01)
    {
      stars[i] <- "***"
      next
    }
  }
  
  return(stars)
}

remove_column <- function(mat, name, action = TRUE)
{
  if (!action)
  {
    return (mat)
  }
  mat <- mat[, -which(colnames(mat) == name), drop = FALSE]
}

#' Modify exogenous variables in data frame
#' @description Change some values of the exogenous variables in a data frame.
#' @param exogenous list such that \code{exogenous[[i]]} represents the value 
#' (or a vector of values of the same size as \code{nrow(newdata)}) which will 
#' be exogenously assigned to the variable \code{names(exogenous)[[i]]} in 
#' \code{newdata} i.e., \code{newdata[, names(exogenous)[i]] <- exogenous[[i]]}.
#' If \code{newdata} is \code{NULL} and \code{exogenous} is not \code{NULL} then
#' \code{newdata} is set to \code{object$data}.
#' This argument is especially useful for causal inference when some endogenous 
#' (dependent) variables should be exogenously assigned with some values i.e.,
#' in the right hand side of the \code{formula} and \code{formula2}.
#' The purpose of the \code{exogenous} argument is just a convenience so
#' equivalently it is possible to exogenously provide the values to variables
#' via \code{newdata} argument.
#' @param newdata data frame.
#' @details This function changes \code{exogenous} variables in \code{newdata}.
#' @return The function returns data frame which is similar to \code{newdata} 
#' but some values of this data frame are set according to \code{exogenous}.
exogenous_fn <- function(exogenous, newdata)
{
  n_exogenous <- length(exogenous)
  exogenous_names <- names(exogenous)
  if (!is.list(exogenous))
  {
    stop ("Argument 'exogenous' should be a list.")
  }
  n_obs <- nrow(newdata)
  for (i in 1:n_exogenous)
  {
    n_exogenous_i <- length(exogenous[[i]])
    if (!is.numeric(exogenous[[i]]) | ((n_exogenous_i > 1) & 
                                       (n_exogenous_i != n_obs)))
    {
      stop (paste0("Elements of 'exogenous' argument should be ",
                   "numeric values of appropriate size but ",
                   "exogenous[[", i, "]] is not."))
    }
    if (!(exogenous_names[i] %in% colnames(newdata)))
    {
      stop (paste0("Incorrect value of exogenous[[", i, "]] since there ",
                   "is no variable ", exogenous_names[i], " in the data."))
    }
    newdata[, exogenous_names[i]] <- exogenous[[i]]
  }
  
  return (as.data.frame(newdata))
}

# Create the clusters
cluster_msel <- function(object, cluster = NULL)
{
  # Take the default cluster
  if (is.null(cluster))
  {
    cluster <- object$cluster
  }
  
  # Check for character
  if (is.character(cluster))
  {
    cluster_str <- "~"
    for (i in 1:length(cluster))
    {
      cluster_str <- paste0(cluster_str, "+", cluster[i])
    }
    cluster <- as.formula(cluster_str)
  }
  
  # Check whether clustering is required
  is_cluster <- is(object = cluster, class2 = "formula")
  
  # Set the default values
  n_cluster   <- object$other$n_obs
  ind_cluster <- 1:n_cluster
  
  # Transform formula to variable
  if (is_cluster)
  {
    data_cluster        <- model.frame(as.formula(cluster), object$data, 
                                       na.action = na.pass)
    ind_cluster         <- as.factor(do.call(paste0, data_cluster))
    n_cluster           <- length(levels(ind_cluster))
    levels(ind_cluster) <- 1:n_cluster
    ind_cluster         <- as.numeric(ind_cluster)
  }
  
  # Validate clustering
  if (is_cluster & any(is.na(ind_cluster)))
  {
    stop (paste0("There are undefined clusters. ",
                 "Please, check that variables in 'cluster' have ",
                 "no missing values."))
  }
  
  # Store to the output
  out <- list(ind_cluster = ind_cluster, 
              n_cluster   = n_cluster, 
              is_cluster  = is_cluster)
  
  # Return the results
  return (out)
}

#' Formulas of msel model.
#' @description Provides formulas associated with the 
#' object of class 'msel'.
#' @param x object of class 'msel'.
#' @param ... further arguments (currently ignored).
#' @param type character; 
#' if \code{type = "formula"} or \code{type = 1} then function returns formulas 
#' of the ordinal equations. 
#' If \code{type = "formula2"} or \code{type = 2} then function returns formulas 
#' of the continuous equations.
#' If \code{type = "formula3"} or \code{type = 3} then function returns formula 
#' of the multinomial equation.
#' @param eq positive integer representing the index of the equation which
#' formula should be returned. If \code{NULL} (default) then formulas for each
#' equation will be returned as a list which \code{i}-th element associated 
#' with \code{i}-th equation.
#' @return Returns a formula or a list of formulas depending on \code{eq} value.
formula.msel <- function(x, ..., type = "formula", eq = NULL) 
{
  if (length(list(...)) > 0)
  {
    warning("Additional arguments passed through ... are ignored.")   
  }
  
  # Validation
  if (type == 1)
  {
    type <- "formula"
  }
  if (type == 2)
  {
    type <- "formula2"
  }
  if (type == 3)
  {
    type <- "formula3"
  }
  if (type == "formula")
  {
    if (!is.null(eq))
    {
      if ((eq <= 0) | (eq > x$control_lnL$n_eq))
      {
        stop("Incorrect 'eq' value.")
      }
    }
  }
  if (type == "formula2")
  {
    if (!x$other$is2)
    {
      return (NULL)
    }
    if (!is.null(eq))
    {
      if ((eq <= 0) | (eq > x$control_lnL$n_eq2))
      {
        stop("incorrect 'eq' value.")
      }
    }
  }
  if (type == "formula3")
  {
    if (!x$other$is3)
    {
      return (NULL)
    }
  }
  if (!(type %in% c("formula", "formula2", "formula3")))
  {
    stop("Incorrect 'type' argument.")
  }
  
  # Formula of the ordinal equation
  if (type == "formula")
  {
    if (x$control_lnL$n_eq == 1)
    {
      eq <- 1
    }
    if (!is.null(eq))
    {
      return(x$formula[[eq]])
    }
    return(x$formula)
  }
  
  # Formula of the continuous equation
  if (type == "formula2")
  {
    if (x$control_lnL$n_eq2 == 1)
    {
      eq <- 1
    }
    if (!is.null(eq))
    {
      return(x$formula2[[eq]])
    }
    return(x$formula2)
  }
  
  # Formula of the multinomial equation
  if (type == "formula3")
  {
    return(x$formula3)
  }
  
  return (NULL)
}

#' Merge formulas
#' @description This function merges all variables of several formulas
#' into a single formula.
#' @param ... formulas to be merged such that there is a single element
#' on the left hand side and various elements on the right hand side.
#' @param type string representing the type of merge to be used.
#' If \code{type = "all"} then both right hand side and left hand side
#' elements of the formulas will be merged on the right hand side.
#' If \code{type = "terms"} then only right hand side elements of the
#' formulas will be merged on the right hand side.
#' If \code{type = "var-terms"} then the result is the same as in case
#' when \code{type = "terms"} but there will be left hand side element
#' of the first formula on the left hand side of the merged formula.
#' @details Merged formulas should have a single element on the left hand
#' side and arbitrary number of elements on the right hand side.
#' @return This function returns a formula which form depends on 
#' \code{type} input argument value. See 'Details' for additional information.
#' @examples 
#' # Consider three formulas
#' f1 <- as.formula("y1 ~ x1 + x2")
#' f2 <- as.formula("y2 ~ x2 + x3")
#' f3 <- as.formula("y3 ~ y2 + x6")
#' # Merge these formulas in a various ways
#' formula_merge(f1, f2, f3, type = "all")
#' formula_merge(f1, f2, f3, type = "terms")
#' formula_merge(f1, f2, f3, type = "var-terms")
#' 
formula_merge <- function(..., type = "all")
{
  formulas <- list(...)
  if (is.list(formulas[[1]]))
  {
    formulas <- formulas[[1]]
  }
  f.y <- NULL
  f.x <- NULL
  for (i in 1:length(formulas))
  {
    f.x <- c(f.x, labels(terms(formulas[[i]])))
    f.y <- c(f.y, all.vars(formulas[[i]])[1])
  }
  
  if (type == "all")
  {
    f.c <- unique(c(f.x, f.y))
  }
  if (type %in% c("terms", "var-terms"))
  {
    f.c <- unique(f.x)
  }
  
  frm <- do.call(paste, c(as.list(f.c), sep = " + "))
  frm <- paste("~ ", frm)
  if(type == "var-terms")
  {
    frm <- paste(f.y[1], frm)
  }
  frm <- as.formula(frm)
  
  return(frm)
}

#' Split formula by symbol
#' @description This function splits one formula into two formulas
#' by symbol.
#' @param formula an object of class \code{formula}.
#' @param symbol a string that is used to split \code{formula} into
#' two formulas.
#' @details The \code{symbol} should be on the right hand side of
#' the formula. 
#' @return This function returns a list of two formulas.
#' @examples 
#' formula_split("y ~ x1 + x2 | x2 + x3")
#' formula_split("y ~ x1 + x2 : x2 + x3", symbol = ":")
#' 
formula_split <- function(formula, symbol = "|")
{
  formula <- as.formula(formula)
  fs <- paste(formula[2], formula[3], sep = '~')
  formula <- strsplit(x = fs, split = symbol, fixed = TRUE)
  
  f1y <- strsplit(x = formula[[1]][1], split = "~", fixed = TRUE) 
  y <- f1y[[1]][1]
  f1 <- f1y[[1]][2]
  f2 <- formula[[1]][2]
  
  f1 <- as.formula(paste(y, "~", f1))
  f2 <- as.formula(paste(y, "~", f2))
  
  f_out <- list(f1 = f1,
                f2 = f2)
  return(f_out)
}

# Create formulas
formula_msel <- function(object, formula, formula2, formula3, 
                         degrees = NULL, degrees3 = NULL)
{
  # Get some variables
  is1        <- object$other$is1
  is2        <- object$other$is2
  is3        <- object$other$is3
  estimator  <- object$estimator
  n_eq       <- object$other$n_eq
  n_eq2      <- object$other$n_eq2
  n_eq3      <- object$other$n_eq3
  
  # Include lambda into formula2 according to degrees
  if ((estimator == "2step") & is1 & (!is.null(degrees)))
  {
    for (v in seq_len(n_eq2))
    {
      for (j in seq_len(n_eq))
      {
        if (degrees[v, j, drop = FALSE] != 0)
        {
          for (j1 in seq_len(degrees[v, j]))
          {
            # non-interaction terms
            if (j1 == 1)
            {
              formula2[[v]] <- update(formula2[[v]], paste0("~. + lambda", j))
            }
            else
            {
              formula2[[v]] <- update(formula2[[v]], 
                                      paste0("~. + I(lambda", j, "^", j1, ")"))
            }
          }
        }
      }
    }
  }
  
  # Include lambda_mn into formula3 according to degrees3
  if ((estimator == "2step") & is3 & (!is.null(degrees3)))
  {
    for (v in seq_len(n_eq2))
    {
      for (j in seq_len(ncol(degrees3)))
      {
        if (degrees3[v, j, drop = FALSE] != 0)
        {
          for (j1 in seq_len(degrees3[v, j]))
          {
            # non-interaction terms
            if (j1 == 1)
            {
              formula2[[v]] <- update(formula2[[v]], 
                                      paste0("~. + lambda", j, "_mn"))
            }
            else
            {
              formula2[[v]] <- update(formula2[[v]], 
                                      paste0("~. + I(lambda", j, "_mn", 
                                             "^", j1, ")"))
            }
          }
        }
      }
    }
  }
  
  # Coerce formula to type 'formula' and check whether it has "|" symbol
  # which indicates the presence of heteroscedasticity.
  # If there is heteroscedasticity then store separate formulas for
  # mean and variance equations.
  is_het       <- rep(FALSE, n_eq)
  formula_mean <- vector(mode = "list", length = n_eq)
  formula_var  <- vector(mode = "list", length = n_eq)
  if (is1)
  {
    for (i in seq_len(n_eq))
    {
      formula[[i]] <- as.formula(formula[[i]])
      frm_tmp      <- paste(formula[[i]][2], formula[[i]][3], sep = '~')
      is_het[i]    <- grepl(x = frm_tmp, pattern = "|", fixed = TRUE)
      if (is_het[i])
      {
        frm_tmp           <- formula_split(formula[[i]])
        formula_mean[[i]] <- frm_tmp[[1]]
        formula_var[[i]]  <- frm_tmp[[2]]
      }
      else
      {
        formula_mean[[i]] <- formula[[i]]
      }
    }
  }
  
  # Find selectivity terms
  coef_lambda_ind <- list()
  if (estimator == "2step")
  {
    coef_lambda_ind <- vector(mode = "list", length = n_eq2)
    for (v in 1:n_eq2)
    {
      formula2_terms <- attr(terms(formula2[[v]]), "term.labels")
      # add 1 for the intercept
      coef_lambda_ind[[v]] <- which(grepl("lambda", formula2_terms, 
                                          fixed = TRUE)) + 1
    }
  }
  
  # Return the results
  out <- list(is_het       = is_het,       formula     = formula,
              formula2     = formula2,     formula3    = formula3,
              formula_mean = formula_mean, formula_var = formula_var,
              coef_lambda_ind = coef_lambda_ind)
}

#' Bootstrap covariance matrix for least squares estimates of linear regression
#' @description This function calculates bootstrapped covariance matrix
#' for least squares estimates of linear regression. The estimates should be
#' obtained via \code{lm} function.
#' @param model object of class \code{lm}.
#' @param iter positive integer representing the number of bootstrap iterations.
#' @details Calculations may take long time for high \code{iter} value.
#' @return This function returns a bootstrapped covariance matrix of the
#' least squares estimator.
#' @examples 
#' set.seed(123)
#' # Generate data according to linear regression
#' n <- 20
#' eps <- rnorm(n)
#' x <- runif(n)
#' y <- x + eps
#' # Estimate the model
#' model <- lm(y ~ x)
#' # Calculate bootstrap covariance matrix
#' boot(model, iter = 50)
boot <- function(model, iter = 100)
{
  if (class(model)[1] != "lm")
  {
    stop(paste0("Argument 'model' should be an object of class 'lm', ", 
                "not '", class(model)[1], "'.\n"))
  }
  if (((iter %% 1) != 0) | (iter <= 0))
  {
    stop(paste0("Wrong 'iter' argument. ", 
                "Please ensture that it is a positive integer.\n"))
  }
  data <- model.frame(model)
  coef.names <- names(data)
  n <- nrow(data)
  model.boot <- vector(mode = "list", length = iter)
  coef.boot <- matrix(NA, nrow = iter, ncol = ncol(data))
  for (i in 1:iter)
  {
    ind.boot <- sample(1:n, size = n, replace = TRUE)
    data.boot <- data[ind.boot, ]
    model.boot[[i]] <- lm(paste0(coef.names[1], " ~ ."), 
                          data = data.boot)
    coef.boot[i, ] <- coef(model.boot[[i]])
  }
  coef.names[1] <- "(Intercept)"
  cov.boot <- cov(coef.boot)
  colnames(cov.boot) <- coef.names
  rownames(cov.boot) <- coef.names
  return(cov.boot)
}

#' Leave-one-out cross-validation
#' @description This function calculates root mean squared error (RMSE) for
#' leave-one-out cross-validation of linear regression estimated via
#' least squares method.
#' @param fit object of class \code{lm}.
#' @details Fast analytical formula is used.
#' @return This function returns a numeric value representing root mean squared 
#' error (RMSE) of leave-one-out cross-validation (LOOCV).
#' @examples 
#' set.seed(123)
#' # Generate data according to linear regression
#' n <- 100
#' eps <- rnorm(n)
#' x <- runif(n)
#' y <- x + eps
#' # Estimate the model
#' model <- lm(y ~ x)
#' # Perform cross-validation
#' loocv(model)
loocv <- function(fit)
{
  if (class(fit)[1] != "lm")
  {
    stop(paste0("Argument 'fit' should be an object of class 'lm', ", 
                "not '", class(fit)[1], "'.\n"))
  }
  return(sqrt(mean((residuals(fit) / (1 - lm.influence(fit)$h)) ^ 2)))
}

#' Coefficients extraction method for msel.
#' @description Extract coefficients and other estimates from msel object.
#' @param object an object of class "msel".
#' @param ... further arguments (currently ignored).
#' @param eq an integer representing the index of the ordered equation.
#' @param eq2 an integer representing the index of the continuous equation.
#' @param eq3 an integer representing the index of the alternative of the
#' multinomial equation.
#' @param regime an integer representing a regime of the continuous equation.
#' @param type a character representing a type of the output. Possible options
#' are \code{"coef"}, \code{"coef2"}, \code{"coef_lambda"}, 
#' \code{"coef_var"}, \code{"coef3"}, \code{"cuts"}, \code{"cov"}, 
#' \code{"cov1"}, \code{"cov12"}, \code{"var"}, \code{"cov2"}, \code{"cov3"}, 
#' and \code{"marginal"}.
#' See 'Details' for additional information.
#' @template coef_msel_details_Template
#' @returns See 'Details' section.
coef.msel <- function (object, ..., 
                       eq = NULL, eq2 = NULL, eq3 = NULL,
                       regime = NULL, type = "coef")
{
  # Validate dots
  if (length(list(...)) > 0)
  {
    warning("Additional arguments passed through ... are ignored.")   
  }
  
  # Some variables
  n_eq            <- object$other$n_eq
  n_eq2           <- object$other$n_eq2
  n_eq3           <- object$other$n_eq3
  is1             <- object$other$is1
  is2             <- object$other$is2
  is3             <- object$other$is1
  n_regimes       <- object$other$n_regimes
  z_names         <- object$other$z_names
  y_names         <- object$other$y_names
  estimator       <- object$estimator
  coef_lambda_ind <- object$other$coef_lambda_ind
  
  # Validation of the type
  type <- tolower(type)
  if ((type == "coefficients") | (type == "coeff"))
  {
    type <- "coef"
    warning("It is assumed that 'type' is 'coef'.")
  }
  if ((type == "coefficients2") | (type == "coeff2"))
  {
    type <- "coef2"
    warning("It is assumed that 'type' is 'coef2'.")
  }
  if ((type == "lambda") | (type == "coef2_lambda"))
  {
    type <- "coef_lambda"
    warning("It is assumed that 'type' is 'coef_lambda'.")
  }
  if ((type == "coefficients3") | (type == "coeff3") |
      (type == "coef_mn")       | (type == "coef_mult"))
  {
    type <- "coef3"
    warning("It is assumed that 'type' is 'coef3'.")
  }
  if ((type == "coef") & !is.null(eq2))
  {
    type <- "coef2"
    warning("Since 'eq2' is not 'NULL' it is assumed that 'type' is 'coef2'.")
  }
  if ((type == "coefficients_var") | (type == "coeff_var"))
  {
    type <- "coef_var"
    warning("It is assumed that 'type' is 'coef_var'.")
  }
  if (type == "marginal_par")
  {
    type <- "marginal"
    warning("It is assumed that 'type' is 'marginal'.")
  }
  if (type %in% c("cut", "threshold", "thresholds"))
  {
    type <- "cuts"
    warning("It is assumed that 'type' is 'cuts'.")
  }
  if (type %in% c("sigma", "sigma1"))
  {
    type <- "cov1"
    warning("It is assumed that 'type' is 'cov1'.")
  }
  if (type %in% c("sigma2"))
  {
    type <- "cov2"
    warning("It is assumed that 'type' is 'cov2'.")
  }
  if (type %in% c("sigma3"))
  {
    type <- "cov3"
    warning("It is assumed that 'type' is 'cov3'.")
  }
  
  # Convert eq into numeric if needed
  if (is1)
  {
    if (!is.null(eq))
    {
      if (is.character(eq))
      {
        eq <- which(z_names %in% eq)
      }
      if (any(eq < 1) | any(eq > n_eq))
      {
        stop("Incorrect 'eq' value.")
      }
    }
  }
  
  # Convert eq2 into numeric if needed
  if (is2)
  {
    if (!is.null(eq2))
    {
      if (!is.numeric(eq2))
      {
        eq2 <- which(y_names %in% eq2)
      }
      if (any(eq2 < 0) | any(eq2 > n_eq2))
      {
        stop("Incorrect 'eq2' value.")
      }
    }
  }
  
  # Convert eq3 into numeric if needed
  if (is3)
  {
    if (!is.null(eq3))
    {
      if (any(eq3 < 0) | any(eq3 >= n_eq3))
      {
        stop("Incorrect 'eq3' value.")
      }
    }
  }
  
  # Validate the regime
  if (is2)
  {
    if (!is.null(regime) & !is.null(eq2))
    {
      if (any(regime < 0) | any(regime >= n_regimes[eq2]))
      {
        stop("Incorrect 'regime' value.")
      }
    }
  }
  
  # Coefficients of the ordinal equations
  if (type == "coef")
  {
    if (is.null(eq))
    {
      return (object$coef)
    }
    return (object$coef[[eq]])
  }
  
  # Coefficients of the continuous equations
  if (type %in% c("coef2", "coef_lambda", "coef2_all"))
  {
    if (!is2)
    {
      stop("Available only when there is at least one continuous equation.")
    }
    if (estimator == "2step")
    {
      if (type != "coef2_all")
      {
        for (v in 1:n_eq2)
        {
          if (length(coef_lambda_ind[[v]]) > 0)
          {
            s                 <- ifelse(type == "coef_lambda", 1, -1)
            object$coef2[[v]] <- object$coef2[[v]][, s * coef_lambda_ind[[v]],
                                                     drop = FALSE]
          }
        }
      }
    }
    if (is.null(eq2) & !is.null(regime))
    {
      eq2 <- 1
    }
    if (is.null(eq2))
    {
      return (object$coef2)
    }
    if (is.null(regime))
    {
      return (object$coef2[[eq2]])
    }
    return (object$coef2[[eq2]][regime + 1, ])
  }
  
  # Coefficients of the variance equation
  if (type == "coef_var")
  {
    if (is.null(eq))
    {
      return (object$coef_var)
    }
    return (object$coef_var[[eq]])
  }
  
  # Cuts
  if (type == "cuts")
  {
    if (is.null(eq))
    {
      return (object$cuts)
    }
    return (object$cuts[[eq]])
  }
  
  # Asymptotic covariance matrix of the estimates
  if ((type == "vcov") | (type == "cov"))
  {
    return (object$cov)
  }
  
  # Covariances between the ordinal equations
  if (type == "cov1")
  {
    if (length(eq) > 1)
    {
      return(object$sigma[eq[1], eq[2], drop = FALSE])
    }
    return(object$sigma)
  }
  
  # Covariances between the continuous and ordinal equations
  if (type == "cov12")
  {
    if (!is.null(eq) & !is.null(regime) & is.null(eq2))
    {
      eq2 <- 1
    }
    if (is.null(eq) | is.null(eq2))
    {
      return (object$cov2)
    }
    if (is.null(regime))
    {
      return (object$cov2[[eq2]])
    }
    return (object$cov2[[eq2]][regime + 1, eq, drop = FALSE])
  }
  
  # Variances of the continuous equations
  if ((type == "var") | (type == "variance"))
  {
    if (!is2)
    {
      stop("Available only when there is a continous equation.\n")
    }
    if (estimator != "ml")
    {
      stop("Available only when 'estimator = ml'.\n")
    }
    if (is.null(eq2) & !is.null(regime))
    {
      eq2 <- 1
    }
    if (is.null(eq2))
    {
      if (n_eq2 == 1)
      {
        return (object$var2[[1]])
      }
      return (object$var2)
    }
    if (is.null(regime))
    {
      return (object$var2[[eq2]])
    }
    if ((regime < 0) | (regime >= n_regimes[eq2]))
    {
      stop("Incorrect 'regime' value.")
    }
    return(object$var2[[eq2]][regime[1] + 1])
  }
  
  # Covariances between the continuous equations
  if (type == "cov2")
  {
    if (n_eq2 < 2)
    {
      stop("Available only when there are more than one continuous equations.")
    }
    if (is.null(eq2))
    {
      return (object$sigma2)
    }
    if ((length(regime) != 2) | (length(eq2) != 2))
    {
      stop("Arguments 'regime' and 'eq2' should be of length '2'.")
    }
    if (eq2[1] == eq2[2])
    {
      stop("Indexes of the continuous equations 'eq2' should be distinct.")
    }
    counter <- 1
    for (i in 2:n_eq2)
    {
      for (j in 1:(i - 1))
      {
        if (((i == eq2[1]) & (j == eq2[2])) | ((i == eq2[2]) & (j == eq2[1])))
        {
          pairs <- object$other$regimes_pair[[counter]]
          ind <- which(((pairs[, 1] == regime[1]) & (pairs[, 2] == regime[2])) |
                         ((pairs[, 1] == regime[2]) & (pairs[, 2] == regime[1])))
          if (length(ind) > 0)
          {
            return (object$sigma2[[counter]][ind])
          }
          return (NA)
        }
        counter <- counter + 1
      }
    }
  }
  
  # Parameters of the marginal distribution
  if (type == "marginal")
  {
    if (is.null(eq))
    {
      return(object$marginal_par)
    }
    return(object$marginal_par[[eq]])
  }
  
  # Coefficients of the multinomial equation
  if (type == "coef3")
  {
    if (is.null(eq3))
    {
      return (object$coef3)
    }
    return (object$coef3[eq3 + 1, ])
  }
  
  # Estimate of the covariance matrix of the 
  # random errors of the multinomial equations
  if (type == "cov3")
  {
    if (length(eq3) == 1)
    {
      return (object$sigma3)
    }
    return (object$sigma3[eq3[1] + 1, eq3[2] + 1, drop = FALSE])
  }
  
  stop("Incorrect 'type' argument.")
}

#' Summary for an Object of Class msel
#' @description Provides summary for an object of class 'msel'.
#' @param object object of class 'msel'
#' @param ... further arguments (currently ignored)
#' @param vcov positively defined numeric matrix representing
#' asymptotic variance-covariance matrix of the estimator to be
#' used for calculation of standard errors and p-values. It may also be a 
#' character. Then the \code{\link[switchSelection]{vcov.msel}} function
#' will be used, whose input argument \code{type} will be set to \code{vcov}.
#' If \code{estimator = "2step"} then \code{vcov} should be an estimate of the 
#' asymptotic covariance matrix of the first step estimator.
#' @param show_ind logical; if \code{TRUE} then indexes of parameters will be
#' shown. Particularly, these indexes may be used in \code{ind} element of
#' \code{regularization} parameter of \code{\link[switchSelection]{msel}}.
#' @details If \code{vcov} is \code{NULL} then this function just changes the 
#' class of the 'msel' object to 'summary.msel'. Otherwise it 
#' additionally changes \code{object$cov} to \code{vcov} and uses it to
#' recalculate \code{object$se}, \code{object$p_value} and \code{object$tbl} 
#' values. It also adds the value of \code{show_ind} argument to the object.
#' @return Returns an object of class 'summary.msel'.
summary.msel <- function(object, ..., vcov = NULL, show_ind = FALSE) 
{
  if (length(list(...)) > 0)
  {
    warning("Additional arguments passed through ... are ignored.")   
  }
  
  # Add information on the indexes
  object$show_ind <- show_ind
  
  if (!is.null(vcov))
  {
    if (is.character(vcov))
    {
      vcov <- vcov.msel(object, type = vcov)
    }
    if (any(dim(vcov) != dim(object$cov)))
    {
      stop("Incorrect size of 'vcov' matrix.")
    }
    tbl_list       <- tbl_msel(object = object, vcov = vcov)
    object$tbl     <- tbl_list$tbl
    object$se      <- tbl_list$se
    object$p_value <- tbl_list$p_value
  }
  
  class(object) <- "summary.msel"
  
  return(object)
}

#' Print summary for an Object of Class msel
#' @description Prints summary for an object of class 'msel'.
#' @param x object of class 'msel'
#' @param ... further arguments (currently ignored)
#' @return The function returns \code{x}.
print.summary.msel <- function(x, ...)
{
  # Assign the class
  class(x) <- "msel"
  
  # Get some data
  show_ind         <- x$show_ind
  tbl_coef         <- x$tbl$coef
  tbl_coef_var     <- x$tbl$coef_var
  tbl_sigma        <- x$tbl$sigma
  tbl_cuts         <- x$tbl$cuts
  tbl_coef2        <- x$tbl$coef2
  tbl_lambda       <- x$tbl$lambda
  tbl_var2         <- x$tbl$var2
  tbl_cov2         <- x$tbl$cov2
  tbl_sigma2       <- x$tbl$sigma2
  tbl_sigma3       <- x$tbl$sigma3
  tbl_marginal_par <- x$tbl$marginal_par
  tbl_coef3        <- x$tbl$coef3
  n_eq             <- x$other$n_eq
  n_eq2            <- x$other$n_eq2
  n_eq3            <- x$other$n_eq3
  coef_ind         <- x$ind$coef
  coef_var_ind     <- x$ind$coef_var
  cuts_ind         <- x$ind$cuts
  coef2_ind        <- x$ind$coef2
  sigma_ind        <- x$ind$sigma
  sigma2_ind       <- x$ind$sigma2
  sigma3_ind       <- x$ind$sigma3
  var2_ind         <- x$ind$var2
  cov2_ind         <- x$ind$cov2
  marginal_par_ind <- x$ind$marginal_par
  n_obs            <- x$other$n_obs
  is_het           <- x$other$is_het
  n_regimes        <- x$other$n_regimes
  marginal         <- x$marginal
  marginal_par     <- x$marginal_par
  marginal_names   <- names(marginal)
  is_marginal      <- length(marginal) > 0
  is1              <- x$other$is1
  is2              <- x$other$is2
  is3              <- x$other$is3
  type3            <- x$type3
  estimator        <- x$estimator
  coef_lambda_ind  <- x$other$coef_lambda_ind
  z_mn_names       <- x$other$z_mn_names
  
  # Provide the name to the model
  cat(paste0(ifelse(estimator == "ml", "Maximum-likelihood ", "Two-step "),
             "estimator is used\n"))
  cat("--- \n")
  if (!is.na(x$logLik))
  {
    cat(paste0("Log-likelihood = ", round(x$logLik, 4), " \n"))
  }
  if (!is.na(AIC(x)))
  {
    cat(paste0("AIC = ", round(AIC(x), 4), "\n"))
  }
  cat(paste0("Observations = ", n_obs, "\n"))
  
  if (is1)
  {
    cat("--- \n")
    cat("Ordinal equations \n")
    for (i in 1:n_eq)
    {
      cat("-- \n")
      cat(paste0(x$formula[[i]][[2]], " equation \n"))
      cat("- \n");
      if (is_marginal)
      {
        cat(paste0("Distribution: ", marginal_names[i], "\n"))
        cat("- \n")
      }
      cat("Coefficients: \n")
      printCoefmat(remove_column(tbl_coef[[i]], "ind", !show_ind), 
                   has.Pvalue = TRUE, signif.legend = FALSE, P.values = TRUE)
      if (is_het[i])
      {
        cat("- \n")
        cat("Coefficients of the variance equation: \n")
        printCoefmat(remove_column(tbl_coef_var[[i]], "ind", !show_ind),
                     has.Pvalue = TRUE, signif.legend = FALSE, P.values = TRUE)
      }
      cat("- \n")
      cat("Cuts (thresholds): \n")
      printCoefmat(remove_column(tbl_cuts[[i]], "ind", !show_ind),
                   has.Pvalue = TRUE, signif.legend = FALSE, P.values = TRUE)
      if (length(marginal_par[[i]]) > 0)
      {
        cat("- \n")
        cat(paste0("Parameters of the ", marginal_names[i], 
                   " distribution: \n"))
        printCoefmat(remove_column(tbl_marginal_par[[i]], "ind", !show_ind),
                     has.Pvalue = TRUE, signif.legend = FALSE, P.values = TRUE)
      }
    }
    if ((n_eq >= 2) & (length(x$other$sigma_vec_ind) > 0))
    {
      cat("--- \n")
      cat("Correlations between the ordinal equations: \n")
      printCoefmat(remove_column(tbl_sigma, "ind", !show_ind), 
                   has.Pvalue = TRUE, signif.legend = FALSE, P.values = TRUE)
    }
  }
  
  if (is3)
  {
    cat("--- \n")
    cat(paste0(z_mn_names[1], " multinomial ", type3, " equation \n"))
    
    # Coefficients
    for (i in 1:(n_eq3 - 1))
    {
      cat("-- \n")
      cat(paste0("Alternative ", i - 1, "\n"))
      cat("- \n")
      cat(paste0("Coefficients: \n"))
      printCoefmat(remove_column(tbl_coef3[[i]], "ind", !show_ind),
                   has.Pvalue = TRUE, signif.legend = FALSE, 
                   P.values = TRUE)
    }
    
    # Covariances between the alternatives
    if (type3 == "probit")
    {
      cat("- \n")
      cat("Covariances between the alternatives: \n")
      printCoefmat(remove_column(tbl_sigma3, "ind", !show_ind), 
                   has.Pvalue = TRUE, signif.legend = FALSE, P.values = TRUE)
    }
  }
  
  if (is2)
  {
    cat("--- \n")
    cat("Continuous equations \n")
    for (i in 1:n_eq2)
    {
      cat("-- \n")
      cat(paste0(all.vars(x$formula2[[i]])[1], " equation \n"))
      for (j in 1:n_regimes[i])
      {
        cat("-- \n")
        if (n_regimes[i] > 1)
        {
          cat(paste0("Regime ", j - 1, ": \n"))
          cat("- \n")
        }
        if (estimator == "2step")
        {
          r_squared <- round(summary(x$twostep[[i]][[j]])$r.squared, 4)
          r_adj_squared <- round(summary(x$twostep[[i]][[j]])$adj.r.squared, 4)
          loocv_val <- round(loocv(x$twostep[[i]][[j]]), 4)
          cat(paste0("R-squared: ", r_squared, "\n"))
          cat(paste0("Adjusted R-squared: ", r_adj_squared, "\n"))
          cat(paste0("Leave-one-out cross-validation RMSE: ", loocv_val, "\n"))
          cat("- \n")
        }
        cat("Coefficients: \n")
        printCoefmat(remove_column(tbl_coef2[[i]][[j]], "ind", !show_ind),
                     has.Pvalue = TRUE, signif.legend = FALSE, P.values = TRUE)
        if ((estimator == "2step"))
        {
          if (length(coef_lambda_ind[[i]]) > 0)
          {
            cat("- \n")
            cat("Selectivity terms: \n")
            printCoefmat(remove_column(tbl_lambda[[i]][[j]], "ind", !show_ind),
                         has.Pvalue = TRUE, signif.legend = FALSE, 
                         P.values = TRUE)
          }
        }
        if (estimator != "2step")
        {
          cat("- \n")
          cat(paste0("Variance: \n"))
          printCoefmat(remove_column(tbl_var2[[i]][j, , drop = FALSE], "ind", 
                                     !show_ind),
                       has.Pvalue = TRUE, signif.legend = FALSE, 
                       P.values = TRUE)
          if (is1)
          {
            cat("- \n")
            cat(paste0("Covariances with the ordinal equations: \n"))
            printCoefmat(remove_column(tbl_cov2[[i]][[j]], "ind", !show_ind),
                         has.Pvalue = TRUE, signif.legend = FALSE, 
                         P.values = TRUE)
          }
        }
      }
    }
  }
  
  if ((n_eq2 >= 2) & (estimator == "ml"))
  {
    cat("--- \n")
    cat("Covariances between the continuous equations: \n")
    counter <- 1
    for (i in 2:n_eq2)
    {
      for (j in 1:(i - 1))
      {
        if (length(x$other$regimes_pair[[counter]]) > 0)
        {
          cat("- \n")
          cat(paste0("Between ", x$formula2[[i]][[2]], 
                     " and ", x$formula2[[j]][[2]], "\n"))
          printCoefmat(remove_column(tbl_sigma2[[counter]], "ind", !show_ind),
                       has.Pvalue = TRUE, signif.legend = FALSE, 
                       P.values = TRUE)
        }
        counter <- counter + 1
      }
    }
  }
  
  if (getOption("show.signif.stars"))
  {
    cat("--- \n")
    cat("Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1 \n")
  }
  
  return(x)
}

#' Print for an Object of Class msel
#' @description Prints information on the object of class 'msel'.
#' @param x object of class 'msel'
#' @param ... further arguments (currently ignored)
#' @return The function returns \code{NULL}.
print.msel <- function(x, ...)
{
  if (length(list(...)) > 0)
  {
    warning("Additional arguments passed through ... are ignored.")   
  }
  
  cat("This object of class 'msel' contains the following elements: \n")
  cat("--- \n")
  print(summary.default(x))
  cat("--- \n")
  print(struct_msel(x))
  cat("--- \n")
  cat("Use 'summary()' to get more detailed information on the output. \n")
  return(NULL)
}

#' Structure of the Object of Class msel
#' @description Prints information on the structure of the model.
#' @param x object of class 'msel'
#' @return The function returns a numeric matrix whose columns are 
#' \code{groups}, \code{groups2}, and \code{groups3}, respectively. 
#' It also has additional (last) column with the number of observations 
#' associated with the corresponding combinations of the groups.
struct_msel <- function(x)
{
  # Prepare the variable to store the output
  out <- NULL
  
  # Combine the groups
  if (x$other$is1)
  {
    out <- cbind(out, x$groups)
  }
  if (x$other$is2)
  {
    out <- cbind(out, x$groups2)
  }
  if (x$other$is3)
  {
    out                             <- cbind(out, x$groups3)
    colnames(out)[ncol(out)] <- "alt"
  }
  
  # Add the number of observations
  out <- cbind(out, obs = x$control_lnL$n_obs_g)
  rownames(out) <- rep("", nrow(out))
  
  # Assign the class
  class(out) <- "struct_msel"
  
  # Return the result
  return(out)
}

#' Print for an Object of Class struct_msel
#' @description Prints information on the object of class 'struct_msel'.
#' @param x object of class 'struct_msel'
#' @param ... further arguments (currently ignored)
#' @return The function returns \code{NULL}.
print.struct_msel <- function(x, ...)
{
  if (length(list(...)) > 0)
  {
    warning("Additional arguments passed through ... are ignored.")   
  }
  
  cat(paste0("Structure of the model i.e., correspondence between the ",
             "possible values\n",
             "of the ordinal equations and the regimes of the ",
             "continuous equations: \n"))
  cat("--- \n")
  attr(x,"class") <- NULL
  print.default(x)
  cat("obs - the number of observations with corresponding values. \n")
  
  return(NULL)
}

#' Extract Model Fitted Values
#' @description Extracts fitted values from 'msel' object
#' @param object object of class 'msel'.
#' @param ... further arguments (currently ignored).
#' @param newdata an optional data frame in which to look for variables 
#' with which to predict. If omitted, the original data frame is used. 
#' This data frame should contain values of dependent variables even if they 
#' are not actually needed for prediction (simply assign them with 0 values).
#' @returns Returns a numeric matrix. Its columns whose names coincide with the
#' names of the ordinal and multinomial equations provide the index of the most 
#' probable category for each observation.
#' Columns whose names coincide with the names of the continuous equations 
#' provide conditional expectations of the dependent variables in observable 
#' regimes for each observation.
fitted.msel <- function (object, ..., newdata = NULL)
{
  # Validate dots
  if (length(list(...)) > 0)
  {
    warning("Additional arguments passed through ... are ignored.")   
  }
  
  # Some variables
  n_eq       <- object$other$n_eq
  n_eq2      <- object$other$n_eq2
  n_eq3      <- object$other$n_eq3
  is1        <- object$other$is1
  is2        <- object$other$is2
  is3        <- object$other$is3
  n_regimes  <- object$other$n_regimes
  z_names    <- object$other$z_names
  y_names    <- object$other$y_names
  z_mn_names <- object$other$z_mn_names
  if (is.null(newdata))
  {
    newdata <- object$data
  }
  n          <- nrow(newdata)
  n_cuts_eq  <- object$other$n_cuts_eq
  
  # Output variable
  out           <- matrix(NA, nrow = n, ncol = n_eq + n_eq2 + is3)
  colnames(out) <- rep("val", ncol(out))
  rownames(out) <- 1:n
  
  # Most probable categories of the ordinal equations
  if (is1)
  {
    group <- rep(-1, n_eq)
    for (i in 1:n_eq)
    {
      probs <- matrix(NA, nrow = n, ncol = n_cuts_eq[i] + 1)
      for (j in 0:n_cuts_eq[i])
      {
        group[i]         <- j
        probs[, j + 1]   <- predict(object, type = "prob", newdata = newdata, 
                                    group = group)
        group[i]         <- -1
      }
      probs_obs               <- complete.cases(probs)
      probs_tmp               <- probs
      probs_tmp[!probs_obs, ] <- 0
      out[, i]                <- apply(X = probs_tmp, MARGIN = 1, 
                                       FUN = which.max) - 1
      out[!probs_obs, i]      <- NA
    }
    colnames(out)[1:n_eq] <- z_names
  }
  
  # Most probable categories of the multinomial equation
  if (is3)
  {
    probs <- matrix(NA, nrow = n, ncol = n_eq3)
    for (j in 1:n_eq3)
    {
      probs[, j] <- predict(object, type = "prob_mn", 
                            newdata = newdata, group3 = j - 1)
    }
    out[, ncol(out)]         <- apply(X = probs, MARGIN = 1, 
                                      FUN = which.max) - 1
    colnames(out)[ncol(out)] <- z_mn_names
  }
  
  # Fitted values of the continuous equations
  if (is2)
  {
    ind                <- (n_eq + 1):(n_eq + n_eq2)
    out[, ind]         <- predict(object, type = "val", newdata = newdata)
    colnames(out)[ind] <- y_names
  }
  
  # Return the result
  return(out)
}

#' Likelihood ratio test 
#' @description This function performs chi-squared test for nested models.
#' @param model1 the first model.
#' @param model2 the second model.
#' @details Arguments \code{model1} and \code{model2} should be objects
#' of class that has implementations of 
#' \code{\link[stats]{logLik}} and 
#' \code{\link[stats]{nobs}} methods. It is assumed that either \code{model1}
#' is nested into \code{model2} or vice versa. More precisely it is assumed
#' that the model with smaller log-likelihood value is nested into the model
#' with greater log-likelihood value.
#' 
#' Arguments \code{model1} and \code{model2} may be the lists of models.
#' If \code{model1} is a list of models then it is assumed that the number
#' of degrees of freedom and log-likelihood of the first model are just a sum
#' of degrees of freedom and log-likelihoods of the models in this list.
#' Similarly for \code{model2}. 
#' 
#' If \code{model1} or \code{model2} is a list then the number of observations 
#' of the associated models is calculated as the sum of the numbers of 
#' observations of the models in corresponding lists.
#' However sometimes it may be misleading. For example, when bivariate probit
#' model (full) is tested against two independent probit models (restricted). 
#' Then it will be assumed that the number of observations in the restricted
#' model is twice the number of observations in the full model that is not the
#' case. 
#' Fortunately it will not affect the results of the likelihood ratio test.
#' @return The function returns an object of class \code{'lrtest_msel'} that is
#' a list with the following elements:
#' \itemize{
#' \item \code{n1} - the number of observations in the first model.
#' \item \code{n2} - the number of observations in the second model.
#' \item \code{ll1} - log-likelihood value of the first model.
#' \item \code{ll2} - log-likelihood value of the second model.
#' \item \code{df1} - the number of parameters in the first model.
#' \item \code{df2} - the number of parameters in the second model.
#' \item \code{restrictions} - the number of restrictions in the nested model.
#' \item \code{value} - chi-squared (likelihood ratio) test statistic value.
#' \item \code{p_value} - p-value of the chi-squared (likelihood ratio) test.
#' }
#' @examples 
#' set.seed(123)
#' # Generate data according to linear regression
#' n   <- 100
#' eps <- rnorm(n)
#' x1  <- runif(n)
#' x2  <- runif(n)
#' y   <- x1 + 0.2 * x2 + eps
#' # Estimate full model
#' model1 <- lm(y ~ x1 + x2)
#' # Estimate restricted (nested) model
#' model2 <- lm(y ~ x1)
#' # Likelihood ratio test results
#' lrtest_msel(model1, model2)
lrtest_msel <- function(model1, model2)
{
  # List to save the results
  out <- list()
  
  # Control for multiple models input
  multiple1 <- !(paste0("nobs.", class(model1)) %in% 
                 methods(class = class(model1)))
  multiple2 <- !(paste0("nobs.", class(model2)) %in% 
                 methods(class = class(model2)))
  
  # First model
  if (!multiple1)
  {
    out$n1 <- nobs(model1)                                 
    out$ll1 <- logLik(model1)                            
    out$df1 <- attr(out$ll1, which = "df")                 
  }
  else
  {
    out$n1 <- 0
    out$ll1 <- 0
    out$df1 <- 0
    for (i in 1:length(model1))
    {
      out$n1 <- out$n1 + as.numeric(nobs(model1[[i]])) 
      ll1 <- logLik(model1[[i]])
      out$ll1 <- out$ll1 + as.numeric(ll1)                  
      out$df1 <- out$df1 + as.numeric(attr(ll1, which = "df"))
    }
  }
  
  # Second model
  if (!multiple2)
  {
    out$n2 <- nobs(model2)                                 
    out$ll2 <- logLik(model2)                             
    out$df2 <- attr(out$ll2, which = "df")                
  }
  else
  {
    out$n2 <- 0
    out$ll2 <- 0
    out$df2 <- 0
    for (i in 1:length(model2))
    {
      out$n2 <- out$n2 + as.numeric(nobs(model2[[i]]))    
      ll2 <- logLik(model2[[i]])
      out$ll2 <- out$ll2 + as.numeric(ll2) 
      out$df2 <- out$df2 + as.numeric(attr(ll2, which = "df"))  
    }
  }
  
  # Validation
  if (!(multiple1 | multiple2))
  {
    if (out$n1 != out$n2)
    {
      stop("Models should have the same number of observations.")
    }
    
    if ((which.max(c(out$ll1, out$ll2)) != 
         which.max(c(out$df1, out$df2))) |
        (out$df1 == out$df2))
    {
      stop(paste0("Models should be nested."))
    }
  }
  
  # Calculation of the test statistics and p-value
  out$value <- as.numeric(2 * abs(out$ll1 - out$ll2))
  out$restrictions <- abs(out$df1 - out$df2)
  out$p_value <- 1 - pchisq(out$value, df = out$restrictions)
  class(out) <- "lrtest_msel";
  
  return(out)
}

#' Print Method for Likelihood Ratio Test
#' @description Prints summary for an object of class 'lrtest_msel'.
#' @param x object of class "summary.lrtest_msel".
#' @param ... further arguments (currently ignored).
#' @return The function returns the input argument \code{x}.
print.lrtest_msel <- function(x, ...)
{
  cat("LR-test results\n")
  cat("---\n")
  cat(paste0("It is assumed that model",
             which.min(c(x$df1, x$df2)),
             " is nested into model",
             which.max(c(x$df1, x$df2)), "\n"))
  cat(paste0("H0: model", which.min(c(x$df1, x$df2)), " is correct\n"))
  cat("--\n")
  cat(paste0("Restrictions = ", x$restrictions, "\n"))
  cat(paste0("Test statistic = ", round(x$value, 3), "\n"))
  if (x$p_value >= 0.00001)
  {
    cat(paste0("p-value = ", round(x$p_value, 5), starsVector(x$p_value), "\n"))
  }
  else
  {
    cat(paste0("p-value < 0.00001 ", starsVector(x$p_value), "\n"))
  }
  if (getOption("show.signif.stars"))
  {
    cat("--\n")
    cat("Signif. codes:  0 '***' 0.01 '**' 0.05 '*' 0.1 \n")
  }
  return(x)
}

#' Summary Method for Likelihood Ratio Test
#' @description Provides summary for an object of class 'lrtest_msel'.
#' @param object object of class "lrtest_msel"
#' @param ... further arguments (currently ignored)
#' @details This function just changes the class of the 'lrtest_msel'
#' object to 'summary.lrtest_msel'.
#' @return Returns an object of class 'summary.lrtest_msel'.
summary.lrtest_msel <- function(object, ...) 
{
  if (length(list(...)) > 0)
  {
    warning("Additional arguments passed through ... are ignored.")   
  }
  
  class(object) <- "summary.lrtest_msel";
  
  return(object)
}

#' Print Summary Method for Likelihood Ratio Test
#' @description Prints summary for an object of class 'lrtest_msel'.
#' @param x object of class "lrtest_msel"
#' @param ... further arguments (currently ignored)
#' @return The function returns input argument \code{x} changing
#' its class to \code{lrtest_msel}.
print.summary.lrtest_msel <- function(x, ...)
{
  class(x) <- "lrtest_msel"
  print(x)
  return(x)
}

#' Predict method for msel function
#' @description Predicted values based on the object of class 'msel'.
#' @template predict.msel_param_Template
#' @template predict.msel_details_Template
#' @template predict.msel_return_Template
predict.msel <- function(object, ..., 
                         newdata   = NULL, 
                         given_ind = numeric(),
                         group     = NA,
                         group2    = NA,
                         group3    = NA,
                         type      = ifelse(any(is.na(group2)), 
                                            "prob", "val"),
                         me        = NULL,
                         eps       = NULL,
                         control   = list(),
                         test      = FALSE,
                         exogenous = NULL,
                         factors   = NULL)
{
  # -------------------------------------------------------
  # Deal with the data and the variables
  # -------------------------------------------------------

  # Deal with the groups
  is_na_group  <- c(any(is.na(group)), any(is.na(group2)), any(is.na(group3)))
  
  # Get some values
  groups    <- object$groups
  groups2   <- object$groups2
  groups3   <- object$groups3
  formula   <- object$formula
  formula2  <- object$formula2
  formula3  <- object$formula3
  ind_g     <- object$other$ind_g
  ind_eq    <- object$other$ind_eq
  n_par     <- length(object$par)
  n_eq      <- object$other$n_eq
  n_eq2     <- object$other$n_eq2
  n_eq3     <- object$other$n_eq3
  is1       <- object$other$is1
  is2       <- object$other$is2
  is3       <- object$other$is3
  n_groups  <- object$other$n_groups
  estimator <- object$estimator
  type3     <- object$type3
  
  # Provide 'data' as 'newdata' if needed
  is_newdata <- TRUE
  if (is.null(newdata))
  {
    is_newdata <- FALSE
    newdata    <- object$data
  }
  
  # Add the intercept to the newdata
  if (is2 | is3)
  {
    newdata[, '(Intercept)'] <- 1
  }

  # Deal with the 'exogenous' argument
  n_exogenous  <- length(exogenous)
  is_exogenous <- n_exogenous > 0
  if (is_exogenous)
  {
    newdata    <- exogenous_fn(exogenous = exogenous, newdata = newdata)
    is_newdata <- TRUE
  }
  
  # Determine whether recalculation of selectivity terms is required
  is_recalculate_lambda <- TRUE
  if (hasName(control, "is_recalculate_lambda"))
  {
    is_recalculate_lambda <- control$is_recalculate_lambda
  }

  # Recalculate selectivity terms for the two-step estimator
  if ((type == "val") & (estimator == "2step") & 
      ((is1 & !is_na_group[1]) | (is3 & !is_na_group[3])) & 
      is_recalculate_lambda)
  {
    # Set newdata
    is_newdata <- TRUE
    
    # Remove the data associated with the selectivity terms
    for (v in 1:n_eq2)
    {
      formula2_terms <- attr(terms(formula2[[v]]), "term.labels")
      formula2_terms <- formula2_terms[formula2_terms %in% colnames(newdata)]
      newdata[, formula2_terms[grepl("lambda", 
                                     formula2_terms, 
                                     fixed = TRUE)]] <- NULL
      if (is1)
      {
        for (j in seq_len(n_eq))
        {
          newdata[[paste0("lambda", j)]] <- 0
        }
      }
      if (is3)
      {
        for (j in seq_len(n_eq3 - (type3 == "probit")))
        {
          newdata[[paste0("lambda", j, "_mn")]] <- 0
        }
      }
    }
    
    # Calculate the selectivity terms of the ordinal equations
    lambda <- NULL
    if (is1)
    {
      lambda <- predict(object, group = group, group3 = -1, 
                        type = "lambda", newdata = newdata)
      for (j in seq_len(n_eq))
      {
        if (group[j] != -1)
        {
          newdata[[paste0("lambda", j)]] <- lambda[, j]
        }
      }
    }
    
    # Calculate the selectivity terms of the multinomial equation
    lambda_mn <- NULL
    if (is3)
    {
      lambda_mn <- predict(object, group = group, group3 = group3, 
                           type = "lambda_mn", newdata = newdata)
      for (j in seq_len(n_eq3 - (type3 == "probit")))
      {
        if (group3 != -1)
        {
          newdata[[paste0("lambda", j, "_mn")]] <- lambda_mn[, j]
        }
      }
    }
  }

  # Adjust the new data
  if (is_newdata)
  {
    # Deal with the omitted variables
    all_names                <- object$other$all_names
    lambda_names             <- object$other$lambda_names
    omitted_names            <- all_names[!(all_names %in% colnames(newdata))]
    if (estimator == "2step")
    {
      omitted_names <- omitted_names[!(omitted_names %in% lambda_names)]
    }
    newdata[, omitted_names] <- 0

    # Remove the missing values
    newdata <- complete_msel(object = object, data = newdata)
  }

  # Split the newdata according to the formulas
  if (!is_newdata)
  {
    W_mean <- object$W_mean
    W_var  <- object$W_var
    W_mn   <- object$W_mn
    X      <- object$X
    y      <- object$y
  }
  else
  {
    newdata_list <- data_msel(object = object, data = newdata)
    W_mean       <- newdata_list$W_mean
    W_var        <- newdata_list$W_var
    X            <- newdata_list$X
    W_mn         <- newdata_list$W_mn
    z            <- newdata_list$z
    y            <- newdata_list$y
    z_mn         <- newdata_list$z_mn
    newdata_list <- NULL
  }
  
  # Get the number of observations
  n_obs <- nrow(newdata)
  
  # Get the information on the marginal distributions
  marginal    <- object$marginal
  is_marginal <- length(object$marginal) > 0
  
  # Get estimates of the parameters and some related variables
  coef        <- object$coef
  coef_var    <- object$coef_var
  coef3       <- object$coef3
  sigma       <- object$sigma
  sigma3      <- object$sigma3
  cuts        <- object$cuts
  sigma_omit  <- object$other$sigma_omit
  n_eq        <- object$other$n_eq
  n_cuts_eq   <- object$other$n_cuts_eq
  is_het      <- object$other$is_het
  
  # Estimate the value
  if (all(is_na_group))
  {
    # Get some variables
    groups   <- object$groups
    groups2  <- object$groups2
    groups3  <- object$groups3
    n_groups <- object$other$n_groups
    ind_g    <- object$other$ind_g
    
    # Change the groups related data if needed
    if (is_newdata)
    {
      groups_list <- groups_msel(object  = object, data    = newdata, 
                                 groups  = groups, groups2 = groups2,
                                 groups3 = groups3)
      groups      <- groups_list$groups
      groups2     <- groups_list$groups2
      groups3     <- groups_list$groups3
      n_groups    <- groups_list$n_groups
      ind_g       <- groups_list$ind_g
    }
    
    # Calculate the value for all groups and
    # aggregate the result
    val <- NULL
    if (n_groups > 1)
    {
      for (i in 1:n_groups)
      {
        groups_i  <- NA
        groups2_i <- NA
        groups3_i <- NA
        if (is1)
        {
          groups_i  <- groups[i, ]
        }
        if (is2)
        {
          groups2_i <- groups2[i, ]
        }
        if (is3)
        {
          groups3_i <- groups3[i]
        }
        val_tmp <- predict(object, 
                           newdata = newdata[ind_g[[i]], ],
                           group   = groups_i,
                           group2  = groups2_i,
                           group3  = groups3_i,
                           type    = type)
        if (i == 1)
        {
          val           <- matrix(0, nrow = n_obs, ncol = ncol(val_tmp))
          colnames(val) <- colnames(val_tmp)
        }
        val[ind_g[[i]], ] <- val_tmp
      }
    }
    
    # Return the results
    return(val)
  }
  
  # -------------------------------------------------------
  # Statistical test
  # -------------------------------------------------------
  
  # Deal with the input
  test_fn <- NULL
  if (!is.logical(test))
  {
    test_fn <- test
    test    <- TRUE
  }
  
  # Perform the test
  if (test)
  {
    # Get arguments of the function and change 'test'
    # argument to false to prevent the eternal cycle
    predict_args         <- c(as.list(environment()), list(...))
    predict_args$newdata <- newdata
    predict_args$test    <- FALSE

    # Apply the test
    predict_fn <- function(object)
    {
      predict_args$object <- object
      val <- do.call(what = predict.msel, args = predict_args)
      if (!is.null(test_fn))
      {
        if (is.function(test_fn))
        {
          val <- test_fn(val)
        }
        else
        {
          val <- val[, test_fn]
        }
      }
      return(val)
    }
    out <- test_msel(object = object, fn = predict_fn)
    
    return(out)
  }
  
  # -------------------------------------------------------
  # Validation
  # -------------------------------------------------------
  
  # Validate type
  if (type == "lp")
  {
    type <- "li"
  }
  if (type == "lp_mn")
  {
    type <- "li_mn"
  }
  
  # Validate group
  if (!is_na_group[1])
  {
    if (any((group %% 1) != 0))
    {
      stop(paste0("Invalid 'group' argument. Please, ensure that 'group' is ",
                  "a vector of integers.\n"))
    }
    if (length(group) != n_eq)
    {
      stop(paste0("Invalid 'group' argument. Please, ensure that length ",
                  "of 'group' equals to the number of ordered equations i.e. ",
                  "'length(group) == length(formula)'.\n"))
    }
    if (any(group < -1))
    {
      stop(paste0("Invalid 'group' argument. Please, ensure that it ", 
                  "does not contain any negative values other than -1.\n"))
    }
    if (any(group >= n_groups))
    {
      stop(paste0("Invalid 'group' argument. Please, ensure that it ", 
                  "does not contain any values greater than the number of ", 
                  "the maximum category of corresponding equation.\n"))
    }
  }
  
  # Validate group2
  if (is2 & (type == "val") & !is_na_group[1] & is_na_group[2])
  {
    group2 <- rep(0, n_eq2)
    warning(paste0("It is assumed that 'group2' is a vector of zeros.\n"))
  }
  if (is2 & !is_na_group[2] & (length(group2) != n_eq2))
  {
    stop(paste0("Invalid 'group2' argument. ",
                "It should be a vector of length ", n_eq2, ".\n"))
  }
  
  # Validate group3
  if (!is_na_group[3])
  {
    if (((group3 %% 1) != 0) & length(group3) > 1)
    {
      stop(paste0("Invalid 'group3' argument. Please, ensure that 'group3' is ",
                  "an integer.\n"))
    }
    if (any(group3 < -1))
    {
      stop(paste0("Invalid 'group3' argument. Please, ensure that it ", 
                  "is not a negative value different from -1.\n"))
    }
    if (group3 > (n_eq3 - 1))
    {
      stop(paste0("Invalid 'group3' argument. Please, ensure that its ", 
                  "value is not larger than the number of alternatives ",
                  "in the multinomial equation.\n"))
    }
  }
  
  # Additional validation for groups and groups3
  if (is1 & is3)
  {
    if (is_na_group[1])
    {
      group          <- rep(-1, n_eq)
      is_na_group[1] <- FALSE
    }
    if (is_na_group[3])
    {
      group3         <- -1
      is_na_group[3] <- FALSE
    }
  }
  
  # Validate control
  is_scores <- FALSE
  if (hasName(control, "is_scores"))
  {
    is_scores <- control$is_scores
  }
  
  # Assign 'group' for some 'type'
  if (((type == "li") | (type == "sd")) & is_na_group[1])
  {
    group          <- rep(0, n_eq)
    is_na_group[1] <- FALSE
  }
  if ((type == "li_mn") & is_na_group[3])
  {
    group3         <- 0
    is_na_group[3] <- FALSE
  }
  
  # Some additional variables
  is_eq       <- group != -1
  ind_eq      <- which(is_eq)
  ind_eq_omit <- which(!is_eq)
  n_eq_g      <- length(ind_eq)
  y_names     <- object$other$y_names
  
  # Transform marginal argument into mnorm format
  marginal_mnorm <- object$marginal
  if (is_marginal)
  {
    for (i in 1:n_eq)
    {
      if (length(object$marginal_par[[i]]) > 0)
      {
        marginal_mnorm[[i]] <- object$marginal_par[[i]]
      }
    }
  }
  
  # Adjust for the marginal distributions of the observable equations only
  marginal_mnorm_g <- marginal_mnorm
  if (is_marginal)
  {
    marginal         <- marginal[ind_eq]
    marginal_mnorm_g <- marginal_mnorm[ind_eq]
  }
  
  # Covariance matrix for the multinomial probit
  sigma3_alt    <- NULL
  transform_mat <- NULL
  if (is3 & (type3 == "probit"))
  {
    transform_mat <- matrix(0, nrow = n_eq3, ncol = n_eq3)
    diag(transform_mat) <- 1
    if (group3 != (n_eq3 - 1))
    {
      transform_mat[, group3 + 1] <- -1
    }
    transform_mat <- transform_mat[-(group3 + 1), , drop = FALSE]
    transform_mat <- transform_mat[, -n_eq3, drop = FALSE]
    sigma3_alt    <- transform_mat %*% sigma3 %*% t(transform_mat)
  }
  
  # -------------------------------------------------------
  # Marginal effects
  # -------------------------------------------------------
  
  # Calculate marginal effects
  if (!is.null(me))
  {
    # If several marginal effects should be calculated
    n_me <- length(me)
    if (n_me > 1)
    {
      # Remake factors variable
      if (!is.list(factors))
      {
        factors <- list(factors)
      }
      n_factors <- length(factors)
      factors0  <- vector(mode = "list", length = n_me)
      for (i in 1:n_me)
      {
        if (n_factors != 0)
        {
          for (j in 1:n_factors)
          {
            if (me[i] %in% factors[[j]])
            {
              factors0[[i]] <- factors[[j]]
            }
          }
        }
      }
      
      # Calculate the marginal effects for each variable
      val           <- matrix(NA, nrow = n_obs, ncol = n_me)
      colnames(val) <- me
      for (i in 1:n_me)
      {
        val[, i] <- predict(object, ..., 
                            newdata   = newdata,
                            given_ind = given_ind, 
                            group     = group,
                            group2    = group2,
                            group3    = group3,
                            type      = type, 
                            me        = me[i], 
                            eps       = eps,
                            control   = control,
                            factors   = factors0[[i]])
      }
      return(val)
    }
    
    # Deal with factors
    if (!is.null(factors))
    {
      if (is.list(factors))
      {
        factors <- factors[[1]]
      }
      eps <- c(0, 1)
      if (!(me %in% factors))
      {
        stop (paste0("Variable ", me, 
                     " is not a category of the factor variable: ",
                     paste0(factors, collapse = ", ")))
      }
      for (i in factors)
      {
        newdata[i] <- 0
      }
    }
    
    # Determine the type of the marginal effect
    is_discrete <- length(eps) > 1
    
    # Adjust epsilon if needed
    if (is.null(eps))
    {
      eps <- newdata[[me]] * sqrt(.Machine$double.eps)
    }
    
    # Calculate the values before and after the increment
    if (is_discrete)
    {
      newdata[me] <- eps[1]
    }
    p0 <- predict.msel(object, 
                            newdata   = newdata,
                            given_ind = given_ind,
                            group     = group,
                            group2    = group2,
                            group3    = group3,
                            type      = type,
                            me        = NULL,
                            control   = control)
    if (is_discrete)
    {
      newdata[me] <- eps[2]
    }
    else
    {
      newdata[me] <- newdata[me] + eps
    }
    p1 <- predict.msel(object, 
                       newdata   = newdata,
                       given_ind = given_ind,
                       group     = group,
                       group2    = group2,
                       group3    = group3,
                       type      = type,
                       me        = NULL,
                       control   = control)
    
    # Estimate marginal effect
    val <- p1 - p0
    if (!is_discrete)
    {
      val <- val / eps
    }
    
    # Provide a name for the output
    if (is.matrix(val))
    { 
      colnames(val) <- paste0("d", colnames(val), "/", "d", me)
      if (is_discrete)
      {
        colnames(val) <- paste0(colnames(val), " from ", 
                                eps[1], " to ", eps[2])
      }
    }
    
    # Return marginal effect
    return(val)
  }
  
  # -------------------------------------------------------
  # Conditional probabilities
  # -------------------------------------------------------
  
  # Calculate the conditional probabilities
  if (length(given_ind) > 0)
  {
    if (length(given_ind) >= n_eq_g)
    {
      stop(paste("At least one component should be unconditioned.",
                 "Please, ensure that 'length(given_ind)' is smaller than",
                 "the number of observable equations."))
    }
    
    if (type != "prob")
    {
      warning(paste0("Since 'given_ind' has been provided then 'type'",
                     "will be coerced to 'prob'."))
    }
    
    group_given             <- group
    group_given[-given_ind] <- -1
    
    # Probability of the intersection
    p_intersection <- predict(object, 
                              newdata = newdata, 
                              type    = "prob",
                              group   = group,
                              group3  = group3,
                              control = control)
    
    # Probability of the condition
    p_given <- predict(object, 
                       newdata = newdata, 
                       type    = "prob",
                       group   = group_given,
                       group3  = group3,
                       control = control)
    
    # Conditional probability
    p_cond <- p_intersection / p_given
    
    # Provide the name for the conditional probability
    is_eq_ng            <- is_eq
    is_eq_ng[given_ind] <- FALSE
    colnames(p_cond)[1] <- paste0("P(",
                                  paste0(object$other$z_names[is_eq_ng], "=",
                                         group[is_eq_ng], collapse = ", "),
                                  "|",
                                  paste0(object$other$z_names[given_ind], "=",
                                         group[given_ind], collapse = ", "),
                                  ")")
    rownames(p_cond)   <- 1:length(p_cond)
    
    # Return the results
    return(p_cond)
  }

  # -------------------------------------------------------
  # Linear predictors (indexes)
  # -------------------------------------------------------
  
  # Calculate the linear predictors (indexes) of the ordinal equations
  li_lower <- NULL
  li_upper <- NULL
  li_mean  <- NULL
  li_var   <- NULL
  if (is1)
  {
    if (n_eq_g > 0)
    {
      li_lower <- matrix(NA, nrow = n_obs, ncol = n_eq_g)
      li_upper <- matrix(NA, nrow = n_obs, ncol = n_eq_g)
      li_mean  <- matrix(NA, nrow = n_obs, ncol = n_eq_g)
      li_var   <- matrix(1,  nrow = n_obs, ncol = n_eq_g)
      for (i in 1:n_eq_g)
      {
        # Calculate the linear predictors (indexes) for the 
        # mean and variance parts
        li_mean[, i] <- W_mean[[ind_eq[i]]] %*% coef[[ind_eq[i]]]
        if (is_het[ind_eq[i]])
        {
          li_var[, i] <- exp(W_var[[ind_eq[i]]] %*% coef_var[[ind_eq[i]]])
        }
        # Adjust lower limits for the cuts and heteroscedasticity
        if (group[ind_eq[i]] != 0)
        {
          li_lower[, i] <- cuts[[ind_eq[i]]][group[ind_eq[i]]] - 
            li_mean[, i]
          if (is_het[ind_eq[i]])
          {
            li_lower[, i] <- li_lower[, i, drop = FALSE] / 
                             li_var[, i, drop = FALSE]
          }
        }
        else
        {
          li_lower[, i] <- -Inf
        }
        # Adjust upper limits for the cuts and heteroscedasticity
        if (group[ind_eq[i]] != n_cuts_eq[[ind_eq[i]]])
        {
          li_upper[, i] <- cuts[[ind_eq[i]]][group[ind_eq[i]] + 1] - 
                           li_mean[, i, drop = FALSE]
          if (is_het[ind_eq[i]])
          {
            li_upper[, i] <- li_upper[, i, drop = FALSE] / 
                             li_var[, i, drop = FALSE]
          }
        }
        else
        {
          li_upper[, i] <- Inf
        }
      }
    }
    
    # Return the linear predictors (indexes)
    if (type == "li")
    {
      colnames(li_mean) <- object$other$z_names[is_eq]
      rownames(li_mean) <- 1:nrow(li_mean)
      return(li_mean)
    }
    
    # Return the standard deviations
    if (type == "sd")
    {
      colnames(li_var) <- object$other$z_names[is_eq]
      rownames(li_var) <- 1:nrow(li_var)
      return(li_var)
    }
  }
  
  # Calculate the linear predictors (indexes) of the multinomial equation
  li_mn   <- NULL
  li_diff <- NULL
  if (is3 & (group3 != -1))
  {
    # Linear predictors (indexes)
    li_mn <- matrix(0, nrow = n_obs, ncol = n_eq3)
    if (type %in% c("li_mn", "prob_mn", "lambda_mn"))
    {
      for (i in seq_len(n_eq3 - 1))
      {
        li_mn[, i] <- W_mn %*% t(coef3[i, , drop = FALSE])
      }
    }
    # Differences between the linear predictors (indexes)
    if (type3 == "probit")
    {
      li_mn <- li_mn[, -n_eq3, drop = FALSE]
      li_diff <- matrix(NA, n_obs, n_eq3 - 1)
      if ((group3 + 1) == n_eq3)
      {
        li_diff <- -li_mn
      }
      else
      {
        li_diff[, n_eq3 - 1]    <- li_mn[, group3 + 1, drop = FALSE]
        li_diff[, -(n_eq3 - 1)] <- apply(li_mn[, -(group3 + 1), 
                                                 drop = FALSE], 2, 
                                         function(x)
                                         {
                                           return(li_mn[, (group3 + 1)] - x)
                                         })
      }
    }
    
    # Return the linear predictors (indexes)
    if (type == "li_mn")
    {
      colnames(li_mn) <- paste0("alt", 0:(n_eq3 - 1))
      rownames(li_mn) <- 1:nrow(li_mn)
      return(li_mn)
    }
    
    # Return the differences
    if (type == "li_diff")
    {
      return(li_diff)
    }
  }
  
  # -------------------------------------------------------
  # Probabilities
  # -------------------------------------------------------
  
  # Covariances between random errors of the observable
  # ordinal equations
  sigma_g <- sigma
  if (is1)
  {
    sigma_g <- sigma[ind_eq, ind_eq, drop = FALSE]
    if (any(sigma_omit[ind_eq, ind_eq] == 1))
    {
      sigma_g[sigma_omit[ind_eq, ind_eq]] <- 0
      warning("Unidentified covariances are set to 0.")
    }
  }
  
  # Calculate the probabilities of the ordered equations
  prob <- 0
  if (is1 & (type == "prob") & (n_eq_g > 0))
  {
    # Estimate the probability
    prob <- mnorm::pmnorm(lower    = li_lower, 
                          upper    = li_upper, 
                          mean     = rep(0, n_eq_g), 
                          sigma    = sigma_g,
                          marginal = marginal_mnorm_g)$prob
    
    # Provide the name for the probability
    colnames(prob)[1] <- paste0("P(",
                                paste0(object$other$z_names[is_eq], "=",
                                       group[is_eq], collapse = ", "),
                                ")")
    rownames(prob)    <- 1:nrow(prob)
    
    # Return the result
    return(prob)
  }
  
  # Calculate the probabilities of the multinomial equation
  prob_mn <- 0
  if (is3 & (group3 != -1) &
      ((type == "prob_mn") | (type == "lambda_mn")))
  {
    # Multinomial logit
    if (type3 == "logit")
    {
      li_exp    <- exp(li_mn)
      prob_mn <- sweep(li_exp, 1, rowSums(li_exp), FUN = '/')
    }
    # Multinomial probit
    if (type3 == "probit")
    {
      lower_neg_inf <- matrix(-Inf, nrow = n_obs, ncol = n_eq3 - 1)
      prob_mn       <- mnorm::pmnorm(lower         = lower_neg_inf, 
                                     upper         = li_diff,
                                     mean          = rep(0, n_eq3 - 1), 
                                     sigma         = sigma3_alt,
                                     is_validation = FALSE)$prob
    }
  }
  
  # Return the result
  if  (type == "prob_mn")
  {
    if (type3 == "logit")
    {
      prob_mn <- prob_mn[, (group3 + 1), drop = FALSE]
    }
    colnames(prob_mn) <- paste0("P(", object$other$z_mn_names, 
                                " = ", group3, ")")
    rownames(prob_mn) <- 1:nrow(prob_mn)
    return(prob_mn)
  }
  
  # -------------------------------------------------------
  # Lambda
  # -------------------------------------------------------

  # Calculate lambda of the ordered equations
  lambda <- NULL
  if (is1)
  {
    lambda <- matrix(0, nrow = n_obs, ncol = n_eq_g)
    if (n_eq_g > 0 & ((type == "lambda") | 
                      ((type == "val") & (estimator == "ml"))))
    {
      # Prepare the matrix to store the final results
      lambda_mat           <- matrix(0, nrow = n_obs, ncol = n_eq)
      colnames(lambda_mat) <- paste0("lambda", 1:n_eq)
      if (n_eq_g > 0)
      {
        # Calculate lambdas for the conditional expectations
        grads <- mnorm::pmnorm(lower              = li_lower, 
                               upper              = li_upper, 
                               mean               = rep(0, n_eq_g), 
                               sigma              = sigma_g,
                               log                = TRUE, 
                               grad_upper         = TRUE, 
                               grad_lower         = TRUE,
                               grad_sigma         = FALSE,
                               marginal           = marginal_mnorm_g,
                               grad_marginal      = is_marginal,
                               grad_marginal_prob = is_marginal)
        lambda_lower <- NULL
        lambda_upper <- NULL
        if (!is_marginal)
        {
          lambda_lower <- -grads$grad_lower
          lambda_upper <-  grads$grad_upper
        }
        else
        {
          lambda_lower <- -grads$grad_lower_marginal
          lambda_upper <-  grads$grad_upper_marginal
        }
        if (hasName(control, name = "adj"))
        {
          if (control$adj)
          {
            li_lower_adj                        <- li_lower
            li_lower_adj[is.infinite(li_lower)] <- 0
            li_upper_adj                        <- li_upper
            li_upper_adj[is.infinite(li_upper)] <- 0
            lambda                              <- lambda_lower * li_lower_adj - 
                                                   lambda_upper * li_upper_adj
          }
          else
          {
            stop("Parameter 'adj' in 'control' should be 'TRUE'.")
          }
        }
        else
        {
          lambda <- lambda_lower - lambda_upper
        }
      }
      
      # Return lambda
      if (type == "lambda")
      {
        lambda_mat[, ind_eq] <- lambda
        return(lambda_mat)
      }
    }
    else
    {
      if (type == "lambda")
      {
        return(matrix(NA, nrow = n_obs, ncol = n_eq))
      }
    }
  }
  
  # Calculate lambda for the multinomial equation
  lambda_mn <- NULL
  if (is3 & (group3 == -1) & (type == "lambda_mn"))
  {
    if (type3 == "logit")
    {
      lambda_mn <- matrix(0, nrow = n_obs, ncol = n_eq3)
    }
    if (type3 == "probit")
    {
      lambda_mn <- matrix(0, nrow = n_obs, ncol = n_eq3 - 1)
    }
    return(lambda_mn)
  }
  if (is3 & (group3 != -1) & (type == "lambda_mn"))
  {
    lambda_mn <- matrix(0, nrow = n_obs, ncol = n_eq3)
    # Multinomial logit
    if (type3 == "logit")
    {
      lambda_mn                  <- matrix(NA, nrow = n_obs, ncol = n_eq3)
      lambda_mn[, group3 + 1]    <- -log(prob_mn[, group3 + 1])
      lambda_mn[, -(group3 + 1)] <- prob_mn[, -(group3 + 1)] * 
                                      log(prob_mn[, -(group3 + 1)]) / 
                                      (1 - prob_mn[, -(group3 + 1)])
    }
    # Multinomial probit
    if (type3 == "probit")
    {
      lower_neg_inf <- matrix(-Inf, nrow = n_obs, ncol = n_eq3 - 1)
      lambda_mn     <- mnorm::pmnorm(lower         = lower_neg_inf, 
                                     upper         = li_diff,
                                     mean          = rep(0, n_eq3 - 1), 
                                     sigma         = sigma3_alt,
                                     grad_upper    = TRUE,
                                     log           = TRUE, 
                                     is_validation = FALSE)$grad_upper
      A <- matrix(0, nrow = n_eq3 - 1, ncol = n_eq3 - 1)
      for (i in 1:(n_eq3 - 1))
      {
        for (j in 1:(n_eq3 - 1))
        {
          if (i == (group3 + 1))
          {
            A[i, j] <- 1
          }
          if (((i < (group3 + 1)) & (i == j)) | 
              ((i > (group3 + 1)) & (i == (j + 1))))
          {
            A[i, j] <- -1
          }
        }
      }
      lambda_mn <- lambda_mn %*% t(A)
    }
    
    # Return the result
    if (type == "lambda_mn")
    {
      return(lambda_mn)
    }
  }
  
  # -------------------------------------------------------
  # Predictions for the continuous equations
  # -------------------------------------------------------
  
  # Prediction for the continuous equations
  y_pred    <- NULL
  scores    <- vector(mode = "list", length = n_eq2)
  if (!is_na_group[2])
  {
    # Matrix to store the predictions
    y_pred           <- matrix(NA, nrow = n_obs, ncol = n_eq2)
    colnames(y_pred) <- object$other$y_names
    
    # Predictions
    for (i in 1:n_eq2)
    {
      X_i <- as.matrix(X[[i]])
      if (group2[i] != -1)
      {
        # substitute data with the predictions
        if (i >= 2)
        {
          # predictions of the previous equation
          for (v0 in 1:(i - 1))
          {
            if (group2[v0] != -1)
            {
              if (y_names[v0] %in% colnames(X_i))
              {
                X_i[, y_names[v0]] <- y_pred[, v0]
              }
            }
          }
        }
        # predictions
        y_pred[, i] <- X_i %*% matrix(object$coef2[[i]][group2[i] + 1, ], 
                                      ncol = 1)
        if (!is_na_group[1])
        {
          if (object$estimator == "ml")
          {
            y_pred[, i] <- y_pred[, i] + lambda %*% 
                           matrix(object$cov2[[i]][group2[i] + 1, ind_eq], 
                                  ncol = 1)
          }
        }
      }
      # provide the names
      is1_cond <- FALSE
      if (!is_na_group[1])
      {
        if (any(is_eq))
        {
          is1_cond <- TRUE
        }
      }
      is3_cond <- FALSE
      if (!is_na_group[3])
      {
        if (group3 != -1)
        {
          is3_cond <- TRUE
        }
      }
      E_y_names  <- paste0("E(", object$other$y_names)
      if (!is_na_group[2])
      {
        group2_tmp                   <- group2
        group2_tmp[group2_tmp == -1] <- ""
        E_y_names                    <- paste0(E_y_names, group2_tmp)
      }
      cond_names <- ""
      if (is1_cond | is3_cond)
      {
        E_y_names <- paste0(E_y_names, "|")
        if (is1_cond)
        {
          cond_names <- paste0(cond_names, 
                               paste0(object$other$z_names[is_eq], "=", 
                                      group[is_eq], collapse = ", "))
          if (is3_cond)
          {
            cond_names <- paste0(cond_names, ",")
          }
        }
        if (is3_cond)
        {
          cond_names <- paste0(object$other$z_mn_names, "=", 
                               group3, collapse = ", ")
        }
      }
      E_y_names           <- paste0(paste0(E_y_names, cond_names), ")")
      colnames(y_pred)    <- E_y_names
      rownames(y_pred)    <- 1:nrow(y_pred)
      # calculate the scores
      if (is_scores)
      {
        scores[[i]] <- as.vector(y[, i] - y_pred[, i]) * X_i
        scores[[i]][is.infinite(scores[[i]]) | is.na(scores[[i]])] <- 0
      }
    }
  }
    
  # If the scores should be returned
  if (is_scores)
  {
    names(scores) <- object$other$y_names
    return(scores)
  }
    
  # Return prediction for the continuous equation
  if (type == "val")
  {
    return(y_pred)
  }
  
  # -------------------------------------------------------
  # Normally user should not pass to this section
  # -------------------------------------------------------

  stop("Wrong 'type' argument.")
}

#' Multivariate and multinomial sample selection and endogenous switching models
#' with multiple outcomes.
#' @description This function allows to estimate parameters of the 
#' multivariate and multinomial sample selection and endogenous switching models
#' with multiple outcomes. Both maximum-likelihood and two-step estimators are
#' implemented.
#' @template msel_param_Template
#' @template msel_details_Template
#' @template msel_return_Template
#' @template msel_references_Template
#' @template msel_examples_cps_Template
#' @template msel_examples1_Template
#' @template msel_examples2_Template
#' @template msel_examples3_Template
#' @template msel_examples4_Template
#' @template msel_examples5_Template
#' @template msel_examples6_Template
#' @template msel_examples7_Template
#' @template msel_examples8_Template
#' @template msel_examples9_Template
#'
msel <- function(formula        = NA,
                 formula2       = NA,
                 formula3       = NA,
                 data           = NULL,
                 groups         = NA,
                 groups2        = NA,
                 groups3        = NA,
                 marginal       = list(),
                 opt_type       = "optim",
                 opt_args       = NA,
                 start          = NULL,
                 estimator      = "ml",
                 cov_type       = "mm",
                 degrees        = NA,
                 degrees3       = NA,
                 n_sim          = 1000, 
                 n_cores        = 1,
                 control        = list(),
                 regularization = list(),
                 type3          = "logit",
                 cluster        = NA)
{
  # List to store the output
  out        <- list()
  class(out) <- "msel"
  out$other  <- list(n_cores = n_cores, n_sim = n_sim)
  
  # Get the first step model if it has been provided
  model1 <- NULL
  if (is(object = formula, class2 = "msel"))
  {
    model1  <- formula
    formula <- model1$formula
  }
  if (is(object = formula3, class2 = "msel"))
  {
    model1   <- formula3
    formula3 <- model1$formula3
  }
  if (!is.null(model1))
  {
    formula  <- model1$formula
    formula3 <- model1$formula3
    marginal <- model1$marginal
    if (!any(is.na(groups2)))
    {
      groups  <- model1$groups
      groups3 <- model1$groups3
    }
    estimator <- "2step"
  }
  
  # Find the groups which have not been provided by the users
  is_na_group           <- c(any(is.na(groups)), 
                             any(is.na(groups2)), 
                             any(is.na(groups3)))
  out$other$is_na_group <- is_na_group
  
  # Deal with the control variables
  out_type <- "default"
  if (hasName(x = control, name = "out_type"))
  {
    out_type <- control$out_type
  }
  
  # -------------------------------------------------------
  # Validation
  # -------------------------------------------------------
  
  # Validate data
  if (!is.data.frame(data))
  {
    stop(paste0("Argument 'data' is missing or wrong. ",
                "Please, ensure that 'data' is a dataframe ",
                "containing variables described in 'formula', 'formula2', ",
                "and 'formula3'.\n"))
  }
  
  # Validate estimator
  estimator <- tolower(estimator)
  if (estimator %in% c("two-step", "twostep","2-step", "2st", "2", 'mm', 'gmm'))
  {
    warning(paste0("It is assumed that the 'estimator' '", estimator,
                   "' is '2step'.\n"))
    estimator <- "2step"
  }
  if (estimator %in% c("mle", "likelihood","maximum-likelihood", 
                       "1step", "maxlik", "fiml"))
  {
    warning(paste0("It is assumed that the 'estimator' '", estimator,
                   "' is 'ml'.\n"))
    estimator <- "ml"
  }
  if (!(estimator %in% c("2step", "ml")))
  {
    stop(paste0("Wrong 'estimator' argument. ",
                "Please, ensure that it is either '2step' or 'ml'.\n"))
  }
  out$estimator <- estimator
  
  # Validate formula
  is1 <- is.list(formula) | is.language(formula)
  if (is.null(formula))
  {
    formula <- NA
  }
  if (is1)
  {
    if (!is.list(formula))
    {
      tryCatch(formula <- as.formula(formula),
               error = function(e) {stop("Invalid 'formula' argument.\n")})
      formula <- list(formula)
    }
  }
  out$other$is1 <- is1
  
  # Validate formula2
  is2 <- is.list(formula2) | is.language(formula2)
  if (is.null(formula2))
  {
    formula2 <- NA
  }
  if (is2)
  {
    if (!is.list(formula2))
    {
      tryCatch(formula2 <- as.formula(formula2),
               error = function(e) {stop("Invalid 'formula2' argument.\n")})
      formula2 <- list(formula2)
    }
  }
  if (!is2 & (estimator != "ml"))
  {
    estimator <- "ml"
    warning(paste0("Since 'formula2' have not been provided it is assumed ",
                   "that 'estimator' is  'ml'.\n"))
  }
  out$other$is2 <- is2
  
  # Validate formula3
  is3 <- is.list(formula3) | is.language(formula3)
  if (is.null(formula3))
  {
    formula3 <- NA
  }
  if (is3)
  {
    tryCatch(formula3 <- as.formula(formula3),
             error = function(e) {stop("Invalid 'formula3' argument.\n")})
  }
  out$other$is3 <- is3
  
  # Validate the estimator one more time
  if (is2 & is3 & (estimator == "ml"))
  {
    estimator     <- "2step"
    out$estimator <- estimator
    warning("It is assumed that the 'estimator' is '2step'.")
  }
    
  # Check that at least some formulas have been provided
  if (!is1 & !is2 & !is3)
  {
    stop("No 'formula', 'formula2', or 'formula3' have been provided.\n")
  }
  
  # Get the number of equations
  n_eq  <- 0
  n_eq2 <- 0
  if (is1)
  {
    n_eq <- length(formula)
  }
  if (is2)
  {
    n_eq2 <- length(formula2)
  }
  out$other$n_eq  <- n_eq
  out$other$n_eq2 <- n_eq2
  
  # Validate groups
  if (is.null(groups))
  {
    groups <- NA
  }
  if (!is_na_group[1])
  {
    if (!is.matrix(groups))
    {
      if (is.vector(groups))
      {
        groups <- matrix(groups, ncol = 1)
      }
      else
      {
        groups <- as.matrix(groups)
      }
    }
    if (ncol(groups) != length(formula))
    {
      stop(paste0("Argument 'groups' is wrong. ",
                  "Please, ensure that 'groups' is a matrix ",
                  "whose number of columns equals the length ",
                  "of 'formula'.",
                  "\n"))
    }
  }
  
  # Validate groups2
  if (is.null(groups2))
  {
    groups2 <- NA
  }
  if (!is_na_group[2])
  {
    if (!is.matrix(groups2))
    {
      if (is.vector(groups2))
      {
        groups2 <- matrix(groups2, ncol = 1)
      }
      else
      {
        groups2 <- as.matrix(groups2)
      }
    }
    if (ncol(groups2) != length(formula2))
    {
      stop(paste0("Argument 'groups2' is wrong. ",
                  "Please, ensure that 'groups2' is a matrix ",
                  "whose number of columns equals the length ",
                  "of 'formula2'.",
                  "\n"))
    }
  }
  
  # Validate opt_type
  opt_type_vec <- c("optim", "gena", "pso")
  if (!(opt_type %in% opt_type_vec))
  {
    stop(paste0("Argument 'opt_type' is wrong. ",
                "Please, ensure that it is one of: ",
                paste0(opt_type_vec, collapse = ", "),
                ".\n"))
  }
  
  # Validate cov_type
  cov_type     <- tolower(cov_type)
  cov_type_vec <- NULL
  if (estimator == "ml")
  {
    cov_type_vec <- c("sandwich", "hessian", "gop", "no", "mm")
  }
  else
  {
    cov_type_vec <- c("mm", "gmm", "no")
  }
  if (!is.matrix(cov_type))
  {
    if (!(cov_type %in% cov_type_vec))
    {
      stop(paste0("Argument 'cov_type' is wrong. ",
                  "Please, ensure that it is one of: ",
                  paste0(cov_type_vec, collapse = ", "),
                  ".\n"))
    }
  }
  if (cov_type == "gmm")
  {
    cov_type <- "mm"
  }
  if (cov_type == "opg")
  {
    cov_type <- "gop"
    warning("It is assumed that cov_type = 'gop'.\n")
  }
  
  # Validate marginal
  n_marginal <- length(marginal)
  if (n_marginal > 0)
  {
    # transform vector of characters into the list of NULL values
    if (!is.list(marginal))
    {
      if (is.character(marginal))
      {
        marginal_list        <- vector(mode = "list", length = n_marginal)
        names(marginal_list) <- marginal
        marginal             <- marginal_list
      }
      else
      {
        stop("Argument 'marginal' should be either character vector or a list.")
      }
    }
    if (n_marginal != length(formula))
    {
      stop(paste0("The number of elements in 'marginal' should be the same ",
                  "as the number of ordered equations. ",
                  "Please, ensure that 'length(marginal) == length(formula)' ",
                  "or 'marginal' is an empty character vector."))
    }
  }
  
  # Validate degrees
  if ((estimator == "2step") & is1)
  {
    if (is.null(degrees))
    {
      degrees <- NA
    }
    if (any(is.na(degrees)))
    {
      degrees <- matrix(data = 1, nrow = n_eq2, ncol = n_eq)
    }
    if (is.vector(degrees))
    {
      if (length(degrees) != n_eq)
      {
        stop(paste0("Invalid 'degrees' argument. Please, ensure that ", 
                    "if 'degrees' is a vector then its length equals ",
                    "the number of selection equations."))
      }
      degrees <- matrix(rep(degrees, n_eq2), nrow = n_eq2, byrow = TRUE)
    }
    if (!is.matrix(degrees))
    {
      stop(paste0("Invalid 'degrees' argument. Please, ensure that ", 
                  "it is a matrix."))
    }
    if (nrow(degrees) > n_eq2)
    {
      stop(paste0("Invalid 'degrees' argument since it has ", nrow(degrees),
                  " rows. Please, ensure that it has ", n_eq2, " rows."))
    }
    if (ncol(degrees) > n_eq)
    {
      stop(paste0("Invalid 'degrees' argument since it has ", ncol(degrees),
                  " columns. Please, ensure that it has ", n_eq, " columns."))
    }
    if (any((degrees %% 1) != 0) | any(degrees < 0))
    {
      stop(paste0("Invalid 'degrees' argument. Please, ensure that ", 
                  "all elements of degrees are non-negative integers."))
    }
    for (v in seq_len(n_eq2))
    {
      formula2_terms <- attr(terms(formula2[[v]]), "term.labels")
      if (any(grepl("lambda", formula2_terms, fixed = TRUE)))
      {
        degrees[v, ] <- 0
      }
      # remove redundant selectivity terms
      if (!is_na_group[1] & !is_na_group[2])
      {
        for (j in 1:n_eq)
        {
          if (all(groups[groups2[, v] != -1, j] == -1))
          {
            degrees[v, j] <- 0
          }
        }
      }
    }
  }
  out$other$degrees <- degrees
  
  # Validate type3
  type3_vec <- c("logit", "probit")
  if (type3 %in% c("logistic", "mlogit", "mnlogit", "multinomial logit"))
  {
    type3 <- "logit"
    warning("It is assumed that 'type3' is 'logit'.")
  }
  if (type3 %in% c("normal", "mprobit", "mnprobit", "normal", "gaussian",
                   "norm", "multinomial probit"))
  {
    type3 <- "probit"
    warning("It is assumed that 'type3' is 'probit'.")
  }
  if (!(type3 %in% type3_vec))
  {
    stop(paste0("Wrong 'type3' argument. ",
                "Please, ensure that it is either 'logit' or 'probit'.\n"))
  }
  out$type3 <- type3
  
  # Validate degrees3
  n_degrees3 <- 0
  if ((estimator == "2step") & is3)
  {
    z_mn_name  <- all.vars(formula3)[1]
    n_degrees3 <- sum(unique(na.omit(data[, z_mn_name])) >= 0)
    if (type3 == "probit")
    {
      n_degrees3 <- n_degrees3 - 1
    }
    if (is.null(degrees3))
    {
      degrees3 <- NA
    }
    if (any(is.na(degrees3)))
    {
      degrees3 <- matrix(data = 1, nrow = n_eq2, ncol = n_degrees3)
    }
    if (is.vector(degrees3))
    {
      if (length(degrees3) != n_degrees3)
      {
        stop(paste0("Invalid 'degrees3' argument. Please, ensure that ", 
                    "if 'degrees3' is a vector then its length equals ",
                    "the number of multinomial equations."))
      }
      degrees3 <- matrix(rep(degrees3, n_eq2), nrow = n_eq2, byrow = TRUE)
    }
    if (!is.matrix(degrees3))
    {
      stop(paste0("Invalid 'degrees3' argument. Please, ensure that ", 
                  "it is a matrix."))
    }
    if (nrow(degrees3) > n_eq2)
    {
      stop(paste0("Invalid 'degrees3' argument since it has ", nrow(degrees3),
                  " rows. Please, ensure that it has ", n_eq2, " rows."))
    }
    if (ncol(degrees3) > n_degrees3)
    {
      stop(paste0("Invalid 'degrees3' argument since it has ", ncol(degrees3),
                  " columns. Please, ensure that it has ", 
                  model1$other$n_eq3, " columns."))
    }
    if (any((degrees3 %% 1) != 0) | any(degrees3 < 0))
    {
      stop(paste0("Invalid 'degrees3' argument. Please, ensure that ", 
                  "all elements of degrees3 are non-negative integers."))
    }
    for (v in seq_len(n_eq2))
    {
      formula2_terms <- attr(terms(formula2[[v]]), "term.labels")
      if (any(grepl("lambda_mn", formula2_terms, fixed = TRUE)))
      {
        degrees3[v, ] <- 0
      }
    }
  }
  out$other$degrees3   <- degrees3
  
  # Validate groups3
  if (is.matrix(groups3))
  {
    groups3 <- as.vector(groups3)
  }

  # -------------------------------------------------------
  # Formulas
  # -------------------------------------------------------
  
  # Adjust the formulas
  formula_list <- formula_msel(object   = out,      formula  = formula, 
                               formula2 = formula2, formula3 = formula3,
                               degrees  = degrees,  degrees3 = degrees3)
  
  # Get the results
  is_het          <- formula_list$is_het
  formula         <- formula_list$formula
  formula2        <- formula_list$formula2
  formula3        <- formula_list$formula3
  formula_mean    <- formula_list$formula_mean
  formula_var     <- formula_list$formula_var
  coef_lambda_ind <- formula_list$coef_lambda_ind
  
  # Clear the memory
  formula_list <- NULL
  
  # Save the results to the output
  out$other$is_het          <- is_het
  out$formula               <- formula
  out$formula2              <- formula2
  out$formula3              <- formula3
  out$other$formula_mean    <- formula_mean
  out$other$formula_var     <- formula_var
  out$other$coef_lambda_ind <- coef_lambda_ind
  
  # -------------------------------------------------------
  # The first step
  # -------------------------------------------------------
  
  # Prepare the variables to store the selectivity terms
  lambda    <- NULL
  lambda_mn <- NULL
  
  # Get estimates of the first step
  if ((estimator == "2step") & (is1 | is3))
  {
    if (is.null(model1))
    {
      # Multivariate and multinomial models
      model1 <- msel(formula   = formula,  formula3  = formula3,
                     groups    = groups,   groups3   = groups3,
                     data      = data,     estimator = "ml",
                     marginal  = marginal, opt_type  = opt_type,
                     opt_args  = opt_args, n_sim     = n_sim,
                     n_cores   = n_cores,  cov_type  = "mm",
                     type3     = type3,    cluster   = cluster)
      if (!is(object = model1, class2 = "msel"))
      {
        return(model1)
      }
    }
    out$model1 <- model1
    data       <- model1$data
    # Selectivity terms of the ordinal equations
    if (is1)
    {
      lambda <- model1$lambda
      for (i in seq_len(model1$other$n_eq))
      {
        data[[paste0("lambda", i)]] <- lambda[, i]
      }
    }
    # Selectivity term of the multinomial equation
    if (is3)
    {
      lambda_mn <- model1$lambda_mn
      for (i in seq_len(model1$other$n_eq3 - (type3 == "probit")))
      {
        data[[paste0("lambda", i, "_mn")]] <- lambda_mn[, i]
      }
    }
  }

  # -------------------------------------------------------
  # Missings
  # -------------------------------------------------------
  
  # Determine the groups before removing the missing values
  if (all(is_na_group))
  {
    groups_list           <- groups_msel(object  = out,    data    = data, 
                                         groups  = groups, groups2 = groups2, 
                                         groups3 = groups3)
    groups                <- groups_list$groups
    groups2               <- groups_list$groups2
    groups3               <- groups_list$groups3
    is_na_group           <- c(!is1, !is2, !is3)
    out$other$is_na_group <- is_na_group
    groups_list           <- NULL
  }

  # Get data including only the complete observations 
  data <- complete_msel(object = out, data = data)
  
  # Need to run second time because of the endogenous regressors
  data <- complete_msel(object = out, data = data)

  # -------------------------------------------------------
  # Groups
  # -------------------------------------------------------

  # Get the groups associated with the data
  groups_list <- groups_msel(object  = out,    data    = data, 
                             groups  = groups, groups2 = groups2, 
                             groups3 = groups3)

  # Store to the variables
  groups     <- groups_list$groups
  groups2    <- groups_list$groups2
  groups3    <- groups_list$groups3
  ind_g      <- groups_list$ind_g
  ind_eq     <- groups_list$ind_eq
  ind_eq2    <- groups_list$ind_eq2
  ind_eq3    <- groups_list$ind_eq3
  ind_eq_all <- groups_list$ind_eq_all
  n_cuts_eq  <- groups_list$n_cuts_eq
  n_eq3      <- groups_list$n_eq3
  n_groups   <- groups_list$n_groups
  data       <- groups_list$data
  n_obs      <- groups_list$n_obs
  n_obs_g    <- groups_list$n_obs_g
  n_eq_g     <- groups_list$n_eq_g
  n_eq2_g    <- groups_list$n_eq2_g
  n_eq_all_g <- groups_list$n_eq_all_g
  ind_g_all  <- groups_list$ind_g_all
  
  # Clear the memory
  groups_list <- NULL
  
  # Store to the outup
  out$groups           <- groups
  out$groups2          <- groups2
  out$groups3          <- groups3
  out$other$ind_g      <- ind_g
  out$other$ind_eq     <- ind_eq
  out$other$ind_eq2    <- ind_eq2
  out$other$ind_eq3    <- ind_eq3
  out$other$ind_eq_all <- ind_eq_all
  out$other$n_cuts_eq  <- n_cuts_eq
  out$other$n_eq3      <- n_eq3
  out$other$n_groups   <- n_groups
  out$data             <- data
  out$other$n_obs      <- n_obs
  out$other$n_obs_g    <- n_obs_g
  out$other$n_eq_g     <- n_eq_g
  out$other$n_eq2_g    <- n_eq2_g
  out$other$n_eq_all_g <- n_eq_all_g
  out$other$ind_g_all  <- ind_g_all
  
  # -------------------------------------------------------
  # Variables
  # -------------------------------------------------------
  
  # Get dependent and independent variables
  data_list <- data_msel(object = out, data = data)
  
  # Store to the variables
  W_mean <- data_list$W_mean
  W_var  <- data_list$W_var
  X      <- data_list$X
  W_mn   <- data_list$W_mn
  z      <- data_list$z
  y      <- data_list$y
  z_mn   <- data_list$z_mn
  
  # Clear the memory
  data_list <- NULL
  
  # Store to the output
  out$W_mean <- W_mean
  out$W_var  <- W_var
  out$X      <- X
  out$W_mn   <- W_mn
  out$z      <- z
  out$y      <- y
  out$z_mn   <- z_mn
  
  # -------------------------------------------------------
  # Names of the variables
  # -------------------------------------------------------

  names_list <- names_msel(object = out)

  # Store to the variables
  z_names         <- names_list$z_names
  y_names         <- names_list$y_names
  z_mn_names      <- names_list$z_mn_names
  colnames_X      <- names_list$colnames_X 
  colnames_W_mean <- names_list$colnames_W_mean
  colnames_W_var  <- names_list$colnames_W_var
  colnames_W_mn   <- names_list$colnames_W_mn
  
  # Clear the memory
  names_list <- NULL
  
  # Store to the output
  out$other$z_names         <- z_names
  out$other$y_names         <- y_names
  out$other$z_mn_names      <- z_mn_names
  out$other$colnames_X      <- colnames_X 
  out$other$colnames_W_mean <- colnames_W_mean
  out$other$colnames_W_var  <- colnames_W_var
  out$other$colnames_W_mn   <- colnames_W_mn
  
  # All the names
  all_names           <- c(z_names,        y_names,        z_mn_names, 
                           unlist(colnames_X),     unlist(colnames_W_mean), 
                           unlist(colnames_W_var), colnames_W_mn)
  all_names           <- na.omit(unique(all_names))
  out$other$all_names <- all_names
  
  # Names of the selectivity correction terms
  lambda_names <- numeric()
  if (estimator == "2step")
  {
    lambda_names <- all_names[grepl("lambda", all_names, fixed = TRUE)]
  }
  out$other$lambda_names <- lambda_names
  
  # Name the groups
  if (is1)
  {
    colnames(groups) <- z_names
    out$groups       <- groups
  }
  if (is2)
  {
    colnames(groups2) <- y_names
    out$groups2       <- groups2
  }
  
  # -------------------------------------------------------
  # Clusters
  # -------------------------------------------------------
  
  # Apply the clustering
  cluster_list <- cluster_msel(object = out, cluster = cluster)
  
  # Store to the variables
  ind_cluster <- cluster_list$ind_cluster
  n_cluster   <- cluster_list$n_cluster
  is_cluster  <- cluster_list$is_cluster
  
  # Store to the output
  out$cluster           <- cluster
  out$other$ind_cluster <- ind_cluster
  out$other$n_cluster   <- n_cluster
  out$other$is_cluster  <- is_cluster
  
  # -------------------------------------------------------
  # Indexes of the parameters
  # -------------------------------------------------------
  
  # Calculate the number of the regimes for each continuous variable
  n_regimes <- vector(mode = "numeric", length = 0)
  if (is2)
  {
    n_regimes <- vector(mode = "numeric", length = n_eq2)
    for (i in seq_len(n_eq2))
    {
      n_regimes[i] <- max(groups2[, i] + 1)
    }
  }
  out$other$n_regimes <- n_regimes

  # Get the number of coefficients (regressors) for each equation and 
  # determine their indexes in the parameters vector
  n_coef   <- vector("mode" = "numeric", length = n_eq)
  coef_ind <- vector("mode" = "list",    length = n_eq)
  if (is1)
  {
    for(i in seq_len(n_eq))
    {
      n_coef[i] <- ncol(W_mean[[i]])
      if (i != 1)
      {
        coef_ind[[i]] <- (coef_ind[[i - 1]][n_coef[i - 1]] + 1):
                         ((coef_ind[[i - 1]][n_coef[i - 1]] + n_coef[i]))
      }
      else
      {
        coef_ind[[i]] <- seq_len(n_coef[i])
      }
    }
  }
  
  # Get sigma elements which are not identified
  # and store them into the matrix
  sigma_omit <- matrix(0, nrow = n_eq, ncol = n_eq)
  if (is1)
  {
    if (n_eq > 1)
    {
      for (i in seq_len(n_eq - 1))
      {
        for(j in (i + 1):n_eq)
        {
          sigma_omit[i, j] <- as.numeric(all((groups[, i, drop = FALSE] == -1) | 
                                             (groups[, j, drop = FALSE] == -1)))
          sigma_omit[j, i] <- sigma_omit[i, j]
        }
      }
    }
  }
  out$other$sigma_omit <- sigma_omit
  
  # The number of the unidentified sigma elements
  n_sigma_omit <- sum(sigma_omit) / 2
  
  # Calculate the total number of the estimated coefficients
  n_coef_total <- sum(n_coef)
  
  # Get the number of the covariance matrix elements to be estimated
  n_sigma           <- (n_eq ^ 2 - n_eq) / 2 - n_sigma_omit
  out$other$n_sigma <- n_sigma
  
  # Get the number of the cut points to be considered
  n_cuts <- sum(n_cuts_eq)
  
  # Get indexes of the sigma elements in the parameters vector
  sigma_ind <- is1
  if (n_sigma > 0)
  {
    sigma_ind <- (n_coef_total + 1):(n_coef_total + n_sigma)
  }

  # Store the indexes of the sigma elements in a matrix form
  sigma_ind_mat <- matrix(1, nrow = n_eq, ncol = n_eq)
  if (is1)
  {
    if (n_eq >= 2)
    {
      counter <- sigma_ind[1]
      for (i in 2:n_eq)
      {
        for (j in seq_len(i - 1))
        {
          if (sigma_omit[i, j] == 0)
          {
            sigma_ind_mat[i, j] <- counter
            sigma_ind_mat[j, i] <- sigma_ind_mat[i, j]
            counter             <- counter + 1
          }
          else
          {
            # special number for omitted covariances
            sigma_ind_mat[i, j] <- 0
          }
        }
      }
    }
  }

  # Store the indexes of the cuts in a list form
  cuts_ind <- vector(mode = "list", length = n_eq)
  n_par    <- n_coef_total + n_sigma
  if (is1)
  {
    for(i in seq_len(n_eq))
    {
      for (j in seq_len(n_cuts_eq[i]))
      {
        n_par            <- n_par + 1
        cuts_ind[[i]][j] <- n_par
      }
    }
  }
  
  # Deal with the coefficients of the variance equation
  n_coef_var   <- vector("mode" = "numeric", length = n_eq)
  coef_var_ind <- vector("mode" = "list",    length = n_eq)
  if (is1)
  {
    for(i in seq_len(n_eq))
    {
      if(is_het[i])
      {
        n_coef_var[i]     <- ncol(W_var[[i]])
        coef_var_ind[[i]] <- (n_par + 1):(n_par + n_coef_var[i])
        n_par             <- n_par + n_coef_var[i]
      }
      else
      {
        coef_var_ind[[i]] <- as.vector(1)
      }
    }
  }

  # Deal with the coefficients of the continuous equations
  n_coef2   <- vector(mode = "numeric", length = 0)
  coef2_ind <- list(matrix(1))
  if (is2)
  {
    n_coef2   <- vector(mode = "numeric", length = n_eq2)
    coef2_ind <- vector(mode = "list",    length = n_eq2)
    for(i in seq_len(n_eq2))
    {
      n_coef2[i] <- ncol(X[[i]])
      # coefficients for each regime are located in the different rows
      coef2_ind[[i]] <- matrix((n_par + 1):(n_par + n_coef2[i] * n_regimes[i]), 
                               nrow = n_regimes[i], ncol = n_coef2[i], 
                               byrow = TRUE)
      n_par          <- n_par + length(coef2_ind[[i]])
    }
  }
  
  # Determine the structure of the covariances between the continuous
  # equations for different regimes
  regimes         <- as.matrix(t(hpa::polynomialIndex(n_regimes - 1)))
  n_regimes_total <- nrow(regimes)
  
  # Get cov2 elements which are not identified
  # and store them into the list
  cov2_omit      <- vector(mode = "list", length = 1)
  cov2_omit[[1]] <- matrix(0, nrow = 1, ncol = 1)
  if (is1 & is2 & (estimator == "ml"))
  {
    cov2_omit <- vector(mode = "list", length = n_eq2)
    for (v in seq_len(n_eq2))
    {
      cov2_omit[[v]] <- matrix(0, ncol = n_eq, nrow = n_regimes[v])
      for (r in seq_len(n_regimes[v]))
      {
        for (j in 1:n_eq)
        {
          cov2_omit[[v]][r, j] <- all(groups[groups2[, v] == (r - 1), j] == -1) 
        }
      }
    }
  }
  out$other$cov2_omit <- cov2_omit
  
  # If needed, add the covariances related to the continuous equations
  var2_ind       <- list(as.vector(1))
  cov2_ind       <- list(matrix(1))
  sigma2_ind     <- list(as.vector(1))
  sigma2_ind_mat <- list(matrix(1))
  if (is2 & (estimator == "ml"))
  {
    # variances
    for (i in seq_len(n_eq2))
    {
      var2_ind[[i]] <- (n_par + 1):(n_par + n_regimes[i])
      n_par         <- n_par + n_regimes[i]
    }
    # ordinal equations
    if (is1)
    {
      cov2_ind <- vector(mode = "list", length = n_eq2)
      for (i in seq_len(n_eq2))
      {
        cov2_ind[[i]] <- matrix(1, ncol = n_eq, nrow = n_regimes[i])
        for (j in seq_len(n_regimes[i]))
        {
          n_cov2_omit <- sum(cov2_omit[[i]][j, ])
          if (any(cov2_omit[[i]][j, ] == 0))
          {
            cov2_ind[[i]][
              j, cov2_omit[[i]][j, ] == 0] <- (n_par + 1):(n_par + n_eq - 
                                                           n_cov2_omit)
            n_par <- n_par + n_eq - n_cov2_omit
          }
        }
      }
    }
      # multinomial equations
    if (is3 & (type3 == "probit"))
    {
      for (i in seq_len(n_eq2))
      {
        cov2_ind[[i]] <- matrix(ncol = n_eq3 - 1, nrow = n_regimes[i])
        for (j in seq_len(n_regimes[i]))
        {
          cov2_ind[[i]][j, ] <- (n_par + 1):(n_par + n_eq3 - 1)
          n_par              <- n_par + n_eq3 - 1
        }
      }
    }
    # covariances between the continuous equations
    if (n_eq2 >= 2)
    {
      n_sigma2           <- (n_eq2 ^ 2 - n_eq2) / 2
      out$other$n_sigma2 <- n_sigma2
      sigma2_ind         <- vector(mode = "list", length = n_sigma2)
      regimes_pair       <- vector(mode = "list", length = n_sigma2)
      sigma2_ind_mat     <- vector(mode = "list", length = n_groups)
      for (g in seq_len(n_groups))
      {
        sigma2_ind_mat[[g]] <- matrix(1, nrow = n_eq2, ncol = n_eq2)
      }
      # indexes for each covariance depending on the regime
      counter <- 1
      for (i in 2:n_eq2)
      {
        for (j in seq_len(i - 1))
        {
          sigma2_ind[[counter]]   <- numeric()
          regimes_pair[[counter]] <- numeric()
          # note that i > j so in regimes_pair equation with
          # the greater index goes first
          pairs          <- unique(groups2[, c(i, j), drop = FALSE])
          pairs          <- pairs[(pairs[, 1, drop = FALSE] != -1) & 
                                  (pairs[, 2, drop = FALSE] != -1), , 
                                  drop = FALSE]
          regimes_pair_n <- ifelse(is.matrix(pairs), nrow(pairs), 0)
          if (regimes_pair_n > 0)
          {
            regimes_pair[[counter]] <- pairs
            sigma2_ind[[counter]]   <- (n_par + 1):(n_par + regimes_pair_n)
            n_par                   <- n_par + regimes_pair_n
            # indexes of covariances depending on group
            for (g in seq_len(n_groups))
            {
              cond <- which((regimes_pair[[counter]][, 1] == groups2[g, i]) &
                            (regimes_pair[[counter]][, 2] == groups2[g, j]))
              if (length(cond) > 0)
              {
                sigma2_ind_mat[[g]][i, j] <- sigma2_ind[[counter]][cond]
                sigma2_ind_mat[[g]][j, i] <- sigma2_ind[[counter]][cond]
              }
            }
          }
          counter <- counter + 1
        }
      }
      out$other$regimes_pair <- regimes_pair
    }
  }

  # Parameters associated with the marginal distributions
  is_marginal              <- length(marginal) > 0
  out$other$is_marginal    <- is_marginal
  marginal_par_ind         <- list(as.vector(1))
  marginal_par_n           <- rep(0, n_eq)
  out$other$marginal_par_n <- marginal_par_n
  marginal_names           <- character()
  if (is_marginal)
  {
    marginal_names <- names(marginal)
    for (i in seq_len(n_eq))
    {
      marginal_par_n[i] <- ifelse((length(marginal[[i]]) != 0) &
                                  !is.null(marginal[[i]]), 
                                  as.numeric(marginal[[i]]), 0)
      if (marginal_par_n[i] > 0)
      {
        marginal_par_ind[[i]] <- (n_par + 1):(n_par + marginal_par_n[i])
        n_par                 <- n_par + length(marginal_par_ind[[i]])
      }
    }
  }
  out$marginal <- marginal
  
  # Round the number of parameters to prevent the bugs
  marginal_par_n           <- round(marginal_par_n)
  out$other$marginal_par_n <- marginal_par_n
  
  # Coefficients of the multinomial equation
  n_coef3   <- 0
  coef3_ind <- matrix(1)
  if (is3)
  {
    n_coef3   <- ncol(W_mn)
    coef3_ind <- matrix(nrow = n_eq3 - 1, ncol = n_coef3)
    for(i in seq_len(n_eq3 - 1))
    {
      coef3_ind[i, ] <- n_par + (((i - 1) * n_coef3 + 1):(i * n_coef3))
    }
    n_par <- n_par + length(coef3_ind)
  }
  out$other$n_coef3 <- n_coef3
  
  # Covariances of the multinomial equations
  # Note: if sigma3_ind_mat[k, ] = c(i, j) it means
  # that sigma3_ind[k] refers to the sigma3[i, j]
  n_sigma3       <- ((n_eq3 - 1) ^ 2 - n_eq3 + 1) / 2 + (n_eq3 - 2)
  sigma3_ind     <- 1
  sigma3_ind_mat <- matrix(1)
  if (is3 & (type3 == "probit"))
  {
    if (n_sigma3 > 0)
    {
      sigma3_ind     <- (n_par + 1):(n_par + n_sigma3)
      sigma3_ind_mat <- matrix(1, nrow = n_sigma3, ncol = 2)
    }
    if (n_eq3 > 2)
    {
      counter <- 1
      for (i in seq_len(n_eq3 - 1))
      {
        for (j in 1:i)
        {
          if (!((i == 1) & (j == 1)))
          {
            sigma3_ind_mat[counter, 1] <- i
            sigma3_ind_mat[counter, 2] <- j
            counter                    <- counter + 1
          }
        }
      }
    }
    n_par <- n_par + length(sigma3_ind)
  }
  
  # Save the number of the estimated parameters
  out$other$n_par <- n_par
  
  # Save the indexes
  out$ind <- list(coef         = coef_ind,      coef_var     = coef_var_ind,
                  cuts         = cuts_ind,      sigma        = sigma_ind,
                  g            = ind_g,         eq           = ind_eq,
                  var2         = var2_ind,      sigma2       = sigma2_ind,
                  coef2        = coef2_ind,     cov2         = cov2_ind,
                  sigma2       = sigma2_ind,    marginal_par = marginal_par_ind,
                  sigma_mat    = sigma_ind_mat, coef3        = coef3_ind,
                  sigma3       = sigma3_ind,    sigma3_mat   = sigma3_ind_mat)
  
  # -------------------------------------------------------
  # Maximum-likelihood estimator
  # -------------------------------------------------------

  # Store the information needed for the maximum-likelihood 
  # estimator into the list
  groups_tmp <- groups
  if (any(is.na(groups)))
  {
    groups_tmp <- matrix()
  }
  groups2_tmp <- groups2
  if (any(is.na(groups2)))
  {
    groups2_tmp <- matrix()
  }
  groups3_tmp <- groups3
  if (any(is.na(groups3)))
  {
    groups3_tmp <- vector(mode = "numeric")
  }
  control_lnL <- list(n_par            = n_par,
                      n_obs            = n_obs,
                      n_eq             = n_eq,
                      n_eq2            = n_eq2,
                      n_eq3            = n_eq3,
                      n_coef           = n_coef,
                      n_coef2          = n_coef2,
                      n_coef3          = n_coef3,
                      n_regimes        = n_regimes,
                      coef_ind         = lapply(coef_ind, function(x){x - 1}),
                      coef_var_ind     = lapply(coef_var_ind, 
                                                function(x){x - 1}),
                      coef2_ind        = lapply(coef2_ind, function(x){x - 1}),
                      sigma_ind        = sigma_ind - 1,
                      sigma2_ind       = lapply(sigma2_ind, function(x){x - 1}),
                      sigma_ind_mat    = sigma_ind_mat - 1,
                      sigma_omit       = sigma_omit,
                      sigma2_ind_mat   = lapply(sigma2_ind_mat, 
                                                function(x){x - 1}),
                      sigma3_ind       = sigma3_ind - 1,
                      sigma3_ind_mat   = sigma3_ind_mat - 1,
                      var2_ind         = lapply(var2_ind, function(x){x - 1}),
                      cov2_ind         = lapply(cov2_ind, function(x){x - 1}),
                      cov2_omit        = cov2_omit,
                      cuts_ind         = lapply(cuts_ind, function(x){x - 1}),
                      marginal_par_ind = lapply(marginal_par_ind, 
                                                function(x){x - 1}),
                      coef3_ind        = coef3_ind - 1,
                      n_groups         = n_groups,
                      ind_g            = lapply(ind_g, function(x){x - 1}),
                      ind_eq           = lapply(ind_eq, function(x){x - 1}),
                      ind_eq2          = lapply(ind_eq2, function(x){x - 1}),
                      ind_eq_all       = lapply(ind_eq_all, function(x){x - 1}),
                      n_cuts_eq        = n_cuts_eq,
                      groups           = groups_tmp,
                      groups2          = groups2_tmp,
                      groups3          = groups3_tmp,
                      n_obs_g          = n_obs_g,
                      n_eq_g           = n_eq_g,
                      n_eq2_g          = n_eq2_g,
                      n_eq_all_g       = n_eq_all_g,
                      is_het           = is_het,
                      is1              = is1,
                      is2              = is2,
                      is3              = is3,
                      y                = y,
                      X                = X,
                      W                = W_mean,
                      W_var            = W_var,
                      W_mn             = W_mn,
                      marginal_names   = marginal_names,
                      marginal_par_n   = marginal_par_n,
                      type3            = type3)
  out$control_lnL <- control_lnL
  
  # Validate the regularization
  regularization <- regularization_validate(regularization = regularization, 
                                            n_par          = n_par, 
                                            estimator      = estimator)
  out$other$regularization <- regularization
  
  # Vector of the estimates of the estimated parameters
  par <- rep(NA, n_par)
  
  # Create a starting point for the numeric optimization
  if (is.null(start) & (estimator == "ml"))
  {
    start <- rep(0, n_par)
    if (is1)
    {
      for (i in seq_len(n_eq))
      {
        # adjusted ordinal outcome
        z_tmp                <- z[, i]
        z_tmp[z_tmp == -1]   <- NA
        is_na_z              <- is.na(z[, i])
        z_tmp                <- na.omit(z[, i])
        # cuts
        for (j in seq_len(n_cuts_eq[i]))
        {
          start[cuts_ind[[i]]][j] <- qnorm(mean(na.omit(z_tmp) <= (j - 1)))
        }
        # coefficients
        z_tmp_table          <- table(z_tmp)
        z_tmp_ind            <- which.min(abs((cumsum(z_tmp_table) / 
                                               sum(z_tmp_table)) - 0.5)) - 1
        z_tmp_cond           <- z_tmp <= z_tmp_ind
        z_tmp[z_tmp_cond]    <- 0
        z_tmp[!z_tmp_cond]   <- 1
        data_tmp             <- data[!is_na_z, ]
        data_tmp[z_names[i]] <- z_tmp
        model_tmp            <- glm(formula_mean[[i]], data = data_tmp,
                                    family = binomial(link = "logit"))
        start[coef_ind[[i]]] <- coef(model_tmp)[-1] * (sqrt(3) / pi)
        # heteroscedasticity
        if (is_het[i])
        {
          n_coef_var_ind_i <- length(coef_var_ind[[i]])
          for (j in 1:n_coef_var_ind_i)
          {
            W_var_j_mean <- mean(W_var[[i]][, j], na.rm = TRUE)
            val_tmp <- 1
            if (abs(W_var_j_mean) > 1)
            {
              val_tmp <- val_tmp / W_var_j_mean
            }
            val_tmp <- val_tmp / sd(W_var[[i]][, j], na.rm = TRUE)
            start[coef_var_ind[[i]][j]] <- val_tmp
          }
          start[coef_var_ind[[i]]] <- start[coef_var_ind[[i]]] / 
                                      (5 * n_coef_var_ind_i)
        }
      }
    }
    for (i in seq_len(n_eq2))
    {
      data_tmp <- data.frame(y = y[, i], X[[i]])
      data_tmp <- data_tmp[rowSums(is.na(X[[i]])) == 0, ]
      for (j in seq_len(n_regimes[i]))
      {
        ind_g_include_tmp <- which(groups2[, i] == (j - 1))
        ing_g_tmp         <- NULL
        for (t in ind_g_include_tmp)
        {
          ing_g_tmp <- c(ing_g_tmp, ind_g[[t]])
        }
        data_tmp_g <- data_tmp[ing_g_tmp, ]
        model_tmp  <- lm(y ~ . + 0, 
                         data = data_tmp_g[!is.na(data_tmp_g$y), ])
        start[coef2_ind[[i]][j, ]] <- coef(model_tmp)
        start[var2_ind[[i]][j]]    <- sigma(model_tmp) ^ 2
      }
    }
    if (is_marginal)
    {
      for (i in seq_len(n_eq))
      {
        if ((marginal_names[i] == "PGN") | (marginal_names[i] == "hpa"))
        {
          start[marginal_par_ind[[i]]] <- 1e-8
        }
        if (marginal_names[i] == "student")
        {
          start[marginal_par_ind[[i]]] <- 30
        }
      }
    }
    if (is3 & (type3 == "probit"))
    {
      if (n_eq3 > 2)
      {
        for (i in seq_len(n_sigma3))
        {
          if (sigma3_ind_mat[i, 1] == sigma3_ind_mat[i, 2])
          {
            start[sigma3_ind[i]] <- 1
          }
        }
      }
    }
  }
  out$start <- start
  
  # Code to test the derivatives
  if (hasName(control, "test_grad"))
  {
    f0 <- lnL_msel(par = start,
                   n_sim = n_sim, n_cores = n_cores,
                   control_lnL = control_lnL)
    grad.a <- lnL_msel(par = start,
                       n_sim = n_sim, n_cores = n_cores,
                       control_lnL = control_lnL,
                       out_type = "grad")
    grad.n <- rep(NA, n_par)
    for (i in seq_len(n_par))
    {
      delta <- 1e-6
      start.delta <- start
      start.delta[i] <- start[i] + delta
      f1 <- lnL_msel(par = start.delta,
                          n_sim = n_sim, n_cores = n_cores,
                          control_lnL = control_lnL)
      grad.n[i] <- (f1 - f0) / delta
    }
    return(cbind(analytical = as.vector(grad.a), numeric = as.vector(grad.n)))
  }

  # Optimization
  opt    <- NULL
  is_opt <- TRUE
  if (hasName(control, "is_opt"))
  {
    is_opt <- control$is_opt
    par    <- start
  }
  if ((estimator == "ml") & is_opt)
  {
    opt <- opt_switchSelection(opt_args       = opt_args, 
                               control_lnL    = control_lnL,
                               n_sim          = n_sim, 
                               n_cores        = n_cores,
                               type           = "msel", 
                               start          = start, 
                               opt_type       = opt_type, 
                               regularization = regularization)
    par <- opt$par
  }
  gc()

  # -------------------------------------------------------
  # Two-step estimator
  # -------------------------------------------------------

  # List to store the least squares estimators
  model2_list <- NULL
  
  # List to store the indexes of the observations for different regimes
  ind_regime  <- NULL
  
  # Matrix to store the predictions of the continuous equations
  # used by V-stage least squares estimator
  y_pred  <- NULL

  # The main routine
  if (estimator == "2step")
  {
    # Initialize the matrix to store the predictions
    y_pred           <- matrix(NA, nrow = n_obs, ncol = n_eq2)
    colnames(y_pred) <- y_names
    
    # Store the estimates of the first step
    if (is1)
    {
      for (i in seq_len(n_eq))
      {
        par[cuts_ind[[i]]] <- model1$cuts[[i]]
        par[coef_ind[[i]]] <- model1$coef[[i]]
        if (is_het[i])
        {
          par[coef_var_ind[[i]]] <- model1$coef_var[[i]]
        }
        if (is_marginal)
        {
          if (model1$other$marginal_par_n[i] > 0)
          {
            par[marginal_par_ind[[i]]] <- model1$par[
              model1$ind$marginal_par[[i]]]
          }
        }
      }
      if (n_sigma > 0)
      {
        par[sigma_ind] <- model1$par[model1$ind$sigma]
      }
    }
    if (is3)
    {
      for (i in seq_len(n_eq3 - 1))
      {
        par[coef3_ind[i, ]] <- model1$par[model1$ind$coef3[i, ]]
      }
      par[sigma3_ind] <- model1$par[model1$ind$sigma3]
    }

    # List to store the least squares models
    model2_list <- vector(mode = "list", length = n_eq2)
    
    # List to store the indexes of the observations in each regime
    ind_regime <- vector(mode = "list", length = n_eq2)
    
    # Two-step procedure for each equation
    for (v in seq_len(n_eq2))
    {
      # Conduct two-step procedure for each regime
      ind_regime[[v]] <- vector(mode = "list", length = n_regimes[v])
      for (i in seq_len(n_regimes[v]))
      {
        # Indexes of the observations corresponding to the regime
        for (j in seq_len(n_groups))
        {
          if (groups2[j, v] == (i - 1))
          {
            ind_regime[[v]][[i]] <- c(ind_regime[[v]][[i]], ind_g[[j]])
          }
        }
        
        # Regime specific data
        data_regime <- data[ind_regime[[v]][[i]], ]
        if (v >= 2)
        {
          # predictions of the previous equation
          for (v0 in seq_len(v - 1))
          {
            if (y_names[v0] %in% colnames(data_regime))
            {
              data_regime[, y_names[v0]] <- y_pred[ind_regime[[v]][[i]], v0]
            }
          }
        }
        
        # Estimate least squares regression
        model2_list[[v]][[i]] <- lm(formula2[[v]], data = data_regime, 
                                    na.action = na.exclude)
        
        # Save predictions of the regression
        y_pred[ind_regime[[v]][[i]], v] <- predict(model2_list[[v]][[i]])
        
        # Store the coefficients
        par[coef2_ind[[v]][i, ]] <- coef(model2_list[[v]][[i]])
      }
    }
  }
  out$twostep <- model2_list
  out$y_pred  <- y_pred
  gc()
  
  # Store parameters into the variables
    # get the list of the parameters
  par_list <- par_msel(object = out, par = par)
    # assign them to the variables
  par             <- par_list$par
  coef            <- par_list$coef
  coef_var        <- par_list$coef_var
  cuts            <- par_list$cuts
  coef2           <- par_list$coef2
  sigma           <- par_list$sigma
  var2            <- par_list$var2
  cov2            <- par_list$cov2
  sigma2          <- par_list$sigma2
  marginal_par    <- par_list$marginal_par
  sigma_vec_ind   <- par_list$sigma_vec_ind
  coef3           <- par_list$coef3
  sigma3          <- par_list$sigma3
  
  # Store the variables in the output list
  out$par                 <- par
  out$coef                <- coef
  out$coef_var            <- coef_var
  out$cuts                <- cuts
  out$coef2               <- coef2
  out$sigma               <- sigma
  out$var2                <- var2
  out$cov2                <- cov2
  out$sigma2              <- sigma2
  out$marginal_par        <- marginal_par
  out$other$sigma_vec_ind <- sigma_vec_ind
  out$coef3               <- coef3
  out$sigma3              <- sigma3
  
  # Clear the memory
  par_list <- NULL

  # Estimate lambda for the sample selection models
  if (is1)
  {
    out$lambda <- predict(out, type = "lambda")
  }
  if (is3)
  {
    out$lambda_mn <- predict(out, type = "lambda_mn")
  }

  # Estimate the asymptotic covariance matrix
    # Prepare some values
  H     <- NULL                                   # Hessian
  H_inv <- NULL                                   # Inverse Hessian
  J     <- NULL                                   # Jacobian (or scores for MM)
  cov   <- matrix(NA, nrow = n_par, ncol = n_par) # Asymptotic covariance matrix
    # Manual covariance matrix
  if (is.matrix(cov_type))
  {
    cov <- cov_type
  }
  else
  {
    cov_type <- tolower(cov_type)
  }
    # Maximum-likelihood asymptotic covariance matrix estimator
  if (!is.matrix(cov_type) & (estimator == "ml"))
  {
    vcov_object <- vcov_ml(object = out, type = cov_type, 
                           n_cores = n_cores, n_sim = n_sim)
    cov <- vcov_object$vcov
    if (hasName(vcov_object, "H"))
    {
      H     <- vcov_object$H
      out$H <- H
    }
    if (hasName(vcov_object, "J"))
    {
      J     <- vcov_object$J
      out$J <- J
    }
    if (hasName(vcov_object, "J"))
    {
      out$J_cluster <- vcov_object$J_cluster
    }
  }
    # Two-step asymptotic covariance matrix estimator
  if (estimator == "2step" & (cov_type != "no"))
  {
    is_cov_simple <- FALSE
    if (hasName(control, "is_cov_simple"))
    {
      is_cov_simple <- control$is_cov_simple
    }
    vcov_object <- vcov_2step(object        = out,
                              n_cores       = n_cores, 
                              n_sim         = n_sim,
                              is_cov_simple = is_cov_simple)
    cov         <- vcov_object$vcov
    out$J       <- vcov_object$scores
    out$H       <- vcov_object$scores_jac
    if (is_cluster)
    {
      out$J_cluster <- vcov_object$scores_cluster
    }
  }
  out$cov_type <- cov_type
  out$cov      <- cov

  # Create tables to store the results in a format used by summary function
  tbl_list    <- tbl_msel(object = out)
  out$tbl     <- tbl_list$tbl
  out$se      <- tbl_list$se
  out$p_value <- tbl_list$p_value
  
  # Calculate the log-likelihood value
  logLik_val <- NA
  if (estimator == "ml")
  {
    if (length(regularization) == 0)
    {
      if (is_opt)
      {
        logLik_val <- opt$value
      }
    }
    else
    {
      # because of the regularization
      logLik_val <- lnL_msel(par = par, control_lnL = control_lnL)
    }
  }
  out$logLik <- logLik_val

  return(out)
}

# Differentiate function with respect to the parameters
# estimated via msel function
deriv_msel <- function(object, 
                       fn, 
                       fn_args = list(), 
                       eps     = max(1e-4, sqrt(.Machine$double.eps) * 10),
                       type    = "default",
                       n_sim   = 10000,
                       n_cores = 1,
                       is.gc   = FALSE)
{
  # Deal with several objects
  if (!is(object = object, class2 = "msel"))
  {
    # Get the number of objects
    n_objects <- length(object)
    
    # Setup the arguments of the function
    fn_args$object <- object
    
    # List to store the outputs of single calls
    d <- vector(mode = "list", length = n_objects)
    
    # Prepare the output
    out <- list(grad = NULL, val = NULL)
    
    # Create a new function
    fn_i <- function(object, fn_args = fn_args, i = 1)
    {
      # Call the initial function
      fn_args$object[[i]] <- object
      val                 <- do.call(what = fn, args = fn_args)

      # Return the results
      return (val)
    }
    
    # Apply the functions
    for (i in 1:n_objects)
    {
      # Check the class of the object
      if (!is(object = object[[i]], class2 = "msel"))
      {
        stop(paste0("Argument object[[", i, "]] is not of class 'msel'."))
      }
      
      # Get eps
      eps_i <- eps
      if (is.list(eps))
      {
        eps_i <- eps[[i]]
      }
      
      # Recursive call with a new function
      d[[i]] <- deriv_msel(object  = object[[i]], 
                           fn      = fn_i,
                           fn_args = list(fn_args = fn_args, i = i), 
                           eps     = eps_i,
                           type    = type,
                           n_sim   = n_sim,
                           n_cores = n_cores,
                           is.gc   = is.gc)
      
      # Combine the results
      out$grad <- cbind(out$grad, d[[i]]$grad)
    }
    out$val <- d[[1]]$val    
    
    # Return the results
    return (out)
  }
  
  # Set some properties of the object
  object$n_sim   <- n_sim
  object$n_cores <- n_cores
  
  # Get some variables
  n_par      <- object$other$n_par
  n_eq       <- object$other$n_eq
  n_eq2      <- object$other$n_eq2
  n_eq3      <- object$other$n_eq3
  is1        <- object$other$is1
  is2        <- object$other$is2
  is3        <- object$other$is3
  type3      <- object$type3
  sigma_omit <- object$other$sigma_omit
  cov2_omit  <- object$other$cov2_omit
  
  # Deal with eps
  if (length(eps) == 1)
  {
    eps <- rep(eps, n_par)
  }
  
  # Estimate value of the function at initial point
  fn_args$object <- object
  fn_val         <- do.call(what = fn, args = fn_args)
  if (is.matrix(fn_val))
  {
    if (ncol(fn_val) > 1)
    {
      stop ("Function 'fn' should return a vector or a single column matrix.")
    }
  }
  n_val <- length(fn_val)
  
  # Prepare the output matrix
  out           <- matrix(0, nrow = n_val, ncol = n_par)
  colnames(out) <- 1:ncol(out)
  
  # Differentiate respect to the coefficients
  coef_ind <- object$ind$coef
  if (is1)
  {
    for (i in 1:n_eq)
    {
      for (j in 1:length(object$coef[[i]]))
      {
        if (eps[coef_ind[[i]][j]] != 0)
        {
          # save initial value
          par_old_tmp                     <- object$coef[[i]][j] 
          # prepare increment
          eps_tmp                         <- eps[coef_ind[[i]][j]] * 
                                             abs(par_old_tmp)
          # plus
          fn_args$object$coef[[i]][j]     <- par_old_tmp + eps_tmp
          fn_plus                         <- do.call(what = fn, args = fn_args)
          # minus
          fn_args$object$coef[[i]][j]     <- par_old_tmp - eps_tmp
          fn_minus                        <- do.call(what = fn, args = fn_args)
          # derivative
          out[, coef_ind[[i]][j]]         <- (fn_plus - fn_minus) / 
                                             (2 * eps_tmp)
          # names
          colnames(out)[coef_ind[[i]][j]] <- paste0("coef",  j, " of ",
                                                    object$other$z_names[i])
          # set initial value
          fn_args$object$coef[[i]][j]     <- par_old_tmp
        }
      }
      if (is.gc)
      {
        gc()
      }
    }
  }
  
  # Differentiate respect to the cuts
  cuts_ind <- object$ind$cuts
  if (is1)
  {
    for (i in 1:n_eq)
    {
      for (j in 1:length(object$cuts[[i]]))
      {
        if (eps[cuts_ind[[i]][j]] != 0)
        {
          # save initial value
          par_old_tmp <- object$cuts[[i]][j]
          # prepare increment
          eps_tmp <- eps[cuts_ind[[i]][j]] * abs(par_old_tmp)
          # plus
          fn_args$object$cuts[[i]][j] <- par_old_tmp + eps_tmp
          fn_plus <- do.call(what = fn, args = fn_args)
          # minus
          fn_args$object$cuts[[i]][j] <- par_old_tmp - eps_tmp
          fn_minus <- do.call(what = fn, args = fn_args)
          # derivative
          out[, cuts_ind[[i]][j]] <- (fn_plus - fn_minus) / (2 * eps_tmp)
          # names
          colnames(out)[cuts_ind[[i]][j]] <- paste0("cut",  j, " of ",
                                                    object$other$z_names[i])
          # set initial value
          fn_args$object$cuts[[i]][j] <- par_old_tmp
        }
      }
      if (is.gc)
      {
        gc()
      }
    }
  }
  
  # Differentiate respect to the coefficients of the variance equation
  coef_var_ind <- object$ind$coef_var
  if (is1)
  {
    for (i in 1:n_eq)
    {
      if (object$other$is_het[i])
      {
        for (j in 1:length(coef_var_ind[[i]]))
        {
          if (eps[coef_var_ind[[i]][j]] != 0)
          {
            # save initial value
            par_old_tmp <- object$coef_var[[i]][j]
            # prepare increment
            eps_tmp <- eps[coef_var_ind[[i]][j]] * abs(par_old_tmp)
            # plus
            fn_args$object$coef_var[[i]][j] <- par_old_tmp + eps_tmp
            fn_plus <- do.call(what = fn, args = fn_args)
            # minus
            fn_args$object$coef_var[[i]][j] <- par_old_tmp - eps_tmp
            fn_minus <- do.call(what = fn, args = fn_args)
            # derivative
            out[, coef_var_ind[[i]][j]] <- (fn_plus - fn_minus) / (2 * eps_tmp)
            # names
            colnames(out)[coef_var_ind[[i]][j]] <- paste0("coef_var", j, " of ",
                                                          object$other$z_names[i])
            # set initial value
            fn_args$object$coef_var[[i]][j] <- par_old_tmp
          }
        }
      }
      if (is.gc)
      {
        gc()
      }
    }
  }
  
  # Differentiate respect to the elements of the covariance matrix
  # of the ordinal equations
  if (n_eq > 1)
  {
    sigma_ind_mat <- object$ind$sigma_mat
    for (i in 1:(n_eq - 1))
    {
      for (j in (i + 1):n_eq)
      {
        if (eps[sigma_ind_mat[i, j]] != 0 & (sigma_omit[i, j] != 1))
        {
          # save initial value
          par_old_tmp <- object$sigma[i, j]
          # prepare increment
          eps_tmp <- eps[sigma_ind_mat[i, j]] * abs(par_old_tmp)
          # plus
          fn_args$object$sigma[i, j] <- par_old_tmp + eps_tmp
          fn_args$object$sigma[j, i] <- fn_args$object$sigma[i, j]
          fn_plus <- do.call(what = fn, args = fn_args)
          # minus
          fn_args$object$sigma[i, j] <- par_old_tmp - eps_tmp
          fn_args$object$sigma[j, i] <- fn_args$object$sigma[i, j]
          fn_minus <- do.call(what = fn, args = fn_args)
          # derivative
          out[, sigma_ind_mat[i, j]] <- (fn_plus - fn_minus) / (2 * eps_tmp)
          # names
          colnames(out)[sigma_ind_mat[i, j]] <- paste0("cov(",
                                                        object$other$z_names[i],
                                                        ",",
                                                        object$other$z_names[j],
                                                        ")")
          # set initial value
          fn_args$object$sigma[i, j] <- par_old_tmp
          fn_args$object$sigma[j, i] <- par_old_tmp
        }
      }
      if (is.gc)
      {
        gc()
      }
    }
  }
  
  # Differentiate respect to the parameters of the marginal distribution
  if (object$other$is_marginal)
  {
    marginal_par_n   <- object$other$marginal_par_n
    marginal_par_ind <- object$ind$marginal_par
    for (i in 1:n_eq)
    {
      if (marginal_par_n[i] > 0)
      {
        for (j in 1:marginal_par_n[i])
        {
          if (eps[marginal_par_ind[[i]][j]] != 0)
          {
            # save initial value
            par_old_tmp <- object$marginal_par[[i]][j]
            # prepare increment
            eps_tmp <- eps[marginal_par_ind[[i]][j]] * abs(par_old_tmp)
            # plus
            fn_args$object$marginal_par[[i]][j] <- par_old_tmp + eps_tmp
            fn_plus <- do.call(what = fn, args = fn_args)
            # minus
            fn_args$object$marginal_par[[i]][j] <- par_old_tmp - eps_tmp
            fn_minus <- do.call(what = fn, args = fn_args)
            # derivative
            out[, marginal_par_ind[[i]][j]] <- (fn_plus - fn_minus) / 
                                               (2 * eps_tmp)
            # set initial value
            fn_args$object$marginal_par[[i]][j] <- par_old_tmp
          }
        }
      }
      if (is.gc)
      {
        gc()
      }
    }
  }
  
  # Derivatives respect to the parameters of the continuous equations
  if (is2)
  {
    n_eq2     <- object$other$n_eq2 
    n_regimes <- object$other$n_regimes
    
    # Differentiate respect to the coefficients of the continuous equations
    coef2_ind <- object$ind$coef2
    for (i in 1:n_eq2)
    {
      for (j in 1:n_regimes[i])
      {
        for (t in 1:length(object$coef2[[i]][j, ]))
        {
          if (eps[coef2_ind[[i]][j, t]] != 0)
          {
            # save initial value
            par_old_tmp <- object$coef2[[i]][j, t] 
            # prepare increment
            eps_tmp <- eps[coef2_ind[[i]][j, t]] * abs(par_old_tmp)
            # plus
            fn_args$object$coef2[[i]][j, t] <- par_old_tmp + eps_tmp
            fn_plus <- do.call(what = fn, args = fn_args)
            # minus
            fn_args$object$coef2[[i]][j, t] <- par_old_tmp - eps_tmp
            fn_minus <- do.call(what = fn, args = fn_args)
            # derivative
            out[, coef2_ind[[i]][j, t]] <- (fn_plus - fn_minus) / (2 * eps_tmp)
            # names
            colnames(out)[coef2_ind[[i]][j, t]] <- paste0("coef2(", 
                                                          j - 1, ",", 
                                                          t, ")", " of ",
                                                          object$other$y_names[i])
            # set initial value
            fn_args$object$coef2[[i]][j, t] <- par_old_tmp
          }
        }
      }
      if (is.gc)
      {
        gc()
      }
    }
    
    # Differentiate respect to the variances of the continuous equations
    if (object$estimator == "ml")
    {
      var2_ind <- object$ind$var2
      for (i in 1:n_eq2)
      {
        for (j in 1:n_regimes[i])
        {
          if (eps[var2_ind[[i]][j]] != 0)
          {
            # save initial value
            par_old_tmp <- object$var2[[i]][j] 
            # prepare increment
            eps_tmp <- eps[var2_ind[[i]][j]] * abs(par_old_tmp)
            # plus
            fn_args$object$var2[[i]][j] <- par_old_tmp + eps_tmp
            fn_plus <- do.call(what = fn, args = fn_args)
            # plus
            fn_args$object$var2[[i]][j] <- par_old_tmp - eps_tmp
            fn_minus <- do.call(what = fn, args = fn_args)
            # derivative
            out[, var2_ind[[i]][j]] <- (fn_plus - fn_minus) / (2 * eps_tmp)
            # names
            colnames(out)[var2_ind[[i]][j]] <- paste0("var(", 
                                                       object$other$y_names[i],
                                                       ")", j - 1)
            # set initial value
            fn_args$object$var2[[i]][j] <- par_old_tmp
          }
        }
        if (is.gc)
        {
          gc()
        }
      }
    }
    
    # Differentiate respect to the covariances between the continuous 
    # and ordinal equations
    if ((object$estimator == "ml") & is1)
    {
      cov2_ind <- object$ind$cov2
      for (i in 1:n_eq2)
      {
        for (j in 1:n_regimes[i])
        {
          for (t in 1:n_eq)
          {
            if (cov2_omit[[i]][j, t] == 0)
            {
              if (eps[cov2_ind[[i]][j, t]] != 0)
              {
                # save initial value
                par_old_tmp <- object$cov2[[i]][j, t] 
                # prepare increment
                eps_tmp <- eps[cov2_ind[[i]][j, t]] * abs(par_old_tmp)
                # plus
                fn_args$object$cov2[[i]][j, t] <- par_old_tmp + eps_tmp
                fn_plus <- do.call(what = fn, args = fn_args)
                # minus
                fn_args$object$cov2[[i]][j, t] <- par_old_tmp - eps_tmp
                fn_minus <- do.call(what = fn, args = fn_args)
                # derivative
                out[, cov2_ind[[i]][j, t]] <- (fn_plus - fn_minus) / (2 * eps_tmp)
                # names
                colnames(out)[cov2_ind[[i]][j, t]] <- paste0("cov(", 
                                                             object$other$y_names[i],
                                                             "(", j, ")", ",", 
                                                             object$other$z_names[t],
                                                             ")")
                # set initial value
                fn_args$object$cov2[[i]][j, t] <- par_old_tmp
              }
            }
          }
        }
        if (is.gc)
        {
          gc()
        }
      }
    }
    
    # Differentiate respect to the covariances between regimes of the 
    # continuous equations
    if (n_eq2 > 1 & (object$estimator == "ml"))
    {
      sigma2_ind <- lapply(object$control_lnL$sigma2_ind, function(x){x + 1})
      counter <- 1
      regimes_pair <- object$other$regimes_pair
      for (i in 2:n_eq2)
      {
        for (j in 1:(i - 1))
        {
          for (t in 1:nrow(regimes_pair[[counter]]))
          {
            if (eps[sigma2_ind[[counter]][t]] != 0)
            {
              # save initial value
              par_old_tmp <- object$sigma2[[counter]][t]
              # prepare increment
              eps_tmp <- eps[sigma2_ind[[counter]][t]] * abs(par_old_tmp)
              # plus
              fn_args$object$sigma2[[counter]][t] <- par_old_tmp + eps_tmp
              fn_plus <- do.call(what = fn, args = fn_args)
              # minus
              fn_args$object$sigma2[[counter]][t] <- par_old_tmp - eps_tmp
              fn_minus <- do.call(what = fn, args = fn_args)
              # derivative
              out[, sigma2_ind[[counter]][t]] <- (fn_plus - fn_minus) / 
                                                 (2 * eps_tmp)
              # names
              colnames(out)[sigma2_ind[[counter]]] <- paste0(
                  "cov(",
                   object$other$y_names[i],
                   "(",
                   regimes_pair[[counter]][t, 1],
                   ")", ",",
                   "(",
                   regimes_pair[[counter]][t, 2],
                   ")",
                   object$other$y_names[j],
                   ")")
              # set initial value
              fn_args$object$sigma2[[counter]][t] <- par_old_tmp
            }
            counter <- counter + 1
          }
        }
        if (is.gc)
        {
          gc()
        }
      }
    }
  }
  
  # Differentiate respect to the coefficients of the multinomial equations
  if (is3)
  {
    coef3_ind <- object$ind$coef3
    for (i in 1:(n_eq3 - 1))
    {
      for (j in 1:length(object$coef3[i, ]))
      {
        if (eps[coef3_ind[i, j]] != 0)
        {
          # save initial value
          par_old_tmp                     <- object$coef3[i, j]
          # prepare increment
          eps_tmp                         <- eps[coef3_ind[i, j]] * 
                                             abs(par_old_tmp)
          # plus
          fn_args$object$coef3[i, j]      <- par_old_tmp + eps_tmp
          fn_plus                         <- do.call(what = fn, args = fn_args)
          # minus
          fn_args$object$coef3[i, j]      <- par_old_tmp - eps_tmp
          fn_minus                        <- do.call(what = fn, args = fn_args)
          # derivative
          out[, coef3_ind[i, j]]          <- (fn_plus - fn_minus) / 
                                             (2 * eps_tmp)
          # names
          colnames(out)[coef3_ind[i, j]] <- paste0("coef",  j, 
                                                   " of alt", i)
          # set initial value
          fn_args$object$coef3[i, j]     <- par_old_tmp
        }
      }
      if (is.gc)
      {
        gc()
      }
    }
  }
  
  # Differentiate respect to the covariances between the alternatives of 
  # the multinomial equations
  if (is3 & (n_eq3 > 2) & (type3 == "probit"))
  {
    sigma3_ind <- object$ind$sigma3
    counter    <- 1
    for (i in seq_len(n_eq3 - 1))
    {
      for (j in 1:i)
      {
        if (!((i == 1) & (j == 1)))
        {
          if (eps[sigma3_ind[counter]] != 0)
          {
            # save initial value
            par_old_tmp <- object$sigma3[i, j]
            # prepare increment
            eps_tmp                            <- eps[sigma3_ind[counter]] * 
                                                  abs(par_old_tmp)
            # plus
            fn_args$object$sigma3[i, j]        <- par_old_tmp + eps_tmp
            fn_args$object$sigma3[j, i]        <- fn_args$object$sigma3[j, i]
            fn_plus                            <- do.call(what = fn, 
                                                          args = fn_args)
            # minus
            fn_args$object$sigma3[i, j]        <- par_old_tmp - eps_tmp
            fn_args$object$sigma3[j, i]        <- fn_args$object$sigma3[j, i]
            fn_minus                           <- do.call(what = fn, 
                                                          args = fn_args)
            # derivative
            out[, sigma3_ind[counter]]         <- (fn_plus - fn_minus) / 
                                                  (2 * eps_tmp)
            # names
            colnames(out)[sigma3_ind[counter]] <- paste0("cov(alt", i, 
                                                         ",alt", j, ")")
            # set initial value
            fn_args$object$sigma3[i, j]        <- par_old_tmp
            fn_args$object$sigma3[j, i]        <- par_old_tmp
          }
          counter <- counter + 1
        }
      }
      if (is.gc)
      {
        gc()
      }
    }
  }
  
  # Short version of the output
  if (type %in% c("grad", "derivative", "jac", "gradient", "jacobian"))
  {
    return(out)
  }

  out <- list(grad = out, val = fn_val)

  return(out)
}

#' Tests and confidence intervals for the parameters estimated by 
#' the msel function
#' @description This function conducts various statistical tests and calculates
#' confidence intervals for the parameters of the model estimated via the 
#' \code{\link[switchSelection]{msel}} function.
#' @template test_msel_param_Template
#' @template test_msel_details_Template
#' @template test_msel_return_Template
#' @template test_msel_references_Template
#' @template test_msel_examples_Template
test_msel <- function(object, 
                      fn, 
                      fn_args   = list(), 
                      test      = "t",
                      method    = "classic",
                      ci        = "classic",
                      cl        = 0.95,
                      se_type   = "dm",
                      trim      = 0,
                      vcov      = NULL,
                      iter      = 100,
                      generator = rnorm,
                      bootstrap = NULL,
                      par_ind   = NULL,
                      eps       = max(1e-4, sqrt(.Machine$double.eps) * 10),
                      n_sim     = 1000,
                      n_cores   = 1)
{
  # -------------------------------------------------------
  # Several models
  # -------------------------------------------------------
  
  # Get the number of models
  n_objects <- 1
  if (!is(object = object, class2 = "msel"))
  {
    n_objects <- length(object)
  }
  
  # -------------------------------------------------------
  # Likelihood ratio test
  # -------------------------------------------------------
  
  if ((length(object) == 2) & (test %in% c("lr", "lrtest", "lr-test")))
  {
    out <- lrtest_msel(model1 = object[[1]], model2 = object[[2]])
    return(out)
  }
  
  # -------------------------------------------------------
  # Setup
  # -------------------------------------------------------
  
  # List to aggregate the output
  out <- list()
  
  # Get the number of parameters of the model
  n_par       <- NULL
  object_list <- NULL
  if (n_objects == 1)
  {
    n_par <- object$other$n_par
    if (is.null(vcov))
    {
      vcov <- object$cov
    }
  }
  else
  {
    # Calculate asymptotic covariance matrix of the parameters
    # of the models
    object_list <- vcov_combine(object, type = "all")
    n_par       <- object_list$n_par
    if (is.null(vcov))
    {
      vcov <- object_list$cov
    }
  }
  
  # Values for the output
  val     <- NULL
  stat    <- NULL
  se      <- NULL
  p_value <- NULL
  lwr     <- NULL
  upr     <- NULL
  
  # Provide the parameters
  if (n_objects == 1)
  {
    object$n_sim   <- n_sim
    object$n_cores <- n_cores
  }
  else
  {
    for (i in 1:n_objects)
    {
      object[[i]]$n_sim   <- n_sim
      object[[i]]$n_cores <- n_cores
    }
  }
  fn_args$object <- object
  
  # -------------------------------------------------------
  # Validation
  # -------------------------------------------------------
  
  # Validate test
  test     <- tolower(test)
  test_vec <- c("t", "wald")
  if (test %in% c("waldtest", "wald test", "aggregate", "wald-test"))
  {
    test <- "wald"
    warning("It is assumed that 'test' is 'wald' so Wald test is used.")
  }
  if (test %in% c("delta method", "deltamethod", "individual", "ind",
                  "t-test", "t_test"))
  {
    test <- "t"
    warning("It is assumed that 'test' is 't' so t-test is used.")
  }
  if (!(test %in% test_vec))
  {
    stop(paste0("Argument 'test' is wrong. ",
                "Please, ensure that it is one of: ",
                paste0(test_vec, collapse = ", "),
                ".\n"))
  }
  
  # Validate method
  method     <- tolower(method)
  method_vec <- c("classic", "bootstrap", "score")
  if (method %in% c("asymptotic", "parametric", "normal"))
  {
    method <- "classic"
    warning("It is assumed that 'method' is 'classic'.")
  }
  if (method %in% c("boot", "nonparametric", "non-parametric", "percentile"))
  {
    method <- "bootstrap"
    warning("It is assumed that 'method' is 'bootstrap'.")
  }
  if (method %in% c("score bootstrap", "bootstrap score", "score-bootstrap"))
  {
    method <- "score"
    warning("It is assumed that 'method' is 'score'.")
  }
  if (!(method %in% method_vec))
  {
    stop(paste0("Argument 'method' is wrong. ",
                "Please, ensure that it is one of: ",
                paste0(method_vec, collapse = ", "),
                ".\n"))
  }
  
  # Validate ci
  ci     <- tolower(ci)
  ci_vec <- c("classic", "percentile", "bc")
  if (ci %in% c("asymptotic", "normal"))
  {
    ci <- "classic"
    warning("It is assumed that 'ci' is 'classic'.")
  }
  if (ci %in% c("bootstrap", "bootstrap percentile", "percentile bootstrap"))
  {
    ci <- "percentile"
    warning("It is assumed that 'ci' is 'percentile'.")
  }
  if (ci %in% c("bias corrected", "bias-corrected", "bias_corrected",
                "Efron", "Efron1982", "Efron 1982"))
  {
    ci <- "bc"
    warning("It is assumed that 'ci' is 'bc'.")
  }
  if (!(ci %in% ci_vec))
  {
    stop(paste0("Argument 'ci' is wrong. ",
                "Please, ensure that it is one of: ",
                paste0(ci_vec, collapse = ", "),
                ".\n"))
  }
  
  # Validate cl
  if (!is.numeric(cl) | (length(cl) > 1))
  {
    stop("Invalid 'cl' value. It should be a numeric value of length 1.")
  }
  if ((cl <= 0) | (cl >= 1))
  {
    stop("Invalid 'cl' value. It should be a numeric value between 0 and 1.")
  }
  
  # Validate se_type
  se_type     <- tolower(se_type)
  se_type_vec <- c("dm", "bootstrap")
  if (se_type %in% c("delta-method", "delta method", "delta_method"))
  {
    se_type <- "dm"
    warning("It is assumed that 'se_type' is 'dm'.")
  }
  if (se_type %in% c("boot"))
  {
    se_type <- "bootstrap"
    warning("It is assumed that 'se_type' is 'bootstrap'.")
  }
  if ((se_type == "bootstrap") & is.null(bootstrap))
  {
    stop("Invalid 'se_type' since 'bootstrap' is 'NULL'.")
  }
  if (!(se_type %in% se_type_vec))
  {
    stop(paste0("Argument 'se_type' is wrong. ",
                "Please, ensure that it is one of: ",
                paste0(se_type_vec, collapse = ", "),
                ".\n"))
  }
  
  # Validate trim
  if ((trim < 0) | (trim >= 1) | !is.numeric(trim) | (length(trim) > 1))
  {
    stop("Parameter 'trim' should be a numeric value between 0 and 1")
  }
  
  # Validate vcov
  if (n_objects == 1)
  {
    if (!is.matrix(vcov) | any(dim(vcov) != dim(object$cov)))
    {
      stop(paste0("Invalid 'vcov' value. ",
                  "It should be a square matrix of appropriate size."))
    }
  }
  
  # Validate iter
  iter <- round(iter)
  if (iter <= 0)
  {
    stop(paste0("Invalid 'iter' value. ",
                "It should be a positive integer."))
  }
  
  # Validate generator
  if (!is.function(generator))
  {
    stop(paste0("Invalid 'generator' value. ",
                "It should be a function."))
  }
  if (!("n" %in% names(formals(generator))))
  {
    stop(paste0("Function 'generator' should have 'n' input argument ",
                "responsible for the number of generated random variables."))
  }
  
  # Validate par_ind
  if (n_objects == 1)
  {
    if (is.null(par_ind))
    {
      par_ind <- 1:object$other$n_par
    }
    else
    {
      par_ind_unique <- unique(par_ind)
      if (length(par_ind) != length(par_ind_unique))
      {
        warning("Duplicates of some indexes have been removed from 'par_ind'.")
        par_ind <- par_ind_unique
      }
      out_of_range <- (par_ind < 1) | (par_ind > n_par)
      if (any(out_of_range))
      {
        warning(paste0("Indexes ", paste(par_ind[out_of_range], collapse = ", "), 
                       " are out of range so they have been removed",
                       " from 'par_ind'."))
        par_ind <- par_ind[!out_of_range]
      }
      if (length(par_ind) == 0)
      {
        stop("Input argument 'par_ind' should not be empty.")
      }
    }
  }
  else
  {
    if (is.list(par_ind) & (length(par_ind) == n_objects))
    {
      # User have provided par_ind
      for (i in 1:n_objects)
      {
        par_ind[[i]] <- unique(par_ind[[i]])
      }
    }
    else
    {
      # User have not provided par_ind
      par_ind <- vector(mode = "list", length = n_objects)
      for (i in 1:n_objects)
      {
        par_ind[[i]] <- 1:object[[i]]$other$n_par
      }
    }
  }
  
  # Validate eps
  if (n_objects == 1)
  {
    if (length(eps) == 1)
    {
      eps <- rep(eps, length(par_ind))
    }
    if (length(eps) != length(par_ind))
    {
      stop(paste0("If 'eps' has more than 1 element then ",
                  "length of 'eps' and 'par_ind' should be the same. "))
    }
    eps_tmp          <- rep(0, n_par)
    eps_tmp[par_ind] <- eps
    eps              <- eps_tmp
    eps[is.na(eps)]  <- 0
  }
  else
  {
    if (length(eps) == 1)
    {
      eps_list <- vector(mode = "list", length = n_objects)
      for (i in 1:n_objects)
      {
        eps_list[[i]] <- rep(eps, object[[i]]$other$n_par)
      }
      eps <- eps_list
    }
    if (length(eps) != n_objects)
    {
      stop (paste0("Incorrect length of 'eps' argument. It should be a list", 
                   " of ", n_objects, " elements."))
    }
    for (i in 1:n_objects)
    {
      if (length(eps[[i]]) != object[[i]]$other$n_par)
      {
        stop (paste0("Incorrect length of 'eps[['", i, "]]' argument. ",
                     "It should be a numeric vector", " of ", 
                     object[[i]]$other$n_par, " elements."))
      }
      eps[[i]][-par_ind[[i]]] <- 0
    }
  }
  
  # Validate object
  if (!is(object = object, class2 = "msel") & (n_objects == 1))
  {
    stop("Invalid 'object' argument. It should be an object of class 'msel'.")
  }
  
  # Validate bootstrap
  is_bootstrap     <- !is.null(bootstrap)
  if (is_bootstrap)
  {
    if (!is(object = bootstrap, class2 = "bootstrap_msel") & 
        !is(object = bootstrap, class2 = "crossmodel_bootstrap_msel"))
    {
      stop(paste0("Invalid 'bootstrap' argument. ",
                  "It should be an object of class 'bootstrap_msel' or ",
                  "'crossmodel_bootstrap_msel'."))
    }
  }
  if (((method == "bootstrap")         | 
       (ci %in% c("percentile", "bc")) | 
       (se_type == "bootstrap"))       &
      !is_bootstrap)
  {
    stop("Argument 'bootstrap' should be provided.")
  }
  out$is_bootstrap <- is_bootstrap
  
  # Additional validation
  if ((test == "t") & (method == "score"))
  {
    stop("Score bootstrap is not available for the t-test.\n")
  }
  
  # -------------------------------------------------------
  # Values of the function and the derivatives
  # -------------------------------------------------------
  
  # Get the values of the function and its derivatives
  d        <- NULL  # derivatives
  val      <- NULL  # values of the function
  n_val    <- NULL  # dimensions of the function
  if ((method %in% c("classic", "score")) | (ci %in% c("classic")) |
      (se_type == "dm"))
  {
    d_list <- deriv_msel(object  = object,  fn  = fn, 
                         fn_args = fn_args, eps = eps)
    d      <- d_list$grad
    val    <- d_list$val
    n_val  <- nrow(d)
  }
  else
  {
    val   <- do.call(what = fn, args = fn_args)
    n_val <- length(val)
  }
  if (is.null(names(val)))
  {
    names(val) <- 1:n_val
  }
  
  # -------------------------------------------------------
  # Bootstrap values
  # -------------------------------------------------------
  
  val_bootstrap <- NULL   # values of the function for each bootstrap sample
  n_bootstrap   <- NULL   # the number of bootstrap iterations
  if (!is.null(bootstrap))
  {
    if (n_objects == 1)
    {
      n_bootstrap <- bootstrap$iter
    }
    else
    {
      n_bootstrap <- bootstrap[[1]]$iter
    }
    val_bootstrap <- matrix(NA, nrow = n_bootstrap, ncol = n_val)
    for (i in 1:n_bootstrap)
    {
      if (n_objects == 1)
      {
        fn_args$object <- update_msel(object, par = bootstrap$par[i, ])
      }
      else
      {
        for (j in 1:n_objects)
        {
          fn_args$object[[j]] <- update_msel(object[[j]], 
                                             par = bootstrap[[j]]$par[i, ])
        }
      }
      val_bootstrap[i, ] <- do.call(what = fn, args = fn_args)
    }
    fn_args$object <- object
  }
  out$n_bootstrap <- n_bootstrap
  
  # -------------------------------------------------------
  # Standard errors
  # -------------------------------------------------------
  
  # Calculate standard error via the delta method
  se     <- vector(mode = "numeric", length = n_val)
  fn_cov <- NULL
  if (se_type == "dm")
  {
    fn_cov <- d %*% vcov %*% t(d)
    se     <- sqrt(diag(fn_cov))
  }
  
  # Estimate standard error via the bootstrap
  if (se_type == "bootstrap")
  {
    # Apply the trimming if needed
    val_bootstrap_adj <- val_bootstrap
    if(trim != 0)
    {
      trimmed                     <- rep(FALSE, n_bootstrap)
      val_bootstrap_adj           <- sweep(x      = val_bootstrap,           
                                           MARGIN = 2, 
                                           STATS  = colMeans(val_bootstrap), 
                                           FUN    = "-")
      trimmed_val                  <- rowSums(val_bootstrap_adj ^ 2)
      trimmed_crit                 <- quantile(trimmed_val, probs = 1 - trim, 
                                               type = 1)
      trimmed                      <- trimmed_val > trimmed_crit
      val_bootstrap_adj[trimmed, ] <- 0 
    }
    
    # Calculate the standard errors
    fn_cov <- cov(val_bootstrap_adj)
    se     <- sqrt(diag(fn_cov))
  }
  
  # -------------------------------------------------------
  # Classic and bootstrap t-test
  # -------------------------------------------------------
  
  # Conduct t-test
  if (test == "t")
  {
    # Estimate test statistics
    stat <- val / se
    
    # Calculate p-values
    p_value <- rep(NA, n_val)
    if (method == "classic")
    {
      for (i in 1:n_val)
      {
        p_value[i] <- 2 * min(pnorm(stat[i]), 1 - pnorm(stat[i]))
      }
    }
    if (method == "bootstrap")
    {
      for (i in 1:n_val)
      {
        stat_bootstrap <- val_bootstrap[, i] / se[i]
        p_value[i]     <- mean(abs(stat_bootstrap - stat[i]) > abs(stat[i]))
      }
    }
  }
  
  # -------------------------------------------------------
  # Classic and bootstrap Wald test
  # -------------------------------------------------------
  
  # Conduct Wald test
  if ((test == "wald") & (method %in% c("classic", "bootstrap")))
  {
    # Prepare the values
    p_value <- NA
    stat    <- NA
    
    # Estimate test statistics
    val  <- matrix(val, ncol = 1)
    stat <- crossprod(qr.solve(fn_cov, val, tol = 1e-16), val)
    
    # Calculate p-values
    if (method == "classic")
    {
      p_value <- 1 - pchisq(stat, df = n_val)
    }
    if (method == "bootstrap")
    {
      stat_bootstrap <- vector(mode = "numeric", length = n_bootstrap)
      for (i in 1:n_bootstrap)
      {
        val_diff          <- matrix(val_bootstrap[i, ] - val, ncol = 1)
        stat_bootstrap[i] <- crossprod(solve(fn_cov, val_diff, tol = 1e-16), 
                                       val_diff)
      }
      p_value <- mean(stat_bootstrap > stat)
    }
  }
  
  # -------------------------------------------------------
  # Score bootstrap Wald test
  # -------------------------------------------------------

  # Conduct score bootstrap Wald test
  if ((test == "wald") & (method == "score"))
  {
    # Calculate Jacobian and Hessian if needed
    if ((!hasName(object, name = "J") | !hasName(object, name = "H")) & 
        (n_objects == 1))
    {
      vcov_ml_list  <- vcov_ml(object, type = "sandwich", 
                               n_cores = 1, n_sim = 1000)
      object$J      <- vcov_ml_list$J
      object$H      <- vcov_ml_list$H
      if (object$other$is_cluster)
      {
        object$J_cluster <- vcov_ml_list$J_cluster
      }
    }
    
    # Prepare main variables
    scores <- NULL
    H      <- object$H
    if (n_objects == 1)
    {
      if (object$other$is_cluster)
      {
        scores <- object$J_cluster
      }
      else
      {
        scores <- object$J
      }
    }
    else
    {
      scores <- object_list$J
      H      <- object_list$H
    }
    n_obs <- nrow(scores)
    
    # Estimate initial statistic
    val    < - matrix(val, ncol = 1)
    A      <- qr.solve(H, t(scores), tol = 1e-16)
    A_adj  <- d %*% A
    A_svd  <- svd(A_adj)
    A_keep <- A_svd$d > (1e-16 * max(A_svd$d))
    stat   <- sum((t(A_svd$u[, A_keep]) %*% val) ^ 2 / A_svd$d[A_keep] ^ 2)
    
    # Get bootstrapped statistics
    stat_bootstrap <- rep(NA, iter)
    A_boot         <- qr.solve(H, t(d), tol = 1e-16)
    for (i in 1:iter)
    {
      weights           <- generator(n = nrow(scores))
      scores_adj        <- weights * scores
      B                 <- scores_adj %*% A_boot
      S                 <- colSums(B) 
      svd_B             <- svd(B)
      keep_B            <- svd_B$d > (1e-16 * max(svd_B$d))
      stat_bootstrap[i] <- sum((t(svd_B$v[, keep_B]) %*% S) ^ 2 / 
                               svd_B$d[keep_B] ^ 2)
    }
    
    # Calculate the p-value
    p_value <- mean(stat_bootstrap > stat)
  }
  
  # -------------------------------------------------------
  # Confidence intervals
  # -------------------------------------------------------
  
  is_ci     <- FALSE
  if (test == "t")
  {
    is_ci      <- TRUE
    cl_p_lower <- (1 - cl) / 2
    cl_p_upper <- 1 - cl_p_lower
    
    # Classic asymptotic confidence interval
    if (ci == "classic")
    {
      z_q <- qnorm(cl + (1 - cl) / 2) * se
      lwr <- val - z_q
      upr <- val + z_q
    }
    
    # Percentile bootstrap confidence interval
    if (ci == "percentile")
    {
      lwr      <- vector(mode = "numeric", length = n_val)
      upr      <- lwr
      for (i in 1:n_val)
      {
        lwr[i]   <- quantile(val_bootstrap[, i], probs = cl_p_lower, type = 1)
        upr[i]   <- quantile(val_bootstrap[, i], probs = cl_p_upper, type = 1)
      }
    }
    
    # Bias-corrected percentile bootstrap confidence interval
    if (ci == "bc")
    {
      lwr <- vector(mode = "numeric", length = n_val)
      upr <- lwr
      for (i in 1:n_val)
      {
        adj_tmp     <- 2 * qnorm(mean(val_bootstrap[, i] <= val[i]))
        cl_bc_lower <- pnorm(qnorm(cl_p_lower) + adj_tmp)
        cl_bc_upper <- pnorm(qnorm(cl_p_upper) + adj_tmp)
        lwr[i]      <- quantile(val_bootstrap[, i], 
                                probs = cl_bc_lower, type = 1)
        upr[i]      <- quantile(val_bootstrap[, i], 
                                probs = cl_bc_upper, type = 1)
      }
    }
  }
  out$is_ci  <- is_ci
  
  # -------------------------------------------------------
  # Output
  # -------------------------------------------------------
  
  # Aggregate the output
  if ((test == "wald") | (n_val == 1))
  {
    out$tbl <- data.frame(row.names = "test")
  }
  else
  {
    out$tbl <- data.frame(row.names = 1:n_val)
  }

  if (test == "t")
  {
    out$tbl$val       <- val
    out$tbl$se        <- se
    rownames(out$tbl) <- 1:n_val
  }

  out$tbl$stat <- stat
  out$test     <- test
  out$method   <- method
  out$se_type  <- se_type
  out$ci       <- ci
  out$cl       <- cl
  out$n_val    <- n_val
  if (is_ci)
  {
    out$tbl$lwr <- lwr
    out$tbl$upr <- upr
  }
  out$tbl$p_value <- p_value

  # Some other output
  if (method == "score")
  {
    out$iter <- iter
  }
  
  # Return the results
  class(out)  <- "test_msel"
  
  return(out)
}

#' Summary for an Object of Class delta_method
#' @description Provides summary for an object of class 'delta_method'.
#' @param object object of class 'delta_method'
#' @param ... further arguments (currently ignored)
#' @param is_legend a logical; if \code{TRUE} then additional information
#' is shown.
#' @return Returns an object of class 'summary.delta_method'.
summary.test_msel <- function(object, ..., is_legend = TRUE)
{
  if (length(list(...)) > 0)
  {
    warning("Additional arguments passed through ... are ignored.")   
  }
  
  class(object)                         <- "summary.test_msel"
  attr(x = object, which = "is_legend") <- is_legend
  
  return(object)
}

#' Print summary for an Object of Class test_msel
#' @description Prints summary for an object of class 'test_msel'.
#' @param x object of class 'test_msel'
#' @param ... further arguments (currently ignored)
#' @param is_legend a logical; if \code{TRUE} then additional information
#' is shown.
#' @return The function returns input argument \code{x}.
print.summary.test_msel <- function(x, ..., is_legend = TRUE)
{
  if (length(list(...)) > 0)
  {
    warning("Additional arguments passed through ... are ignored.")   
  }
  
  # Assign the class
  class(x) <- "test_msel"
  
  # Assign the is_legend value
  if (!is.null(attr(x = x, "is_legend")))
  {
    is_legend <- attr(x = x, which = "is_legend")
  }
  
  # Header
  if (x$test == "t")
  {
    cat("The results of the t-test\n")
  }
  if (x$test == "wald")
  {
    if (x$method == "score")
    {
      cat("The results of the score bootstrap Wald test\n")
    }
    else
    {
      cat("The results of the Wald test\n")
    }
  }
  cat("---\n")
  
  # Results
  printCoefmat(x$tbl, signif.legend = TRUE, has.Pvalue = TRUE)
  
  if (!is_legend)
  {
    return(x)
  }
  
  # Footnote
  cat("---\n")
  if (x$test == "t")
  {
    cat("H0: val = 0\n")
  }
  if (x$test == "wald")
  {
    cat("H0: val = (0,...,0)\n")
  }
  
  # method
  if (x$method == "classic")
  {
    if (x$test == "t")
    {
      cat(paste0("p-values are calculated under the assumption of the \n",
                 "asymptotic normality of the test statistic.\n"))
    }
    if (x$test == "wald")
    {
      cat(paste0("p-values are calculated under the assumption that \n",
                 "the asymptotic distribution of the test statistic \n",
                 "is chi-squared with ", x$n_val, " degrees of freedom.\n"))
    }
  }
  if (x$method == "bootstrap")
  {
    cat("p-values are calculated via the bootstrap.\n")
  }
  
  # se_type
  if (x$se_type == "dm")
  {
    if (x$test == "t")
    {
      cat("Standard errors are estimated via the delta method.\n")
    }
    if (x$test == "wald")
    {
      cat(paste0("Covariance matrix of the outputs of the 'fn'\n",
                 "has been estimated via the delta method.\n"))
    }
  }
  if (x$se_type == "bootstrap")
  {
    if (x$test == "t")
    {
      cat("Standard errors are estimated via the bootstrap.\n")
    }
    if (x$test == "wald")
    {
      cat(paste0("Covariance matrix of the outputs of the 'fn'\n",
                 "has been estimated via the bootstrap.\n"))
    }
  }
  
  # ci
  if (x$is_ci)
  {
    if (x$ci == "classic")
    {
      cat(paste0("Classic ", x$cl * 100, "% confidence interval is used.\n"))
    }
    if (x$ci == "percentile")
    {
      cat(paste0("Percentile bootstrap ", x$cl * 100, 
                 "% confidence interval is used.\n"))
    }
    if (x$ci == "bc")
    {
      cat(paste0("Bias corrected percentile bootstrap ", x$cl * 100, 
                 "% confidence interval is used.\n"))
    }
  }
  
  # bootstrap
  if (x$is_bootstrap)
  {
    cat(paste0("The number of the bootstrap iterations is ", 
               x$n_bootstrap, ".\n"))
  }
  if (x$method == "score")
  {
    cat(paste0("The number of the score bootstrap iterations is ", 
               x$iter, ".\n"))
  }
  
  return(x)
}

#' Extract Residual Standard Deviation 'Sigma'
#' @description Extract standard deviations of random errors of continuous
#' equations of \code{\link[switchSelection]{msel}} function.
#' @param object object of class "msel".
#' @param use.fallback logical, passed to \code{nobs} (currently ignored).
#' @param ... further arguments (currently ignored).
#' @param eq2 index of continuous equation
#' @param regime regime of continuous equation
#' @details Available only if \code{estimator = "ml"} or all \code{degrees}
#' values are equal to \code{1}.
#' @returns Returns estimates of the standard deviations 
#' of \eqn{\varepsilon_{i}}. 
#' If \code{eq2 = k} then estimates only for \eqn{k}-th continuous equation are 
#' returned. If in addition \code{regime = r} then estimate of 
#' \eqn{\sqrt{Var(\varepsilon_{ri})}} is returned. 
#' Herewith if \code{regime} is not \code{NULL} and \code{eq2} is \code{NULL} 
#' it is assumed that \code{eq2 = 1}.
sigma.msel <- function (object, use.fallback = TRUE, ..., 
                             regime = NULL, eq2 = NULL)
{
  # Validation
  if (object$estimator != "ml")
  {
    if (object$cov_type != "parametric")
    {
      stop (paste0("Available only for maximum-likelihood estimator or if ",
                   "parametric estimator of the asymptotic covariance matrix ",
                   "is used."))
    }
  }
  
  # Estimate variance
  val <- coef(object, type = "var", eq2 = eq2, regime = regime)

  # Get standard deviation
  if (is.list(val))
  {
    val <- lapply(val, function(x){sqrt(x)})
  }
  else
  {
    val <- sqrt(val)
  }
  
  # Return the result
  return(val)
}

# Get the groups related variables
groups_msel <- function(object, data, groups, groups2, groups3)
{
  # Get some variables
  is1         <- object$other$is1
  is2         <- object$other$is2
  is3         <- object$other$is3
  is_het      <- object$other$is_het
  estimator   <- object$estimator
  n_eq        <- object$other$n_eq
  n_eq2       <- object$other$n_eq2
  formula     <- object$formula
  formula2    <- object$formula2
  formula3    <- object$formula3
  is_na_group <- object$other$is_na_group
  if (is.null(data))
  {
    data <- object$data
  }
  n_obs <- nrow(data)
  
  # The number of observations
  n_obs <- nrow(data)
  
  # Get the dependent variable of the multinomial equation
  z <- matrix(NA)
  if (is1)
  {
    z <- matrix(NA, nrow = n_obs, ncol = n_eq)
    for (i in 1:n_eq)
    {
      z[, i] <- data[, all.vars(formula[[i]])[1]]
    }
  }
  
  # Get the dependent variables of the continuous equations
  y <- matrix(NA)
  if (is2)
  {
    y <- matrix(NA, nrow = n_obs, ncol = n_eq2)
    for (i in 1:n_eq2)
    {
      y[, i] <- data[, all.vars(formula2[[i]])[1]]
    }
  }
  
  # Get the dependent variable of the muiltinomial equation
  z_mn <- vector(mode = "numeric")
  if (is3)
  {
    z_mn <- data[, all.vars(formula3)[1]]
  }
  
  # Calculate the number of cuts for each equation
  n_cuts_eq <- rep(0, n_eq)
  if (is1)
  {
    for (i in seq_len(n_eq))
    {
      n_cuts_eq[i] <- max(na.omit(z[, i]))
    }
  }

  # Calculate the number of the alternatives in the 
  # multinomial equation
  alt   <- 0
  n_eq3 <- 0
  if (is3)
  {
    alt <- sort(unique(na.omit(z_mn)))
    n_eq3 <- length(alt[alt != -1])
  }
  
  # Set the groups by default
  groups_all <- NULL
  if ((is_na_group[1] & is1) | 
      (is_na_group[2] & is2) |
      (is_na_group[3] & is3))
  {
    # Generate all groups
    groups_pows <- NULL
    if (is1)
    {
      groups_pows <- n_cuts_eq + 1
    }
    if (is2)
    {
      groups_pows <- c(groups_pows, rep(1, n_eq2))
    }
    if (is3)
    {
      groups_pows <- c(groups_pows, n_eq3)
    }
    groups_all <- as.matrix(t(hpa::polynomialIndex(groups_pows) - 1))

    # Split the groups
    counter <- 1
    if (is1)
    {
      groups <- groups_all[, counter:n_eq, drop = FALSE]
      counter <- n_eq + 1
    }
    if (is2)
    {
      groups2 <- groups_all[, counter:(counter + n_eq2 - 1), drop = FALSE]
      counter <- counter + n_eq2
    }
    if (is3)
    {
      groups3 <- groups_all[, counter, drop = TRUE]
      counter <- counter + 1
    }
  }
  
  # Get the indexes of the observations for each group
  ind_g <- NULL
  
  # Get the indexes of the observations corresponding to each group
  groups123 <- NULL
  data123   <- NULL
  if (is1)
  {
    z_tmp           <- z
    z_tmp[is.na(z)] <- -1
    groups123       <- cbind(groups123, groups)
    data123         <- cbind(data123, z_tmp)
  }
  if (is2)
  {
    y_tmp                        <- y
    y_tmp[is.na(y)]              <- -1
    y_tmp[!is.na(y)]             <- 0
    groups2_tmp                  <- groups2
    groups2_tmp[groups2_tmp > 0] <- 0
    groups123                    <- cbind(groups123, groups2_tmp)
    data123                      <- cbind(data123, y_tmp)
  }
  if (is3)
  {
    z_mn_tmp              <- z_mn
    z_mn_tmp[is.na(z_mn)] <- -1
    groups123             <- cbind(groups123, groups3)
    data123               <- cbind(data123, z_mn_tmp)
  }

  # Get the indexes
  ind_g <- findGroup(data123, groups123)
  
  # Calculate the number of groups
  n_groups <- length(ind_g)
  
  # Calculate the number of observations in each group
  n_obs_g <- rep(0, n_groups)
  for (i in seq_len(n_groups))
  {
    n_obs_g[i] <- length(ind_g[[i]])
  }
  n_obs_g_0 <- n_obs_g == 0
  
  # Get the groups without the observable categories
  g_unobs_1 <- rep(TRUE, n_groups)
  g_unobs_2 <- rep(TRUE, n_groups)
  g_unobs_3 <- rep(TRUE, n_groups)
  for (i in seq_len(n_groups))
  {
    if (is1)
    {
      g_unobs_1[i] <- all(groups[i, , drop = FALSE] == -1)
    }
    if (is2)
    {
      g_unobs_2[i] <- all(groups2[i, , drop = FALSE] == -1)
    }
  }
  if (is3)
  {
    g_unobs_3 <- groups3 == -1
  }
  g_unobs <- g_unobs_1 & g_unobs_2 & g_unobs_3
  
  # Remove the groups with zero observations
  ind_g_include <- !(n_obs_g_0 | g_unobs)
  if (is1)
  {
    groups <- groups[ind_g_include, , drop = FALSE]
  }
  if (is2)
  {
    groups2 <- groups2[ind_g_include, , drop = FALSE]
  }
  if (is3)
  {
    groups3 <- groups3[ind_g_include]
  }
  n_obs_g            <- n_obs_g[ind_g_include]
  ind_g              <- ind_g[ind_g_include]
  n_groups           <- length(n_obs_g)
  
  # Calculate the number of the observed equations per group
  n_eq_g     <- rep(0, n_groups)
  n_eq_all_g <- n_eq_g
  n_eq2_g    <- rep(0, n_groups)
  # ordered equations
  if (is1)
  {
    for (i in seq_len(n_groups))
    {
      n_eq_g[i] <- sum(groups[i, , drop = FALSE] != -1)
    }
    n_eq_all_g <- n_eq_g
  }
  # continuous equations
  if (is2)
  {
    n_eq2_g <- vector(mode = "numeric", length = n_groups)
    for (i in seq_len(n_groups))
    {
      n_eq2_g[i]    <- sum(groups2[i, , drop = FALSE] != -1)
      n_eq_all_g[i] <- n_eq_g[i] + n_eq2_g[i]
    }
  }
  
  # Get the indexes of the observed equations for each group
  ind_eq     <- vector(mode = "list", length = n_groups)
  ind_eq2    <- vector(mode = "list", length = n_groups)
  ind_eq3    <- vector(mode = "list", length = n_groups)
  ind_eq_all <- vector(mode = "list", length = n_groups)
  # ordered equations
  if (is1)
  {
    for (i in seq_len(n_groups))
    {
      ind_eq[[i]] <- which(groups[i, , drop = FALSE] != -1)
    }
    ind_eq_all <- ind_eq
  }
  # continuous equations
  if (is2)
  {
    for (i in seq_len(n_groups))
    {
      ind_eq2[[i]]    <- which(groups2[i, , drop = FALSE] != -1)
      ind_eq_all[[i]] <- c(ind_eq[[i]], ind_eq2[[i]] + n_eq)
    }
  }
  # multinomial equations
  # does not include ind_eq_all for now but
  # may be added later
  if (is3)
  {
    ind_eq3 <- which(groups3 != -1)
  }
  
  # Get the vector of all indexes
  ind_g_all <- NULL
  for (i in 1:n_groups)
  {
    ind_g_all <- c(ind_g_all, ind_g[[i]])
  }
  ind_g_all <- sort(ind_g_all)
  # Important: the vector of all indexes of the second step may be
  #            used to select the scores from the first step
  
  # Logical values related to the observations 
  # associated with some group
  ind_g_logical            <- rep(FALSE, n_obs)
  ind_g_logical[ind_g_all] <- TRUE
  ind_g_logical_cumsum     <- cumsum(ind_g_logical)
  
  # Preserve only the observations associated with some group
  data <- data[ind_g_all, , drop = FALSE]
  for (i in 1:n_groups)
  {
    ind_g[[i]] <- ind_g_logical_cumsum[ind_g[[i]]]
  }
  n_obs <- nrow(data)
  
  # Return the results
  out <- list(groups     = groups,     groups2    = groups2,
              groups3    = groups3,    ind_g      = ind_g,
              ind_eq     = ind_eq,     ind_eq2    = ind_eq2,
              ind_eq3    = ind_eq3,    ind_eq_all = ind_eq_all,
              n_cuts_eq  = n_cuts_eq,  n_eq3      = n_eq3,
              n_groups   = n_groups,   data       = data,
              n_obs      = n_obs,      n_obs_g    = n_obs_g,
              n_eq_g     = n_eq_g,     n_eq2_g    = n_eq2_g,
              n_eq_all_g = n_eq_all_g, ind_g_all  = ind_g_all)
}

#' A subset of data from Current Population Survey (CPS).
#'
#' Labor market data on 18,253 middle age (25-54 years) married women
#' in the year 2022.
#'
#' @docType data
#'
#' @format
#' A data frame with 18,253 rows and 25 columns. It contains information on
#' wages and some socio-demographic characteristics of the middle age
#' (25-54 years) married women:
#' \describe{
#'   \item{age}{the age measured in years.}
#'   \item{sage}{the same as age but for a spouse.}
#'   \item{work}{a binary variable for the employment status 
#'   (0 - unemployed, 1 - employed).}
#'   \item{swork}{the same as work but for a spouse.}
#'   \item{nchild}{the number of children under age 5.}
#'   \item{snchild}{the same as nchild but for a spouse.}
#'   \item{health}{subjective health status 
#'   (1 - poor, 2 - fair, 3 - good, 4 - very good, 5 - excellent).}
#'   \item{shealth}{the same as health but for a spouse.}
#'   \item{basic}{a binary variable which equals 1 for those who have graduated 
#'   from high school or has at least some college or has an associated degree 
#'   and does not have any higher level of education, 0 - otherwise.}
#'   \item{bachelor}{a binary variable which equals 1 for those whose highest 
#'   education level is a bachelor degree.}
#'   \item{master}{a binary variable which equals 1 for those whose highest 
#'   education level is a master degree.}
#'   \item{sbasic}{the same as basic but for a spouse.}
#'   \item{sbachelor}{the same as bachelor but for a spouse.}
#'   \item{smaster}{the same as master but for a spouse.} 
#'   \item{educ}{a categorical variable for the level of education such that 
#'   educ = 0 if basic = 1, 
#'   educ = 1 if bachelor = 1 and 
#'   educ = 2 if master = 1.}
#'   \item{seduc}{the same as educ but for a spouse.}
#'   \item{weeks}{a total number of weeks worked during the year.}
#'   \item{sweeks}{the same as weeks but for a spouse.}
#'   \item{hours}{a usual number of working hours per week.}
#'   \item{shours}{the same as hours but for a spouse.}
#'   \item{wage}{the wage of the individual.}
#'   \item{swage}{the same as wage but for a spouse.}
#'   \item{lwage}{an inverse hyperbolic sine transformation of the hourly wage.}
#'   \item{slwage}{the same as lwage but for a spouse.}
#'   \item{state}{a state of residence.}
#'   ...
#' }
#' @usage data(cps)
#' @references Flood S, King M, Rodgers R, Ruggles S, Warren R, 
#' Westberry M (2022). Integrated Public Use Microdata Series, 
#' Current Population Survey: Version 10.0 [dataset].
#' doi: 10.18128/D030.V10.0.
#' @source <https://www.census.gov/programs-surveys/cps.html>
#' @examples
#' \donttest{
#' data(cps)
#' model <- msel(work ~ age + bachelor + master, data = cps)
#' summary(model)
#' } 
"cps"

# Get the data without the missing observations
complete_msel <- function(object, data = NULL)
{
  # Get some variables
  is1       <- object$other$is1
  is2       <- object$other$is2
  is3       <- object$other$is3
  is_het    <- object$other$is_het
  estimator <- object$estimator
  n_eq      <- object$other$n_eq
  n_eq2     <- object$other$n_eq2
  n_eq3     <- object$other$n_eq3
  formula   <- object$formula
  formula2  <- object$formula2
  formula3  <- object$formula3
  if (is.null(data))
  {
    data <- object$data
  }
  
  # Convert the dependent variable to unobservable if
  # there are some unobservable independent variables
    # ordered equations
  if (is1)
  {
    frm_tmp <- NULL
    for (i in seq_len(n_eq))
    {
      if (is_het[i])
      {
        frm_tmp <- switchSelection::formula_merge(
          switchSelection::formula_split(formula[[i]]), 
          type = "var-terms")
      }
      else
      {
        frm_tmp <- formula[[i]]
      }
      df <- model.frame(frm_tmp, data, na.action = na.pass)
      data[!complete.cases(df), names(df)[1]] <- NA
    }
  }
    # continuous equations
  if (is2)
  {
    for (i in seq_len(n_eq2))
    {
      df2 <- model.frame(formula2[[i]], data, na.action = na.pass)
      data[!complete.cases(df2), names(df2)[1]] <- NA
    }
  }
    # multinomial equations
  if (is3)
  {
    df3 <- model.frame(formula3, data, na.action = na.pass)
    data[!complete.cases(df3), names(df3)[1]] <- NA
  }
  
  # Return the data
  return(data)
}

# Create output tables msel
tbl_msel <- function(object, par = NULL, vcov = NULL)
{
  # Get some variables
  if (!is.null(par))
  {
    object <- par_msel(object = object, par = par, type = "object")
  }
  if (!is.null(vcov))
  {
    object$cov <- vcov
  }
  par              <- object$par
  cov              <- object$cov
  n_par            <- object$other$n_par
  estimator        <- object$estimator
  is1              <- object$other$is1
  is2              <- object$other$is2
  is3              <- object$other$is3
  is_het           <- object$other$is_het
  n_eq             <- object$other$n_eq
  n_eq2            <- object$other$n_eq2
  n_eq3            <- object$other$n_eq3
  n_cuts_eq        <- object$other$n_cuts_eq
  n_regimes        <- object$other$n_regimes
  regimes_pair     <- object$other$regimes_pair
  coef             <- object$coef
  coef_var         <- object$coef_var
  cuts             <- object$cuts
  sigma            <- object$sigma
  coef2            <- object$coef2
  var2             <- object$var2
  cov2             <- object$cov2
  sigma2           <- object$sigma2
  sigma3           <- object$sigma3
  marginal_par     <- object$marginal_par
  coef3            <- object$coef3
  coef_ind         <- object$ind$coef
  coef_var_ind     <- object$ind$coef_var
  cuts_ind         <- object$ind$cuts 
  sigma_ind        <- object$ind$sigma
  sigma_vec_ind    <- object$other$sigma_vec_ind
  coef2_ind        <- object$ind$coef2
  var2_ind         <- object$ind$var2
  cov2_ind         <- object$ind$cov2
  cov2_omit        <- object$other$cov2_omit
  sigma2_ind       <- object$ind$sigma2
  sigma3_ind       <- object$ind$sigma3
  sigma3_ind_mat   <- object$ind$sigma3_mat
  marginal_par_ind <- object$ind$marginal_par
  marginal_par_n   <- object$other$marginal_par_n
  coef3_ind        <- object$ind$coef3
  is_marginal      <- object$other$is_marginal
  z_names          <- object$other$z_names
  y_names          <- object$other$y_names
  type3            <- object$type3
  coef_lambda_ind  <- object$other$coef_lambda_ind
  
  # Calculate standard errors
  se <- sqrt(diag(cov))
  
  # Calculate p-values
  z_value <- par / se
  p_value <- rep(NA, n_par)
  for (i in 1:n_par)
  {
    p_value[i] <- 2 * min(pnorm(z_value[i]), 1 - pnorm(z_value[i]))
  }
  
  # P-values of the one sided test for the variances
  if(is2)
  {
    for (i in 1:n_eq2)
    {
      if (estimator != "2step")
      {
        for (j in var2_ind[[i]])
        {
          p_value[j] <- 1 - pnorm(z_value[j])
        }
      }
    }
  }
  
  # Construct output table
  tbl_coef         <- vector(mode = "list", length = n_eq)
  tbl_coef_var     <- vector(mode = "list", length = n_eq)
  tbl_cuts         <- vector(mode = "list", length = n_eq)
  tbl_sigma        <- vector(mode = "list", length = 1)
  tbl_coef2        <- vector(mode = "list", length = n_eq2)
  tbl_lambda       <- vector(mode = "list", length = n_eq2)
  tbl_var2         <- vector(mode = "list", length = n_eq2)
  tbl_cov2         <- vector(mode = "list", length = n_eq2)
  tbl_sigma2       <- vector(mode = "list", length = n_eq2 * (n_eq2 - 1) / 2)
  tbl_marginal_par <- NA
  tbl_coef3        <- NA
  tbl_sigma3       <- NA
  
  # Ordered equations
  if (is1)
  {
    for (i in 1:n_eq)
    {
      # coefficients of the mean equation
      tbl_coef[[i]] <- as.matrix(cbind(Estimate = coef[[i]],
                                       Std_Error = se[coef_ind[[i]]],
                                       z_value = z_value[coef_ind[[i]]],
                                       ind = coef_ind[[i]],
                                       p_value = p_value[coef_ind[[i]]]))
      rownames(tbl_coef[[i]]) <- names(coef[[i]])
      names(tbl_coef) <- z_names
      
      # coefficients of the variance equation
      if (is_het[i])
      {
        tbl_coef_var[[i]] <- as.matrix(cbind(Estimate = coef_var[[i]],
                                             Std_Error = se[coef_var_ind[[i]]],
                                             z_value = z_value[coef_var_ind[[i]]],
                                             ind = coef_var_ind[[i]],
                                             p_value = p_value[coef_var_ind[[i]]]))
        rownames(tbl_coef_var[[i]]) <- names(coef_var[[i]])
      }
      names(tbl_coef_var) <- z_names
      
      # cuts
      tbl_cuts[[i]] <- as.matrix(cbind(Estimate = cuts[[i]],
                                       Std_Error = se[cuts_ind[[i]]],
                                       z_value = z_value[cuts_ind[[i]]],
                                       ind = cuts_ind[[i]],
                                       p_value = p_value[cuts_ind[[i]]]))
      rownames(tbl_cuts[[i]]) <- paste0("cut", 1:n_cuts_eq[i])
    }
  }
  
  # covariance matrix of ordered equations
  tbl_sigma <- NA
  if (is1)
  {
    if ((n_eq >= 2) & (length(sigma_vec_ind) > 0))
    {
      sigma_vec_names <- paste0("cov(", 
                                z_names[sigma_vec_ind[, 1, drop = FALSE]], ", ",
                                z_names[sigma_vec_ind[, 2, drop = FALSE]],
                                ")")
      tbl_sigma <- as.matrix(cbind(Estimate = par[sigma_ind],
                                   Std_Error = se[sigma_ind],
                                   z_value = z_value[sigma_ind],
                                   ind = sigma_ind,
                                   p_value = p_value[sigma_ind]))
      rownames(tbl_sigma) <- sigma_vec_names
    }
  }
  
  # Continuous equations
  if (is2)
  {
    for (i in 1:n_eq2)
    {
      tbl_coef2[[i]]  <- vector(mode = "list", length = n_regimes[i])
      tbl_lambda[[i]] <- vector(mode = "list", length = n_regimes[i])
      tbl_cov2[[i]]   <- vector(mode = "list", length = n_regimes[i])
      for (j in 1:n_regimes[i])
      {
        # coefficients
        tbl_coef2[[i]][[j]] <- as.matrix(cbind(Estimate = coef2[[i]][j, ],
                                               Std_Error = se[coef2_ind[[i]][j, ]],
                                               z_value = z_value[coef2_ind[[i]][j, ]],
                                               ind = coef2_ind[[i]][j, ],
                                               p_value = p_value[coef2_ind[[i]][j, ]]))
        rownames(tbl_coef2[[i]][[j]]) <- names(coef2[[i]][j, ])
        names(tbl_coef2[[i]]) <- paste0("regime ", 0:(n_regimes[i] - 1))
        if ((estimator == "2step"))
        {
          if (length(coef_lambda_ind[[i]]) > 0)
          {
            tbl_lambda[[i]][[j]] <- tbl_coef2[[i]][[j]][coef_lambda_ind[[i]], ,
                                                        drop = FALSE]
            tbl_coef2[[i]][[j]]  <- tbl_coef2[[i]][[j]][-coef_lambda_ind[[i]], ,
                                                        drop = FALSE]
          }
        }
        
        # covariances with ordered equations
        if (is1 & (estimator == "ml"))
        {
          tbl_cov2[[i]][[j]] <- as.matrix(cbind(
            Estimate  = cov2[[i]][j, cov2_omit[[i]][j, ] == 0],
            Std_Error = se[cov2_ind[[i]][j, cov2_omit[[i]][j, ]  == 0]],
            z_value   = z_value[cov2_ind[[i]][j, cov2_omit[[i]][j, ]  == 0]],
            ind       = cov2_ind[[i]][j, cov2_omit[[i]][j, ]  == 0],
            p_value   = p_value[cov2_ind[[i]][j, cov2_omit[[i]][j, ]  == 0]]))
          rownames(tbl_cov2[[i]][[j]]) <- z_names[!cov2_omit[[i]][j, ] ]
          names(tbl_coef2[[i]]) <- paste0("regime ", 0:(n_regimes[i] - 1))
        }
      }
      if (estimator != "2step")
      {
        tbl_var2[[i]] <- as.matrix(cbind(Estimate = var2[[i]],
                                          Std_Error = se[var2_ind[[i]]],
                                          z_value = z_value[var2_ind[[i]]],
                                          ind = var2_ind[[i]],
                                          p_value = p_value[var2_ind[[i]]]))
        rownames(tbl_var2[[i]]) <- rep("var", n_regimes[i])
      }
    }
    names(tbl_coef2) <- paste0("Equation ", 1:n_eq2)
  }
  
  # Covariances between continuous equations
  if ((n_eq2 >= 2) & (estimator == "ml"))
  {
    counter <- 1
    for (i in 2:n_eq2)
    {
      for (j in 1:(i - 1))
      {
        tbl_sigma2[[counter]] <- as.matrix(cbind(Estimate = sigma2[[counter]],
                                                 Std_Error = se[sigma2_ind[[counter]]],
                                                 z_value = z_value[sigma2_ind[[counter]]],
                                                 ind = sigma2_ind[[counter]],
                                                 p_value = p_value[sigma2_ind[[counter]]]))
        names(tbl_sigma2)[counter] <- paste0(y_names[i], " and ", y_names[j])
        rownames(tbl_sigma2[[counter]]) <- rep("", nrow(tbl_sigma2[[counter]]))
        if (length(regimes_pair[[counter]]) > 0)
        {
          for (t in 1:nrow(regimes_pair[[counter]]))
          {
            rownames(tbl_sigma2[[counter]])[t] <- paste0("(",
                                                         regimes_pair[[counter]][t, 1], 
                                                         ",",
                                                         regimes_pair[[counter]][t, 2],
                                                         ")")
          }
        }
        counter <- counter + 1
      }
    }
  }
  
  # Parameters of the marginal distributions
  if (is_marginal)
  {
    tbl_marginal_par <- vector(mode = "list", length = n_eq)
    for (i in 1:n_eq)
    {
      if (marginal_par_n[i] > 0)
      {
        tbl_marginal_par[[i]] <- as.matrix(
          cbind(Estimate = marginal_par[[i]],
                Std_Error = se[marginal_par_ind[[i]] ],
                z_value = z_value[marginal_par_ind[[i]]],
                ind = marginal_par_ind[[i]],
                p_value = p_value[marginal_par_ind[[i]]]))
        rownames(tbl_marginal_par[[i]]) <- paste0("par", 1:marginal_par_n[i])
        names(marginal_par[[i]]) <- paste0("par", 1:marginal_par_n[i])
      }
    }
  }
  
  # Coefficients of the multinomial equations
  if (is3)
  {
    tbl_coef3        <- vector(mode = "list", length = n_eq3 - 1)
    names(tbl_coef3) <- paste0("Alternative ", 1:(n_eq3 - 1))
    for (i in 1:(n_eq3 - 1))
    {
      tbl_coef3[[i]] <- as.matrix(cbind(Estimate = coef3[i, ],
                                        Std_Error = se[coef3_ind[i, ]],
                                        z_value = z_value[coef3_ind[i, ]],
                                        ind = coef3_ind[i, ],
                                        p_value = p_value[coef3_ind[i, ]]))
    }
  }
  
  # Covariances of the multinomial equations
  if (is3 & (n_eq3 > 2) & (type3 == "probit"))
  {
    sigma3_vec_names     <- paste0("cov(", sigma3_ind_mat[, 1, drop = FALSE], 
                                   ",",
                                   sigma3_ind_mat[, 2, drop = FALSE],
                                   ")")
    tbl_sigma3           <- as.matrix(cbind(Estimate  = par[sigma3_ind],
                                            Std_Error = se[sigma3_ind],
                                            z_value   = z_value[sigma3_ind],
                                            ind       = sigma3_ind,
                                            p_value   = p_value[sigma3_ind]))
    rownames(tbl_sigma3) <- sigma3_vec_names
  }
  
  # Aggregate the results into the table
  tbl <- list(coef         = tbl_coef,
              coef_var     = tbl_coef_var,
              cuts         = tbl_cuts,
              sigma        = tbl_sigma,
              coef2        = tbl_coef2,
              lambda       = tbl_lambda,
              var2         = tbl_var2,
              cov2         = tbl_cov2,
              sigma2       = tbl_sigma2,
              marginal_par = tbl_marginal_par,
              coef3        = tbl_coef3,
              sigma3       = tbl_sigma3)
  
  # Store the output
  out <- list(tbl = tbl, se = se, p_value = p_value)
  
  return (out)
}

# Optimization routine
opt_switchSelection <- function(opt_args = NULL, control_lnL,
                                n_sim = 1000, n_cores = 1,
                                type = "msel", start, 
                                opt_type = "optim", regularization = NULL)
{
  # Adjustment for Rcpp code
  if (!is.null(regularization))
  {
    if (hasName(x = regularization, name = "ridge_ind"))
    {
      regularization$ridge_ind <- regularization$ridge_ind - 1
    }
    if (hasName(x = regularization, name = "lasso_ind"))
    {
      regularization$lasso_ind <- regularization$lasso_ind - 1
    }
  }
  
  # Initialize variable to store optimization results
  opt <- NULL
  
  # Initialize list to store arguments
  optim_args <- list()
  if (hasName(opt_args, "optim"))
  {
    optim_args <- opt_args$optim
  }
    
  # obligatory arguments
  optim_args$par <- start
  optim_args$fn  <- lnL_msel
  optim_args$gr  <- grad_msel
    
  # technical arguments
  if (!hasName(optim_args, "method"))
  {
    optim_args$method <- "BFGS"
  }
  if (!hasName(optim_args, "hessian"))
  {
    optim_args$hessian <- FALSE
  }
  if (!hasName(optim_args, "control"))
  {
    optim_args$control <- list(maxit = 10000000,
                               fnscale = -1,
                               reltol = 1e-10,
                               abstol = 1e-10)
  }
  if (!hasName(optim_args, "disable"))
  {
    optim_args$disable = FALSE
  }
    
  # likelihood arguments
  optim_args$n_sim <- n_sim
  optim_args$n_cores <- n_cores
  optim_args$control_lnL <- control_lnL
  optim_args$regularization <- regularization
  
  # Perform the optimization routine
  opt <- list(par = start)
  if (!optim_args$disable)
  {
    optim_args$disable <- NULL
    opt                <- do.call(what = optim, args = optim_args)
  }
    
  # Add genetic optimization if needed
  if (opt_type == "gena")
  {
    # Initialize list to store arguments
    gena_args <- list()
    if (hasName(opt_args, "gena"))
    {
      gena_args <- opt_args$gena
    }
      
    # obligatory arguments
    gena_args$fn <- lnL_msel
    gena_args$gr <- grad_msel
      
    # technical arguments
    if (!hasName(gena_args, "pop.initial"))
    {
      gena_args$pop.initial <- opt$par
    }
    if (!hasName(gena_args, "mutation.method"))
    {
      gena_args$mutation.method <- "percent"
    }
    if (!hasName(gena_args, "maxiter"))
    {
      gena_args$maxiter <- 100
    }
    if (!hasName(gena_args, "info"))
    {
      gena_args$info <- TRUE
    }
    if (!hasName(gena_args, "lower"))
    {
      gena_args$lower <- -2 * abs(opt$par)
    }
    if (!hasName(gena_args, "upper"))
    {
      gena_args$upper <- 2 * abs(opt$par)
    }
    if (!hasName(gena_args, "hybrid.prob"))
    {
      gena_args$hybrid.prob <- 0.2
    }
      
    # likelihood arguments
    gena_args$n_sim          <- n_sim
    gena_args$n_cores        <- n_cores
    gena_args$control_lnL    <- control_lnL
    gena_args$regularization <- regularization
      
    # Perform the optimization routine
    opt <- do.call(what = gena::gena, args = gena_args)
  }
    
  # Add particle swarm optimization if needed
  if (opt_type == "pso")
  {
    # Initialize list to store arguments
    pso_args <- list()
    if (hasName(opt_args, "pso"))
    {
      pso_args <- opt_args$pso
    }
      
    # obligatory arguments
    if (type == "msel")
    {
      pso_args$fn <- lnL_msel
      pso_args$gr <- grad_msel
    }
      
    # technical arguments
    if (!hasName(pso_args, "pop.initial"))
    {
      pso_args$pop.initial <- opt$par
    }
    if (!hasName(pso_args, "maxiter"))
    {
      pso_args$maxiter <- 100
    }
    if (!hasName(pso_args, "info"))
    {
      pso_args$info <- TRUE
    }
    if (!hasName(pso_args, "lower"))
    {
      pso_args$lower <- -2 * abs(opt$par)
    }
    if (!hasName(pso_args, "upper"))
    {
      pso_args$upper <- 2 * abs(opt$par)
    }
    if (!hasName(pso_args, "hybrid.prob"))
    {
      pso_args$hybrid.prob <- 0
    }
      
    # likelihood arguments
    pso_args$n_sim          <- n_sim
    pso_args$n_cores        <- n_cores
    pso_args$control_lnL    <- control_lnL
    pso_args$regularization <- regularization
      
    # Perform the optimization routine
    opt <- do.call(what = gena::pso, args = pso_args)
  }
    
  return (opt)
}

#' Bootstrap for msel function
#' @description Function \code{\link[switchSelection]{bootstrap_msel}} 
#' provides bootstrap estimates of the parameters of the model estimated via
#' the \code{\link[switchSelection]{msel}} function.
#' Function \code{\link[switchSelection]{bootstrap_combine_msel}} allows to
#' combine several objects of class \code{'bootstrap_msel'}.
#' @name bootstrap
#' @template bootstrap_param_Template
#' @template bootstrap_details_Template
#' @template bootstrap_return_Template
#' @template bootstrap_examples_Template
bootstrap_msel <- function(object, 
                           iter     = 100,
                           opt_type = "optim", 
                           opt_args = NULL,
                           is_ind   = FALSE,
                           n_sim    = 1000, 
                           n_cores  = 1)
{
  # Deal with several objects
  n_objects <- NULL
  if (!is(object = object, class2 = "msel"))
  {
    # Get the number of objects
    n_objects <- length(object)
    
    # Routine for several objects
    if (n_objects > 1)
    {
      # Check the class of the object
      for (i in 1:n_objects)
      {
        if (!is(object = object[[i]], class2 = "msel"))
        {
          stop(paste0("Argument object[[", i, "]] is not of class 'msel'."))
        }
      }
    }
  }
  else
  {
    n_objects <- 1
    object <- list(object)
  }
  
  # Combine the data
  data <- object[[1]]$data
  if (n_objects > 1)
  {
    for (i in 2:n_objects)
    {
      data <- suppressMessages(dplyr::full_join(x = data, y = object[[i]]$data))
    }
  }
  
  # The number of observations
  n <- nrow(data)
  
  # Lists to store the results
  out <- vector(mode = "list", length = n_objects)
  for (i in 1:n_objects)
  {
    # Prepare the output
    out[[i]] <- list(par = matrix(NA, 
                                  nrow = iter, 
                                  ncol = object[[i]]$other$n_par))
    if (is_ind)
    {
      out[[i]]$ind <- matrix(NA, nrow = n, ncol = iter)
    }
    
    # Save the number of iterations
    out[[i]]$iter <- iter
    
    # Assign the class
    class(out[[i]]) <- "bootstrap_msel"
    class(out)      <- "crossmodel_bootstrap_msel"
  }
  
  # Main bootstrap routine
  for (i in 1:iter)
  {
    # Select the indexes of the observations to include into the sample
    ind <- sample(x = 1:n, size = n, replace = TRUE)
    
    # For each model
    for (j in 1:n_objects)
    {
      # Save the indexes if needed
      if (is_ind)
      {
        out[[j]]$ind[, i] <- ind
      }
        
      # Estimate the model
      model <- msel(formula        = object[[j]]$formula, 
                    formula2       = object[[j]]$formula2,
                    formula3       = object[[j]]$formula3,
                    data           = data[ind, ], 
                    groups         = object[[j]]$groups, 
                    groups2        = object[[j]]$groups2,
                    groups3        = object[[j]]$groups3,
                    marginal       = object[[j]]$marginal,
                    opt_type       = opt_type, 
                    opt_args       = opt_args,
                    start          = object[[j]]$par,
                    estimator      = object[[j]]$estimator,
                    cov_type       = "no",
                    degrees        = object[[j]]$degrees,
                    degrees3       = object[[j]]$degrees3,
                    n_sim          = n_sim,
                    n_cores        = n_cores,
                    regularization = object[[j]]$other$regularization,
                    type3          = object[[j]]$type3,
                    cluster        = NA)
        
        # Store the results
        out[[j]]$par[i, ] <- model$par
    }
  }
  
  # Covariance matrix
  for (i in 1:n_objects)
  {
    out[[i]]$cov  <- cov(out[[i]]$par)
  }
  
  # Return the results
  if (n_objects == 1)
  {
    out <- out[[1]]
  }
  return(out)
}

#' Update msel object with the new estimates
#' @description This function updates parameters of the model estimated via 
#' \code{\link[switchSelection]{msel}} function.
#' @param object an object of class \code{'msel'}.
#' @param par a vector of parameters which substitutes \code{object$par} and
#' is used to update the estimates i.e., \code{object$coef}, \code{object$cuts} 
#' and others.
#' @details It may be useful to apply this function to the bootstrap
#' estimates of \code{\link[switchSelection]{bootstrap_msel}}.
#' @return This function returns an object of class \code{'msel'}
#' in which \code{object$par} is substituted with \code{par}. Also, \code{par} 
#' is used to update the estimates i.e., \code{object$coef}, \code{object$cuts} 
#' and others.
update_msel <- function(object, par)
{
  object <- par_msel(object = object, par = par, type = "object")
  return(object)
}

#' @name bootstrap
bootstrap_combine_msel <- function(...)
{
  # Collect the models into the list
  b <- list(...)
  
  # Retrieve the first set of bootstrap results
  out <- b[[1]]
  if (length(b) == 1)
  {
    return (out)
  }
  
  # Combine the first set with other sets
  for (i in 2:length(b))
  {
    for (j in names(b[[i]]))
    {
      if (is.list(b[[i]][[j]]))
      {
        out[[j]] <- c(out[[j]], b[[i]][[j]])
      }
    }
    out$par  <- rbind(out$par, b[[i]]$par)
    out$iter <- out$iter + b[[i]]$iter
  }
  out$cov <- cov(out$par)
  
  # Return the results
  return(out)
}

# Split the data according to the equations
data_msel <- function(object, data = NULL)
{
  # Get some variables
  is1          <- object$other$is1
  is2          <- object$other$is2
  is3          <- object$other$is3
  is_het       <- object$other$is_het
  estimator    <- object$estimator
  n_eq         <- object$other$n_eq
  n_eq2        <- object$other$n_eq2
  n_eq3        <- object$other$n_eq3
  formula      <- object$formula
  formula2     <- object$formula2
  formula3     <- object$formula3
  formula_mean <- object$other$formula_mean
  formula_var  <- object$other$formula_var
  if (is.null(data))
  {
    data <- object$data
  }
  n_obs <- nrow(data)
  
  # Separate the dataframes for the mean and variance equations
  # of the ordered equations
  df_mean <- NULL
  df_var  <- NULL
  if (is1)
  {
    df_mean <- vector(mode = "list", length = n_eq)
    df_var  <- vector(mode = "list", length = n_eq)
    for (i in seq_len(n_eq))
    {
      df_mean[[i]] <- model.frame(formula_mean[[i]], data, 
                                  na.action = na.pass)
      if (is_het[i])
      {
        df_var[[i]] <- model.frame(formula_var[[i]], data, 
                                   na.action = na.pass)
      }
    }
  }
  # dataframe for the continuous equations
  df2 <- NULL
  if (is2)
  {
    df2 <- vector(mode = "list", length = n_eq2)
    for (i in seq_len(n_eq2))
    {
      df2[[i]] <- model.frame(formula2[[i]], data, na.action = na.pass)
    }
  }
  # dataframe for the multinomial equation
  df3 <- NULL
  if (is3)
  {
    df3 <- model.frame(formula3, data, na.action = na.pass)
  }
  
  # Extract the variables of the ordinal equations
  z      <- matrix(NA)
  W_mean <- list(matrix())
  W_var  <- list(matrix())
  if (is1)
  {
    z      <- matrix(NA, nrow = n_obs, ncol = n_eq)
    W_mean <- vector("mode" = "list", length = n_eq)
    W_var  <- vector("mode" = "list", length = n_eq)
    for (i in seq_len(n_eq))
    {
      z[, i]      <- as.vector(df_mean[[i]][, 1])
      W_mean[[i]] <- as.matrix(df_mean[[i]][, -1, drop = FALSE])
      if (is_het[i])
      {
        W_var[[i]] <- as.matrix(df_var[[i]][, -1, drop = FALSE])
      }
      else
      {
        W_var[[i]] <- matrix()
      }
    }
    z[z == -1] <- NA
  }
  
  # Extract the variables of the continuous equations
  y <- matrix(NA)
  X <- list(matrix())
  if (is2)
  {
    y <- matrix(NA, nrow = n_obs, ncol = n_eq2)
    X <- vector("mode" = "list", length = n_eq2)
    for (i in seq_len(n_eq2))
    {
      y[, i]              <- as.vector(df2[[i]][, 1])
      X[[i]]              <- cbind(1, as.matrix(df2[[i]][, -1, drop = FALSE]))
      colnames(X[[i]])[1] <- "(Intercept)"
    }
  }
  
  # Extract the variables of the multinomial equation
  z_mn              <- vector(mode = "numeric")
  W_mn              <- matrix(NA)
  if (is3)
  {
    z_mn              <- df3[, 1]
    z_mn[z_mn == -1]  <- NA
    W_mn              <- as.matrix(cbind(1, df3[, -1, drop = FALSE]))
    colnames(W_mn)[1] <- "(Intercept)"
  }
  
  # Store the data to the output
  out <- list(W_mean = W_mean, W_var = W_var, X    = X,   W_mn = W_mn, 
              z      = z,      y     = y,     z_mn = z_mn)
  return(out)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

findGroup <- function(z, groups, adj = 1L) {
    .Call(`_switchSelection_findGroup`, z, groups, adj)
}

matrixInMatrix <- function(x, y) {
    .Call(`_switchSelection_matrixInMatrix`, x, y)
}

#' Log-likelihood Function of Multivariate Ordered Probit Model
#' @description Calculates log-likelihood function of multivariate ordered
#' probit model.
#' @param par vector of parameters.
#' @param control_lnL list with some additional parameters.
#' @param out_type string representing the output type of the function.
#' @param n_sim the number of random draws for multivariate 
#' normal probabilities.
#' @param n_cores the number of cores to be used. 
#' @param regularization list of regularization parameters.
#' @export
lnL_msel <- function(par, control_lnL, out_type = "val", n_sim = 1000L, n_cores = 1L, regularization = NULL) {
    .Call(`_switchSelection_lnL_msel`, par, control_lnL, out_type, n_sim, n_cores, regularization)
}

#' Gradient of the Log-likelihood Function of Multivariate Ordered
#' Probit Model
#' @description Calculates gradient of the log-likelihood function of 
#' multivariate ordered probit model.
#' @param par vector of parameters.
#' @param control_lnL list with some additional parameters.
#' @param out_type string representing the output type of the function.
#' @param n_sim the number of random draws for multivariate 
#' normal probabilities.
#' @param n_cores the number of cores to be used. 
#' @param regularization list of regularization parameters.
#' @export
grad_msel <- function(par, control_lnL, out_type = "grad", n_sim = 1000L, n_cores = 1L, regularization = NULL) {
    .Call(`_switchSelection_grad_msel`, par, control_lnL, out_type, n_sim, n_cores, regularization)
}

#' Extract the Number of Observations from a Fit of the msel Function.
#' @description Extract the number of observations from a model fit
#' of the \code{\link[switchSelection]{msel}} function.
#' @param object object of class "msel"
#' @param ... further arguments (currently ignored)
#' @details Unobservable values of continuous equations are included in
#' the number of observations.
#' @return A single positive integer number.
nobs.msel <- function(object, ...)
{
  if (length(list(...)) > 0)
  {
    warning("Additional arguments passed through ... are ignored.")   
  }
  
  return(object$control_lnL$n_obs)
}

#' Extract Log-Likelihood from a Fit of the msel Function. 
#' @description Extract Log-Likelihood from a model fit
#' of the \code{\link[switchSelection]{msel}} function.
#' @param object object of class "msel"
#' @param ... further arguments (currently ignored)
#' @details If \code{estimator == "2step"} in 
#' \code{\link[switchSelection]{msel}} then function may return
#' \code{NA} value since the two-step estimator of the covariance matrix may not
#' be positively defined.
#' @return Returns an object of class 'logLik'.
logLik.msel <- function (object, ...)
{
  if (length(list(...)) > 0)
  {
    warning("Additional arguments passed through ... are ignored.")   
  }
  
  lnL <- object$logLik
  attr(lnL, "class") <- "logLik"
  attr(lnL, "df") <- length(as.vector(object$par))
  
  return(lnL)
}

# Get the names associated with the data
names_msel <- function(object, data = NULL)
{
  # Get some variables
  is1       <- object$other$is1
  is2       <- object$other$is2
  is3       <- object$other$is3
  is_het    <- object$other$is_het
  estimator <- object$estimator
  n_eq      <- object$other$n_eq
  n_eq2     <- object$other$n_eq2
  n_eq3     <- object$other$n_eq3
  formula   <- object$formula
  formula2  <- object$formula2
  formula3  <- object$formula3
  y         <- object$y
  z         <- object$z  
  z_mn      <- object$z_mn
  X         <- object$X
  W_mean    <- object$W_mean
  W_var     <- object$W_var
  W_mn      <- object$W_mn
  
  if (is.null(data))
  {
    data <- object$data
  }
  n_obs <- nrow(data)
  
  # Get names of the dependent variables
  z_names      <- NA
  y_names      <- NA
  z_mn_names   <- NA
  if (is1)
  {
    z_names <- rep(NA, n_eq)
    for (i in seq_len(n_eq))
    {
      z_names[i] <- as.character(formula[[i]][[2]])
    }
  }
  if (is2)
  {
    y_names <- rep(NA, n_eq2)
    for (i in seq_len(n_eq2))
    {
      y_names[i] <- all.vars(formula2[[i]])[1]
    }
  }
  if (is3)
  {
    z_mn_names <- all.vars(formula3)[1]
  }
  
  # Get names of the independent variables
  colnames_X                <- lapply(X,      colnames)
  colnames_W_mean           <- lapply(W_mean, colnames)
  colnames_W_var            <- lapply(W_var,  colnames)
  colnames_W_mn             <- colnames(W_mn)
  
  # Return the result
  out <- list(z_names         = z_names,    y_names    = y_names, 
              z_mn_names      = z_mn_names, colnames_X = colnames_X,
              colnames_W_mean = colnames_W_mean, 
              colnames_W_var  = colnames_W_var, 
              colnames_W_mn   = colnames_W_mn)
}

regularization_validate <- function(regularization = NULL, 
                                    n_par, estimator)
{
  # -----------------------------------------------------------
  # Note that indexes are adjusted inside opt_switchSelection
  # -----------------------------------------------------------
  
  # Return if NULL
  if (is.null(regularization) | (length(regularization) == 0))
  {
    return (NULL)
  }
  
  # Stop if incorrect estimator is used
  if (estimator != "ml")
  {
    stop ("Regularization available only for maximum-likelihood estimator.")
  }
  
  # Check that regularization is a list
  if (!is.list(regularization))
  {
    stop ("Argument 'regularization' should be a list.")
  }
  
  # Validate names
  names(regularization)[which(names(regularization) == 
                        "ridge_loc")] <- "ridge_location"
  names(regularization)[which(names(regularization) == 
                        "lasso_loc")] <- "lasso_location"
  names(regularization)[which(names(regularization) == 
                        "ridge_indexes")] <- "ridge_ind"
  names(regularization)[which(names(regularization) == 
                        "lasso_indexes")] <- "lasso_ind"
    
  # Check types of regularization
  is_ridge <- hasName(x = regularization, name = "ridge_ind")
  is_lasso <- hasName(x = regularization, name = "lasso_ind")

  # Get the number of regularized parameters
  n_ridge <- 0
  n_lasso <- 0
  if (is_ridge)
  {
    n_ridge <- length(regularization$ridge_ind)
  }
  if (is_lasso)
  {
    n_lasso <- length(regularization$lasso_ind)
  }
    
  # Validate indexes
  if ((n_ridge > n_par) | (n_ridge < 0))
  {
    stop ("Invalid indexes in 'ridge_ind'.")
  }
  if ((n_lasso > n_par) | (n_lasso < 0))
  {
    stop ("Invalid indexes in 'lasso_ind'.")
  }
    
  # Validate scales
  n_ridge_scale <- 0
  n_lasso_scale <- 0
    # ridge
  if (is_ridge)
  {
    if (!hasName(x = regularization, name = "ridge_scale"))
    {
      stop ("Argument 'regularization' should have 'ridge_scale' element.")
    }
    n_ridge_scale <- length(regularization$ridge_scale)
    if (n_ridge_scale != n_ridge)
    {
      if (n_ridge_scale == 1)
      {
        regularization$ridge_scale <- rep(regularization$ridge_scale, n_ridge)
      }
      else
      {
        stop ("Invalid length of 'ridge_scale'.")
      }
    }
  }
    # lasso
  if (is_lasso)
  {
    if (!hasName(x = regularization, name = "lasso_scale"))
    {
      stop ("Argument 'regularization' should have 'lasso_scale' element.")
    }
    n_lasso_scale <- length(regularization$lasso_scale)
    if (n_lasso_scale != n_lasso)
    {
      if (n_lasso_scale == 1)
      {
        regularization$lasso_scale <- rep(regularization$lasso_scale, n_lasso)
      }
      else
      {
        stop ("Invalid length of 'lasso_scale'.")
      }
    }
  }

  # Validate locations
  n_ridge_location <- 0
  n_lasso_location <- 0
    # ridge
  if (is_ridge)
  {
    if (!hasName(x = regularization, name = "ridge_location"))
    {
      n_ridge_location <- n_ridge
      regularization$ridge_location <- rep(0, n_ridge)
    }
    else
    {
      n_ridge_location <- length(regularization$ridge_location)
      if (n_ridge_location != n_ridge)
      {
        if (n_ridge_location == 1)
        {
          regularization$ridge_location <- rep(regularization$ridge_location, 
                                               n_ridge)
        }
        else
        {
          stop ("Invalid length of 'ridge_location'.")
        }
      }
    }
  }
    # lasso
  if (is_lasso)
  {
    if (!hasName(x = regularization, name = "lasso_location"))
    {
      n_lasso_location <- n_lasso
      regularization$lasso_location <- rep(0, n_lasso)
    }
    else
    {
      n_lasso_location <- length(regularization$lasso_location)
      if (n_lasso_location != n_lasso)
      {
        if (n_lasso_location == 1)
        {
          regularization$lasso_location <- rep(regularization$lasso_location, 
                                               n_lasso)
        }
        else
        {
          stop ("Invalid length of 'lasso_location'.")
        }
      }
    }
  }

  return(regularization)
}

# Estimate the asymptotic covariance matrix of the maximum-likelihood estimator
vcov_ml <- function(object, type, n_cores, n_sim)
{
  # Validate a type of the estimator
  type <- tolower(type)
  
  # List of output values
  out <- list(vcov = NULL)
  
  # Get some variables
  par            <- object$par
  cov_type       <- type
  estimator      <- object$estimator
  control_lnL    <- object$control_lnL
  n_par          <- object$other$n_par
  regularization <- object$other$regularization
  n_obs          <- object$other$n_obs
  is_cluster     <- object$other$is_cluster
  n_cluster      <- object$other$n_cluster
  ind_cluster    <- object$other$ind_cluster
  rm(object)

  # Estimate asymptotic covariance matrix
  H         <- NULL
  J         <- NULL
  J_cluster <- NULL
  cov       <- matrix(NA, nrow = n_par, ncol = n_par)
  if (!is.matrix(cov_type))
  {
    if (cov_type != "no")
    {
      if (cov_type %in% c("sandwich", "hessian", "mm"))
      {
        tryCatch(
          {
            cov_type_old <- cov_type
            cov_type     <- "gop"
            gr.args      <- list(n_sim       = n_sim, 
                                 n_cores     = n_cores,
                                 control_lnL = control_lnL, 
                                 out_type    = "grad")
            H            <- NULL
            if (cov_type_old == "mm")
            {
              gr.args$regularization <- regularization
            }
            H        <- gena::gena.hessian(gr      = lnL_msel,
                                           par     = par,
                                           gr.args = gr.args,
                                           is.gc   = TRUE)
            out$H    <- H
            cov_type <- cov_type_old
          },
          error = function(e) {
            warning(paste0("Problems with numeric hessian calculation. ",
                           "Therefore 'cov_type' has been changed to 'gop'."))
          }
        )
        gc()
      }
      if (cov_type %in% c("sandwich", "gop", "mm"))
      {
        J <- NULL
        if (cov_type == "mm")
        {
          J <- lnL_msel(par            = par,
                        n_sim          = n_sim, 
                        n_cores        = n_cores,
                        control_lnL    = control_lnL, 
                        out_type       = "jac",
                        regularization = regularization)
        }
        else
        {
          J <- lnL_msel(par            = par,
                        n_sim          = n_sim, 
                        n_cores        = n_cores,
                        control_lnL    = control_lnL, 
                        out_type       = "jac")
        }
        out$J <- J
        gc()
        
        # Estimate clustered scores
        J_cluster <- matrix(0, nrow = n_cluster, ncol = n_par)
        if (is_cluster)
        {
          for (i in 1:n_cluster)
          {
            J_cluster[i, ] <- colSums(J[ind_cluster == i, , drop = FALSE])
          }
        }
        out$J_cluster <- J_cluster
      }
      if ((cov_type == "sandwich") | (cov_type == "mm"))
      {
        if (is_cluster)
        {
          cov <- tcrossprod(qr.solve(H, t(J_cluster), tol = 1e-16))
        }
        else
        {
          cov <- tcrossprod(qr.solve(H, t(J), tol = 1e-16))
        }
      }
      if (cov_type == "hessian")
      {
        cov <- -qr.solve(H, tol = 1e-16)
      }
      
      if (any(is.na(cov)) & (cov_type != "gop"))
      {
        warning(paste0("Can't calculate the covariance matrix of type '", 
                       cov_type, "'. ", 
                       "Therefore gop covariance matrix will be used instead."))
        cov_type <- "gop"
      }
      
      if (cov_type == "gop")
      {
        tryCatch(
        {
          H     <- crossprod(J)
          J_tmp <- qr.R(qr(J, LAPACK = TRUE))
          cov   <- chol2inv(diag(sign(diag(J_tmp))) %*% J_tmp)
          out$H <- H
        },
        error = function(e) {
          warning(paste0("Problems with numeric hessian calculation. ",
                         "Therefore 'cov_type' has been changed to 'no'."))
        }
       )
      }
    }
  } 
  
  # Aggregate some output
  out$vcov <- cov
  gc()
  
  return(out)
}

# Estimate the asymptotic covariance matrix of the two-step estimator
vcov_2step <- function(object, type = "default", n_cores = 1, n_sim = 10000,
                       is_cov_simple = FALSE)
{
  # Some variables
  n_eq        <- object$other$n_eq
  n_eq2       <- object$other$n_eq2
  n_eq3       <- object$other$n_eq3
  n_par       <- object$other$n_par
  n_obs       <- object$other$n_obs
  is1         <- object$other$is1
  is3         <- object$other$is3
  type3       <- object$type3
  n_sigma     <- object$other$n_sigma
  is_cluster  <- object$other$is_cluster
  n_cluster   <- object$other$n_cluster
  ind_cluster <- object$other$ind_cluster
  
  # Calculate the scores of the second step
  scores <- scores_2step(object, n_cores = n_cores, n_sim = n_sim)
  
  # Calculate the derivatives with respect to the scores of the second step
  scores_list <- NULL
  if (is_cov_simple)
  {
    # Preliminary values
    n_groups   <- object$other$n_groups
    coef2_ind  <- object$ind$coef2
    groups2    <- object$groups2
    y_names    <- object$other$y_names
    colnames_x <- object$other$colnames_X
    par        <- object$par
    y_pred     <- object$y_pred
    y_names    <- object$other$y_names
    y          <- object$y
    
    # Manual calculations with some simplifications
    H <- matrix(0, nrow = n_par, ncol = n_par)
    for (v in 1:n_eq2)
    {
      X           <- object$X[[v]]
      colnames(X) <- colnames_x[[v]]
      X_Y_names   <- y_names[which(y_names %in% colnames_x[[v]])]
      if (length(X_Y_names) > 0)
      {
        X[, X_Y_names] <- y_pred[, X_Y_names]
      }
      for (i in 1:n_groups)
      {
        # Scores with respect to their own coefficients
        if (groups2[i, v] >= 0)
        {
          X0          <- X[object$other$ind_g[[i]], , drop = FALSE]
          ind         <- coef2_ind[[v]][groups2[i, v] + 1, ]
          H[ind, ind] <- H[ind, ind] - crossprod(X0)
        }
        # Scores with respect to other coefficients
        if (v < n_eq2)
        {
          for (v1 in (v + 1):n_eq2)
          {
            if (groups2[i, v1] != -1)
            {
              indy         <- which(colnames_x[[v1]] == y_names[v])
              X1           <- object$X[[v1]][object$other$ind_g[[i]], , drop = FALSE]
              colnames(X1) <- colnames_x[[v1]]
              X1_Y_names   <- y_names[which(y_names %in% colnames_x[[v1]])]
              if (length(X1_Y_names) > 0)
              {
                X1[, X1_Y_names] <- y_pred[object$other$ind_g[[i]], 
                                           X1_Y_names, drop = FALSE]
              }
              if (length(indy) > 0)
              {
                # (y-yh)' * x
                ind1         <- coef2_ind[[v1]][groups2[i, v1] + 1, ]
                H_new        <- (t(X0) %*% X1) * par[ind1[indy]]
                H[ind1, ind] <- H[ind1, ind]  - t(H_new)
                # (y - yh) * x'
                H_new <- colSums(sweep(X0, MARGIN = 1,
                                 STATS = y[object$other$ind_g[[i]], v1] -
                                         y_pred[object$other$ind_g[[i]], v1],
                                 FUN = "*"))
                H[ind1[indy], ind] <- H[ind1[indy], ind] + H_new
              }
            }
          }
        }
        gc()
      }
    }
    scores_list <- list(val  = matrix(colSums(scores), ncol = 1),
                        grad = H)
  }
  else
  {
    # Full calculations which may take much time in large samples
    scores_list <- deriv_msel(object  = object, 
                              fn      = scores_2step,
                              fn_args = list(type     = "aggregate",
                                             n_cores  = n_cores, 
                                             n_sim    = n_sim))
  }
  scores_sum           <- scores_list$val
  scores_jac           <- scores_list$grad
  rownames(scores_jac) <- colnames(scores_jac)
  names(scores_sum)    <- colnames(scores_jac)

  # Collect the estimates of the first step
  if (is1 | is3)
  {
    # Jacobian and average Hessian
    H1 <- object$model1$H
    J1 <- object$model1$J[object$other$ind_g_all, ]

    # Indexes of the first step in the final model
    coef_ind          <- object$ind$coef
    cuts_ind          <- object$ind$cuts
    coef_var_ind      <- object$ind$coef_var
    marginal_par_ind  <- object$ind$marginal_par
    sigma_ind         <- object$ind$sigma
    coef3_ind         <- object$ind$coef3
    sigma3_ind        <- object$ind$sigma3
    
    # Indexes of the first step in first step model
    coef_ind1         <- object$model1$ind$coef
    cuts_ind1         <- object$model1$ind$cuts
    coef_var_ind1     <- object$model1$ind$coef_var
    marginal_par_ind1 <- object$model1$ind$marginal_par
    sigma_ind1        <- object$model1$ind$sigma
    coef3_ind1        <- object$model1$ind$coef3
    sigma3_ind1       <- object$model1$ind$sigma3
    
    # Assign the scores from the first step
      # ordered equations
    if (is1)
    {
      l  <- list()
      l1 <- list()
      if (n_sigma > 0)
      {
        l[[1]]  <- sigma_ind
        l1[[1]] <- sigma_ind1
      }
      for (i in 1:n_eq)
      {
        l[[length(l) + 1]]   <- coef_ind[[i]]
        l1[[length(l1) + 1]] <- coef_ind1[[i]]
        l[[length(l) + 1]]   <- cuts_ind[[i]]
        l1[[length(l1) + 1]] <- cuts_ind1[[i]]
        if (object$other$is_het[i])
        {
          l[[length(l) + 1]]   <- coef_var_ind[[i]]
          l1[[length(l1) + 1]] <- coef_var_ind1[[i]]
        }
        if (object$other$marginal_par_n[i] > 0)
        {
          l[[length(l) + 1]]   <- marginal_par_ind[[i]]
          l1[[length(l1) + 1]] <- marginal_par_ind1[[i]]
        }
        for(j in seq_len(length(l)))
        {
          scores[, l[[j]]] <- J1[, l1[[j]]]
          for(t in 1:length(l))
          {
            scores_jac[l[[j]], l[[t]]] <- H1[l1[[j]], l1[[t]]]
          }
        }
      }
    }
      # multinomial equations
    if (is3)
    {
      if (type3 == "probit")
      {
        scores[, sigma3_ind] <- J1[, sigma3_ind1]
      }
      for (i in seq_len(n_eq3 - 1))
      {
        scores[, coef3_ind[i, ]] <- J1[, coef3_ind1[i, ]]
        for(j in 1:(n_eq3 - 1))
        {
          scores_jac[coef3_ind[i, ], coef3_ind[j, ]] <- H1[coef3_ind1[i, ], 
                                                           coef3_ind1[j, ]]
          if (type3 == "probit")
          {
            scores_jac[coef3_ind[j, ], sigma3_ind] <- H1[coef3_ind1[j, ], 
                                                         sigma3_ind1]
            scores_jac[sigma3_ind, coef3_ind[j, ]] <- H1[sigma3_ind1, 
                                                         coef3_ind1[j, ]]
            scores_jac[sigma3_ind, sigma3_ind]     <- H1[sigma3_ind1, 
                                                         sigma3_ind1]
          }
        }
      }
    }
    gc()
  }

  # Estimate clustered scores
  scores_cluster <- matrix(0, nrow = n_cluster, ncol = n_par)
  if (is_cluster)
  {
    for (i in 1:n_cluster)
    {
      scores_cluster[i, ] <- colSums(scores[ind_cluster == i, , drop = FALSE])
    }
  }

  # Estimate asymptotic covariance matrix
  vcov <- NULL
  if (is_cluster)
  {
    vcov <- tcrossprod(qr.solve(scores_jac, t(scores_cluster), tol = 1e-16))
  }
  else
  {
    vcov <- tcrossprod(qr.solve(scores_jac, t(scores), tol = 1e-16))
  }
  colnames(vcov) <- NULL
  rownames(vcov) <- NULL

  # Return the results
  return(list(scores_jac     = scores_jac,
              scores         = scores,
              scores_sum     = scores_sum,
              vcov           = vcov,
              scores_cluster = scores_cluster))
}

# Estimate scores associated with the second step
scores_2step <- function(object, type = "obs", n_cores = 1, n_sim = 1000)
{
  # Get some variables
  n_par           <- object$other$n_par
  n_eq            <- object$other$n_eq
  n_eq2           <- object$other$n_eq2
  n               <- object$other$n_obs
  groups          <- object$groups
  groups2         <- object$groups2
  groups3         <- object$groups3
  n_groups        <- object$other$n_groups
  ind_g           <- object$ind$g
  coef2_ind       <- object$ind$coef2
  degrees         <- object$degrees
  is1             <- object$other$is1
  is2             <- object$other$is2
  is3             <- object$other$is3

  # Matrix to store the scores
  scores <- matrix(0, nrow = n, ncol = n_par)
  
  # Get conditional predictions with a new model
  for (i in 1:n_groups)
  {
    groups_i  <- NA
    groups3_i <- NA
    if (is1)
    {
      groups_i <- groups[i, ]
    }
    if (is3)
    {
      groups3_i <- groups3[i]
    }
    scores_group <- predict(object, 
                            type    = "val", 
                            newdata = object$data[ind_g[[i]], , drop = FALSE],
                            group   = groups_i, 
                            group2  = groups2[i, ],
                            group3  = groups3_i,
                            control = list(is_scores = TRUE))
    for (v in 1:n_eq2)
    {
      coef2_ind_regime                     <- coef2_ind[[v]][groups2[i, v] + 1, ]
      scores[ind_g[[i]], coef2_ind_regime] <- scores_group[[v]]
    }
  }
  
  if (type == "aggregate")
  {
    scores_sum <- as.matrix(colSums(scores), ncol = 1)
    return(scores_sum)
  }
  
  return(scores)
}

#' Calculate Variance-Covariance Matrix for a msel Object.
#' @description Return the variance-covariance matrix of the parameters of
#' msel model.
#' @param object an object of class \code{msel}.
#' @param ... further arguments (currently ignored).
#' @param type character representing the type of the asymptotic covariance 
#' matrix estimator. It takes the same values as \code{cov_type} parameter of
#' the \code{\link[switchSelection]{msel}} function.
#' @param n_sim integer representing the number of GHK draws when there are
#' more than 3 ordered equations. Otherwise alternative (much more efficient) 
#' algorithms will be used to calculate multivariate normal probabilities.
#' @param n_cores positive integer representing the number of CPU cores used for 
#' parallel computing. If possible it is highly recommended to set it equal to
#' the number of available physical cores especially when the system of
#' ordered equations has 2 or 3 equations.
#' @param recalculate logical; if \code{TRUE} then covariance matrix will be
#' recalculated even if 'type' is the same as 'cov_type' input argument
#' of the model.
#' @param cluster an object which takes the same values as the \code{cluster} 
#' argument of the \code{\link[switchSelection]{msel}} function. 
#' The only exception is \code{cluster = NULL} which implies 
#' \code{cluster = object$cluster}.
#' @details Argument \code{type} is closely related to the argument 
#' \code{cov_type} of \code{\link[switchSelection]{msel}} function. 
#' See 'Details' and 'Usage' sections of \code{\link[switchSelection]{msel}} 
#' for more information on \code{cov_type} argument.
#' 
#' The user may also estimate asymptotic covariance matrix of the parameters
#' of several models. For more information, see paragraph 'Cross-model tests'
#' in 'Details' section of \code{\link[switchSelection]{test_msel}}.
#' 
#' @return Returns numeric matrix which represents estimate of the asymptotic 
#' covariance matrix of model's parameters. If \code{object} is a list of
#' models then rows and columns of this matrix will have names \code{"mipj"}
#' indicating that corresponding element is associated with \code{j}-th 
#' parameter of the \code{i}-th model.
vcov.msel <- function(object, ..., 
                      type        = object$cov_type, 
                      n_cores     = object$other$n_cores,
                      n_sim       = object$other$n_sim,
                      recalculate = FALSE,
                      cluster     = NULL)
{
  # Validate dots
  if (length(list(...)) > 0)
  {
    warning("Additional arguments passed through ... are ignored.")   
  }
  
  # Deal with several objects
  if (!is(object = object, class2 = "msel"))
  {
    return (vcov_combine(object))
  }
  
  # Clusters
  if (!is.null(cluster))
  {
    object$cluster           <- cluster
    cluster_list             <- cluster_msel(object = object, cluster = cluster)
    object$other$ind_cluster <- cluster_list$ind_cluster
    object$other$n_cluster   <- cluster_list$n_cluster
    object$other$is_cluster  <- cluster_list$is_cluster
    recalculate              <- TRUE
  }
  
  # If no recalculations are required
  if ((type == object$cov_type) & !recalculate)
  {
    return (object$cov)
  }
  
  # Maximum-likelihood
  if (object$estimator == "ml")
  {
    return (vcov_ml(object  = object,  type  = type, 
                    n_cores = n_cores, n_sim = n_sim)$vcov)
  }
  
  # Two-step
  if (object$estimator == "2step")
  {
    return(vcov_2step(object = object,n_cores = n_cores, n_sim = n_sim)$vcov)
  }
  
  # Error
  stop("Incorrect 'object$estimator' value.")
}

#' Combine the asymptotic covariance matrices
#' @param object a list of objects of class \code{msel}.
#' @param type the same as the \code{type} argument of the 
#' \code{\link[switchSelection]{vcov.msel}} function.
vcov_combine <- function(object, type = "cov")
{
  # Get the number of objects
  n_objects <- length(object)
  
  # Total number of parameters and indexes of the parameters
  n_par     <- 0
  ind_par   <- vector(mode = "list", length = n_objects)
  for (i in 1:n_objects)
  {
    # Validation of the class
    if (!is(object = object[[i]], class2 = "msel"))
    {
      stop(paste0("Argument object[[", i, "]] is not of class 'msel'."))
    }
    
    # Validation of the covariance matrix type
    if (!(object[[i]]$cov_type %in% c("sandwich", "mm")))
    {
      stop(paste0("Covariance matrix type of object[[", i, "]] should be ",
                  "either 'sandwich' or 'mm', but '", object[[i]]$cov_type,
                  "' has been supplied."))
    }
    
    # Validation of the crossind variable
    if (!hasName(x = object[[i]]$data, "crossind"))
    {
      stop (paste0("There is no 'crossind' variable in data of object[[", 
                   i, "]]."))
    }
    
    # Indexes
    ind_start    <- n_par + 1
    n_par        <- n_par + object[[i]]$other$n_par
    ind_end      <- n_par
    ind_par[[i]] <- ind_start:ind_end
  }
  
  # Create indexes
  crossind <- numeric()
  for (i in 1:n_objects)
  {
    crossind <- c(crossind, object[[i]]$data$crossind)
  }
  crossind <- unique(crossind)
  
  # Total number of observations
  n <- max(crossind)
  
  # Store the Jacobians and Hessians
  J <- matrix(0, nrow = n,     ncol = n_par)
  H <- matrix(0, nrow = n_par, ncol = n_par)
  for (i in 1:n_objects)
  {
    J[object[[i]]$data$crossind, ind_par[[i]]] <- object[[i]]$J
    H[ind_par[[i]], ind_par[[i]]]              <- object[[i]]$H
  }
  
  # Remove bad values
  J_remove  <- apply(J, 1, function(x) any(is.na(x) | is.infinite(x)))
  if (any(J_remove))
  {
    J <- J[!J_remove, ]
    n <- nrow(J)
    warning("NA, NaN or Inf values in J")
  }

  # Check whether clustering is needed
  is_cluster <- FALSE
  for (i in 1:n_objects)
  {
    if ("crosscluster" %in% colnames(object[[i]]$data))
    {
      is_cluster <- TRUE
    }
    else
    {
      if (is_cluster)
      {
        stop (paste0("There is no 'crosscluster' variable in data of object[[",
                     i, "]]. If clustering of standard errors is not needed, ",
                     "then remove this variable from data of each model. ",
                     "Otherwise, add this variable to data of each model."))
      }
    }
  }
  
  # Stuff for clustered data
  if (is_cluster)
  {
    # Combine indexes of the clusters
    cluster_ind <- vector("numeric", length = n)
    for (i in 1:n_objects)
    {
      cluster_ind[object[[i]]$data$crossind] <- object[[i]]$data$crosscluster
    }
    cluster_unique <- unique(cluster_ind)
    n_cluster      <- length(cluster_unique)
    
    # Estimate clustered scores
    J_cluster  <- matrix(0, nrow = n_cluster, ncol = ncol(J))
    for (i in 1:n_cluster)
    {
      J_cluster[i, ] <- colSums(J[cluster_ind == cluster_unique[i], , 
                                  drop = FALSE])
    }
    
    # Rewrite initial scores with clustered scores
    J <- J_cluster
    n <- nrow(J)
  }
    
  # Remove rows of zeros
  J_zero_row <- apply(X = J, MARGIN = 1, FUN = function(x) all(x == 0))
  J          <- J[!J_zero_row, ]
  n          <- nrow(J)
  
  # Sandwich estimator
  cov <- tcrossprod(qr.solve(H, t(J), tol = 1e-16))
  
  # Names
  colnames(cov) <- 1:n_par
  rownames(cov) <- 1:n_par
  for (i in 1:n_objects)
  {
    cov_names                   <- paste0("m", i, "p", 
                                          1:object[[i]]$other$n_par)
    colnames(cov)[ind_par[[i]]] <- cov_names
    rownames(cov)[ind_par[[i]]] <- cov_names
  }
  
  # Extended return
  if (type == "all")
  {
    out <- list(cov = cov, J = J, H = H, n = n, n_par = n_par)
    return (out)
  }
  
  # Return the results
  return(cov)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formula.R
\name{formula_merge}
\alias{formula_merge}
\title{Merge formulas}
\usage{
formula_merge(..., type = "all")
}
\arguments{
\item{...}{formulas to be merged such that there is a single element
on the left hand side and various elements on the right hand side.}

\item{type}{string representing the type of merge to be used.
If \code{type = "all"} then both right hand side and left hand side
elements of the formulas will be merged on the right hand side.
If \code{type = "terms"} then only right hand side elements of the
formulas will be merged on the right hand side.
If \code{type = "var-terms"} then the result is the same as in case
when \code{type = "terms"} but there will be left hand side element
of the first formula on the left hand side of the merged formula.}
}
\value{
This function returns a formula which form depends on 
\code{type} input argument value. See 'Details' for additional information.
}
\description{
This function merges all variables of several formulas
into a single formula.
}
\details{
Merged formulas should have a single element on the left hand
side and arbitrary number of elements on the right hand side.
}
\examples{
# Consider three formulas
f1 <- as.formula("y1 ~ x1 + x2")
f2 <- as.formula("y2 ~ x2 + x3")
f3 <- as.formula("y3 ~ y2 + x6")
# Merge these formulas in a various ways
formula_merge(f1, f2, f3, type = "all")
formula_merge(f1, f2, f3, type = "terms")
formula_merge(f1, f2, f3, type = "var-terms")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lm.R
\name{loocv}
\alias{loocv}
\title{Leave-one-out cross-validation}
\usage{
loocv(fit)
}
\arguments{
\item{fit}{object of class \code{lm}.}
}
\value{
This function returns a numeric value representing root mean squared 
error (RMSE) of leave-one-out cross-validation (LOOCV).
}
\description{
This function calculates root mean squared error (RMSE) for
leave-one-out cross-validation of linear regression estimated via
least squares method.
}
\details{
Fast analytical formula is used.
}
\examples{
set.seed(123)
# Generate data according to linear regression
n <- 100
eps <- rnorm(n)
x <- runif(n)
y <- x + eps
# Estimate the model
model <- lm(y ~ x)
# Perform cross-validation
loocv(model)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formula.R
\name{formula_split}
\alias{formula_split}
\title{Split formula by symbol}
\usage{
formula_split(formula, symbol = "|")
}
\arguments{
\item{formula}{an object of class \code{formula}.}

\item{symbol}{a string that is used to split \code{formula} into
two formulas.}
}
\value{
This function returns a list of two formulas.
}
\description{
This function splits one formula into two formulas
by symbol.
}
\details{
The \code{symbol} should be on the right hand side of
the formula.
}
\examples{
formula_split("y ~ x1 + x2 | x2 + x3")
formula_split("y ~ x1 + x2 : x2 + x3", symbol = ":")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vcov.R
\name{vcov_combine}
\alias{vcov_combine}
\title{Combine the asymptotic covariance matrices}
\usage{
vcov_combine(object, type = "cov")
}
\arguments{
\item{object}{a list of objects of class \code{msel}.}

\item{type}{the same as the \code{type} argument of the 
\code{\link[switchSelection]{vcov.msel}} function.}
}
\description{
Combine the asymptotic covariance matrices
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lm.R
\name{boot}
\alias{boot}
\title{Bootstrap covariance matrix for least squares estimates of linear regression}
\usage{
boot(model, iter = 100)
}
\arguments{
\item{model}{object of class \code{lm}.}

\item{iter}{positive integer representing the number of bootstrap iterations.}
}
\value{
This function returns a bootstrapped covariance matrix of the
least squares estimator.
}
\description{
This function calculates bootstrapped covariance matrix
for least squares estimates of linear regression. The estimates should be
obtained via \code{lm} function.
}
\details{
Calculations may take long time for high \code{iter} value.
}
\examples{
set.seed(123)
# Generate data according to linear regression
n <- 20
eps <- rnorm(n)
x <- runif(n)
y <- x + eps
# Estimate the model
model <- lm(y ~ x)
# Calculate bootstrap covariance matrix
boot(model, iter = 50)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.R
\name{summary.test_msel}
\alias{summary.test_msel}
\title{Summary for an Object of Class delta_method}
\usage{
\method{summary}{test_msel}(object, ..., is_legend = TRUE)
}
\arguments{
\item{object}{object of class 'delta_method'}

\item{...}{further arguments (currently ignored)}

\item{is_legend}{a logical; if \code{TRUE} then additional information
is shown.}
}
\value{
Returns an object of class 'summary.delta_method'.
}
\description{
Provides summary for an object of class 'delta_method'.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.msel}
\alias{summary.msel}
\title{Summary for an Object of Class msel}
\usage{
\method{summary}{msel}(object, ..., vcov = NULL, show_ind = FALSE)
}
\arguments{
\item{object}{object of class 'msel'}

\item{...}{further arguments (currently ignored)}

\item{vcov}{positively defined numeric matrix representing
asymptotic variance-covariance matrix of the estimator to be
used for calculation of standard errors and p-values. It may also be a 
character. Then the \code{\link[switchSelection]{vcov.msel}} function
will be used, whose input argument \code{type} will be set to \code{vcov}.
If \code{estimator = "2step"} then \code{vcov} should be an estimate of the 
asymptotic covariance matrix of the first step estimator.}

\item{show_ind}{logical; if \code{TRUE} then indexes of parameters will be
shown. Particularly, these indexes may be used in \code{ind} element of
\code{regularization} parameter of \code{\link[switchSelection]{msel}}.}
}
\value{
Returns an object of class 'summary.msel'.
}
\description{
Provides summary for an object of class 'msel'.
}
\details{
If \code{vcov} is \code{NULL} then this function just changes the 
class of the 'msel' object to 'summary.msel'. Otherwise it 
additionally changes \code{object$cov} to \code{vcov} and uses it to
recalculate \code{object$se}, \code{object$p_value} and \code{object$tbl} 
values. It also adds the value of \code{show_ind} argument to the object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootstrap.R
\name{bootstrap}
\alias{bootstrap}
\alias{bootstrap_msel}
\alias{bootstrap_combine_msel}
\title{Bootstrap for msel function}
\usage{
bootstrap_msel(
  object,
  iter = 100,
  opt_type = "optim",
  opt_args = NULL,
  is_ind = FALSE,
  n_sim = 1000,
  n_cores = 1
)

bootstrap_combine_msel(...)
}
\arguments{
\item{object}{an object of class 'msel' or a list of such objects.}

\item{iter}{the number of bootstrap iterations.}

\item{opt_type}{the same as the \code{opt_type} argument of the 
\code{\link[switchSelection]{msel}} function.}

\item{opt_args}{the same as the \code{opt_args} argument of the 
\code{\link[switchSelection]{msel}} function.}

\item{is_ind}{logical; if \code{TRUE} then the function also returns a
numeric matrix of indexes of observations used in the bootstrap samples.}

\item{n_sim}{the same as the \code{n_sim} argument of the 
\code{\link[switchSelection]{msel}} function.}

\item{n_cores}{the same as the \code{n_cores} argument of the 
\code{\link[switchSelection]{msel}} function.}

\item{...}{objects returned by the function 
\code{\link[switchSelection]{bootstrap_msel}} to be combined into a 
single object.}
}
\value{
Function \code{\link[switchSelection]{bootstrap_msel}}
returns an object of class \code{"bootstrap_msel"}.
This object is a list which may contain the following elements:
\itemize{
\item \code{par} - a numeric matrix such that \code{par[b, ]} is a vector 
of the estimates of the parameters of the model estimated via the 
\code{\link[switchSelection]{msel}} function on the \code{b}-th 
bootstrap sample.
\item \code{iter} - the number of the bootstrap iterations.
\item \code{cov} - bootstrap estimate of the covariance matrix which 
equals \code{cov(par)}
\item \code{ind} - a numeric matrix such that \code{ind[, b]} stores the 
indexes of the observations from \code{object$data} included in the 
\code{b}-th bootstrap sample.
}
Function \code{\link[switchSelection]{bootstrap_combine_msel}} returns the 
object which combines several objects returned by the
\code{\link[switchSelection]{bootstrap_msel}} function into a single
object.
}
\description{
Function \code{\link[switchSelection]{bootstrap_msel}} 
provides bootstrap estimates of the parameters of the model estimated via
the \code{\link[switchSelection]{msel}} function.
Function \code{\link[switchSelection]{bootstrap_combine_msel}} allows to
combine several objects of class \code{'bootstrap_msel'}.
}
\details{
The function generates \code{iter} bootstrap samples and 
estimates the parameters \eqn{\theta} of the model by using each of
these samples. The estimate \eqn{\hat{\theta}^{(b)}} from the \eqn{b}-th of 
these samples is stored as the \code{b}-th row of the numeric matrix
\code{par} which is an element of the returned object.

Use the \code{\link[switchSelection]{update_msel}} function to
transfer the bootstrap estimates to the object of class \code{'msel'}.

Argument \code{object} may be a list of objects of class \code{'msel'}.
Then the function returns an object of class 
\code{'crossmodel_bootstrap_msel'} which is a list \code{out} such that 
\code{out[[j]]} is an object of class \code{'bootstrap_msel'}. Notice that
all the models will be estimated on the same bootstrap samples. That is,
\code{out[[j]]$ind} is the same for each model \code{j}. Specifically, 
it may be useful for cross-model hypothesis testing.

See \code{\link[switchSelection]{test_msel}} for hypothesis testing.
}
\examples{
# -------------------------------
# Bootstrap for the probit model
# -------------------------------
\donttest{
# ---
# Step 1
# Simulation of data
# ---

# Load required package
library("mnorm")

# Set seed for reproducibility
set.seed(123)

# The number of observations
n <- 100

# Regressors (covariates)
w1 <- runif(n = n, min = -1, max = 1)
w2 <- runif(n = n, min = -1, max = 1)

# Random errors
u <- rnorm(n = n, mean = 0, sd = 1)

# Coefficients
gamma <- c(-1, 2)

# Linear index
li <- gamma[1] * w1 + gamma[2] * w2

# Latent variable
z_star <- li + u

# Cuts
cuts <- c(-1, 0.5, 2)

# Observable ordered outcome
z                                           <- rep(0, n)
z[(z_star > cuts[1]) & (z_star <= cuts[2])] <- 1
z[(z_star > cuts[2]) & (z_star <= cuts[3])] <- 2
z[z_star > cuts[3]]                         <- 3
table(z)

# Data
data <- data.frame(w1 = w1, w2 = w2, z = z, crossind = 1:n)

# ---
# Step 2
# Estimation of the parameters
# ---

# Estimation
model <- msel(formula = z ~ w1 + w2, data = data)
summary(model)
 
# ---
# Step 3
# Bootstrap
# ---

# Perform the bootstrap
bootstrap <- bootstrap_msel(model)

# Test the hypothesis that H0: gamma[2] = -2gamma[1]
# by using the t-test and with bootstrap p-values
fn_test <- function(object)
{
  gamma <- coef(object, eq = 1)
  return(gamma[2] + 2 * gamma[1])
}
b <- test_msel(object    = model, 
               fn        = fn_test,
               test      = "t", 
               method    = "bootstrap",
               ci        = "percentile",
               se_type   = "bootstrap",
               bootstrap = bootstrap)
summary(b)

# Replicate the analysis with an additional 20 bootstrap iterations
bootstrap2    <- bootstrap_msel(model, iter = 20)
bootstrap_new <- bootstrap_combine_msel(bootstrap, bootstrap2)
b2 <- test_msel(object    = model, 
                fn        = fn_test,
                test      = "t", 
                method    = "bootstrap",
                ci        = "percentile",
                se_type   = "bootstrap",
                bootstrap = bootstrap_new)
summary(b2)

# ---
# Step 4
# Cross-model test
# ---

# Estimate restricted model
model2 <- msel(formula = z ~ w1, data = data)

# Perform a bootstrap for two models
bootstrap2 <- bootstrap_msel(list(model, model2), iter = 10)

# Test the hypothesis that the coefficients in both models are the same
fn_test2 <- function(object)
{
  coef1 <- coef(object[[1]], eq = 1)[1]
  coef2 <- coef(object[[2]], eq = 1)[1]
  return(coef1 - coef2)
}
b <- test_msel(object    = list(model, model2), 
               fn        = fn_test2,
               test      = "t", 
               method    = "bootstrap",
               ci        = "percentile",
               se_type   = "bootstrap",
               bootstrap = bootstrap2)
summary(b)

}
 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrtest.R
\name{print.lrtest_msel}
\alias{print.lrtest_msel}
\title{Print Method for Likelihood Ratio Test}
\usage{
\method{print}{lrtest_msel}(x, ...)
}
\arguments{
\item{x}{object of class "summary.lrtest_msel".}

\item{...}{further arguments (currently ignored).}
}
\value{
The function returns the input argument \code{x}.
}
\description{
Prints summary for an object of class 'lrtest_msel'.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{lnL_msel}
\alias{lnL_msel}
\title{Log-likelihood Function of Multivariate Ordered Probit Model}
\usage{
lnL_msel(
  par,
  control_lnL,
  out_type = "val",
  n_sim = 1000L,
  n_cores = 1L,
  regularization = NULL
)
}
\arguments{
\item{par}{vector of parameters.}

\item{control_lnL}{list with some additional parameters.}

\item{out_type}{string representing the output type of the function.}

\item{n_sim}{the number of random draws for multivariate 
normal probabilities.}

\item{n_cores}{the number of cores to be used.}

\item{regularization}{list of regularization parameters.}
}
\description{
Calculates log-likelihood function of multivariate ordered
probit model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootstrap.R
\name{update_msel}
\alias{update_msel}
\title{Update msel object with the new estimates}
\usage{
update_msel(object, par)
}
\arguments{
\item{object}{an object of class \code{'msel'}.}

\item{par}{a vector of parameters which substitutes \code{object$par} and
is used to update the estimates i.e., \code{object$coef}, \code{object$cuts} 
and others.}
}
\value{
This function returns an object of class \code{'msel'}
in which \code{object$par} is substituted with \code{par}. Also, \code{par} 
is used to update the estimates i.e., \code{object$coef}, \code{object$cuts} 
and others.
}
\description{
This function updates parameters of the model estimated via 
\code{\link[switchSelection]{msel}} function.
}
\details{
It may be useful to apply this function to the bootstrap
estimates of \code{\link[switchSelection]{bootstrap_msel}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cps.R
\docType{data}
\name{cps}
\alias{cps}
\title{A subset of data from Current Population Survey (CPS).}
\format{
A data frame with 18,253 rows and 25 columns. It contains information on
wages and some socio-demographic characteristics of the middle age
(25-54 years) married women:
\describe{
  \item{age}{the age measured in years.}
  \item{sage}{the same as age but for a spouse.}
  \item{work}{a binary variable for the employment status 
  (0 - unemployed, 1 - employed).}
  \item{swork}{the same as work but for a spouse.}
  \item{nchild}{the number of children under age 5.}
  \item{snchild}{the same as nchild but for a spouse.}
  \item{health}{subjective health status 
  (1 - poor, 2 - fair, 3 - good, 4 - very good, 5 - excellent).}
  \item{shealth}{the same as health but for a spouse.}
  \item{basic}{a binary variable which equals 1 for those who have graduated 
  from high school or has at least some college or has an associated degree 
  and does not have any higher level of education, 0 - otherwise.}
  \item{bachelor}{a binary variable which equals 1 for those whose highest 
  education level is a bachelor degree.}
  \item{master}{a binary variable which equals 1 for those whose highest 
  education level is a master degree.}
  \item{sbasic}{the same as basic but for a spouse.}
  \item{sbachelor}{the same as bachelor but for a spouse.}
  \item{smaster}{the same as master but for a spouse.} 
  \item{educ}{a categorical variable for the level of education such that 
  educ = 0 if basic = 1, 
  educ = 1 if bachelor = 1 and 
  educ = 2 if master = 1.}
  \item{seduc}{the same as educ but for a spouse.}
  \item{weeks}{a total number of weeks worked during the year.}
  \item{sweeks}{the same as weeks but for a spouse.}
  \item{hours}{a usual number of working hours per week.}
  \item{shours}{the same as hours but for a spouse.}
  \item{wage}{the wage of the individual.}
  \item{swage}{the same as wage but for a spouse.}
  \item{lwage}{an inverse hyperbolic sine transformation of the hourly wage.}
  \item{slwage}{the same as lwage but for a spouse.}
  \item{state}{a state of residence.}
  ...
}
}
\source{
<https://www.census.gov/programs-surveys/cps.html>
}
\usage{
data(cps)
}
\description{
Labor market data on 18,253 middle age (25-54 years) married women
in the year 2022.
}
\examples{
\donttest{
data(cps)
model <- msel(work ~ age + bachelor + master, data = cps)
summary(model)
} 
}
\references{
Flood S, King M, Rodgers R, Ruggles S, Warren R, 
Westberry M (2022). Integrated Public Use Microdata Series, 
Current Population Survey: Version 10.0 [dataset].
doi: 10.18128/D030.V10.0.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpFunctions.R
\name{exogenous_fn}
\alias{exogenous_fn}
\title{Modify exogenous variables in data frame}
\usage{
exogenous_fn(exogenous, newdata)
}
\arguments{
\item{exogenous}{list such that \code{exogenous[[i]]} represents the value 
(or a vector of values of the same size as \code{nrow(newdata)}) which will 
be exogenously assigned to the variable \code{names(exogenous)[[i]]} in 
\code{newdata} i.e., \code{newdata[, names(exogenous)[i]] <- exogenous[[i]]}.
If \code{newdata} is \code{NULL} and \code{exogenous} is not \code{NULL} then
\code{newdata} is set to \code{object$data}.
This argument is especially useful for causal inference when some endogenous 
(dependent) variables should be exogenously assigned with some values i.e.,
in the right hand side of the \code{formula} and \code{formula2}.
The purpose of the \code{exogenous} argument is just a convenience so
equivalently it is possible to exogenously provide the values to variables
via \code{newdata} argument.}

\item{newdata}{data frame.}
}
\value{
The function returns data frame which is similar to \code{newdata} 
but some values of this data frame are set according to \code{exogenous}.
}
\description{
Change some values of the exogenous variables in a data frame.
}
\details{
This function changes \code{exogenous} variables in \code{newdata}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logLik.R
\name{logLik.msel}
\alias{logLik.msel}
\title{Extract Log-Likelihood from a Fit of the msel Function.}
\usage{
\method{logLik}{msel}(object, ...)
}
\arguments{
\item{object}{object of class "msel"}

\item{...}{further arguments (currently ignored)}
}
\value{
Returns an object of class 'logLik'.
}
\description{
Extract Log-Likelihood from a model fit
of the \code{\link[switchSelection]{msel}} function.
}
\details{
If \code{estimator == "2step"} in 
\code{\link[switchSelection]{msel}} then function may return
\code{NA} value since the two-step estimator of the covariance matrix may not
be positively defined.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logLik.R
\name{nobs.msel}
\alias{nobs.msel}
\title{Extract the Number of Observations from a Fit of the msel Function.}
\usage{
\method{nobs}{msel}(object, ...)
}
\arguments{
\item{object}{object of class "msel"}

\item{...}{further arguments (currently ignored)}
}
\value{
A single positive integer number.
}
\description{
Extract the number of observations from a model fit
of the \code{\link[switchSelection]{msel}} function.
}
\details{
Unobservable values of continuous equations are included in
the number of observations.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpFunctions.R
\name{starsVector}
\alias{starsVector}
\title{Stars for p-values}
\usage{
starsVector(p_value)
}
\arguments{
\item{p_value}{vector of values between 0 and 1 representing p-values.}
}
\value{
The function returns a string vector of stars assigned according
to the rules described in 'Details' section.
}
\description{
This function assigns stars 
(associated with different significance levels) to p-values.
}
\details{
Three stars are assigned to p-values not greater than 0.01.
Two stars are assigned to p-values greater than 0.01 and not greater 
than 0.05.
One star is assigned to p-values greater than 0.05 and not greater than 0.1.
}
\examples{
p_value <- c(0.002, 0.2, 0.03, 0.08)
starsVector(p_value)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{predict.msel}
\alias{predict.msel}
\title{Predict method for msel function}
\usage{
\method{predict}{msel}(
  object,
  ...,
  newdata = NULL,
  given_ind = numeric(),
  group = NA,
  group2 = NA,
  group3 = NA,
  type = ifelse(any(is.na(group2)), "prob", "val"),
  me = NULL,
  eps = NULL,
  control = list(),
  test = FALSE,
  exogenous = NULL,
  factors = NULL
)
}
\arguments{
\item{object}{an object of class "msel".}

\item{...}{further arguments (currently ignored)}

\item{newdata}{an optional data frame in which to look for variables 
with which to predict. If omitted, the original data frame used.
This data frame should contain values of dependent variables even if
they are not actually needed for prediction 
(simply assign them with 0 values).}

\item{given_ind}{a numeric vector of indexes of conditioned components.}

\item{group}{a numeric vector which i-th element represents a value of the
i-th dependent variable. If this value equals -1 then this component
will be ignored (useful for estimation of marginal probabilities).}

\item{group2}{a numeric vector which i-th element represents a value of the
i-th dependent variable of the continuous equation. 
If this value equals -1 then this component will be ignored.}

\item{group3}{an integer representing the index of the alternative of the
multinomial equation. If this value equals -1 then this component will be 
ignored.}

\item{type}{a string representing a type of the prediction.
See 'Details' for more information.}

\item{me}{a string representing the name of the variable for which marginal
effect should be estimated. It may also equal to a character vector. Then
the function returns a numeric matrix which \code{i}-th column contains
estimates of the marginal effects with respect to \code{marginal[i]}.
See 'Details' for more information.}

\item{eps}{a numeric vector of length 1 or 2 used for calculation of
the marginal effects. See 'Details' for more information.}

\item{control}{a list of additional arguments. Currently is not intended
for the users.}

\item{test}{a logical, function or integer. If \code{test = TRUE} then the 
output of the function is supplied to 
\code{\link[switchSelection]{test_msel}} before return to perform a t-test.
If \code{test} is a function it will be applied to the output of the 
\code{predict} before \code{\link[switchSelection]{test_msel}} is called.
If \code{test} is an integer then \code{\link[switchSelection]{test_msel}}
will be applied only to the \code{test}-th column of the output.}

\item{exogenous}{a list such that \code{exogenous[[i]]} represents the value 
(or a vector of values of the same size as \code{nrow(newdata)}) which will 
be exogenously assigned to the variable \code{names(exogenous)[[i]]} in 
\code{newdata} i.e., \code{newdata[, names(exogenous)[i]] <- exogenous[[i]]}.
If \code{newdata} is \code{NULL} and \code{exogenous} is not \code{NULL} then
\code{newdata} is set to \code{object$data}.
This argument is especially useful for the causal inference when some 
endogenous (dependent) variables should be exogenously assigned with some 
values i.e., in the right hand side of the \code{formula}, \code{formula2} 
and \code{formula3}. The purpose of the \code{exogeneous} argument is just a 
convenience so equivalently it is possible to exogenously provide the values
to variables via the \code{newdata} argument.}

\item{factors}{a list which elements are character vectors representing
the variables which should be treated as factors when estimation of 
marginal effects is performed. It is assumed that if one of these variables
equals 1 then other are set to 0. Specifically, if \code{me[i]} is an 
element of \code{factors[[j]]} for some \code{j}, then when estimating 
marginal effect respect to \code{me[i]}, the value of \code{eps} set to 
\code{c(0, 1)} and the  values of other variables in \code{factors[[j]]} are
set (internally by the function) to \code{0}.
If \code{length(me) = 1} then \code{factors} may be just a 
character vector (not a list).}
}
\value{
This function returns predictions for each row of \code{newdata}
or for each observation in the model if \code{newdata} is \code{NULL}.
Structure of the output depends on the \code{type} argument
(see 'Details' section).
}
\description{
Predicted values based on the object of class 'msel'.
}
\details{
See 'Examples' section of \code{\link[switchSelection]{msel}}
for the examples of this function application.

\strong{Probabilities of the multivariate ordinal equations}

If \code{type = "prob"} then the function returns a joint probability that 
the ordinal outcomes will have values assigned in \code{group}. To calculate
marginal probabilities set unnecessary \code{group} values to \code{-1}. 

To estimate conditional probabilities provide indexes of the conditioned 
outcomes through the \code{given_ind} argument. 

For example, if \eqn{z_{1i}}, \eqn{z_{2i}} and \eqn{z_{3i}} are the ordinal
outcomes then to estimate 
\eqn{P(z_{1i}=2 | z_{3i} = 0, w_{1i}, w_{2i}, w_{3i})} set 
\code{given_ind = 3} and \code{groups = c(2, -1, 0)}.

\strong{Linear predictors (indexes) of the multivariate ordinal equations}

If \code{type = "li"} or \code{type = "lp"} then the function returns a 
matrix whose columns are linear predictors (indexes) of the corresponding 
equations. If \code{group[j] = -1} then linear predictors (indexes) 
associated with the \code{j}-th ordinal equation will be omitted from the 
output.

For example, if \code{group = c(0, -1, 1)} then the function returns a
matrix whose first column is \eqn{w_{1i}\hat{\gamma}_{1}} and the second 
column is \eqn{w_{3i}\hat{\gamma}_{3}}.

\strong{Standard deviations of the multivariate ordinal equations}

If \code{type = "sd"} then the function returns a matrix whose columns are
the estimates of the standard deviations of the random errors for 
the corresponding equations. 
If \code{group[j] = -1} then the standard deviations associated with the 
\code{j}-th ordinal equation will be omitted from the output.

For example, if \code{group = c(0, -1, 1)} then the function returns a
matrix whose first column is \eqn{\hat{\sigma}_{1i}^{*}} and the second 
column is \eqn{\hat{\sigma}_{3i}^{*}}.

\strong{Predictions of the continuous outcomes}

If \code{type = "val"} then the function returns the predictions of the 
conditional (on \code{group}) expectation of the continuous outcomes in the 
regimes determined by the \code{group2} argument. To predict unconditional 
expectations set \code{group} to a vector of \code{-1} values.

For example, suppose that there is a single continuous equation \eqn{y_{i}} 
and two ordinal equations \eqn{z_{1i}} and \eqn{z_{2i}}. 
To estimate \eqn{E(y_{2i}|x_{i})} set \code{group = c(-1, -1)} and 
\code{group2 = 2}. 
To estimate \eqn{E(y_{1i}|x_{i}, z_{1i} = 2, z_{2i} = 0)} set 
\code{group = c(2, 0)} and \code{group2 = 1}.
To estimate \eqn{E(y_{0i}|x_{i}, z_{2i} = 1)} set 
\code{group = c(-1, 1)} and \code{group2 = 0}.

Suppose that there are two continuous \eqn{y_{i}^{(1)}},
\eqn{y_{i}^{(2)}} and two ordinal \eqn{z_{1i}}, \eqn{z_{2i}} equations.
If \code{group2 = c(1, 3)} and \code{group = c(3, 0)} then the function 
returns a matrix whose first column contains the estimates of 
\eqn{E(y_{1i}^{(1)}|z_{1i} = 3, z_{2i} = 0, x_{i}^{(1)})} and the second 
column contains the estimates of 
\eqn{E(y_{3i}^{(2)}|z_{1i} = 3, z_{2i} = 0, x_{i}^{(2)})}.

\strong{Selectivity terms}

If \code{type = "lambda"} then the function returns a matrix whose
\code{j}-th column is a numeric vector of estimates of the selectivity terms 
\eqn{\lambda_{ji}} associated with the ordinal equations. 
Similarly if \code{type = "lambda_mn"} then the
function returns a numeric matrix with the selectivity terms of the 
multinomial equations.

\strong{Probabilities of the multinomial equation}

If \code{type = "prob_mn"} and \code{group3 = j} then the function returns 
a vector of the estimates of the probabilities 
\eqn{P(\tilde{z}_{i}=j|\tilde{w}_{i})}.

\strong{Linear indexes (predictors) of the multinomial equation}

If \code{type = "li_mn"} or \code{type = "lp_mn"} then the function returns a 
numeric matrix whose \code{j}-th column is a numeric vector of estimates of 
the linear predictor (index) associated with the \code{(j-1)}-th alternative 
\eqn{\tilde{w}_{i}\tilde{\gamma}_{(j-1)}}.

\strong{Estimation of the marginal effects}

If \code{me} is provided then the function returns the marginal effect
of variable \code{me} with respect to the statistic determined by 
the \code{type} argument. 

For example, if \code{me = "x1"} and \code{type = "prob"} then the function 
returns a marginal effect of \code{x1} on the corresponding probability 
i.e., one that would be estimated if \code{me} is \code{NULL}.

If \code{length(eps) = 1} then \code{eps} is an increment in the 
numeric differentiation procedure. 
If \code{eps} is \code{NULL} then this increment will be selected 
automatically taking into account the scaling of the variables. 
If \code{length(eps) = 2} then marginal effects will be estimated as the
difference between predicted values when variable \code{me} equals 
\code{eps[2]} and \code{eps[1]}, respectively. 

For example, suppose that 
\code{type = "prob"}, \code{me = "x1"}, \code{given_ind = 3} and
\code{groups = c(2, -1, 0)}. Then if \code{eps} is \code{NULL} or a 
small number (something like \code{eps = 0.0001}) then the following marginal
effect will be estimated (via numeric differentiation):
\deqn{\frac{\partial P(z_{1i}=2 | z_{3i} = 0)}{\partial x_{1i}}.}
If \code{eps = c(1, 3)} then the function estimates the following difference
(useful for estimation of marginal effects of ordered covariates):
\deqn{P(z_{1i}=2 | z_{3i} = 0, x_{1i} = 3) - 
      P(z_{1i}=2 | z_{3i} = 0, x_{1i} = 1).}
Notice that the conditioning on \eqn{w_{ji}} has been omitted for brevity.
 
\strong{Causal inference}

Argument \code{exogenous} is useful for the causal inference. 
For example, suppose that there are two binary outcomes \eqn{z_{1i}} and
\eqn{z_{2i}}. Also \eqn{z_{1i}} is the endogenous regressor for \eqn{z_{2i}}.
That is \eqn{z_{1i}} appears both on the left hand side of
\code{formula[[1]]} and on the right hand side of \code{formula[[2]]}. 
Consider the estimation of the average treatment effect:
\deqn{ATE = P(z_{2i} = 1|do(z_{1i}) = 1) - 
P(z_{2i} = 1|do(z_{1i}) = 0),}
where \eqn{do} is a do-calculus operator.
The estimate of the average treatment effect is as follows:
\deqn{\widehat{ATE} = \frac{1}{n}\sum\limits_{i=1}^{n}p_{1i}-p_{0i},}
where:
\deqn{p_{1i} = \hat{P}(z_{2i} = 1|do(z_{1i}) = 1, w_{1i}, w_{2i}^{(*)}),}
\deqn{p_{0i} = \hat{P}(z_{2i} = 1|do(z_{1i}) = 0, w_{1i}, w_{2i}^{(*)}).}

Vector \eqn{w_{2i}^{(*)}} denotes all the regressors \eqn{w_{2i}} except
the endogenous one \eqn{z_{1i}}.

To get \eqn{\widehat{ATE}} it is sufficient to make the following steps. 
First, calculate \eqn{p_{1i}} by setting \code{type = "prob"}, 
\code{group = c(-1, 1)} and providing the value \code{1} to \eqn{z_{1i}} 
through the \code{exogenous} argument. 
Second, calculate \eqn{p_{0i}} by setting \code{type = "prob"}, 
\code{group = c(-1, 0)} and providing the value \code{0} to \eqn{z_{1i}} 
through the \code{exogenous} argument. Third, take the average value of 
\eqn{p_{1i}-p_{0i}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitted.R
\name{fitted.msel}
\alias{fitted.msel}
\title{Extract Model Fitted Values}
\usage{
\method{fitted}{msel}(object, ..., newdata = NULL)
}
\arguments{
\item{object}{object of class 'msel'.}

\item{...}{further arguments (currently ignored).}

\item{newdata}{an optional data frame in which to look for variables 
with which to predict. If omitted, the original data frame is used. 
This data frame should contain values of dependent variables even if they 
are not actually needed for prediction (simply assign them with 0 values).}
}
\value{
Returns a numeric matrix. Its columns whose names coincide with the
names of the ordinal and multinomial equations provide the index of the most 
probable category for each observation.
Columns whose names coincide with the names of the continuous equations 
provide conditional expectations of the dependent variables in observable 
regimes for each observation.
}
\description{
Extracts fitted values from 'msel' object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formula.R
\name{formula.msel}
\alias{formula.msel}
\title{Formulas of msel model.}
\usage{
\method{formula}{msel}(x, ..., type = "formula", eq = NULL)
}
\arguments{
\item{x}{object of class 'msel'.}

\item{...}{further arguments (currently ignored).}

\item{type}{character; 
if \code{type = "formula"} or \code{type = 1} then function returns formulas 
of the ordinal equations. 
If \code{type = "formula2"} or \code{type = 2} then function returns formulas 
of the continuous equations.
If \code{type = "formula3"} or \code{type = 3} then function returns formula 
of the multinomial equation.}

\item{eq}{positive integer representing the index of the equation which
formula should be returned. If \code{NULL} (default) then formulas for each
equation will be returned as a list which \code{i}-th element associated 
with \code{i}-th equation.}
}
\value{
Returns a formula or a list of formulas depending on \code{eq} value.
}
\description{
Provides formulas associated with the 
object of class 'msel'.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrtest.R
\name{summary.lrtest_msel}
\alias{summary.lrtest_msel}
\title{Summary Method for Likelihood Ratio Test}
\usage{
\method{summary}{lrtest_msel}(object, ...)
}
\arguments{
\item{object}{object of class "lrtest_msel"}

\item{...}{further arguments (currently ignored)}
}
\value{
Returns an object of class 'summary.lrtest_msel'.
}
\description{
Provides summary for an object of class 'lrtest_msel'.
}
\details{
This function just changes the class of the 'lrtest_msel'
object to 'summary.lrtest_msel'.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{print.msel}
\alias{print.msel}
\title{Print for an Object of Class msel}
\usage{
\method{print}{msel}(x, ...)
}
\arguments{
\item{x}{object of class 'msel'}

\item{...}{further arguments (currently ignored)}
}
\value{
The function returns \code{NULL}.
}
\description{
Prints information on the object of class 'msel'.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/msel.R
\name{msel}
\alias{msel}
\title{Multivariate and multinomial sample selection and endogenous switching models
with multiple outcomes.}
\usage{
msel(
  formula = NA,
  formula2 = NA,
  formula3 = NA,
  data = NULL,
  groups = NA,
  groups2 = NA,
  groups3 = NA,
  marginal = list(),
  opt_type = "optim",
  opt_args = NA,
  start = NULL,
  estimator = "ml",
  cov_type = "mm",
  degrees = NA,
  degrees3 = NA,
  n_sim = 1000,
  n_cores = 1,
  control = list(),
  regularization = list(),
  type3 = "logit",
  cluster = NA
)
}
\arguments{
\item{formula}{a list which i-th element is an object of class 
\code{"formula"} describing the form of the linear predictor (index) of the  
i-th ordinal equation. Mean and variance equations should be separated 
by the '|' symbol.}

\item{formula2}{a list which i-th element is an object of class 
\code{"formula"} describing the form of the linear predictor (index) of the 
i-th continuous equation.}

\item{formula3}{an object of class \code{"formula"} describing the form of 
the linear predictor (index) of the multinomial equation.}

\item{data}{a data frame containing the variables in the model.}

\item{groups}{a matrix which (i, j)-th element is the j-th ordinal category
(value starting from 0) of the i-th dependent ordinal variable. Each row of 
this matrix describes observable (in data) combination of categories -
values of the ordinal dependent variables i.e., from the left hand side of
\code{formula}. Special category \code{'-1'} means that variable in the
j-th column is unobservable when other dependent variables have particular 
values i.e., given in the same row. See 'Details' for additional information.}

\item{groups2}{the same as the \code{groups} argument but for the continuous
dependent variables from \code{formula2}. See 'Details' for additional 
information.}

\item{groups3}{the same as the \code{groups} argument but for the multinomial
dependent variable from \code{formula3}. See 'Details' for additional 
information.}

\item{marginal}{a list such that \code{marginal[[i]]} represents parameters 
of the marginal distribution of the random error of the \code{i}-th ordered 
equation and \code{names(marginal)[i]} is a name of this distribution. 
Marginal distributions are the same as in \code{\link[mnorm]{pmnorm}}.}

\item{opt_type}{a character representing the optimization function to be 
used. If \code{opt_type = "optim"} then \code{\link[stats]{optim}} will be 
used. If \code{opt_type = "gena"} then \code{\link[gena]{gena}} will be 
applied i.e., a genetic algorithm. If \code{opt_type = "pso"} then 
\code{\link[gena]{pso}} will be used i.e., a particle swarm optimization.}

\item{opt_args}{a list of input arguments for the optimization function
selected via the \code{opt_type} argument. See 'Details' for additional 
information.}

\item{start}{a numeric vector of initial parameters' values. It will be used
as a starting point for the optimization purposes. It is also possible to
provide an object of class \code{'msel'}; then its \code{'par'} element will 
be used as a starting point.}

\item{estimator}{a character determining estimation method.
If \code{estimator = "ml"} then maximum-likelihood method will be used.
If \code{estimator = "2step"} then two-step estimation procedure similar
to Heckman's method will be applied. 
See 'Details' for additional information.}

\item{cov_type}{a character determining the type of the covariance matrix 
estimate to be returned. First, suppose that \code{estimator = "ml"} then the
following estimators are available. If \code{cov_type = "hessian"} then 
negative inverse of Hessian matrix will be applied. If 
\code{cov_type = "gop"} then inverse of Jacobian outer products will be used.
If \code{cov_type = "sandwich"} then sandwich covariance matrix
estimator will be applied. 
If \code{cov_type = "no"} then a matrix of \code{NA} values will be used. 
If \code{cov_type = "mm"} (default) then sandwich estimator is 
used along with the penalized log-likelihood function. Therefore if there is 
no regularization then \code{"mm"} and \code{"sandwich"} estimators are the
same. Second, suppose that \code{estimator = "2step"}. Then only available
options are \code{cov_type = "mm"} and \code{cov_type = "no"}.}

\item{degrees}{a vector of non-negative integers such that \code{degrees[i]}
represents the degree of the polynomial whose elements are selectivity 
terms associated with the \code{i}-th ordered equation. See 'Details' for 
additional information.}

\item{degrees3}{a vector of non-negative integers such that 
\code{degrees3[i]} represents the degree of the polynomial whose elements 
are selectivity terms associated with the \code{i}-th multinomial equation. 
See 'Details' for additional information.}

\item{n_sim}{an integer representing the number of GHK draws when there are
more than 3 ordered equations. Otherwise alternative (much more efficient) 
algorithms will be used to calculate multivariate normal probabilities.}

\item{n_cores}{a positive integer representing the number of CPU cores used 
for the parallel computing. If possible it is highly recommended to set it 
equal to the number of available physical cores.}

\item{control}{a list of control parameters. See 'Details' for additional 
information.}

\item{regularization}{a list of control parameters for regularization.
Element \code{ridge_ind} is a vector of indexes of parameters subject to 
regularization according to the quadratic (ridge) penalty function.
These indexes correspond to parameters from \code{par} output element. 
Set \code{show_ind} argument of \code{\link[switchSelection]{summary.msel}} 
to \code{TRUE} to see these indexes.
Element \code{ridge_scale} is a numeric vector of weights of the ridge
penalty function. 
Element \code{ridge_location} is a numeric vector of values to be subtracted
from the parameters before they pass into the penalty function.
Elements \code{lasso_ind}, \code{lasso_scale} and \code{lasso_location} are
the same but for the lasso penalty term.}

\item{type3}{a character determining the type of the multinomial model to be
considered. If \code{type3 = "logit"} then multinomial logit model will
be used. If \code{type3 = "probit"} then multinomial probit model
will be applied. See 'Details' for additional information.}

\item{cluster}{an object of class \code{"formula"} describing the variables 
which interactions determine the clusters used to estimate the asymptotic 
covariance matrix when \code{cov_type = "sandwich"} or 
\code{cov_type = "mm"}. Alternatively this argument may be specified as a 
character vector of these variables.}
}
\value{
This function returns an object of class \code{"msel"}. 
It is a list which may contain the following elements:
\itemize{
\item \code{par} - a vector of the estimates of the parameters.
\item \code{estimator} - the same as the \code{estimator} input argument.
\item \code{type3} - the same as the \code{type3} input argument.
\item \code{formula} - the same as \code{formula} input argument but all
elements are coerced to \code{"formula"} type.
\item \code{formula2} - the same as \code{formula2} input argument but all
elements are coerced to \code{"formula"} type.
\item \code{formula3} - the same as \code{formula3} input argument but all
elements are coerced to \code{"formula"} type.
\item \code{model1} - an object of class \code{"msel"} with the first step 
estimation results.
\item \code{data} - the same as \code{data} input argument but
without missing (at random) values.
\item \code{W_mean} - a list such that \code{W_mean[[j]]} is a matrix of the 
regressors \eqn{w_{ji}} of the mean equation of \eqn{z_{ji}^{*}}.
\item \code{W_var} - a list such that \code{W_var[[j]]} is a matrix of the 
regressors \eqn{w_{ji}^{*}} of the variance equation of \eqn{z_{ji}^{*}}.
\item \code{X} - a list such that \code{X[[v]]} is a numeric matrix of
regressors \eqn{x_{i}^{(v)}} of the \code{v}-th continuous equation
\eqn{y_{i}^{(v)}}.
\item \code{W_mn} - a matrix of the regressors \eqn{\tilde{w}_{i}} of the 
multinomial equation of \eqn{\tilde{z}_{ji}^{*}}.
\item \code{dependent} - a numeric matrix which \eqn{j}-th column 
\code{dependent[, j]} is a vector of the ordinal dependent variable
\eqn{z_{ji}^{(o)}} values.
\item \code{dependent2} - a numeric matrix which \eqn{v}-th column 
\code{dependent2[, v]} is a vector of the continuous dependent variable
\eqn{y_{i}^{(v)}} values.
\item \code{dependent3} - a numeric vector of values of the multinomial
dependent variable \eqn{\tilde{z}_{j}}.
\item \code{groups} - the same as \code{groups} input argument or 
automatically generated matrix representing the structure of the system 
of equations. Please, see 'Details' section above for more information.
\item \code{groups2} - the same as \code{groups2} input argument or 
automatically generated matrix representing the structure of the system 
of equations. Please, see 'Details' section above for more information.
\item \code{groups3} - the same as \code{groups3} input argument or 
automatically generated matrix representing the structure of the system 
of equations. Please, see 'Details' section above for more information.
\item \code{marginal} - the same as \code{marginal} input argument.
\item \code{ind} - a list containing some indexes partition of the 
model (not intended for the users).
\item \code{start} - the same as the \code{start} input argument.
\item \code{twostep} - a list such that \code{twostep[[v]][[r + 1]]} is an
object of class \code{"lm"} associated with the second step estimates
of the \code{v}-th equation in the regime \code{r}.
\item \code{y_pred} - a numeric matrix such that \code{y_pred[, v]} is a 
second step prediction of the \eqn{y_{i}^{(v)}}.
\item \code{coef} - a list which \eqn{j}-th element \code{coef[[j]]} is 
a numeric vector representing \eqn{\hat{\gamma}_{j}}.
\item \code{coef_var} - a list which \eqn{j}-th element \code{coef_var[[j]]} 
is a numeric vector representing \eqn{\hat{\gamma}_{j}^{*}}.
\item \code{cuts} - a list which \eqn{j}-th element \code{cuts[[j]]} is 
a numeric vector representing \eqn{\hat{c}_{j}}.
\item \code{coef2} - a list of numeric matrices such that 
\code{coef2[[v]][, r + 1]} is a numeric vector representing
\eqn{\hat{\beta}_{r^{(v)}}^{(v)}}.
\item \code{sigma} - a numeric matrix such that \code{sigma[j, t]} is a 
numeric value representing \eqn{\widehat{Cov}(u_{ji}, u_{ti})}.
\item \code{var2} - a list of numeric vectors such that 
\code{var2[[v]][r + 1]} represents 
\eqn{\widehat{Var}(\varepsilon_{ri}^{(v)})}.
\item \code{cov2} - a list of numeric matrices which element 
\code{sigma2[[v]][j, r + 1]} represents
\eqn{\widehat{Cov}(u_{ji}, \varepsilon_{ri}^{(v)})}.
\item \code{sigma2} - a list of numeric matrices representing the estimates
of the covariances between random errors of the continuous equations in 
different regimes 
\eqn{\widehat{Cov}(\varepsilon_{r^{(v)}i}^{(v)}, 
\varepsilon_{r^{(t)}i}^{(t)})}.
\item \code{marginal_par} - a list such that \code{marginal_par[[j]]} is a 
numeric vector of estimates of the parameters of the marginal distribution 
of \eqn{u_{ji}}.
\item \code{coef3} - a numeric matrix such that \code{coef3[j + 1, ]} is 
a numeric vector representing \eqn{\hat{\tilde{\gamma}}_{j}}.
\item \code{sigma3} - a numeric matrix such that \code{sigma3[j + 1, t + 1]} 
is a numeric value representing 
\eqn{\widehat{Cov}(\tilde{u}_{ji}, \tilde{u}_{ti})}.
\item \code{lambda} - a numeric matrix such that \code{lambda[i, j]} 
corresponds to \eqn{\hat{\lambda}_{ji}} of the ordinal equations.
\item \code{lambda_mn} - a numeric matrix such that \code{lambda_mn[i, j]} 
corresponds to \eqn{\hat{\lambda}_{ji}} of the multinomial equation.
\item \code{H} - if \code{estimator = "ml"} then \code{H} is a Hessian matrix 
of the log-likelihood function. If \code{estimator = "2step"} then \code{H}
is a numeric matrix of the derivatives of mean sample scores with respect 
to the estimated parameters.
\item \code{J} - if \code{estimator = "ml"} then \code{J} is a Jacobian 
matrix of the log-likelihood function. If \code{estimator = "2step"} then
\code{J} is a numeric matrix such that \code{J[, s]} is a vector of sample 
scores associated with the parameter \code{par[s]}.
\item \code{cov_type} - the same as \code{cov_type} input argument.
\item \code{cov} - an estimate of the asymptotic covariance matrix of 
the parameters' estimator.
\item \code{tbl} - a list of special tables used to create a summary 
(not intended for the users).
\item \code{se} - a numeric vector of standard errors of the estimates.
\item \code{p_value} - a numeric vector of the p-values of the tests on 
the significance of the estimated parameters where the null hypothesis is 
that corresponding parameter is zero.
\item \code{logLik} - the value of log-likelihood function at \code{par}.
\item \code{other} - a list of additional variables not intended for the 
users.
\item \code{cluster} - the same as \code{cluster} input argument.
}

It is highly recommended to get the estimates of the parameters via
\code{\link[switchSelection]{coef.msel}} function.
}
\description{
This function allows to estimate parameters of the 
multivariate and multinomial sample selection and endogenous switching models
with multiple outcomes. Both maximum-likelihood and two-step estimators are
implemented.
}
\details{
This function allows the estimation of multivariate and multinomial 
sample selection and endogenous switching models with multiple outcomes.
These models are the systems of ordinal, continuous and multinomial equations
described by \code{formula}, \code{formula2} and \code{formula3} 
respectively.

\strong{Ordinal equations}

Argument \code{formula} determines the regressors of the 
multivariate ordered probit model with the heteroscedastic random errors:
\deqn{z_{ji}^{*} = w_{ji}\gamma_{j} + \sigma_{ji}^{*}u_{ji},} 
\deqn{\sigma_{ji}^{*} = \exp(w_{ji}^{*}\gamma_{j}^{*}), \quad 
      u_{i}\sim N\left(\begin{bmatrix}0\\ \vdots\\ 0\end{bmatrix}, 
                       \Sigma\right), i.i.d.,}
\deqn{z_{ji}=\begin{cases}
                   0\text{, if }z_{ji}^{*}\leq c_{j1}\\
                   1\text{, if }c_{j1}<z_{ji}^{*}\leq c_{j2}\\
                   2\text{, if }c_{j2}<z_{ji}^{*}\leq c_{j3}\\
                   \vdots\\
                   m_{J-1}\text{, if }z_{(J-1)i}^{*}>c_{(J-1)m_{j-1}}\\
                   \end{cases},}
 \deqn{z_{i}=(z_{1i},...,z_{Ji})^{T},\quad
       u_{i} = (u_{1i},u_{2i},...,u_{Ji})^{T},}
\deqn{i\in\{1,2,...,n\},\quad j\in\{1,2,...,J\},}
where:
\itemize{
\item \eqn{n} - the number of observations. If there are no omitted 
observations then \eqn{n} equals \code{nrow(data)}.
\item \eqn{J} - the number of ordinal equations which equals 
\code{length(formula)}.
\item \eqn{z_{ji}^{*}} - an unobservable (latent) value of the \code{j}-th
dependent ordinal variable.
\item \eqn{z_{ji}} - an observable (ordinal) value of the \code{j}-th
dependent ordinal variable.
\item \eqn{m_{j}} - the number of categories of
\eqn{z_{ji}\in\{0,1,...,m_{j}\}}.
\item \eqn{c_{jk}} - the \eqn{k}-th cut (threshold) of the \code{j}-th
 dependent ordinal variable.
\item \eqn{w_{ji}} - the regressors of the \eqn{j}-th mean equation 
which should be described in \code{formula[[j]]}.
\item \eqn{\gamma_{j}} - the regression coefficients of the \eqn{j}-th 
mean equation.
\item \eqn{w_{ji}\gamma_{j}} - the linear predictor (index) of the \eqn{j}-th 
mean equation.
\item \eqn{u_{i}} - a multivariate normal random vector whose elements are
standard normal random variables.
\item \eqn{\Sigma} - a correlation matrix of \eqn{u_{i}} so
\eqn{\Sigma_{t_{1}t_{2}}=\text{corr}\left(u_{it_{1}}, u_{it_{2}}\right)}.
\item \eqn{\sigma_{ji}^{*}} - a heteroscedastic standard deviation.
\item \eqn{\sigma_{ji}^{*}u_{ji}} - the heteroscedastic random errors.
\item \eqn{w_{ji}^{*}} - the regressors of the \eqn{j}-th variance equation 
which should be described in \code{formula[[j]]} after '|' symbol.
\item \eqn{\gamma_{j}^{*}} - the regression coefficients of the \eqn{j}-th 
variance equation.
\item \eqn{w_{ji}^{*}\gamma_{j}^{*}} - the linear predictor (index) of the 
\eqn{j}-th variance equation.
}

Constant terms (intercepts) are excluded from the model for 
identification purposes. If \eqn{z_{ji}} is a binary variable then 
\eqn{-c_{j1}} may be interpreted as a constant term of the \eqn{j}-th ordinal
equation. If all \eqn{z_{ji}} are binary variables then the model becomes a
multivariate probit.

By default the joint distribution of \eqn{u_{i}} is multivariate normal.
However, by using the \code{marginal} argument, it is possible to consider 
the joint distribution that is determined by the Gaussian copula and possibly
non-normal marginal distributions. Specifically, \code{names(marginal)[i]} 
is the name of the marginal distribution of \eqn{u_{ji}} and 
\code{marginal[[i]]} is the number of parameters of this distribution.
The marginal distributions are the same as in \code{\link[mnorm]{pmnorm}}.

\strong{Multinomial equation}

Argument \code{formula3} determines the regressors of the multinomial 
equation:
\deqn{\tilde{z}_{ji}^{*} = \tilde{w}_{i}\tilde{\gamma}_{j} + \tilde{u}_{ji},
\qquad j\in\{0,1,...,\tilde{J} - 1\},}
\deqn{\tilde{z}_{i}=\underset{t\in\{0,...,\tilde{J}-1\}}{\text{argmax}}
\text{ }\tilde{z}_{ti}^{*}, \qquad 
\tilde{u}_{i} = (\tilde{u}_{0i},\tilde{u}_{1i},...,
\tilde{u}_{(\tilde{J}-1)i})^{T},}

where:
\itemize{
\item \eqn{\tilde{J}} - the number of the alternatives i.e., possible values
of the dependent variable of the multinomial equation.
\item \eqn{\tilde{z}_{ji}^{*}} - an unobservable (latent) value of the 
\code{j}-th alternative. Usually \eqn{\tilde{z}_{ji}^{*}} is interpreted as 
a utility of the \code{j}-th alternative.
\item \eqn{\tilde{z}_{i}} - a selected alternative i.e., one which provides 
the greatest utility \eqn{\tilde{z}_{ji}^{*}}.
\item \eqn{\tilde{w}_{i}} - the regressors of the multinomial equation
which should be described in \code{formula3} and are assumed to be the same 
for all the alternatives.
\item \eqn{\tilde{\gamma}_{j}} - the regression coefficients of the 
\eqn{j}-th alternative's equation.
\item \eqn{\tilde{w}_{i}\tilde{\gamma}_{j}} - the linear predictor (index) 
of the \eqn{j}-th alternative's equation.
\item \eqn{\tilde{u}_{i}} - a vector of random errors.
}

For identification purposes it is assumed that the regression 
coefficients of the last alternative are zero 
\eqn{\tilde{\gamma}_{\tilde{J} - 1} = (0,...,0)^{T}}.

The joint distribution of \eqn{\tilde{u}_{i}} depends on the value of the 
\code{type3} argument. If \code{type3 = "logit"} then a multinomial logit
model is considered. It assumes that \eqn{\tilde{u}_{ji}} are independent
and their marginal distribution is Gumbel (error value distribution).
If \code{type3 = "probit"} then a multinomial probit model is used, so 
it is assumed that the joint distribution of \eqn{\tilde{u}_{i}} is 
multivariate normal with zero mean and the covariance matrix 
\eqn{\tilde{\Sigma}}. For identification purposes it is also assumed 
that \eqn{\tilde{\Sigma}_{11} = 1} so \eqn{Var(\tilde{u}_{0i}) = 1}. 
In addition, \eqn{\tilde{u}_{(\tilde{J}-1)i}=0} which implies 
\eqn{\tilde{\Sigma}_{t(\tilde{J})}=\tilde{\Sigma}_{(\tilde{J})t}=0} for all 
\eqn{t\in\{0,...,\tilde{J}-1\}}.

\strong{Continuous equations}

Argument \code{formula3} determines the regressors of the continuous 
equations:
\deqn{y_{r^{(v)}i}^{(v)} = x_{i}^{(v)}\beta_{r^{(v)}}^{(v)} + 
\varepsilon_{r^{(v)}i}^{(v)},}
\deqn{r^{(v)}\in\{0,1,...,R^{(v)} - 1\}, \quad v\in \{1,...,V\},}

where:
\itemize{
\item \eqn{V} - the number of continuous equations.
\item \eqn{r^{(v)}} - the regime of the \eqn{v}-th continuous equation.
\item \eqn{y_{r^{(v)}i}^{(v)}} - the \eqn{r^{(v)}}-th potential outcome
(in the sense of the Neyman-Rubin framework) of the \eqn{v}-th continuous 
equation.
\item \eqn{R^{(v)}} - the number of potential outcomes of the 
\eqn{v}-th continuous equation.
\item \eqn{x_{i}^{(v)}} - the regressors of the \eqn{v}-th continuous 
equation. They are provided via \code{formula2[[v]]}.
\item \eqn{\beta_{r^{(v)}}^{(v)}} - regression coefficients of the \eqn{v}-th 
continuous equation in the regime \eqn{r^{(v)}}.
\item \eqn{\varepsilon_{r^{(v)}i}^{(v)}} - a random error of the \eqn{v}-th 
continuous equation in the regime \eqn{r^{(v)}}.
}

\strong{Estimation of the model with multivariate ordinal and 
multinomial equations}

If \code{formula2} is not provided then maximum-likelihood estimator is used 
with \code{estimator = "ml"} to estimate the parameters of the multivariate 
ordinal and multinomial equations.

If both \code{formula} and \code{formula3} are provided then parameters of
the multivariate ordered and multinomial equations are estimated under the
assumption that \eqn{u_{i}} is independent of \eqn{\tilde{u}_{i}}.
Therefore the estimates are identical to those obtained by separate
estimation of these models. We may relax the independence assumption
during future updates.

\strong{Estimation of the model with continuous equations}

If \code{formula} and \code{formula3} are not provided then it is assumed
that each continuous equation has only one regime so \eqn{R^{(v)} = 1} for
each \eqn{v\in\{1,...,V\}}.

If \code{estimator = "ml"} then the maximum-likelihood estimator is used 
under the assumption that the distribution of random errors is multivariate 
normal. If \code{estimator = "2step"} then the \eqn{V}-stage least squares 
estimator is used. In the latter case \eqn{v}-th equation is estimated by the
least squares estimator and for every \eqn{v_{0}} such that 
\eqn{v_{0} < v} if \eqn{y_{0i}^{(v_{0})}} is included in \eqn{x_{i}^{(v)}} 
then \eqn{y_{0i}^{(v_{0})}} is substituted with its estimate 
\eqn{\hat{y}_{0i}^{(v_{0})}} obtained on the \eqn{v_{0}}-th step.
Therefore if \eqn{v_{0} < v} then if some endogenous variable appears on the
left hand side of \code{formula2[[v]]} it should not appear on the right hand 
side of \code{formula2[[v0]]}.

\strong{Estimation of the model with ordinal outcomes and 
multivariate ordered selection}

Suppose that \eqn{z_{i}} represents the ordinal potential outcomes while 
the observable values are \eqn{z_{i}^{(o)} = g^{(1)}(z_{i})}, where the 
function \eqn{g^{(1)}(z_{i})} converts unobservable values of \eqn{z_{i}} to 
\eqn{-1}. Therefore \eqn{z_{ji}^{(o)}} instead of \eqn{z_{i}} appears on the 
left hand side of the \code{formula[[j]]}.

For example, consider a binary variable for the employment status 
(0 - unemployed, 1 - employed) and an ordinal variable \eqn{z_{2i}} 
(ranging from 0 to 2) for job satisfaction 
(0 - unsatisfied, 1- satisfied, 2 - highly satisfied). Then \eqn{z_{2i}} is 
observable only when \eqn{z_{1i}} equals 1 since job satisfaction is 
observable only for working individuals. Consequently \eqn{z_{2i}^{(o)}} 
should be equal to -1 (minus one) whenever \eqn{z_{1i}} equals 0:
\deqn{z_{i}^{(o)}=g^{(1)}(z_{i})=
\begin{cases}
(1, 2)\text{, if }z_{i} = (1, 2)\\
(1, 1)\text{, if }z_{i} = (1, 1)\\
(1, 0)\text{, if }z_{i} = (1, 0)\\
(0, -1)\text{, otherwise}\\
\end{cases}}

Rows of the matrix \code{groups} determine all possible values of the 
function \eqn{g^{(1)}(z_{i})}. In this particular example matrix 
\code{groups} should have the following form:
\deqn{\text{groups} =
\begin{bmatrix}
1 & 2\\
1 & 1\\
1 & 0\\
0 & -1                     
\end{bmatrix}.}

Notice that in this particular example it is necessary to ensure that
in \code{data} if \eqn{z_{1i}^{(o)}} equals 0 then \eqn{z_{2i}^{(o)}}
equals -1. Also in this case matrix \code{groups} will be created 
automatically so there is no need to provide it manually.

By using the \code{groups} argument it is straightforward to consider 
various other models with ordinal outcomes and multivariate ordered 
selection mechanisms.

If some values of the ordinal variables \eqn{z_{ji}} are missing (at random) 
i.e., take an \code{NA} value then the contribution of other ordinal 
dependent variables (for the i-th observation) still may be included into the 
likelihood function by manually substituting \code{NA} with -1 in the 
\code{data}. However, ensure that this particular (missing) \eqn{z_{ji}} is 
not a regressor for another dependent variable that may happen in the 
hierarchical systems.

It is useful to use the \code{groups} argument to consider causal inference
models. For example, suppose that \eqn{z_{1i}} represents a binary treatment 
variable for the university diploma (0 - no diploma, 1 - has diploma). Also, 
\eqn{z_{2i}} is a binary potential outcome for the employment status 
(0 - unemployed, 1 - employed) of the individual if she has university 
diploma. Finally, \eqn{z_{3i}} is a binary potential outcome for the 
employment status (0 - unemployed, 1 - employed) if the individual does not 
have a university diploma. Since \eqn{z_{2i}} is observable only if 
\eqn{z_{1i} = 1} and \eqn{z_{3i}} is observable only when \eqn{z_{2i} = 0} 
we get:
\deqn{z_{i}^{(o)} = 
\begin{cases}
(1, 1, -1)\text{, if }z_{1i} = 1\text{ and }z_{2i}=1\\
(1, 0, -1)\text{, if }z_{1i} = 1\text{ and }z_{2i}=0\\
(0, -1, 1)\text{, if }z_{1i} = 0\text{ and }z_{3i}=1\\
(0, -1, 0)\text{, if }z_{1i} = 0\text{ and }z_{3i}=0
\end{cases}.}

Therefore to estimate this model it is necessary to ensure that in 
\code{data} we have \eqn{z_{2i}^{(o)} = -1} when \eqn{z_{1i}^{(o)} = 0} and 
\eqn{z_{3i}^{(o)} = -1} if \eqn{z_{1i}^{(o)} = 1}. Also the \code{groups}
argument should include all possible values of \eqn{z_{i}^{(o)}}:
\deqn{\text{groups} =
\begin{bmatrix}
1 &  1 & -1\\
1 &  0 & -1\\
0 & -1 & 1\\
0 & -1 & 0                    
\end{bmatrix}.}

\strong{Estimation of the model with continuous outcomes and 
multivariate ordered selection}

To simplify the notation, suppose that there is only one continuous equation
with multiple regimes:

\deqn{y_{ri} = x_{i}\beta_{r} + \varepsilon_{ri},}
\deqn{y_{i} =
\begin{cases}
\text{unobservable, if }g^{(2)}\left(z_{i}^{(o)}\right) = -1\\
y_{0i}\text{, if }g^{(2)}\left(z_{i}^{(o)}\right) = 0\\
y_{1i}\text{, if }g^{(2)}\left(z_{i}^{(o)}\right) = 1\\
\vdots\\
y_{(R^{*}-1)i}\text{, if }g^{(2)}\left(z_{i}^{(o)}\right) = R^{*} - 1\\
\end{cases},}
\deqn{\varepsilon_{i} = (\varepsilon_{0i},\varepsilon_{1i},...,\varepsilon_{(R^{*}-1)i}),\quad
r\in\{0,1,...,R^{*} - 1\},}

where:
\itemize{
\item \eqn{y_{i}} - an observable continuous outcome.
\item \eqn{r} - the index of the potential outcome.
\item \eqn{R^{*}} - the number of the regimes.
\item \eqn{y_{ri}} - a continuous potential outcome i.e., the value of
\eqn{y_{i}} in the regime \eqn{r}.
\item \eqn{x_{i}} - the vector of regressors provided in \code{formula2}.
\item \eqn{\beta_{r}} - the regression coefficients in the \eqn{r}-th regime. 
\item \eqn{g^{(2)}(z_{i}^{(o)})} - a function determining which potential 
outcome is observable depending on the observable values of the ordinal 
variables \eqn{z_{i}^{(o)}}.
\item \eqn{\varepsilon_{i}} - a vector of random errors.
}

The value of the \code{groups2[i]} argument specifies the value 
of \eqn{g^{(2)}(z_{i}^{(o)})} when \eqn{z_{i}^{(o)}} equals 
\code{groups[i, ]}. The values of \eqn{y_{i}} in \code{data} such that
\eqn{g^{(2)}(z_{i}^{(o)})=-1} should be set to \code{NA}.

For example, consider a system of equations for wage \eqn{y_{i}},
employment status \eqn{z_{1i}} (0 - unemployed, 1 - employed)
and job satisfaction \eqn{z_{2i}} 
(0 - unsatisfied, 1- satisfied, 2 - highly satisfied). Notice that wage and
job satisfaction are observable only for working individuals. Also suppose
that wage is unobservable for unsatisfied workers and observable in different
regimes for other workers. Namely, for satisfied workers \eqn{z_{2i} = 1} we 
observe \eqn{y_{0i}} and for highly satisfied workers \eqn{z_{2i} = 2} we 
observe \eqn{y_{1i}}.

To estimate this model it is necessary to  manually specify the structure of 
the equations via the \code{groups} and \code{groups2} arguments by 
providing all possible combinations of the ordinal variables and the regimes 
of the continuous equation:

\deqn{\text{groups} =
\begin{bmatrix}
1 & 2\\
1 & 1\\
1 & 0\\
0 & -1                     
\end{bmatrix}, \text{groups2} =
\begin{bmatrix}
1\\
0\\
-1\\
-1                     
\end{bmatrix}.}

Notice that \code{groups2[1] = 1} indicates that when 
\code{groups[1, ] = c(1, 2)} i.e. \eqn{z_{1i} = 1} and \eqn{z_{2i} = 2} 
we observe \eqn{y_{i}} in regime \code{1} corresponding to the wage of highly 
satisfied workers \eqn{y_{1i}}. 
Similarly \code{groups2[2] = 0} indicates that when 
\code{groups[2, ] = c(1, 1)} i.e., \eqn{z_{1i} = 1} and \eqn{z_{2i} = 1} we 
observe \eqn{y_{i}} in the regime \code{0} corresponding to the wage of the
satisfied workers  \eqn{y_{0i}}. Also, \code{groups2[3] = -1} means that when 
\code{groups[3, ] = c(1, 0)} i.e., \eqn{z_{1i} = 1} and \eqn{z_{2i} = 0} we
do not observe the wage of the worker \eqn{y_{i}} since he is unsatisfied.
Finally, \code{groups2[4] = -1} means that when 
\code{groups[4, ] = c(0, -1)} i.e., \eqn{z_{1i} = 0} and 
\eqn{z_{2i}^{(o)} = -1} we do not observe the wage of the worker 
\eqn{y_{i}} since he is unemployed.

If the joint distribution of \eqn{\varepsilon_{i}} and \eqn{u_{i}} is 
multivariate normal then according to Kossova and Potanin (2018):
\deqn{y_{ri}=x_{i}\beta_{r}+\sum\limits_{j=1}^{J} 
\Sigma^{(12)}_{j(r+1)}\lambda_{ji}+\varepsilon_{ri}^{*},}
where:
\deqn{\varepsilon_{ri}^{*} = \varepsilon_{ri} - 
E(\varepsilon_{i}|z_{1i},...z_{Ji})=\varepsilon_{ri} - 
\sum\limits_{j=1}^{J} \Sigma^{(12)}_{j(r+1)}\lambda_{ji},}
\deqn{\lambda_{ji}=
\lambda_{ji}^{(1)} + \lambda_{ji}^{(2)},\qquad 
\Sigma^{(12)}_{j(r+1)} = cov(u_{ji}, \varepsilon_{r+1}),}
\deqn{\lambda_{ji}^{(1)}=
\begin{cases}
0\text{, if }z_{ji} = 0\\
-\frac{\partial \ln P_{i}^{*}}{\partial a_{ji}}\text{, otherwise }
\end{cases},
\qquad
\lambda_{ji}^{(2)}=
  \begin{cases}
0\text{, if }z_{ji} = m_{j} - 1\\
-\frac{\partial \ln P_{i}^{*}}{\partial b_{ji}}\text{, otherwise }
\end{cases},}
\deqn{P_{i}^{*}(a_{1i},...,a_{Ji};b_{1i},...,b_{Ji})=
P(a_{1i}\leq u_{1i}\leq b_{1i},....,a_{Ji}\leq u_{Ji}\leq b_{Ji}),}
\deqn{a_{ji}=
\begin{cases}
-\infty\text{, if }z_{ji} = 0\\
\frac{c_{jz_{ji}}-w_{ji}\gamma_{j}}{\sigma_{ji}^{*}}\text{, otherwise }
\end{cases},
\qquad 
b_{ji}=
  \begin{cases}
\infty\text{, if }z_{ji} = m_{j} - 1\\
\frac{c_{j(z_{ji}+1)}-w_{ji}\gamma_{j}}{\sigma_{ji}^{*}}\text{, otherwise }
\end{cases}.}

Notice that the regression equation has selectivity terms \eqn{\lambda_{ji}}
which may be correlated with \eqn{x_{i}}. Therefore if random errors
\eqn{u_{i}} and \eqn{\varepsilon_{i}} are correlated, the least squares 
estimator of \eqn{x_{i}} on \eqn{y_{ri}} is inconsistent. 
To get consistent estimates of \eqn{\beta_{r}} it is possible to use 
maximum-likelihood \code{estimator = "ml"} or two-step (method of moments) 
\code{estimator = "2step"} estimator.

If the two-step estimator is used then on the first step \eqn{\lambda_{ji}}
are estimated as the functions of the estimates of the multinomial 
heteroscedastic ordered probit model. On the second step least squares 
regression of \eqn{y_{ri}} on \eqn{x_{i}} and the first step estimates 
\eqn{\hat{\lambda}_{ji}} is used to estimate \eqn{\beta_{r}} and 
\eqn{\Sigma^{(12)}_{j(r+1)}}.

If the joint distribution of random errors \eqn{\varepsilon_{i}}, \eqn{u_{i}} 
is not multivariate normal then \eqn{\lambda_{ji}} terms may enter the 
continuous equation non-linearly. Following Newey (2009) and 
E. Kossova, L. Kupriianova, and B. Potanin (2020) it is assumed that:
\deqn{y_{ri}= x_{i}\beta_{r} + \zeta_{i}\tau_{r} + \varepsilon_{i}^{*},}
where \eqn{\tau_{r}} is a \eqn{n_{\lambda}}-dimensional column vector and:
\deqn{\zeta_{i} = (\zeta_{1}(\lambda_{i}),...,
\zeta_{n_{\lambda}}(\lambda_{i})),\qquad \lambda_{i} = (\lambda_{1i},...,
\lambda_{Ji}).}

Functions \eqn{\zeta_{t}(\lambda_{i})} are specified manually by the user in
the \code{formula2} inside \code{I()}. For example, to specify
\eqn{\zeta_{t}(\lambda_{i}) = \lambda_{1i}\times \lambda_{2i}} it is 
sufficient to have a term \code{I(lambda1 * lambda2)} in \code{formula2}. 
Notice that to avoid confusion, no variables in \code{data} should have 
the names containing \code{"lambda"}. Otherwise these variables will be 
dropped.

It is possible to specify \eqn{\zeta_{t}(\lambda_{i})} functions as a 
polynomial without interaction terms by using the \code{degrees} argument.
Specifically, if \code{degrees[j] = t} then \code{lambdaj}, 
\code{I(lambdaj ^ 2)},...,\code{I(lambdaj ^ t)} terms are added to
\code{formula2}. However, if the \code{degrees} argument is used then no 
functions of \code{lambdaj} should be provided manually in \code{formula2}.
Otherwise it will be assumed that \code{degrees} is a vector of zeros.
Also, if \code{estimator = "2step"}, there are no \code{lambdaj} terms in 
\code{formula2} and \code{degrees} is \code{NA}, then \code{degrees} will 
be converted to a vector of ones.

If there are multiple continuous equations then \code{formula2} should be 
a list of formulas. Further, if \code{estimator = "2step"} then the second 
step is a \eqn{V}-stage least squares estimator with \code{lambda} terms.
If they are provided via the \code{degrees} argument, then it should be a 
matrix whose \code{v}-th row corresponds to the \code{v}-th continuous 
equation.

For example, consider the previous example with additional continuous 
equation for working hours
\eqn{y_{i}^{(2)}} which does not vary with the satisfaction of the workers
\eqn{z_{2i}} but is observable only for the employed individuals 
\eqn{z_{1i} = 1}. To estimate the system with this additional continuous 
equation simply substitute all \eqn{y_{i}^{(2)}} (such that \eqn{z_{1i}=0}) 
in \code{data} with \code{NA} and specify:

\deqn{\text{groups} =
\begin{bmatrix}
1 & 2\\
1 & 1\\
1 & 0\\
0 & -1                     
\end{bmatrix}, \text{groups2} =
\begin{bmatrix}
1 & 0\\
0 & 0\\
-1 & 0\\
-1 & -1                    
\end{bmatrix}.}

Notice that \code{groups2[, 1]} describes the regimes of the wage equation 
\eqn{y_{i}^{(1)}} while \code{groups[, 2]} contains the regimes of the hours 
equation \eqn{y_{i}^{(2)}}. Note that the formula of the first equation
(wage) should be specified in \code{formula2[[1]]} and the formula of the 
second equation should be provided via \code{formula2[[2]]} i.e., as the
first and the second elements in a \code{formula2} list correspondingly.

If the \code{marginal} argument is used, then aforementioned formula of 
\eqn{\lambda_{ji}} is slightly modified to address the non-normal 
marginal distribution of \eqn{u_{ji}}.

\strong{Estimation of the model with continuous outcomes and multinomial 
selection}

The only difference with the previous model is that the observable value of
the continuous equation depends on the value of the multinomial equation
described in \code{formula3}. Therefore \eqn{g^{(2)}(z_{i}^{(o)})} is 
substituted with \eqn{g^{(3)}(\tilde{z}_{i})}. Also \code{groups3} argument 
instead of \code{groups} is used. Since there is only a single multinomial 
equation argument \code{groups3} is a vector.
If \code{groups3[k] = q} and \code{groups2[k, v] = r} then
\eqn{\tilde{z}_{i} = q} implies \eqn{y_{i}^{(v)} = y_{ri}^{(v)}}. 
Recall that a special value \code{r = -1} implies that \eqn{y_{i}^{(v)}} is
unobservable.

Only the two-step \code{estimator = "2step"} estimator of this model is 
available, which is similar to the one described above. The only difference 
is the formula used to estimate selectivity terms. If \code{type = "logit"}, 
then the two-step estimator of Dubin-McFadden (1984) is used, so selectivity 
terms are as follows:

\deqn{\lambda_{ji} = 
\begin{cases}
-\ln P(\tilde{z}_{i} = j)\text{, if }\tilde{z}_{i} = j\\
\frac{P(\tilde{z}_{i} = j)\ln P(\tilde{z}_{i} = j)}{1 - 
P(\tilde{z}_{i} = j)}\text{, otherwise}
\end{cases},
}
where \eqn{j\in\{0,...,\tilde{J} - 1\}} and:
\deqn{P(\tilde{z}_{i} = j) = 
\begin{cases}
\frac{e^{(\tilde{w}_{i}\tilde{\gamma}_{j})}}{1 + 
\sum\limits_{q=0}^{\tilde{J} - 2}e^{(\tilde{w}_{i}\tilde{\gamma}_{q})}}
\text{, if }j\ne \tilde{J} - 1\\
\frac{1}{1 + 
\sum\limits_{q=0}^{\tilde{J} - 2}e^{(\tilde{w}_{i}\tilde{\gamma}_{q})}}
\text{, otherwise}\\
\end{cases}.}

If \code{type = "probit"}, then the two-step estimator of Kossova and 
Potanin (2022) is used with the selectivity terms of the form:

\deqn{\lambda_{ji} = \left(A^{(\tilde{z}_{i})}\lambda_{i}^{*}\right)_{j},}

where \eqn{j\in\{0,...,\tilde{J} - 2\}} and:

\deqn{A^{(j)}_{t_{1}t_{2}} = 
\begin{cases}
1\text{, if }t_{1} = j + 1\\
-1\text{, if }t_{1} < j + 1\text{ and }t_{1} = t_{2}\\
-1\text{, if }t_{1} > j + 1\text{ and }t_{1} = t_{2} + 1\\
0\text{, otherwise }
\end{cases},
}
\deqn{t_{1},t_{2}\in\{1,...,\tilde{J} - 1\},}
\deqn{\lambda_{i}^{*} = 
\nabla \ln P\left(\tilde{z}_{i}\right) = 
      \nabla \ln F_{\tilde{u}^{(ji)}}\left(\tilde{z}_{1}^{(ji)},...,
      \tilde{z}_{\tilde{J}-1}^{(ji)}\right)=
      \left(\lambda_{1i}^{*},...,\lambda_{(\tilde{J}-1)i}^{*}  \right)^{T},
}
\deqn{
\tilde{u}^{(ji)} = 
\left(\tilde{u}_{0i}-\tilde{u}_{ji},\tilde{u}_{1i}-\tilde{u}_{ji},...,
      \tilde{u}_{(j-1)i}-\tilde{u}_{ji},\tilde{u}_{(j+1)i}-\tilde{u}_{ji},...,
      \tilde{u}_{(\tilde{J} - 1)i}-\tilde{u}_{ji}\right),
}
\deqn{
\tilde{z}^{(ji)} = \tilde{w}_{i}\left(
   \left(\tilde{\gamma}_{j}-\tilde{\gamma}_{0}\right), 
   \left(\tilde{\gamma}_{j}-\tilde{\gamma}_{1}\right),...,
   \left(\tilde{\gamma}_{j}-\tilde{\gamma}_{j-1}\right),
   \left(\tilde{\gamma}_{j}-\tilde{\gamma}_{j+1}\right),...,
   \left(\tilde{\gamma}_{j}-\tilde{\gamma}_{\tilde{J} - 1}\right)\right).}

Probabilities \eqn{P(z_{i})} are calculated by using a cumulative 
distribution function of the multivariate normal distribution with zero mean
and the covariance matrix of \eqn{\tilde{u}^{(ji)}}.

In the \code{formula2}, selectivity terms associated with the 
multinomial equation should be named \code{lambdaj_mn} instead of 
\code{lambdaj}. Argument \code{degrees3} is similar to \code{degrees}.

Consider a simple example of this model. Suppose that \eqn{\tilde{z}_{i}}
is a multinomial variable for the employment status 
(0 - unemployed, 1 - working in IT sector, 2 - working in other sector). 
Wage \eqn{y_{i}} is unobservable for the unemployed \eqn{\tilde{z}_{i} = 0}, 
equals \eqn{y_{0i}} in the IT sector \eqn{\tilde{z}_{i} = 1}, and equals 
\eqn{y_{1i}} in other sectors \eqn{\tilde{z}_{i} = 2}. 
To estimate this model set \code{groups3 = (0, 1, 2)} and 
\code{groups2 = (-1, 0, 1)}. Then substitute all \eqn{y_{i}} such that 
\eqn{\tilde{z}_{i} = 0} with \code{NA}.

\strong{Estimation of the model with continuous outcomes and mixed selection}

It is possible to consider the model with continuous outcomes and both 
multinomial and ordinal selection equations. Recall that it is assumed that
random errors of the ordered and multinomial equations are independent. 
Therefore if \code{formula} and \code{formula3} are provided then both 
\code{lambdaj} and \code{lambdaj_mn} are included in \code{formula2}. Only 
the two-step estimator \code{estimator = "2step"} is available for 
this model.

\strong{Missing values}

If any of the left hand side variables (regressors) of \code{formula[[j]]}
is missing then the right hand side variable of \code{formula[[j]]} will
be set to \code{NA} in \code{data}. Similar is true for \code{formula2}
and \code{formula3}.

\strong{Additional information}

Functions \code{\link[mnorm]{pmnorm}} and \code{\link[mnorm]{dmnorm}} are 
used internally for calculation of multivariate normal probabilities, 
densities and their derivatives.

Currently \code{control} has no input arguments intended for the users. 
This argument is used for some internal purposes of the package.

Optimization always starts with \code{\link[stats]{optim}}. If
\code{opt_type = "gena"} or \code{opt_type = "pso"} then 
\code{\link[gena]{gena}} or \code{\link[gena]{pso}} is used
to proceed with optimization starting from the initial point provided by 
\code{optim}. Manual arguments to \code{\link[stats]{optim}}
should be provided in the form of a list through \code{opt_args$optim}.
Similarly \code{opt_args$gena} and \code{opt_args$pso} provide manual
arguments to \code{\link[gena]{gena}} and \code{\link[gena]{pso}}.
For example, to provide the Nelder-Mead optimizer to 
\code{\link[stats]{optim}} and
restrict the number of genetic algorithm iterations to \eqn{10} make
\code{opt_args = list(optim = list(method = "Nelder-Mead"), 
gena = list(maxiter = 10))}.

If \code{estimator = "2step"} then it is possible to precalculate the first 
step model with \code{\link[switchSelection]{msel}} function
and pass it through the \code{formula} argument. It allows one to experiment
with various \code{formula2} and \code{degrees} specifications without
extra computational burden associated with the first step estimation.

If \code{estimator = "2step"} then the method of moments estimator of the 
asymptotic covariance matrix is used as described in 
Meijer and Wansbeek (2007).
}
\examples{
\donttest{
# ---------------------------------------
# CPS data example
# ---------------------------------------

# Set seed for reproducibility
set.seed(123)

# Load data
data(cps)

# Prepare the variable for education
cps$educ                    <- NA
cps$educ[cps$basic == 1]    <- 0
cps$educ[cps$bachelor == 1] <- 1
cps$educ[cps$master == 1]   <- 2

# Labor supply (probit) model
f_work <- work ~ age + I(age ^ 2) + bachelor + master + health + 
                 slwage + nchild
model1 <- msel(f_work, data = cps)
summary(model1)
    
# Education choice (ordered probit) model
f_educ <- educ ~ age + I(age ^ 2) + sbachelor + smaster
model2 <- msel(f_educ, data = cps)
summary(model2)   

# Education choice (multinomial logit) model
model3 <- msel(formula3 = f_educ, data = cps, type3 = "logit")
summary(model3)  

# Education choice (multinomial probit) model
model4 <- msel(formula3 = f_educ, data = cps, type3 = "probit")
summary(model4) 

# Labor supply with endogenous ordinal education 
# treatment (recursive or hierarchical ordered probit) model
model5 <- msel(list(f_work, f_educ), data = cps)  
summary(model5)         

# Sample selection (on employment) Heckman's model
f_lwage                  <- lwage ~ age + I(age ^ 2) + 
                                    bachelor + master + health
model6                   <- msel(f_work, f_lwage, data = cps)
summary(model6)

# Ordinal endogenous education treatment with non-random 
# sample selection into employment
model7 <- msel(list(f_work, f_educ), f_lwage, data = cps)
summary(model7)
 
# Ordinal endogenous switching on education model with 
# non-random selection into employment
groups   <- cbind(c(1, 1, 1, 0, 0, 0),
                  c(0, 1, 2, 0, 1, 2))
groups2  <- matrix(c(0, 1, 2, -1, -1, -1), ncol = 1)
f_lwage2 <- lwage ~ age + I(age ^ 2) + health
model8   <- msel(list(f_work, f_educ), f_lwage2,
                      groups = groups, groups2 = groups2,
                      data   = cps)
summary(model8)

# Multinomial endogenous switching on education model with 
# non-random selection into employment
groups  <- matrix(c(1, 1, 1, 0, 0, 0), ncol = 1)
groups2 <- matrix(c(0, 1, 2, -1, -1, -1), ncol = 1)
groups3 <- c(0, 1, 2, 0, 1, 2)
model9  <- msel(f_work, f_lwage2, f_educ,
                groups    = groups,  groups2 = groups2, 
                groups3   = groups3, data    = cps,
                estimator = "2step")
summary(model9)
}

# ---------------------------------------
# Simulated data example 1
# Ordered probit and other univariate
# ordered choice models
# --------------------------------------

# ---
# Step 1
# Simulation of the data
# ---

# Load required package
library("mnorm")

# Set seed for reproducibility
set.seed(123)

# The number of observations
n <- 1000

# Regressors (covariates)
w1 <- runif(n = n, min = -1, max = 1)
w2 <- runif(n = n, min = -1, max = 1)

# Random errors
u <- rnorm(n = n, mean = 0, sd = 1)

# Coefficients
gamma <- c(-1, 2)

# Linear predictor (index)
li <- gamma[1] * w1 + gamma[2] * w2

# Latent variable
z_star <- li + u

# Cuts
cuts <- c(-1, 0.5, 2)

# Observable ordinal outcome
z                                           <- rep(0, n)
z[(z_star > cuts[1]) & (z_star <= cuts[2])] <- 1
z[(z_star > cuts[2]) & (z_star <= cuts[3])] <- 2
z[z_star > cuts[3]]                         <- 3
table(z)

# Data
data <- data.frame(w1 = w1, w2 = w2, z = z)

# ---
# Step 2
# Estimation of the parameters
# ---

# Estimation
model <- msel(z ~ w1 + w2, data = data)
summary(model)
\donttest{
# Compare the estimates and true values of the parameters
  # regression coefficients
gamma_est <- coef(model, type = "coef", eq = 1)
cbind(true = gamma, estimate = gamma_est)
  # cuts
cuts_est <- coef(model, type = "cuts", eq = 1)
cbind(true = cuts, estimate = cuts_est)   

# ---
# Step 3
# Estimation of the probabilities and marginal effects
# ---

# Predict conditional probability of the dependent variable
# equals 2 for every observation in a sample.
# P(z = 2 | w)
prob <- predict(model, group = 2, type = "prob")
head(prob)

# Calculate mean marginal effect of w2 on P(z = 1 | w)
mean(predict(model, group = 1, type = "prob", me = "w2"))

# Calculate probabilities and marginal effects
# for manually provided observations.
  # new data
newdata <- data.frame(z = c(1, 1), 
                      w1 = c(0.5, 0.2), 
                      w2 = c(-0.3, 0.8))
  # probability P(z = 2 | w)
predict(model, group = 2, type = "prob", newdata = newdata)
  # linear predictor (index)
predict(model, type = "li", newdata = newdata)  
  # marginal effect of w1 on P(z = 2 | w)
predict(model, group = 2, type = "prob", newdata = newdata, me = "w1")
  # marginal effect of w1 and w2 on P(z = 3 | w)
predict(model, group   = 3,       type = "prob", 
               newdata = newdata, me   = c("w1", "w2"))
  # marginal effect of w2 on the linear predictor (index)
predict(model, group = 2, type = "li", newdata = newdata, me = "w2")
  # discrete marginal effect:
  # P(z = 2 | w1 = 0.5, w2) - P(z = 2 | w1 = 0.2, w2)
predict(model, group = 2,    type = "prob", newdata = newdata, 
               me    = "w2", eps  = c(0.2, 0.5))

# ---
# Step 4
# Ordered logit model
# ---

# Estimate ordered logit model with a unit variance
# that is just a matter of reparametrization i.e.,
# does not affect signs and significance of the coefficients
# and does not affect at all the marginal effects
logit <- msel(z ~ w1 + w2, data = data, marginal = list("logistic" = 0))
summary(logit)

# Compare ordered probit and ordered logit models
# using Akaike and Bayesian information criteria
  # AIC
c(probit = AIC(model), logit = AIC(logit))
  # BIC
c(probit = BIC(model), logit = BIC(logit))

# Estimate some probabilities and marginal effects
  # probability P(z = 1 | w)
predict(logit, group = 1, type = "prob", newdata = newdata)
  # marginal effect of w2 on P(z = 1 | w)
predict(logit, group = 1, type = "prob", newdata = newdata, me = "w2")

# ---
# Step 5
# Semiparametric ordered choice model with 
# Gallant and Nychka distribution
# ---

# Estimate semiparametric model
pgn <- msel(z ~ w1 + w2, data = data, marginal = list("PGN" = 2))
summary(pgn)

# Estimate some probabilities and marginal effects
  # probability P(z = 3 | w)
predict(pgn, group = 3, type = "prob", newdata = newdata)
  # marginal effect of w2 on P(z = 3 | w)
predict(pgn, group = 3, type = "prob", newdata = newdata, me = "w2")

# Test the normality assumption via the likelihood ratio test
summary(lrtest_msel(model, pgn))

# Test the normality assumption via the Wald test
test_fn <- function(object)
{
  marginal_par <- coef(object, type = "marginal", eq = 1)
  return(marginal_par)
}
test_result <- test_msel(object = pgn, test = "wald", fn = test_fn)
summary(test_result)
}
 
\donttest{
# ---------------------------------------
# Simulated data example 2
# Heteroscedastic ordered probit model
# ---------------------------------------

# Load required package
library("mnorm")

# ---
# Step 1
# Simulation of the data
# ---

# Set seed for reproducibility
set.seed(123)

# The number of observations
n <- 1000

# Regressors (covariates)
w1 <- runif(n = n, min = -1, max = 1)
w2 <- runif(n = n, min = -1, max = 1)
w3 <- runif(n = n, min = -1, max = 1)

# Random errors
u <- rnorm(n, mean = 0, sd = 1)

# Coefficients of the mean equation
gamma <- c(-1, 2)

# Coefficients of the variance equation
gamma_het <- c(0.5, -1)

# Linear predictor (index) of the mean equation
li <- gamma[1] * w1 + gamma[2] * w2

# Linear predictor (index) of the variance equation
li_het <- gamma_het[1] * w2 + gamma_het[2] * w3

# Heteroscedastic standard deviation
# i.e., value of the variance equation
sd_het <- exp(li_het)

# Latent variable
z_star <- li + u * sd_het

# Cuts
cuts <- c(-1, 0.5, 2)

# Observable ordinal outcome
z                                           <- rep(0, n)
z[(z_star > cuts[1]) & (z_star <= cuts[2])] <- 1
z[(z_star > cuts[2]) & (z_star <= cuts[3])] <- 2
z[z_star > cuts[3]]                         <- 3
table(z)

# Data
data <- data.frame(w1 = w1, w2 = w2, w3 = w3, z = z)

# ---
# Step 2
# Estimation of the parameters
# ---

# Estimation
model <- msel(z ~ w1 + w2 | w2 + w3,
                   data = data)
summary(model)

# Compare the estimates and true values of the parameters
  # regression coefficients of the mean equation
gamma_est <- coef(model, type = "coef", eq = 1)
cbind(true = gamma, estimate = gamma_est)
  # regression coefficients of the variance equation
gamma_het_est <- coef(model, type = "coef_var", eq = 1)
cbind(true = gamma_het, estimate = gamma_het_est)  
  # cuts
cuts_est <- coef(model, type = "cuts", eq = 1)
cbind(true = cuts, estimate = cuts_est)   

# Likelihood-ratio test for the homoscedasticity
model0 <- msel(z ~ w1 + w2, data = data)
summary(lrtest_msel(model, model0))

# Wald test for the homoscedasticity
test_fn <- function(object)
{
  val <- coef(object, type = "coef_var", eq = 1)
  return(val)
}
test_result <- test_msel(model, test = "wald", fn = test_fn)
summary(test_result)

# ---
# Step 3
# Estimation of the probabilities and marginal effects
# ---

# Predict probability that the dependent variable
# equals 2 for every observation in a sample
# P(z = 2 | w)
prob <- predict(model, group = 2, type = "prob")
head(prob)

# Calculate mean marginal effect of w2 on P(z = 1 | w)
mean(predict(model, group = 1, type = "prob", me = "w2"))

# Estimate conditional probabilities, linear predictors (indexes) and 
# heteroscedastic standard deviations for manually 
# provided observations.
  # new data
newdata <- data.frame(z = c(1, 1), 
                      w1 = c(0.5, 0.2), 
                      w2 = c(-0.3, 0.8),
                      w3 = c(0.6, 0.1))
  # probability P(z = 2 | w)
predict(model, group = 2, type = "prob", newdata = newdata)
  # linear predictor (index)
predict(model, type = "li", newdata = newdata, group = 0)
  # standard deviation
predict(model, type = "sd", newdata = newdata, group = 0)
  # marginal effect of w3 on P(z = 3 | w)
predict(model, group = 3, type = "prob", newdata = newdata, me = "w3")
  # marginal effect of w2 on the standard error
predict(model, group = 2, type = "sd", newdata = newdata, me = "w2")
  # discrete marginal effect: 
  # P(Z = 2 | w1 = 0.5, w2) - P(Z = 2 | w1 = 0.2, w2)
predict(model, group = 2, type = "prob", newdata = newdata,
        me = "w2", eps = c(0.2, 0.5))
}

\donttest{
# ---------------------------------------
# Simulated data example 3
# Bivariate ordered probit model with 
# heteroscedastic second equation
# ---------------------------------------

# Load required package
library("mnorm")

# ---
# Step 1
# Simulation of the data
# ---

# Set seed for reproducibility
set.seed(123)

# The number of observations
n <- 1000

# Regressors (covariates)
w1 <- runif(n = n, min = -1, max = 1)
w2 <- runif(n = n, min = -1, max = 1)
w3 <- runif(n = n, min = -1, max = 1)
w4 <- runif(n = n, min = -1, max = 1)

# Covariance matrix of random errors
rho <- 0.5
sigma <- matrix(c(1,   rho,
                  rho, 1), 
                nrow = 2)

# Random errors
u <- mnorm::rmnorm(n = n, mean = c(0, 0), sigma = sigma)

# Coefficients
gamma1 <- c(-1, 2)
gamma2 <- c(1, 1.5)

# Coefficients of the variance equation
gamma2_het <- c(0.5, -1)

# Linear predictors (indexes)
li1 <- gamma1[1] * w1 + gamma1[2] * w2
li2 <- gamma2[1] * w2 + gamma2[2] * w3

# Linear predictor (index) of the variance equation
li2_het <- gamma2_het[1] * w2 + gamma2_het[2] * w4

# Heteroscedastic standard deviation
# i.e., the value of the variance equation
sd2_het <- exp(li2_het)

# Latent variables
z1_star <- li1 + u[, 1]
z2_star <- li2 + u[, 2] * sd2_het

# Cuts
cuts1 <- c(-1, 0.5, 2)
cuts2 <- c(-2, 0)

# Observable ordinal outcome
  # first outcome
z1                                               <- rep(0, n)
z1[(z1_star > cuts1[1]) & (z1_star <= cuts1[2])] <- 1
z1[(z1_star > cuts1[2]) & (z1_star <= cuts1[3])] <- 2
z1[z1_star > cuts1[3]]                           <- 3
  # second outcome
z2                                               <- rep(0, n)
z2[(z2_star > cuts2[1]) & (z2_star <= cuts2[2])] <- 1
z2[z2_star > cuts2[2]]                           <- 2
  # distribution
table(z1, z2)

# Data
data <- data.frame(w1 = w1, w2 = w2, w3 = w3, 
                   w4 = w4, z1 = z1, z2 = z2)

# ---
# Step 2
# Estimation of the parameters
# ---

# Estimation
model <- msel(list(z1 ~ w1 + w2,
                   z2 ~ w2 + w3 | w2 + w4),
              data = data)
summary(model)

# Compare the estimates and true values of the parameters
  # regression coefficients of the first equation
gamma1_est <- coef(model, type = "coef", eq = 1)
cbind(true = gamma1, estimate = gamma1_est)
  # regression coefficients of the second equation
gamma2_est <- coef(model, type = "coef", eq = 2)
cbind(true = gamma2, estimate = gamma2_est)
  # cuts of the first equation
cuts1_est <- coef(model, type = "cuts", eq = 1)
cbind(true = cuts1, estimate = cuts1_est)   
  # cuts of the second equation
cuts2_est <- coef(model, type = "cuts", eq = 2)
cbind(true = cuts2, estimate = cuts2_est) 
  # correlation coefficients
rho_est <- coef(model, type = "cov1", eq = c(1, 2))
cbind(true = rho, estimate = rho_est)  
  # regression coefficients of the variance equation
gamma2_het_est <- coef(model, type = "coef_var", eq = 2)
cbind(true = gamma2_het, estimate = gamma2_het_est)

# ---
# Step 3
# Estimation of the probabilities and linear predictors (indexes)
# ---

# Predict probability P(z1 = 2, z2 = 0 | w)
prob <- predict(model, group = c(2, 0), type = "prob")
head(prob)

# Calculate mean marginal effect of w2 on:
  # P(z1 = 1 | w)
mean(predict(model, group = c(1, -1), type = "prob", me = "w2"))
  # P(z1 = 1, z2 = 0 | w)
mean(predict(model, group = c(1, 0), type = "prob", me = "w2"))

# Calculate conditional probabilities and linear predictors (indexes)
# for the manually provided observations.
  # new data
newdata <- data.frame(z1 = c(1, 1), 
                      z2 = c(1, 1),
                      w1 = c(0.5, 0.2), 
                      w2 = c(-0.3, 0.8),
                      w3 = c(0.6, 0.1),
                      w4 = c(0.3, -0.5))
  # probability P(z1 = 2, z2 = 0 | w)
predict(model, group = c(2, 0), type = "prob", newdata = newdata)
  # linear predictor (index)
predict(model, type = "li", newdata = newdata, group = c(0, 0))  
  # marginal probability P(z2 = 1 | w)
predict(model, group = c(-1, 1), type = "prob", newdata = newdata) 
  # marginal probability P(z1 = 3 | w)
predict(model, group = c(3, -1), type = "prob", newdata = newdata)
  # conditional probability P(z1 = 2 | z2 = 0, w)
predict(model, group = c(2, 0), given_ind = 2,
        type = "prob", newdata = newdata) 
  # conditional probability P(z2 = 1 | z1 = 3, w)
predict(model, group = c(3, 1), given_ind = 1,
        type = "prob", newdata = newdata) 
  # marginal effect of w4 on P(Z2 = 2 | w)
predict(model, group = c(-1, 2),
        type = "prob", newdata = newdata, me = "w4")   
  # marginal effect of w4 on P(z1 = 3, Z2 = 2 | w)
predict(model, group = c(3, 2),
        type = "prob", newdata = newdata, me = "w4") 
  # marginal effect of w4 on P(z1 = 3 | z2 = 2, w)
predict(model, group = c(3, 2), given_ind = 2,
        type = "prob", newdata = newdata, me = "w4")         

# ---
# Step 4
# Replication under the non-random sample selection
# ---

# Suppose that z2 is unobservable when z1 = 1 or z1 = 3
z2[(z1 == 1) | (z1 == 3)] <- -1
data$z2 <- z2

# Replicate the estimation procedure
model <- msel(list(z1 ~ w1 + w2,
                   z2 ~ w2 + w3 | w2 + w4),
              cov_type = "gop", data = data)
summary(model)

# Compare the estimates and true values of the parameters
  # regression coefficients of the first equation
gamma1_est <- coef(model, type = "coef", eq = 1)
cbind(true = gamma1, estimate = gamma1_est)
  # regression coefficients of the second equation
gamma2_est <- coef(model, type = "coef", eq = 2)  
cbind(true = gamma2, estimate = gamma2_est)
  # cuts of the first equation
cuts1_est <- coef(model, type = "cuts", eq = 1)  
cbind(true = cuts1, estimate = cuts1_est)   
  # cuts of the second equation
cuts2_est <- coef(model, type = "cuts", eq = 2)  
cbind(true = cuts2, estimate = cuts2_est) 
  # correlation coefficient
rho_est <- coef(model, type = "cov1", eq = c(1, 2))
cbind(true = rho, estimate = rho_est)  
  # regression coefficients of the variance equation
gamma2_het_est <- coef(model, type = "coef_var", eq = 2)  
cbind(true = gamma2_het, estimate = gamma2_het_est)

# ---
# Step 5
# Semiparametric model with marginal logistic and PGN distributions
# ---

# Estimate the model
model <- msel(list(z1 ~ w1 + w2,
                   z2 ~ w2 + w3 | w2 + w4),
              data = data, marginal = list(PGN = 3, logistic = NULL))
summary(model)
}

\donttest{
# ---------------------------------------
# Simulated data example 4
# Heckman model with ordinal 
# selection mechanism
# ---------------------------------------

# Load required package
library("mnorm")

# ---
# Step 1
# Simulation of the data
# ---

# Set seed for reproducibility
set.seed(123)

# The number of observations
n <- 1000

# Regressors (covariates)
w1 <- runif(n = n, min = -1, max = 1)
w2 <- runif(n = n, min = -1, max = 1)
w3 <- runif(n = n, min = -1, max = 1)

# Random errors
rho    <- 0.5
var.y  <- 0.3
sd.y   <- sqrt(var.y)
sigma  <- matrix(c(1,          rho * sd.y,
                   rho * sd.y, var.y),
                 nrow = 2)
errors <- mnorm::rmnorm(n = n, mean = c(0, 0), sigma = sigma)
u      <- errors[, 1]
eps    <- errors[, 2]

# Coefficients
gamma <- c(-1, 2)
beta  <- c(1, -1, 1)

# Linear predictor (index)
li   <- gamma[1] * w1 + gamma[2] * w2
li.y <- beta[1] + beta[2] * w1 + beta[3] * w3

# Latent variable
z_star <- li + u
y_star <- li.y + eps

# Cuts
cuts <- c(-1, 0.5, 2)

# Observable ordered outcome
z                                           <- rep(0, n)
z[(z_star > cuts[1]) & (z_star <= cuts[2])] <- 1
z[(z_star > cuts[2]) & (z_star <= cuts[3])] <- 2
z[z_star > cuts[3]]                         <- 3
table(z)

# Observable continuous outcome such
# that outcome 'y' is observable only 
# when 'z > 1' and unobservable otherwise
# i.e., when 'z <= 1' we code 'y' as 'NA'
y         <- y_star
y[z <= 1] <- NA

# Data
data <- data.frame(w1 = w1, w2 = w2, w3 = w3, 
                   z = z, y = y)

# ---
# Step 2
# Estimation of the parameters
# ---

# Estimation
model <- msel(z ~ w1 + w2, y ~ w1 + w3, data = data)
summary(model)

# Compare estimates and true values of the parameters
  # regression coefficients of the ordinal equation
gamma_est <- coef(model, type = "coef", eq = 1)
cbind(true = gamma, estimate = gamma_est)
  # cuts
cuts_est <- coef(model, type = "cuts", eq = 1)
cbind(true = cuts, estimate = cuts_est)   
  # regression coefficients of the continuous equation
beta_est <- coef(model, type = "coef2", eq2 = 1, regime = 0)
cbind(true = beta, estimate = beta_est)  
  # variance
var.y_est <- coef(model, type = "var", eq2 = 1, regime = 0)
cbind(true = var.y, estimate = var.y_est)
  # covariance
cov_est <- coef(model, type = "cov12", eq = 1, eq2 = 1)
cbind(true = rho * sd.y, estimate = cov_est)

# ---
# Step 3
# Estimation of the expectations and marginal effects
# ---

# New data
newdata <- data.frame(z = 1, 
                      y = 1,
                      w1 = 0.1, 
                      w2 = 0.2,
                      w3 = 0.3)

# Predict the unconditional expectation of the continuous outcome
# E(y | w)
predict(model, group = -1, group2 = 0, newdata = newdata)

# Predict the conditional expectations of the continuous outcome
  # E(y | z = 2, w)
predict(model, group = 2, group2 = 0, newdata = newdata)
  # E(y | z = 0, w)
predict(model, group = 0, group2 = 0, newdata = newdata)

# ---
# Step 4
# The classic Heckman two-step estimation procedure
# ---

# Estimate the model by using the two-step estimator
model_ts <- msel(z ~ w1 + w2, y ~ w1 + w3,
                 data = data, estimator = "2step")
summary(model_ts)

# Check the accuracy of the estimates
tbl <- cbind(true    = beta, 
             ml      = model$coef2[[1]][1, ],
             twostep = model_ts$coef2[[1]][1, -4])  
print(tbl)

# ---
# Step 5
# Semiparametric estimation procedure
# ---

# Estimate the model using Lee's method 
# assuming logistic distribution of the
# random errors of the selection equation
model_Lee <- msel(z ~ w1 + w2, 
                  y ~ w1 + w3,
                  data = data, estimator = "2step",
                  marginal = list(logistic = NULL))
summary(model_Lee)

# One-step estimation is available as well as
# more complex marginal distributions.
# Consider random errors in the selection equation
# following a PGN distribution with three parameters.
model_sp <- msel(z ~ w1 + w2, 
                 y ~ w1 + w3,
                 data = data, marginal = list(PGN = 3))
summary(model_sp)

# To additionally relax the normality assumption of
# the random error of the continuous equation, it is possible
# to use Newey's two-step procedure.
model_Newey <- msel(z ~ w1 + w2, 
                    y ~ w1 + w3,
                    data      = data,    marginal = list(logistic = 0),
                    estimator = "2step", degrees  = 2)
summary(model_Newey)
}

\donttest{
# ---------------------------------------
# Simulated data example 5
# Endogenous switching model with 
# heteroscedastic ordered selection 
# mechanism
# ---------------------------------------

# Load required package
library("mnorm")

# ---
# Step 1
# Simulation of the data
# ---

# Set seed for reproducibility
set.seed(123)

# The number of observations
n <- 1000

# Regressors (covariates)
w1 <- runif(n = n, min = -1, max = 1)
w2 <- runif(n = n, min = -1, max = 1)
w3 <- runif(n = n, min = -1, max = 1)

# Random errors
rho_0    <- -0.8
rho_1    <- -0.7
var2_0   <- 0.9
var2_1   <- 1
sd_y_0   <- sqrt(var2_0)
sd_y_1   <- sqrt(var2_1)
cor_y_01 <- 0.7
cov2_01  <- sd_y_0 * sd_y_1 * cor_y_01
cov2_z_0 <- rho_0 * sd_y_0
cov2_z_1 <- rho_1 * sd_y_1
sigma    <- matrix(c(1,        cov2_z_0, cov2_z_1,
                     cov2_z_0, var2_0,   cov2_01,
                     cov2_z_1, cov2_01,  var2_1),
                   nrow = 3)
errors   <- mnorm::rmnorm(n = n, mean = c(0, 0, 0), sigma = sigma)
u        <- errors[, 1]
eps_0    <- errors[, 2]
eps_1    <- errors[, 3]

# Coefficients
gamma     <- c(-1, 2)
gamma_het <- c(0.5, -1)
beta_0    <- c(1, -1, 1)
beta_1    <- c(2, -1.5, 0.5)

# Linear predictor (index) of the ordinal equation
  # mean
li <- gamma[1] * w1 + gamma[2] * w2
  # variance
li_het <- gamma_het[1] * w2 + gamma_het[2] * w3

# Linear predictor (index) of the continuous equation
  # regime 0
li_y_0 <- beta_0[1] + beta_0[2] * w1 + beta_0[3] * w3
  # regime 1
li_y_1 <- beta_1[1] + beta_1[2] * w1 + beta_1[3] * w3

# Latent variable
z_star   <- li + u * exp(li_het)
y_0_star <- li_y_0 + eps_0
y_1_star <- li_y_1 + eps_1

# Cuts
cuts <- c(-1, 0.5, 2)

# Observable ordinal outcome
z                                           <- rep(0, n)
z[(z_star > cuts[1]) & (z_star <= cuts[2])] <- 1
z[(z_star > cuts[2]) & (z_star <= cuts[3])] <- 2
z[z_star > cuts[3]]                         <- 3
table(z)

# Observable continuous outcome such that y is 
# observable in regime 1 when 'z = 1', 
# observable in regime 0 when 'z <= 1',
# unobservable           when 'z = 0'
y         <- rep(NA, n)
y[z == 0] <- NA
y[z == 1] <- y_0_star[z == 1]
y[z > 1]  <- y_1_star[z > 1]

# Data
data <- data.frame(w1 = w1, w2 = w2, w3 = w3, 
                   z  = z,  y  = y)

# ---
# Step 2
# Estimation of the parameters
# ---

# Assign groups
groups <- matrix(0:3, ncol = 1)
groups2 <- matrix(c(-1, 0, 1, 1), ncol = 1)

# Estimation
model <- msel(list(z ~ w1 + w2 | w2 + w3),
                   list(y ~ w1 + w3),
                   groups = groups, groups2 = groups2,
                   data = data)
summary(model)

# Compare the estimates and true values of the parameters
  # regression coefficients of the ordinal equation
gamma_est     <- coef(model, type = "coef",     eq = 1)
gamma_het_est <- coef(model, type = "coef_var", eq = 1)
cbind(true = gamma,     estimate = gamma_est)
cbind(true = gamma_het, estimate = gamma_het_est)
  # cuts
cuts_est <- coef(model, type = "cuts", eq = 1)
cbind(true = cuts, estimate = cuts_est)   
  # regression coefficients of the continuous equation
beta_0_test <- coef(model, type = "coef2", eq2 = 1, regime = 0)
beta_1_test <- coef(model, type = "coef2", eq2 = 1, regime = 1)
cbind(true = beta_0, estimate = beta_0_test) 
cbind(true = beta_1, estimate = beta_1_test)
  # variances
var2_0_est <- coef(model, type = "var", eq2 = 1, regime = 0)
var2_1_est <- coef(model, type = "var", eq2 = 1, regime = 1)
cbind(true = c(var2_0, var2_1), estimate = c(var2_0_est, var2_1_est)) 
  # covariances between the random errors
cov2_z_0_est <- coef(model, type = "cov12", eq = 1, eq2 = 1, regime = 0)
cov2_z_1_est <- coef(model, type = "cov12", eq = 1, eq2 = 1, regime = 1)
cbind(true     = c(cov2_z_0,     cov2_z_1), 
      estimate = c(cov2_z_0_est, cov2_z_1_est))

# ---
# Step 3
# Estimation of the expectations and marginal effects
# ---

# New data
newdata <- data.frame(z  = 1,   y  = 1,
                      w1 = 0.1, w2 = 0.2, w3 = 0.3)

# Predict the unconditional expectation of the 
# continuous outcome E(yr | w)
  # regime 0 
predict(model, group = -1, group2 = 0, newdata = newdata)
  # regime 1
predict(model, group = -1, group2 = 1, newdata = newdata)

# Predict the conditional expectations of the continuous outcome
# given condition 'z == 0' for regime 1 i.e., E(y1 | z = 0, w)
predict(model, group = 0, group2 = 1, newdata = newdata)
}

\donttest{
# ---------------------------------------
# Simulated data example 6
# Endogenous switching model with 
# multivariate heteroscedastic ordered 
# selection mechanism
# ---------------------------------------

# Load required package
library("mnorm")

# ---
# Step 1
# Simulation of the data
# ---

# Set seed for reproducibility
set.seed(123)

# The number of observations
n <- 1000

# Regressors (covariates)
w1 <- runif(n = n, min = -1, max = 1)
w2 <- runif(n = n, min = -1, max = 1)
w3 <- runif(n = n, min = -1, max = 1)
w4 <- runif(n = n, min = -1, max = 1)

# Random errors
rho_z1_z2 <- 0.5
rho_y0_z1 <- 0.6
rho_y0_z2 <- 0.7
rho_y1_z1 <- 0.65
rho_y1_z2 <- 0.75
var20     <- 0.9
var21     <- 1
sd_y0     <- sqrt(var20)
sd_y1     <- sqrt(var21)
cor_y01   <- 0.7
cov201    <- sd_y0 * sd_y1 * cor_y01
cov20_z1  <- rho_y0_z1 * sd_y0
cov21_z1  <- rho_y1_z1 * sd_y1
cov20_z2  <- rho_y0_z2 * sd_y0
cov21_z2  <- rho_y1_z2 * sd_y1
sigma     <- matrix(c(1,         rho_z1_z2,  cov20_z1,  cov21_z1, 
                      rho_z1_z2, 1,          cov20_z2,  cov21_z2,
                      cov20_z1,  cov20_z2,   var20,     cov201,
                      cov21_z1,  cov21_z2,   cov201,    var21),
                    nrow = 4)
errors    <- mnorm::rmnorm(n = n, mean = c(0, 0, 0, 0), sigma = sigma)
u1        <- errors[, 1]
u2        <- errors[, 2]
eps0      <- errors[, 3]
eps1      <- errors[, 4]

# Coefficients
gamma1     <- c(-1, 2)
gamma1_het <- c(0.5, -1)
gamma2     <- c(1, 1)
beta0      <- c(1, -1, 1, -1.2)
beta1      <- c(2, -1.5, 0.5, 1.2)

# Linear index (predictor) of the ordinal equation
  # mean
li1 <- gamma1[1] * w1 + gamma1[2] * w2
li2 <- gamma2[1] * w1 + gamma2[2] * w3
  # variance
li1_het <- gamma1_het[1] * w2 + gamma1_het[2] * w3

# Linear predictor (index) of the continuous equation
  # regime 0
li_y0 <- beta0[1] + beta0[2] * w1 + beta0[3] * w3 + beta0[4] * w4
  # regime 1
li_y1 <- beta1[1] + beta1[2] * w1 + beta1[3] * w3 + beta1[4] * w4

# Latent variables
z1_star <- li1 + u1 * exp(li1_het)
z2_star <- li2 + u2
y0_star <- li_y0 + eps0
y1_star <- li_y1 + eps1

# Cuts
cuts1 <- c(-1, 1)
cuts2 <- c(0)

# Observable ordered outcome
  # first
z1                                               <- rep(0, n)
z1[(z1_star > cuts1[1]) & (z1_star <= cuts1[2])] <- 1
z1[z1_star > cuts1[2]]                           <- 2
  # second
z2                     <- rep(0, n)
z2[z2_star > cuts2[1]] <- 1
table(z1, z2)

# Observable continuous outcome such
# that outcome 'y' is 
# in regime 0  when 'z1 == 1', 
# in regime 1  when 'z1 == 0' or 'z1 == 2',
# unobservable when 'z2 == 0'
y          <- rep(NA, n)
y[z1 == 1] <- y0_star[z1 == 1]
y[z1 != 1] <- y1_star[z1 != 1]
y[z2 == 0] <- NA

# Data
data <- data.frame(w1 = w1, w2 = w2, w3 = w3, w4 = w4,
                   z1 = z1, z2 = z2, y  = y)

# ---
# Step 2
# Estimation of the parameters
# ---

# Assign groups
groups <- matrix(c(0, 0,
                   0, 1,
                   1, 0,
                   1, 1,
                   2, 0,
                   2, 1), 
                 byrow = TRUE, ncol = 2)
groups2 <- matrix(c(-1, 1, -1, 0, -1, 1), ncol = 1)

# Estimation
model <- msel(list(z1 ~ w1 + w2 | w2 + w3,
                   z2 ~ w1 + w3),
              y ~ w1 + w3 + w4,
              groups = groups, groups2 = groups2,
              data   = data)
summary(model)

# Compare estimates and true values of the parameters
  # regression coefficients of the first ordered equation
gamma1_est      <- coef(model, type = "coef",     eq = 1)
gamma1_het_est <- coef(model, type = "coef_var", eq = 1)
cbind(true = gamma1,     estimate = gamma1_est)
cbind(true = gamma1_het, estimate = gamma1_het_est)
  # regression coefficients of the second ordered equation
gamma2_est <- coef(model, type = "coef", eq = 2)
cbind(true = gamma2, estimate = gamma2_est)
  # cuts
cuts1_est <- coef(model, type = "cuts", eq = 1)
cuts2_est <- coef(model, type = "cuts", eq = 2)
cbind(true = cuts1, estimate = cuts1_est)   
cbind(true = cuts2, estimate = cuts2_est)  
  # regression coefficients of the continuous equation
beta0_est <- coef(model, type = "coef2", eq2 = 1, regime = 0)
beta1_est <- coef(model, type = "coef2", eq2 = 1, regime = 1)
cbind(true = beta0, estimate = beta0_est) 
cbind(true = beta1, estimate = beta1_est)
  # variances
var20_est <- coef(model, type = "var", eq2 = 1, regime = 0)
var21_est <- coef(model, type = "var", eq2 = 1, regime = 1)
cbind(true = c(var20, var21), estimate = c(var20_est, var21_est)) 
  # covariances
cov_y0_z1_est <- coef(model, type = "cov12", eq = 1, eq2 = 1, regime = 0)
cov_y0_z2_est <- coef(model, type = "cov12", eq = 2, eq2 = 1, regime = 0)
cov_y1_z1_est <- coef(model, type = "cov12", eq = 1, eq2 = 1, regime = 1)
cov_y1_z2_est <- coef(model, type = "cov12", eq = 2, eq2 = 1, regime = 1)
cbind(true     = c(cov20_z1, cov20_z2), 
      estimate = c(cov_y0_z1_est, cov_y0_z2_est))
cbind(true     = c(cov21_z1, cov21_z2), 
      estimate = c(cov_y1_z1_est,  cov_y1_z2_est))

# ---
# Step 3
# Estimation of the expectations and marginal effects
# ---

# New data
newdata <- data.frame(z1 = 1,   z2 = 1,   y  = 1,
                      w1 = 0.1, w2 = 0.2, w3 = 0.3, w4 = 0.4)

# Predict the unconditional expectation of the continuous outcome
  # regime 0 
predict(model, group = c(-1, -1), group2 = 0, newdata = newdata)
  # regime 1
predict(model, group = c(-1, -1), group2 = 1, newdata = newdata)

# Predict the conditional expectations of the continuous outcome
# E(y1 | z1 = 2, z2 = 1, w)
predict(model, group = c(2, 1), group2 = 1, newdata = newdata)

# Marginal effect of w3 on E(y1 | z1 = 2, z2 = 1, w)
predict(model, group = c(2, 1), group2 = 1, newdata = newdata, me = "w3")

# ---
# Step 4
# Two-step estimation procedure
# ---

# For comparison reasons, let's estimate the model
# via the least squares
model.ls.0 <- lm(y ~ w1 + w3 + w4,
                 data = data[!is.na(data$y) & (data$z1 == 1), ])
model.ls.1 <- lm(y ~ w1 + w3 + w4,
                 data = data[!is.na(data$y) & (data$z1 != 1), ])


# Apply the two-step procedure
model_ts <-  msel(list(z1 ~ w1 + w2 | w2 + w3,
                       z2 ~ w1 + w3),
                  y ~ w1 + w3 + w4,
                  groups    = groups,  groups2 = groups2,
                  estimator = "2step", data    = data)
summary(model_ts)

# Use the two-step procedure with logistic marginal distributions
# that is a multivariate generalization of Lee's method
model_Lee <-  msel(list(z1 ~ w1 + w2 | w2 + w3,
                        z2 ~ w1 + w3),
                   y ~ w1 + w3 + w4,
                   marginal = list(logistic = NULL, logistic = NULL),
                   groups    = groups,  groups2 = groups2,
                   estimator = "2step", data    = data)
                        
# Apply Newey's method
model_Newey <-  msel(list(z1 ~ w1 + w2 | w2 + w3,
                          z2 ~ w1 + w3),
                     y ~ w1 + w3 + w4,
                     marginal = list(logistic = NULL, logistic = NULL),
                     degrees = c(2, 3), groups = groups, groups2 = groups2,
                     estimator = "2step", data = data)

# Compare the accuracy of the methods
  # beta0
tbl0 <- cbind(true    = beta0, 
              ls      = coef(model.ls.0),
              ml      = model$coef2[[1]][1, 1:length(beta0)],
              twostep = model_ts$coef2[[1]][1, 1:length(beta0)],
              Lee     = model_Lee$coef2[[1]][1, 1:length(beta0)],
              Newey   = model_Newey$coef2[[1]][1, 1:length(beta0)])  
print(tbl0)
  # beta1
tbl1 <- cbind(true    = beta1, 
              ls      = coef(model.ls.1),
              ml      = model$coef2[[1]][2, 1:length(beta1)],
              twostep = model_ts$coef2[[1]][2, 1:length(beta1)],
              Lee     = model_Lee$coef2[[1]][2, 1:length(beta1)],
              Newey   = model_Newey$coef2[[1]][2, 1:length(beta1)])  
print(tbl1)
}

\donttest{
# ---------------------------------------
# Simulated data example 7
# Endogenous multivariate switching model 
# with multivariate heteroscedastic
# ordered selection mechanism
# ---------------------------------------

# Load required package
library("mnorm")

# ---
# Step 1
# Simulation of the data
# ---

# Set seed for reproducibility
set.seed(123)

# The number of observations
n <- 1000

# Regressors (covariates)
w1 <- runif(n = n, min = -1, max = 1)
w2 <- runif(n = n, min = -1, max = 1)
w3 <- runif(n = n, min = -1, max = 1)
w4 <- runif(n = n, min = -1, max = 1)
w5 <- runif(n = n, min = -1, max = 1)

# Random errors
var_y0 <- 0.9
var_y1 <- 1
var_g0 <- 1.1
var_g1 <- 1.2
var_g2 <- 1.3
A      <- rWishart(1, 7, diag(7))[, , 1]
B      <- diag(sqrt(c(1, 1, var_y0, var_y1, 
                      var_g0, var_g1, var_g2)))
sigma  <-  B  \%*\% cov2cor(A) \%*\% B
errors <- mnorm::rmnorm(n = n, mean = rep(0, nrow(sigma)), sigma = sigma)
u1     <- errors[, 1]
u2     <- errors[, 2]
eps0_y <- errors[, 3]
eps1_y <- errors[, 4]
eps0_g <- errors[, 5]
eps1_g <- errors[, 6]
eps2_g <- errors[, 7]

# Coefficients
gamma1     <- c(-1, 2)
gamma1_het <- c(0.5, -1)
gamma2     <- c(1, 1)
beta0_y    <- c(1, -1, 1, -1.2)
beta1_y    <- c(2, -1.5, 0.5, 1.2)
beta0_g    <- c(-1, 1, 1, 1)
beta1_g    <- c(1, -1, 1, 1)
beta2_g    <- c(1, 1, -1, 1)

# Linear predictor (index) of the ordinal equation
  # mean
li1 <- gamma1[1] * w1 + gamma1[2] * w2
li2 <- gamma2[1] * w1 + gamma2[2] * w3
  # variance
li1_het <- gamma1_het[1] * w2 + gamma1_het[2] * w3

# Linear predictor (index) of the first continuous equation
  # regime 0
li_y0 <- beta0_y[1] + beta0_y[2] * w1 + beta0_y[3] * w3 + beta0_y[4] * w4
  # regime 1
li_y1 <- beta1_y[1] + beta1_y[2] * w1 + beta1_y[3] * w3 + beta1_y[4] * w4

# Linear predictor (index) of the second continuous equation
  # regime 0
li_g0 <- beta0_g[1] + beta0_g[2] * w2 + beta0_g[3] * w3 + beta0_g[4] * w5
  # regime 1
li_g1 <- beta1_g[1] + beta1_g[2] * w2 + beta1_g[3] * w3 + beta1_g[4] * w5
  # regime 2
li_g2 <- beta2_g[1] + beta2_g[2] * w2 + beta2_g[3] * w3 + beta2_g[4] * w5

# Latent variables
z1_star <- li1 + u1 * exp(li1_het)
z2_star <- li2 + u2
y0_star <- li_y0 + eps0_y
y1_star <- li_y1 + eps1_y
g0_star <- li_g0 + eps0_g
g1_star <- li_g1 + eps1_g
g2_star <- li_g2 + eps2_g

# Cuts
cuts1 <- c(-1, 1)
cuts2 <- c(0)

# Observable ordered outcome
  # first
z1                                               <- rep(0, n)
z1[(z1_star > cuts1[1]) & (z1_star <= cuts1[2])] <- 1
z1[z1_star > cuts1[2]]                           <- 2
  # second
z2                     <- rep(0, n)
z2[z2_star > cuts2[1]] <- 1
table(z1, z2)

# Observable continuous outcome such that outcome 'y' is 
# in regime 0 when 'z1 == 1', 
# in regime 1 when 'z1 == 0' or 'z1 == 2',
# unobservable when 'z2 == 0'
y          <- rep(NA, n)
y[z1 == 1] <- y0_star[z1 == 1]
y[z1 != 1] <- y1_star[z1 != 1]
y[z2 == 0] <- NA

# Observable continuous outcome such that outcome 'g' is 
# in regime 0 when 'z1 == z2', 
# in regime 1 when 'z1 > z2',
# in regime 2 when 'z1 < z2',
g           <- rep(NA, n)
g[z1 == z2] <- g0_star[z1 == z2]
g[z1 > z2]  <- g1_star[z1 > z2]
g[z1 < z2]  <- g2_star[z1 < z2]

# Data
data <- data.frame(w1 = w1, w2 = w2, w3 = w3, w4 = w4, w5 = w5,
                   z1 = z1, z2 = z2, y  = y,  g  = g)

# ---
# Step 2
# Estimation of the parameters
# ---

# Assign groups
groups <- matrix(c(0, 0,
                   0, 1,
                   1, 0,
                   1, 1,
                   2, 0,
                   2, 1), 
                 byrow = TRUE, ncol = 2)

# Assign groups 2
  # prepare the matrix
groups2 <- matrix(NA, nrow = nrow(groups), ncol = 2)
  # fill the matrix
groups2[groups[, 1] == 1, 1]                        <- 0
groups2[(groups[, 1] == 0) | (groups[, 1] == 2), 1] <- 1
groups2[groups[, 2] == 0, 1]                        <- -1
groups2[groups[, 1] == groups[, 2], 2]              <- 0
groups2[groups[, 1] > groups[, 2], 2]               <- 1
groups2[groups[, 1] < groups[, 2], 2]               <- 2

# The structure of the model
cbind(groups, groups2)

# Estimation
model <- msel(list(z1 ~ w1 + w2 | w2 + w3, z2 ~ w1 + w3),
              list(y ~ w1 + w3 + w4, g ~ w2 + w3 + w5),
              groups = groups, groups2 = groups2, data = data)
summary(model)

# Compare the estimates and true values of the parameters
  # regression coefficients of the first ordered equation
gamma1_est     <- coef(model, type = "coef", eq = 1)
gamma1_het_est <- coef(model, type = "coef_var", eq = 1)
cbind(true = gamma1,     estimate = gamma1_est)
cbind(true = gamma1_het, estimate = gamma1_het_est)
  # regression coefficients of the second ordered equation
gamma2_est <- coef(model, type = "coef", eq = 2)
cbind(true = gamma2, estimate = gamma2_est)
  # cuts
cuts1_est <- coef(model, type = "cuts", eq = 1)
cuts2_est <- coef(model, type = "cuts", eq = 2)
cbind(true = cuts1, estimate = cuts1_est)   
cbind(true = cuts2, estimate = cuts2_est)  
  # regression coefficients of the first continuous equation
beta0_y_est <- coef(model, type = "coef2", eq2 = 1, regime = 0)
beta1_y_est <- coef(model, type = "coef2", eq2 = 1, regime = 1)
cbind(true = beta0_y, estimate = beta0_y_est) 
cbind(true = beta1_y, estimate = beta1_y_est)
  # regression coefficients of the second continuous equation
beta0_g_est <- coef(model, type = "coef2", eq2 = 2, regime = 0)
beta1_g_est <- coef(model, type = "coef2", eq2 = 2, regime = 1)
beta2_g_est <- coef(model, type = "coef2", eq2 = 2, regime = 2)
cbind(true = beta0_g, estimate = beta0_g_est) 
cbind(true = beta1_g, estimate = beta1_g_est)
cbind(true = beta2_g, estimate = beta2_g_est)
  # variances of the first continuous equation
var_y0_est <- coef(model, type = "var", eq2 = 1, regime = 0)
var_y1_est <- coef(model, type = "var", eq2 = 1, regime = 1)
cbind(true = c(var_y0, var_y1), estimate = c(var_y0_est, var_y1_est)) 
  # variances of the second continuous equation
var_g0_est <- coef(model, type = "var", eq2 = 2, regime = 0)
var_g1_est <- coef(model, type = "var", eq2 = 2, regime = 1)
var_g2_est <- coef(model, type = "var", eq2 = 2, regime = 2)
cbind(true     = c(var_g0, var_g1, var_g2), 
      estimate = c(var_g0_est, var_g1_est, var_g2_est)) 
  # correlation between the ordinal equations
sigma12_est <- coef(model, type = "cov1", eq = c(1, 2))
cbind(true = c(sigma[1, 2]), estimate = sigma12_est)
  # covariances between the continuous and ordinal equations
cbind(true = sigma[1:2, 3], estimate = model$cov2[[1]][1, ])
cbind(true = sigma[1:2, 4], estimate = model$cov2[[1]][2, ])
cbind(true = sigma[1:2, 5], estimate = model$cov2[[2]][1, ])
cbind(true = sigma[1:2, 6], estimate = model$cov2[[2]][2, ])
cbind(true = sigma[1:2, 7], estimate = model$cov2[[2]][3, ])
  # covariances between the continuous equations
sigma2_est <- coef(model, type = "cov2")[[1]]
cbind(true     = c(sigma[4, 7], sigma[3, 5], sigma[4, 6]), 
      estimate = sigma2_est) 
      
# ---
# Step 3
# Estimation of the expectations and marginal effects
# ---

# New data
newdata <- data.frame(z1 = 1,   z2 = 1,   y  = 1,   g  = 1,
                      w1 = 0.1, w2 = 0.2, w3 = 0.3, w4 = 0.4, w5 = 0.5)

# Predict unconditional expectation of the dependent variable
  # regime 0 for 'y' and regime 1 for 'g' i.e. E(y0 | w), E(g1 | w)
predict(model, group = c(-1, -1), group2 = c(0, 1), newdata = newdata)

# Predict conditional expectations of the dependent variable
# E(y0 | z1 = 2, z2 = 1, w), E(g1 | z1 = 2, z2 = 1, w)
predict(model, group = c(2, 1), group2 = c(0, 1), newdata = newdata)

# Marginal effect of w3 on 
# E(y1 | z1 = 2, z2 = 1, w) and E(g1 | z1 = 2, z2 = 1, w)
predict(model, group   = c(2, 1), group2 = c(0, 1), 
               newdata = newdata, me     = "w3")

# ---
# Step 4
# Two-step estimation procedure
# ---

# Manually provide selectivity terms
model2 <- msel(list(z1 ~ w1 + w2 | w2 + w3, z2 ~ w1 + w3),
               list(y ~ w1 + w3 + w4 + 
                        lambda1 + lambda2 + I(lambda1 * lambda2), 
                    g ~ w2 + w3 + w5 + lambda1 + lambda2),
               groups = groups, groups2   = groups2,
               data   = data,   estimator = "2step")
summary(model2)
}

\donttest{
# ---------------------------------------
# Simulated data example 8
# Multinomial endogenous switching and 
# selection model (probit)
# ---------------------------------------

# Load required package
library("mnorm")

# ---
# Step 1
# Simulation of the data
# ---

# Set seed for reproducibility
set.seed(123)

# The number of observations
n <- 10000

# Random errors
  # variances and correlations
sd.z2    <- sqrt(0.9)
cor.z    <- 0.3
sd.y0    <- sqrt(2)
cor.z1y0 <- 0.4
cor.z2y0 <- 0.7
sd.y1    <- sqrt(1.8)
cor.z1y1 <- 0.3
cor.z2y1 <- 0.6
cor.y    <- 0.8
  # the covariance matrix
sigma <- matrix(c(
1,                cor.z * sd.z2,            cor.z1y0 * sd.y0,         cor.z1y1 * sd.y1,
cor.z * sd.z2,    sd.z2 ^ 2,                cor.z2y0 * sd.z2 * sd.y0, cor.z2y1 * sd.z2 * sd.y1,
cor.z1y0 * sd.y0, cor.z2y0 * sd.z2 * sd.y0, sd.y0 ^ 2,                cor.y * sd.y0 * sd.y1,
cor.z1y1 * sd.y1, cor.z2y1 * sd.z2 * sd.y1, cor.y * sd.y0 * sd.y1,    sd.y1 ^ 2),
                ncol = 4, byrow = TRUE)
colnames(sigma) <- c("z1", "z2", "y0", "y1")
rownames(sigma) <- colnames(sigma)

# Simulate the random errors
errors <- rmnorm(n, c(0, 0, 0, 0), sigma)
u      <- errors[, 1:2]
eps    <- errors[, 3:4]

# Regressors (covariates)
x1 <- runif(n, -1, 1)
x2 <- runif(n, -1, 1)
x3 <- (x2 + runif(n, -1, 1)) / 2
W  <- cbind(1, x1, x2)
X  <- cbind(1, x1, x3)

# Coefficients
gamma0 <- c(0.1,    1,   1)
gamma1 <- c(0.2, -1.2, 0.8)
beta0  <- c(1,     -3,   4) 
beta1  <- c(1,      4,  -3)

# Linear predictors (indexes)
z1.li <- W \%*\% gamma0
z2.li <- W \%*\% gamma1
y0.li <- X \%*\% beta0
y1.li <- X \%*\% beta1

# Latent variables
z1.star <- z1.li + u[, 1]
z2.star <- z2.li + u[, 2]
y0.star <- y0.li + eps[, 1]
y1.star <- y1.li + eps[, 2]

# Observable variable as a dummy
z1 <- (z1.star > z2.star) & (z1.star > 0)
z2 <- (z2.star > z1.star) & (z2.star > 0)
z3 <- (z1 != 1) & (z2 != 1)

# Observable multinomial variable
z     <- rep(0, n)
z[z1] <- 0
z[z2] <- 1
z[z3] <- 2
table(z)

# Make unobservable values of the continuous outcome
y         <-  rep(NA, n)
y[z == 1] <- y0.star[z == 1]
y[z == 2] <- y1.star[z == 2]

# Data
data <- data.frame(z = z, y = y, x1 = x1, x2 = x2, x3 = x3)
            
# ---
# Step 2
# Estimation of the parameters
# ---

# Define the groups
groups3 <- c(0, 1, 2)
groups2 <- matrix(c(-1, 0, 1), ncol = 1)

# Two-step method
model <- msel(formula3  = z ~ x1 + x2, formula2 = y ~ x1 + x3, 
              groups3   = groups3,     groups2   = groups2,
              data      = data,        estimator = "2step",
              type3     = "probit")
summary(model)

# Get the coefficients of the selectivity terms
coef_lambda0_est <- coef(model, type = "coef_lambda", eq2 = 1, regime = 0)
coef_lambda1_est <- coef(model, type = "coef_lambda", eq2 = 1, regime = 1)

# Compare the estimates and true values of the parameters
  # regression coefficients of the continuous equation
beta0_est <- coef(model, type = "coef2", eq2 = 1, regime = 0)
beta1_est <- coef(model, type = "coef2", eq2 = 1, regime = 1)
cbind(true = beta0, est = beta0_est[1:length(beta0)])
cbind(true = beta1, est = beta1_est[1:length(beta1)])
  # regression coefficients of the multinomial equations
gamma0_est <- coef(model, type = "coef3", eq3 = 0)
gamma1_est <- coef(model, type = "coef3", eq3 = 1)
cbind(true = gamma0, est = gamma0_est)
cbind(true = gamma1, est = gamma1_est)
  # compare the covariances between
  # z1 and z2
cbind(true = cor.z * sd.z2, 
      est  = coef(model, type = "cov3", eq3 = c(0, 1)))
  # z1 and y0
cbind(true = cor.z1y0 * sd.y0, 
      est  = coef_lambda0_est["lambda1_mn"]) 
  # z2 and y0
cbind(true = cor.z2y0 * sd.y0, 
      est  = coef_lambda0_est["lambda2_mn"])   
  # z1 and y1
cbind(true = cor.z1y1 * sd.y1, 
      est  = coef_lambda1_est["lambda1_mn"]) 
  # z2 and y1
cbind(true = cor.z2y1 * sd.y1, 
      est  = coef_lambda1_est["lambda2_mn"]) 
            
# ---
# Step 3
# Predictions and marginal effects
# ---  

# Unconditional expectation E(y1 | w) for every observation in the sample
predict(model, type = "val", group2 = 1, group3 = -1)

# The marginal effect of x1 on conditional expectation E(y0 | z = 1, w) 
# for every observation in a sample
predict(model, type = "val", group2 = 0, group3 = 1, me = "x1")    

# Calculate predictions and marginal effects
# for manually provided observations
# using the aforementioned models
newdata <- data.frame(z  = c(1, 1), 
                      y  = c(1, 1), 
                      x1 = c(0.5, 0.2), 
                      x2 = c(-0.3, 0.8),
                      x3 = c(0.6, -0.7))
                      
# Unconditional expectation E(y0 | w)
predict(model, type = "val", group2 = 0, group3 = -1, newdata = newdata)

# Conditional expectation E(y1 | z=2, w)
predict(model, type = "val", group2 = 1, group3 = 2, newdata = newdata)

# Marginal effect of x2 on E(y0 | z = 1, w)
predict(model, type = "val", group2  = 0, group3 = 1, 
               me   = "x2",  newdata = newdata)
  
# ---
# Step 4
# Multinomial logit selection
# ---                
        
# Two-step method
model2 <- msel(formula3  = z ~ x1 + x2, formula2 = y ~ x1 + x3, 
               groups3   = groups3,     groups2   = groups2,
               data      = data,        estimator = "2step",
               type3     = "logit")
summary(model2)    

# Compare the estimates
beta0_est2 <- coef(model2, type = "coef2", eq2 = 1, regime = 0)[]
beta1_est2 <- coef(model2, type = "coef2", eq2 = 1, regime = 1)

# beta0 coefficients
cbind(true = beta0, probit = beta0_est[1:3], logit = beta0_est2[1:3])
   
# beta1 coefficients
cbind(true = beta1, probit = beta1_est[1:3], logit = beta1_est2[1:3])            
}
             
\donttest{
# ---------------------------------------
# Simulated data example 9
# Multinomial endogenous switching and 
# selection model (logit)
# ---------------------------------------

# ---
# Step 1
# Simulation of the data
# ---

# Set seed for reproducibility
set.seed(123)

# The number of observations
n <- 10000

# Random errors
u      <- matrix(-log(-log(runif(n * 3))), nrow = n, ncol = 3)
tau0   <- matrix(c(0.6, -0.4, 0.3), ncol = 1)
tau1   <- matrix(c(-0.3, 0.5, 0.2), ncol = 1)
eps0   <- (u - 0.57721656649) \%*\% tau0 + rnorm(n)
eps1   <- (u - 0.57721656649) \%*\% tau1 + rnorm(n)

# Regressors (covariates)
x1 <- runif(n = n, min = -1, max = 1)
x2 <- runif(n = n, min = -1, max = 1)
x3 <- runif(n = n, min = -1, max = 1)

# Coefficients
gamma.0 <- c(0.2, -2,  2)
gamma.1 <- c(0.1,  2, -2)
beta.0  <- c(2,    2,  2)
beta.1  <- c(1,   -2,  2)

# Linear predictors (indexes)
z0.li <- gamma.0[1] + gamma.0[2] * x1 + gamma.0[3] * x2
z1.li <- gamma.1[1] + gamma.1[2] * x1 + gamma.1[3] * x2

# Latent variables
z0.star <- z0.li + u[, 1]
z1.star <- z1.li + u[, 2]
z2.star <- u[, 3]
y0.star <- beta.0[1] + beta.0[2] * x1 + beta.0[3] * x3 + eps0
y1.star <- beta.1[1] + beta.1[2] * x1 + beta.1[3] * x3 + eps1

# Observable multinomial variable
z                                            <- rep(2, n)
z[(z0.star > z1.star) & (z0.star > z2.star)] <- 0
z[(z1.star > z0.star) & (z1.star > z2.star)] <- 1
table(z)

# Unobservable values of the continuous outcome
y         <- rep(NA, n)
y[z == 0] <- y0.star[z == 0]
y[z == 1] <- y1.star[z == 1]

# Data
data <- data.frame(x1 = x1, x2 = x2, x3 = x3, z = z, y = y)
            
# ---
# Step 2
# Estimation of the parameters
# ---

# Define the groups
groups3 <- c(0, 1, 2)
groups2 <- c(0, 1, -1)

# Two-step estimator of Dubin-McFadden
model <- msel(formula3  = z ~ x1 + x2, formula2 = y ~ x1 + x3, 
              groups3   = groups3,     groups2   = groups2,
              data      = data,        estimator = "2step",
              type3     = "logit")
summary(model)

# Least squares estimates (benchmark)
lm0 <- lm(y ~ x1 + x3, data = data[data$z == 0, ])
lm1 <- lm(y ~ x1 + x3, data = data[data$z == 1, ])


# Compare the estimates of beta0
cbind(true = beta.0,
      DMF  = coef(model, type = "coef2", eq2 = 1, regime = 0),
      LS   = coef(lm0))
      
# Compare the estimates of beta1
cbind(true = beta.1,
      DMF  = coef(model, type = "coef2", eq2 = 1, regime = 1),
      LS   = coef(lm1))
}
             
}
\references{
W. K. Newey (2009). 
Two-step series estimation of sample selection models.
The Econometrics Journal, vol. 12(1), pages 217-229.

E. Kossova, B. Potanin (2018). 
Heckman method and switching regression model multivariate generalization.
Applied Econometrics, vol. 50, pages 114-143.

E. Kossova, L. Kupriianova, B. Potanin (2020). 
Parametric and semiparametric multivariate sample selection models 
estimators' accuracy: Comparative analysis on simulated data.
Applied Econometrics, vol. 57, pages 119-139.

E. Kossova, B. Potanin (2022). 
Estimation of Gaussian multinomial endogenous switching model.
Applied Econometrics, vol. 67, pages 121-143.

E. Meijer, T. Wansbeek (2007). 
The sample selection model from a method of moments perspective.
Econometric Reviews, vol. 26(1), pages 25-51.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{struct_msel}
\alias{struct_msel}
\title{Structure of the Object of Class msel}
\usage{
struct_msel(x)
}
\arguments{
\item{x}{object of class 'msel'}
}
\value{
The function returns a numeric matrix whose columns are 
\code{groups}, \code{groups2}, and \code{groups3}, respectively. 
It also has additional (last) column with the number of observations 
associated with the corresponding combinations of the groups.
}
\description{
Prints information on the structure of the model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrtest.R
\name{print.summary.lrtest_msel}
\alias{print.summary.lrtest_msel}
\title{Print Summary Method for Likelihood Ratio Test}
\usage{
\method{print}{summary.lrtest_msel}(x, ...)
}
\arguments{
\item{x}{object of class "lrtest_msel"}

\item{...}{further arguments (currently ignored)}
}
\value{
The function returns input argument \code{x} changing
its class to \code{lrtest_msel}.
}
\description{
Prints summary for an object of class 'lrtest_msel'.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sigma.R
\name{sigma.msel}
\alias{sigma.msel}
\title{Extract Residual Standard Deviation 'Sigma'}
\usage{
\method{sigma}{msel}(object, use.fallback = TRUE, ..., regime = NULL, eq2 = NULL)
}
\arguments{
\item{object}{object of class "msel".}

\item{use.fallback}{logical, passed to \code{nobs} (currently ignored).}

\item{...}{further arguments (currently ignored).}

\item{regime}{regime of continuous equation}

\item{eq2}{index of continuous equation}
}
\value{
Returns estimates of the standard deviations 
of \eqn{\varepsilon_{i}}. 
If \code{eq2 = k} then estimates only for \eqn{k}-th continuous equation are 
returned. If in addition \code{regime = r} then estimate of 
\eqn{\sqrt{Var(\varepsilon_{ri})}} is returned. 
Herewith if \code{regime} is not \code{NULL} and \code{eq2} is \code{NULL} 
it is assumed that \code{eq2 = 1}.
}
\description{
Extract standard deviations of random errors of continuous
equations of \code{\link[switchSelection]{msel}} function.
}
\details{
Available only if \code{estimator = "ml"} or all \code{degrees}
values are equal to \code{1}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vcov.R
\name{vcov.msel}
\alias{vcov.msel}
\title{Calculate Variance-Covariance Matrix for a msel Object.}
\usage{
\method{vcov}{msel}(
  object,
  ...,
  type = object$cov_type,
  n_cores = object$other$n_cores,
  n_sim = object$other$n_sim,
  recalculate = FALSE,
  cluster = NULL
)
}
\arguments{
\item{object}{an object of class \code{msel}.}

\item{...}{further arguments (currently ignored).}

\item{type}{character representing the type of the asymptotic covariance 
matrix estimator. It takes the same values as \code{cov_type} parameter of
the \code{\link[switchSelection]{msel}} function.}

\item{n_cores}{positive integer representing the number of CPU cores used for 
parallel computing. If possible it is highly recommended to set it equal to
the number of available physical cores especially when the system of
ordered equations has 2 or 3 equations.}

\item{n_sim}{integer representing the number of GHK draws when there are
more than 3 ordered equations. Otherwise alternative (much more efficient) 
algorithms will be used to calculate multivariate normal probabilities.}

\item{recalculate}{logical; if \code{TRUE} then covariance matrix will be
recalculated even if 'type' is the same as 'cov_type' input argument
of the model.}

\item{cluster}{an object which takes the same values as the \code{cluster} 
argument of the \code{\link[switchSelection]{msel}} function. 
The only exception is \code{cluster = NULL} which implies 
\code{cluster = object$cluster}.}
}
\value{
Returns numeric matrix which represents estimate of the asymptotic 
covariance matrix of model's parameters. If \code{object} is a list of
models then rows and columns of this matrix will have names \code{"mipj"}
indicating that corresponding element is associated with \code{j}-th 
parameter of the \code{i}-th model.
}
\description{
Return the variance-covariance matrix of the parameters of
msel model.
}
\details{
Argument \code{type} is closely related to the argument 
\code{cov_type} of \code{\link[switchSelection]{msel}} function. 
See 'Details' and 'Usage' sections of \code{\link[switchSelection]{msel}} 
for more information on \code{cov_type} argument.

The user may also estimate asymptotic covariance matrix of the parameters
of several models. For more information, see paragraph 'Cross-model tests'
in 'Details' section of \code{\link[switchSelection]{test_msel}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coef.R
\name{coef.msel}
\alias{coef.msel}
\title{Coefficients extraction method for msel.}
\usage{
\method{coef}{msel}(
  object,
  ...,
  eq = NULL,
  eq2 = NULL,
  eq3 = NULL,
  regime = NULL,
  type = "coef"
)
}
\arguments{
\item{object}{an object of class "msel".}

\item{...}{further arguments (currently ignored).}

\item{eq}{an integer representing the index of the ordered equation.}

\item{eq2}{an integer representing the index of the continuous equation.}

\item{eq3}{an integer representing the index of the alternative of the
multinomial equation.}

\item{regime}{an integer representing a regime of the continuous equation.}

\item{type}{a character representing a type of the output. Possible options
are \code{"coef"}, \code{"coef2"}, \code{"coef_lambda"}, 
\code{"coef_var"}, \code{"coef3"}, \code{"cuts"}, \code{"cov"}, 
\code{"cov1"}, \code{"cov12"}, \code{"var"}, \code{"cov2"}, \code{"cov3"}, 
and \code{"marginal"}.
See 'Details' for additional information.}
}
\value{
See 'Details' section.
}
\description{
Extract coefficients and other estimates from msel object.
}
\details{
Consider the notations from the 'Details' section of 
\code{\link[switchSelection]{msel}}.

\strong{Mean coefficients of the ordinal equations}

Suppose that \code{type = "coef"}. Then estimates of the \eqn{\gamma_{j}} 
coefficients are returned for each \eqn{j\in\{1,...,J\}}. 
If \code{eq = j} then only estimates of the \eqn{\gamma_{j}} coefficients 
are returned.

\strong{Variance coefficients of the ordinal equations}

Suppose that \code{type = "coef_var"}. Then estimates of the
\eqn{\gamma_{j}^{*}} coefficients are returned for each 
\eqn{j\in\{1,...,J\}}. If \code{eq = j} then only estimates of 
\eqn{\gamma_{j}^{*}} coefficients are returned.

\strong{Coefficients of the continuous equations}

Suppose that \code{type = "coef2"}. Then estimates of the \eqn{\beta_{r}} 
coefficients are returned for each \eqn{r\in\{0,...,R - 1\}}. 
If \code{eq2 = k} then only estimates for the \eqn{k}-th continuous equation 
are returned. If \code{regime = r} then estimates of the \eqn{\beta_{r}}
coefficients are returned for the \code{eq2}-th continuous equation. 
Herewith if \code{regime} is not \code{NULL} and \code{eq2} is \code{NULL} 
it is assumed that \code{eq2 = 1}.

\strong{Selectivity terms}

Suppose that \code{type = "coef_lambda"}. Then estimates of the coefficients
associated with the selectivity terms are returned for each 
\eqn{r\in\{0,...,R - 1\}}. If \code{eq2 = k} then only estimates for the 
\eqn{k}-th continuous equation are returned. If \code{regime = r} then 
estimates of the coefficients of the selectivity terms are returned for the 
\code{eq2}-th continuous equation.

\strong{Thresholds of the ordinal equations}

Suppose that \code{type = "cuts"} or \code{type = "thresholds"}. Then 
estimates of the \eqn{c_{j}} cuts (thresholds) are returned for each 
\eqn{j\in\{1,...,J\}}. If \code{eq = j} then only estimates of the 
\eqn{c_{j}} cuts are returned.


\strong{Covariances between the random errors of the ordinal equations}

Suppose that \code{type = "cov1"}. Then the estimate of the covariance matrix 
of \eqn{u_{i}} is returned. If \code{eq = c(a, b)} then the function returns
\eqn{(a, b)}-th element of this matrix i.e. an element from the
\code{a}-th row and the \code{b}-th column which represents an estimate of
\eqn{Cov(u_{ai}, u_{bi})}.

\strong{Covariances between the random errors of the ordinal and continuous 
equations}

Suppose that \code{type = "cov12"}. Then estimates of the covariances between
random errors of the ordinal \eqn{u_{i}} and continuous \eqn{\varepsilon_{i}} 
equations are returned. If \code{eq2 = k} then covariances with random errors 
of the \code{k}-th continuous equation are returned. If in addition 
\code{eq = j} and \code{regime = r} then the function returns an estimate of 
\eqn{Cov(u_{ji}, \varepsilon_{ri})} for the \code{k}-th continuous equation. 
If \code{eq2 = NULL} it is assumed that \code{eq2 = 1}.

\strong{Variances of the random errors of the continuous equations}

Suppose that \code{type = "var"}. Then estimates of the variances of 
\eqn{\varepsilon_{i}} are returned. If \code{eq2 = k} then estimates only for
the \eqn{k}-th continuous equation are returned. If in addition 
\code{regime = r} then the estimate of the \eqn{Var(\varepsilon_{ri})} is 
returned. Herewith if \code{regime} is not \code{NULL} and \code{eq2} is 
\code{NULL} it is assumed that \code{eq2 = 1}.

\strong{Covariances between the random errors of the continuous equations}

Suppose that \code{type = "cov2"}. Then estimates of the covariances between
random errors of different continuous equations in different regimes are 
returned. If \code{eq2 = c(a, b)} and \code{regime = c(c, d)} then the 
function returns an estimate of the covariance of random errors of the 
\code{a}-th and \code{b}-th continuous equations in the regimes 
\code{c} and \code{d} correspondingly. If this covariance is not 
identifiable then an \code{NA} value is returned.

\strong{Coefficients of the multinomial equation}

Suppose that \code{type = "coef3"}. Then estimates of the 
\eqn{\tilde{\gamma}_{j}} coefficients are returned for each 
\eqn{j\in\{0,...,\tilde{J} - 1\}}. If \code{eq3 = j} then only estimates of 
the \eqn{\tilde{\gamma}_{j}} coefficients are returned.

\strong{Covariances between the random errors of the multinomial equations}

Suppose that \code{type = "cov3"}. Then the estimate of the covariance matrix 
of \eqn{\tilde{u}_{i}} is returned. If \code{eq3 = c(a, b)} then the function 
returns \eqn{(a, b)}-th element of this matrix i.e. an element from the
\code{a}-th row and the \code{b}-th column which represents an estimate of
\eqn{Cov(\tilde{u}_{(a+1)i}, \tilde{u}_{(b+1)i})}.

\strong{Parameters of the marginal distributions}

Suppose that \code{type = "marginal"}. Then a list is returned whose
\code{j}-th element is a numeric vector of estimates of the parameters
of the marginal distribution of \eqn{u_{ji}}.

\strong{Asymptotic covariance matrix}

Suppose that \code{type = "cov"}. Then the estimate of the asymptotic 
covariance matrix of the estimator is returned. Note that this estimate 
depends on the \code{cov_type} argument of the 
\code{\link[switchSelection]{msel}} function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{print.summary.msel}
\alias{print.summary.msel}
\title{Print summary for an Object of Class msel}
\usage{
\method{print}{summary.msel}(x, ...)
}
\arguments{
\item{x}{object of class 'msel'}

\item{...}{further arguments (currently ignored)}
}
\value{
The function returns \code{x}.
}
\description{
Prints summary for an object of class 'msel'.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrtest.R
\name{lrtest_msel}
\alias{lrtest_msel}
\title{Likelihood ratio test}
\usage{
lrtest_msel(model1, model2)
}
\arguments{
\item{model1}{the first model.}

\item{model2}{the second model.}
}
\value{
The function returns an object of class \code{'lrtest_msel'} that is
a list with the following elements:
\itemize{
\item \code{n1} - the number of observations in the first model.
\item \code{n2} - the number of observations in the second model.
\item \code{ll1} - log-likelihood value of the first model.
\item \code{ll2} - log-likelihood value of the second model.
\item \code{df1} - the number of parameters in the first model.
\item \code{df2} - the number of parameters in the second model.
\item \code{restrictions} - the number of restrictions in the nested model.
\item \code{value} - chi-squared (likelihood ratio) test statistic value.
\item \code{p_value} - p-value of the chi-squared (likelihood ratio) test.
}
}
\description{
This function performs chi-squared test for nested models.
}
\details{
Arguments \code{model1} and \code{model2} should be objects
of class that has implementations of 
\code{\link[stats]{logLik}} and 
\code{\link[stats]{nobs}} methods. It is assumed that either \code{model1}
is nested into \code{model2} or vice versa. More precisely it is assumed
that the model with smaller log-likelihood value is nested into the model
with greater log-likelihood value.

Arguments \code{model1} and \code{model2} may be the lists of models.
If \code{model1} is a list of models then it is assumed that the number
of degrees of freedom and log-likelihood of the first model are just a sum
of degrees of freedom and log-likelihoods of the models in this list.
Similarly for \code{model2}. 

If \code{model1} or \code{model2} is a list then the number of observations 
of the associated models is calculated as the sum of the numbers of 
observations of the models in corresponding lists.
However sometimes it may be misleading. For example, when bivariate probit
model (full) is tested against two independent probit models (restricted). 
Then it will be assumed that the number of observations in the restricted
model is twice the number of observations in the full model that is not the
case. 
Fortunately it will not affect the results of the likelihood ratio test.
}
\examples{
set.seed(123)
# Generate data according to linear regression
n   <- 100
eps <- rnorm(n)
x1  <- runif(n)
x2  <- runif(n)
y   <- x1 + 0.2 * x2 + eps
# Estimate full model
model1 <- lm(y ~ x1 + x2)
# Estimate restricted (nested) model
model2 <- lm(y ~ x1)
# Likelihood ratio test results
lrtest_msel(model1, model2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.R
\name{test_msel}
\alias{test_msel}
\title{Tests and confidence intervals for the parameters estimated by 
the msel function}
\usage{
test_msel(
  object,
  fn,
  fn_args = list(),
  test = "t",
  method = "classic",
  ci = "classic",
  cl = 0.95,
  se_type = "dm",
  trim = 0,
  vcov = NULL,
  iter = 100,
  generator = rnorm,
  bootstrap = NULL,
  par_ind = NULL,
  eps = max(1e-04, sqrt(.Machine$double.eps) * 10),
  n_sim = 1000,
  n_cores = 1
)
}
\arguments{
\item{object}{an object of class 'msel'. It also may be a list of two 
objects. Then \code{object[[1]]} and \code{object[[2]]} are supplied to the
arguments \code{model1} and \code{model2} of the 
\code{\link[switchSelection]{lrtest_msel}} function.}

\item{fn}{a function which returns a numeric vector and should depend on the 
elements of \code{object}. These elements should be accessed via 
\code{\link[switchSelection]{coef.msel}} or 
\code{\link[switchSelection]{predict.msel}} functions.
The first argument of \code{fn} should be an \code{object}.
Therefore \code{coef} and \code{predict} functions in \code{fn} should also
depend on \code{object}.}

\item{fn_args}{a list of additional arguments of \code{fn}.}

\item{test}{a character representing the test to be used.
If \code{test = "t"} then t-test is used.
If \code{test = "wald"} then Wald test is applied.}

\item{method}{a character representing a method used to conduct a test.
If \code{test = "t"} or \code{test = "wald"} and \code{method = "classic"} 
then p-values are calculated by using the quantiles of the standard normal 
distribution.
If \code{test = "t"} or \code{test = "wald"} and \code{method = "bootstrap"} 
then p-values are calculated by using the bootstrap as described 
in Hansen (2022).
If \code{test = "wald"} and \code{method = "score"} then score 
bootstrap Wald test of P. Kline and A. Santos (2012) is used.}

\item{ci}{a character representing the type of the confidence interval
used. Available only if \code{test = "t"}.
If \code{ci = "classic"} then quantiles of the standard normal distribution
are used to build an asymptotic confidence interval.
If \code{ci = "percentile"} then percentile bootstrap interval is applied.
If \code{ci = "bc"} then the function constructs a bias-corrected 
percentile bootstrap confidence interval of Efron (1982) as described in
Hansen (2022).}

\item{cl}{a numeric value between \code{0} and \code{1} representing
a confidence level of the confidence interval.}

\item{se_type}{a character representing a method used to estimate 
the standard errors of the outputs of \code{fn}. 
If \code{se_type = "dm"} then delta method is used.
If \code{se_type = "bootstrap"} then bootstrap is applied.}

\item{trim}{a numeric value between \code{0} and \code{1} representing
the share of bootstrap estimates to be nullified when standard errors are
estimated for \code{se_type = "bootstrap"}.}

\item{vcov}{an estimate of the asymptotic covariance matrix of the 
parameters of the model.}

\item{iter}{the number of iterations used by the score bootstrap Wald test.}

\item{generator}{a function which is used by the score bootstrap to generate
random weights. It should have an argument \code{n} representing the
number of random weights to generate. Other arguments are ignored.}

\item{bootstrap}{an object of class \code{'bootstrap_msel'} which is an 
output of the \code{\link[switchSelection]{bootstrap_msel}} function.
This object is used to retrieve the estimates of the bootstrap samples.}

\item{par_ind}{a vector of indexes of the model parameters used 
in the calculation of \code{fn}. If only necessary indexes are included then
in some cases estimation time may greatly decrease. Set \code{ind = TRUE}
in \code{\link[switchSelection]{summary.msel}} to see the indexes of the 
model parameters. If \code{eps} is a vector then \code{eps[i]} determines 
the increment used to differentiate \code{fn} with respect to the parameter
with \code{par_ind[i]}-th index.}

\item{eps}{a positive numeric value representing the increment used for the
numeric differentiation of \code{fn}. It may also be a numeric vector
such that \code{eps[i]} is an increment used to differentiate the 
\code{fn} with respect to the \code{par_ind[i]}-th parameter of the model.
Set \code{ind = TRUE} in \code{\link[switchSelection]{summary.msel}}, to see 
the indexes of the model parameters. If \code{eps[i] = 0} then derivative of 
\code{fn} with respect to \code{par_ind[i]}-th parameter is assumed 
to be zero.}

\item{n_sim}{the value passed to the \code{n_sim} argument of the
\code{\link[switchSelection]{msel}} function.}

\item{n_cores}{the value passed to the \code{n_cores} argument of the
\code{\link[switchSelection]{msel}} function.}
}
\value{
This function returns an object of class \code{'test_msel'} which is
a list. It may have the following elements:
\itemize{
\item \code{tbl} - a list with the elements described below.
\item \code{is_bootstrap} - a logical value which equals \code{TRUE} if 
bootstrap has been used.
\item \code{is_ci} - a logical value which equals \code{TRUE} if
confidence intervals were used.
\item \code{test} - the same as the input argument \code{test}.
\item \code{method} - the same as the input argument \code{method}.
\item \code{se_type} - the same as the input argument \code{se_type}.
\item \code{ci} - the same as the input argument \code{ci}.
\item \code{cl} - the same as the input argument \code{cl}.
\item \code{iter} - the same as the input argument \code{iter}.
\item \code{n_bootstrap} - an integer representing the number of 
bootstrap iterations used.
\item \code{n_val} - the length of the vector returned by \code{fn}.
}

A list \code{tbl} may have the following elements:
\itemize{
\item \code{val} - an output of the \code{fn} function.
\item \code{se} - a numeric vector such that \code{se[i]} represents 
a standard error associated with \code{val[i]}.
\item \code{p_value} - a numeric vector of p-values.
\item \code{lwr} - a numeric vector such that \code{lwr[i]} is the 
realization of the lower (left) bound of the confidence interval for the
true value of \code{val[i]}.
\item \code{upr} - a numeric vector such that \code{upr[i]} is the 
realization of the upper (right) bound of the confidence interval for the
true value of \code{val[i]}.
\item \code{stat} - a numeric vector of values of the test statistics.
}

An object of class \code{'test_msel'} has an implementation of the
\code{summary} method
\code{\link[switchSelection]{summary.test_msel}}.

In a special case when \code{test = "lr"} and \code{object} is a list of 
length \code{2} the function returns an object of class \code{'lrtest_msel'} 
since the function \code{\link[switchSelection]{lrtest_msel}} is called 
internally.
}
\description{
This function conducts various statistical tests and calculates
confidence intervals for the parameters of the model estimated via the 
\code{\link[switchSelection]{msel}} function.
}
\details{
Suppose that \eqn{\theta} is a vector of parameters of the model 
estimated via the \code{\link[switchSelection]{msel}} function and 
\eqn{g(\theta)} is a differentiable function representing \code{fn} which 
returns a \eqn{m}-dimensional vector of real values:
\deqn{g(\theta) = (g_{1}(\theta),...,g_{m}(\theta))^{T}.}

\strong{Classic and bootstrap t-test}
 
If \code{test = "t"} then for each \eqn{j\in \{1,...,m\}} the following 
hypothesis is tested:
\deqn{H_{0}:g_{j}(\theta) = 0,\qquad H_{1}:g_{j}(\theta)\ne 0.}

The test statistic is: 
\deqn{T = g_{j}(\hat{\theta})/\hat{\sigma}_{j},}

where \eqn{\hat{\sigma}} is a standard error of \eqn{g_{j}(\hat{\theta})}.

If \code{se_type = "dm"} then delta method is used to estimate 
this standard error:

\deqn{\hat{\sigma}_{j} = \sqrt{\nabla g_{j}(\hat{\theta})^{T} 
\widehat{As.Cov}(\hat{\theta})
\nabla g_{j}(\hat{\theta})},
}

where \eqn{\nabla g_{j}(\hat{\theta})} is a gradient as a column vector and
the estimate of the asymptotic covariance matrix of the estimates 
\eqn{\widehat{As.Cov}(\hat{\theta})} is provided via the \code{vcov} 
argument. Numeric differentiation is used to calculate 
\eqn{\nabla g_{j}(\hat{\theta})}.

If \code{se_type = "bootstrap"} then bootstrap is 
applied to estimate the standard error:
\deqn{\hat{\sigma}_{j} = \sqrt{\frac{1}{B - 1}\sum\limits_{b = 1}^{B} 
(g_{j}(\hat{\theta}^{(b)}) - \overline{g_{j}(\hat{\theta}^{(b)}}))^2},}

where \eqn{B} is the number of the bootstrap iterations 
\code{bootstrap$iter}, \eqn{\hat{\theta}^{(b)}} is the estimate 
associated with the \eqn{b}-th of these iterations \code{bootstrap$par[b, ]},
and \eqn{\overline{g_{j}(\hat{\theta}^{(b)})}} is a sample mean of the 
bootstrap estimates estimates:

\deqn{\overline{g_{j}(\hat{\theta}^{(b)}}) = 
\frac{1}{B}\sum\limits_{b = 1}^{B}g_{j}(\hat{\theta}^{(b)}).}

If \code{method = "classic"} it is assumed that if the null hypothesis is 
true then the asymptotic distribution of the test statistic is standard 
normal. This distribution is used for the calculation of the p-value:

\deqn{p-value = 2\min(\Phi(T), 1 - \Phi(T)),}

where \eqn{\Phi()} is a cumulative distribution function of the standard
normal distribution.

If \code{method = "bootstrap"} then p-value is calculated via the bootstrap 
as suggested by Hansen (2022):
\deqn{
p-value = \frac{1}{B}\sum\limits_{b=1}^{B} I(|T_{b} - T| > |T|),
}

where \eqn{T_{b} = g_{j}(\hat{\theta}^{(b)})/\hat{\sigma}_{j}} is the value of 
the test statistic estimated on the \code{b}-th bootstrap sample and 
\eqn{I(q)} is an indicator function which equals \eqn{1} when \eqn{q} is a 
true statement and \eqn{0} - otherwise.

\strong{Classic and bootstrap Wald test}

Suppose that \code{method = "classic"} or \code{method = "bootstrap"}.
If \code{test = "wald"} then the null hypothesis is:
\deqn{H_{0}:
\begin{cases}
g_{1}(\theta) = 0\\
g_{2}(\theta) = 0\\
\vdots\\
g_{m}(\theta) = 0\\
\end{cases}.
}

The alternative hypothesis is that there is such \eqn{j\in\{1,...,m\}} that:
\deqn{H_{1}:g_{j}(\theta)\ne 0.}

The test statistic is:
\deqn{T = g(\hat{\theta})^{T}\widehat{As.Cov}(g(\hat{\theta}))^{-1}
g(\hat{\theta}),}

where \eqn{\widehat{As.Cov}(g(\hat{\theta}))} is the estimate of the 
asymptotic covariance matrix of \eqn{g(\hat{\theta})}. 

If \code{se_type = "dm"} then delta method is used to estimate this matrix:
\deqn{\widehat{As.Cov}(g(\hat{\theta})) 
= g'(\hat{\theta})\widehat{As.Cov}(\hat{\theta}) g'(\hat{\theta})^{T},}

where \eqn{g'(\hat{\theta})} is a Jacobian matrix. Numeric differentiation
is used to calculate its elements:
\deqn{g'(\hat{\theta})_{ij} = 
\frac{\partial g_{i}(\theta)}{\partial \theta_{j}}|_{\theta = \hat{\theta}}.}

If \code{se_type = "bootstrap"} then bootstrap is used to estimate this 
matrix:
\deqn{\widehat{As.Cov}(g(\hat{\theta}))=
\frac{1}{B-1}\sum\limits_{i=1}^{B}q_{b}q_{b}^{T},}
where:
\deqn{q_{b} = (g(\hat{\theta}^{(b)}) - 
\overline{g(\hat{\theta}^{(b)})}),}
\deqn{\overline{g(\hat{\theta}^{(b)})} = \frac{1}{B}\sum\limits_{i=1}^{B}
g(\hat{\theta}^{(b)}).}

If \code{method = "classic"} then it is assumed that if the null hypothesis 
is true then the asymptotic distribution of the test statistic is chi-squared
with \eqn{m} degrees of freedom. This distribution is used for the 
calculation of the p-value:

\deqn{p-value = 1 - F_{m}(T),}

where \eqn{F_{m}} is a cumulative distribution function of the chi-squared
distribution with \eqn{m} degrees of freedom.

If \code{method = "bootstrap"} then p-value is calculated via the bootstrap 
as suggested by Hansen (2022):
\deqn{
p-value = \frac{1}{B}\sum\limits_{b=1}^{B} I(T_{b} > T),
}
where:
\deqn{T_{b} = s_{b}^{T}\widehat{As.Cov}(g(\hat{\theta}))^{-1}s_{b},}
\deqn{s_{b} = g(\hat{\theta}^{(b)}) - g(\hat{\theta}).}

\strong{Score bootstrap Wald test}

If \code{method = "score"} and \code{test = "Wald"} then score bootstrap
Wald test of Kline and Santos (2012) is used.

Consider \eqn{B} independent samples of \eqn{n} independent identically 
distributed random weights with zero mean and unit variance. 
Let \eqn{w_{ib}} denote the \eqn{i}-th weight of the \eqn{b}-th sample. 
Argument \code{generator} is used to supply a function which generates these 
weights \eqn{w_{ib}} and \code{iter} argument represents \eqn{B}. 
Also \eqn{n} is the number of observations in the model 
\code{object$other$n_obs}.

Let \eqn{J} denote a matrix of sample scores \code{object$J}. 
Further, denote by \eqn{J_{b}} a matrix such that its \eqn{b}-th row is a 
product of the \eqn{w_{ib}} and the \eqn{b}-th row of \eqn{J}. 
Also, denote by \eqn{H} a matrix of mean values of the derivatives of 
sample scores with respect to the estimated parameters \code{object$H}.

In addition consider the following notations:
\deqn{A = g'(\theta) H^{-1}, \qquad S_{b} = A J^{(c)}_{b},}
where \eqn{J^{(c)}_{b}} is a vector of the column sums of \eqn{J_{b}}.

The test statistic is as follows:
\deqn{T = g(\hat{\theta})^{T}(A\widehat{Cov}(J)
A^{T})^{-1}g(\hat{\theta}) / n,}
where \eqn{\widehat{Cov}(J)} is a sample covariance matrix of the sample
scores of the model \code{cov(object$J)}.

The test statistic on the \eqn{b}-th bootstrap sample is similar:
\deqn{T_{b} = S^{T}(A\widehat{Cov}(J_{b})
A^{T})^{-1}S / n.}

The p-value is estimated as follows:
\deqn{
p-value = \frac{1}{B}\sum\limits_{b=1}^{B} I(T_{b} > T).
}

\strong{Confidence intervals}

If \code{test = "t"} then the function also returns the realizations of the 
lower and upper bounds of the \eqn{100 \times}\code{cl} percent symmetric 
asymptotic confidence interval of \eqn{g_{j}(\theta)}.

If \code{ci = "classic"} then classic confidence interval is used which
assumes asymptotic normality of \eqn{g_{j}(\hat{\theta})}:
\deqn{(g_{j}(\hat{\theta}) + z_{(1 - cl) / 2}\hat{\sigma}_{j}, 
g_{j}(\hat{\theta}) + z_{1 - (1 - cl) / 2}\hat{\sigma}_{j}),}
where \eqn{z_{q}} is the \eqn{q}-th quantile of the standard normal 
distribution and \eqn{cl} is a confidence level \code{cl}. The method used 
to estimate \eqn{\hat{\sigma}_{j}} depends on the \code{se_type} argument as 
described above.

If \code{ci = "percentile"} then percentile bootstrap confidence interval 
is used. Therefore the sample quantiles of \eqn{g_{j}(\hat{\theta}^{(b)})} 
are used as the realizations of the lower and upper bounds of the confidence 
interval.

If \code{ci = "bc"} then bias-corrected percentile bootstrap confidence 
interval of Efron (1982) is used as described in Hansen (2022). The default
percentile bootstrap confidence interval uses sample quantiles of levels 
\eqn{(1 - cl)/2} and \eqn{1 - (1 - cl)/2}. 
Bias-corrected version uses the sample quantiles of the
following levels:
\deqn{(1 - cl)/2 + \Phi(\Phi^{-1}((1 - cl)/2) + s),}
\deqn{1 - (1 - cl)/2 + \Phi(\Phi^{-1}(1 - (1 - cl)/2) + s),}
where:
\deqn{s = 2\Phi^{-1}(\frac{1}{B}\sum\limits_{b = 1}^{B}
I(g_{j}(\hat{\theta}^{(b)})\leq g_{j}(\hat{\theta}))).}

\strong{Trimming}

If \code{se_type = "bootstrap"} and \code{trim > 0} then trimming is used as
described in Hansen (2022) to estimate \eqn{\hat{\sigma}_{j}} and 
\eqn{\widehat{As.Cov}(g(\hat{\theta}))}. The algorithm is as follows. 
First, nullify 100\code{trim} percent of \eqn{g(\hat{\theta}^{(b)})} with the 
greatest values of the L2-norm of \eqn{q_{b}} (defined above). 
Then use this 'trimmed' sample to estimate the standard error and the 
asymptotic covariance matrix.

\strong{Cross-model tests}

It is also possible to test the hypothesis on the parameters estimated
via several different models. To do so it is necessary to provide a list of 
models (objects of class \code{msel}) via the \code{object} argument. 
Particularly, cross-model tests are useful for comparing marginal and 
treatment effects among the different models. In addition, cross-model
tests may be used for testing over-identifying restrictions.

If \code{se_type = "dm"} then the joint covariance matrix of the estimators 
is obtained via the stacking method similar to one implemented by 
\code{suest} postestimation command in \code{STATA}. Note that there are 
the following requirements:
\itemize{
\item Covariance matrix type \code{cov_type} of each \code{object[[i]]} 
should be either \code{"mm"} or \code{"sandwich"}. 
\item Argument \code{data}  of each \code{object[[i]]} should contain a 
special variable \code{crossind}. Suppose that each \code{object[[i]]$data} 
is a subsample of some \code{data0}. Then \code{crossind} argument should 
take different values in range between \code{1} and \code{nrow(data0)}. 
In addition, observations with the same value of \code{crossind} in 
\code{object[[i]]$data} should correspond to the same observation 
in \code{data0}. A simple way to create \code{crossind} variable is just to
set it equal to \code{1:nrow(data0)} in \code{data0} before splitting it
into subsamples used by the models.
\item Argument \code{cluster} of each \code{object[[i]]} is ignored. However,
clustering may be provided via \code{crosscluster} variable in 
\code{object[[i]]$data}. Then observations with the same value of 
\code{crosscluster} are considered to be from the same cluster.
}

Arguments \code{eps} and \code{par_ind} may be lists whose elements are 
associated with the corresponding models. If \code{eps} is a scalar, then the 
same increment will be used for the numeric differentiation with respect to 
parameters of all models. Also, by default indexes of all parameters of all 
the models will be included in \code{par_ind}.
}
\examples{
\donttest{

# -------------------------------
# CPS data example
# -------------------------------

# Set seed for reproducibility
set.seed(123)

# Load the data
data(cps)

# Estimate the employment model
model <- msel(work ~ age + I(age ^ 2) + bachelor + master, data = cps)
summary(model)    

# Use Wald test to test the hypothesis that age has no 
# effect on the conditional probability of employment:
# H0: coef age     = 0
#     coef age ^ 2 = 0
age_fn <- function(object)
{
  lwage_coef <- coef(object, type = "coef")[[1]]
  val        <- c(lwage_coef["age"], lwage_coef["I(age^2)"])
  return(val)
}
age_test <- test_msel(object = model, fn = age_fn, test = "wald")
summary(age_test)

# Use t-test to test for each individual the hypothesis:
# P(work = 1 | x) = 0.8
prob_fn <- function(object)
{
  prob <- predict(object, group = 1, type = "prob")
  val  <- prob - 0.8
  return(val)
}
prob_test <- test_msel(object = model, fn = prob_fn, test = "t")
summary(prob_test)

# -------------------------------
# Simulated data example
# Model with continuous outcome
# and ordinal selection
# -------------------------------

# ---
# Step 1
# Simulation of the data
# ---

# Set seed for reproducibility
set.seed(123)

# Load required package
library("mnorm")

# The number of observations
n <- 10000

# Regressors (covariates)
s1 <- runif(n = n, min = -1, max = 1)
s2 <- runif(n = n, min = -1, max = 1)
s3 <- runif(n = n, min = -1, max = 1)
s4 <- runif(n = n, min = -1, max = 1)

# Random errors
sigma <- matrix(c(1,    0.4,  0.45, 0.7,
                  0.4,  1,    0.54, 0.8,
                  0.45, 0.54, 0.81, 0.81,
                  0.7,  0.8,  0.81, 1), nrow = 4)
errors <- mnorm::rmnorm(n = n, mean = c(0, 0, 0, 0), sigma = sigma)
u1   <- errors[, 1]
u2   <- errors[, 2]
eps0 <- errors[, 3]
eps1 <- errors[, 4]

# Coefficients
gamma1     <- c(-1, 2)
gamma2     <- c(1, 1)
gamma1_het <- c(0.5, -1)
beta0      <- c(1, -1, 1, -1.2)
beta1      <- c(2, -1.5, 0.5, 1.2)
# Linear index of the ordinal equations
# mean part
li1 <- gamma1[1] * s1 + gamma1[2] * s2
li2 <- gamma2[1] * s1 + gamma2[2] * s3
# variance part
li1_het <- gamma1_het[1] * s2 + gamma1_het[2] * s3

# Linear index of the continuous equation
# regime 0
li_y0 <- beta0[1] + beta0[2] * s1 + beta0[3] * s3 + beta0[4] * s4
# regime 1
li_y1 <- beta1[1] + beta1[2] * s1 + beta1[3] * s3 + beta1[4] * s4

# Latent variables
z1_star <- li1 + u1 * exp(li1_het)
z2_star <- li2 + u2
y0_star <- li_y0 + eps0
y1_star <- li_y1 + eps1

# Cuts
cuts1 <- c(-1)
cuts2 <- c(0, 1)

# Observable ordinal outcome
# first
z1                     <- rep(0, n)
z1[z1_star > cuts1[1]] <- 1
# second
z2                                               <- rep(0, n)
z2[(z2_star > cuts2[1]) & (z2_star <= cuts2[2])] <- 1
z2[z2_star > cuts2[2]]                           <- 2
z2[z1 == 0]                                      <- NA

# Observable continuous outcome
y                 <- rep(NA, n)
y[which(z2 == 0)] <- y0_star[which(z2 == 0)]
y[which(z2 != 0)] <- y1_star[which(z2 != 0)]
y[which(z1 == 0)] <- NA

# Data
data <- data.frame(s1 = s1, s2 = s2, s3 = s3, s4 = s4,
                   z1 = z1, z2 = z2, y = y,   crossind = 1:n)

# ---
# Step 2
# Estimation of the parameters
# ---

# Assign the groups
groups  <- matrix(c(1, 2,
                    1, 1,
                    1, 0,
                    0, -1), 
                  byrow = TRUE, ncol = 2)
groups2 <- matrix(c(1, 1, 0, -1), ncol = 1)

# Estimate the model
model <- msel(list(z1 ~ s1 + s2 | s2 + s3,
                   z2 ~ s1 + s3),
              list(y  ~ s1 + s3 + s4),
              groups = groups, groups2 = groups2, 
              data   = data)
                   
# ---
# Step 3
# Hypotheses testing
# ---

# Use t-test to test for each observation the hypothesis
# H0: P(z1 = 1, z2 = 0 | Xi) = 0 
prob02_fn <- function(object)
{
   val <- predict(object, group = c(1, 0))
   
   return(val)
}
prob02_test <- test_msel(object = model, fn = prob02_fn, test = "t")
summary(prob02_test)

# Use t-test to test the hypothesis
# H0: E(y1|z1=0, z2=2) - E(y0|z1=0, z2=2)
ATE_fn <- function(object)
{
   val1 <- predict(object, group = c(0, 2), group2 = 1)
   val0 <- predict(object, group = c(0, 2), group2 = 0)
   val <- mean(val1 - val0)
   
   return(val)
}
ATE_test <- test_msel(object = model, fn = ATE_fn)
summary(ATE_test)

# Use Wald test to test the hypothesis
# H0: beta1 = beta0
coef_fn <- function(object)
{
   coef1 <- coef(object, regime = 1, type = "coef2")
   coef0 <- coef(object, regime = 0, type = "coef2")
   coef_difference <- coef1 - coef0
   
   return(coef_difference)
}
coef_test <- test_msel(object = model, fn = coef_fn, test = "wald")
summary(coef_test)

# Use t-test to test for each 'k' the hypothesis
# H0: beta1k = beta0k
coef_test2 <- test_msel(object = model, fn = coef_fn, test = "t")
summary(coef_test2)

# Use Wald test to test the hypothesis
# H0: beta11 + beta12 - 0.5 = 0
#     beta11 * beta13 - beta03 = 0
test_fn <- function(object)
{
  coef1 <- coef(object, regime = 1, type = "coef2")
  coef0 <- coef(object, regime = 0, type = "coef2")
  val   <- c(coef1[1] + coef1[2] - 0.5, 
  coef1[1] * coef1[3] - coef0[3])
     
  return(val)
}
# classic Wald test
wald1 <- test_msel(object = model,  fn     = test_fn, 
                   test   = "wald", method = "classic")
summary(wald1)
# score bootstrap Wald test
wald2 <- test_msel(object = model,  fn     = test_fn, 
                   test   = "wald", method = "score")
summary(wald2)

# Replicate the latter test with the 2-step estimator
model2 <- msel(list(z1 ~ s1 + s2 | s2 + s3,
                         z2 ~ s1 + s3),
               list(y ~ s1 + s3 + s4),
               groups    = groups, groups2   = groups2, 
               data      = data,   estimator = "2step")
# classic Wald test
wald1_2step <- test_msel(object = model2, fn     = test_fn, 
                         test   = "wald", method = "classic")
summary(wald1_2step)
# score bootstrap Wald test
wald2_2step <- test_msel(object = model2, fn     = test_fn, 
                         test   = "wald", method = "score")
summary(wald2_2step)    

# ---
# Step 4
# Cross-model tests
# ---

# Use Wald test to test the hypothesis that coefficients
# in regime 1 are the same in both models
cmtest_fn <- function(object)
{
  coef1 <- coef(object[[1]], regime = 1, type = "coef2")
  coef2 <- coef(object[[2]], regime = 1, type = "coef2")
  val   <- coef1 - coef2
  
  return(val)
}

# Classic Wald test
test_wald <- test_msel(object = list(model, model2), fn     = cmtest_fn, 
                       test   = "wald",              method = "classic")
summary(test_wald)


# Score bootstrap Wald test
test_score <- test_msel(object = list(model, model2), fn     = cmtest_fn, 
                        test   = "wald",              method = "score")
summary(test_score)

}
             
}
\references{
B. Efron (1982). The Jackknife, the Bootstrap, and Other 
Resampling Plans. Society for Industrial and Applied Mathematics.

B. Hansen (2022). Econometrics. Princeton University Press.

P. Kline, A. Santos (2012). 
A Score Based Approach to Wild Bootstrap Inference.
Journal of Econometric Methods, vol. 67, no. 1, pages 23-41.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.R
\name{print.summary.test_msel}
\alias{print.summary.test_msel}
\title{Print summary for an Object of Class test_msel}
\usage{
\method{print}{summary.test_msel}(x, ..., is_legend = TRUE)
}
\arguments{
\item{x}{object of class 'test_msel'}

\item{...}{further arguments (currently ignored)}

\item{is_legend}{a logical; if \code{TRUE} then additional information
is shown.}
}
\value{
The function returns input argument \code{x}.
}
\description{
Prints summary for an object of class 'test_msel'.
}
